function f=appenddatagui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%make figure:
f=figure('units','pixels','position',[0 0 500 330],'name',...
    'Add Rows to Data Set','numbertitle','off','menubar','none','resize','off','integerhandle','off','handlevisibility','callback');
handles.gui=f;
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
load raveprefs
handles.prefs=raveprefs;
handles.colors=handles.prefs.colors;
handles.fonts=handles.prefs.fonts;

set(f,'color',handles.colors.frame)


%make initial uicontrols:
handles.background=uipanel('units','pixels','position',[50 55 406 220],'visible','on','backgroundcolor',[1 1 1]);
handles.titleaxes=axes('units','pixels','position',[11 280 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265-100-40 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Ok','callback',@donebutton_callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438-100-40 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Cancel','callback',@cancelbutton_callback);

text(0,.2,'Add Rows to Data Set','parent',handles.titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

selectexisting=uicontrol('units','pixel','position',[50 20 200 22],'parent',handles.background,'style','checkbox',...
    'string','Select existing data','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectexisting');

selectnew=uicontrol('units','pixel','position',[50 40 200 22],'parent',handles.background,'style','checkbox',...
    'string','Select newly added data','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectnew');

optionstext=uicontrol('units','pixel','position',[35 60 200 22],'parent',handles.background,'style','text',...
    'string','Options','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

colortestbutton=zeros(1,10);

for c=1:10
    colortestbutton(c)=uicontrol('position',[50+22*c 90 18 18],'parent',handles.background,...
    'style','togglebutton','string','','value',0,'backgroundcolor',handles.prefs.rowcolors(c,:),...
    'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
    'tag','rowcolorblob','userdata',c,'callback',@colorbutton_callback);
end
handles.colortestbutton=colortestbutton;

set(colortestbutton(handles.prefs.defaultrowcolor),'value',1);

colortext=uicontrol('units','pixel','position',[35 110 300 22],'parent',handles.background,'style','text',...
    'string','Row color of newly added rows:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

appendbutton=uicontrol('units','pixel','position',[50 40+120 350 22],'parent',handles.background,'style','radiobutton','value',1,...
    'string','Append new rows to existing data set','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','append','callback',@append_callback);

replacebutton=uicontrol('units','pixel','position',[50 20+120 350 22],'parent',handles.background,'style','radiobutton',...
    'string','Replace data set with newly added rows (delete old data)','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','replace','callback',@append_callback);

optionstext=uicontrol('units','pixel','position',[35 60+120 200 22],'parent',handles.background,'style','text',...
    'string','Append or Replace:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');


% Choose default command line output
handles.output = f;



% Update handles structure
guidata(f, handles);

set(0,'userdata',[]);


function colorbutton_callback(src,ev)
handles=guidata(src);
set(handles.colortestbutton,'value',0)
set(src,'value',1)




function donebutton_callback(src,ev)
handles=guidata(src);

thiscolor=findobj(handles.colortestbutton,'value',1);
outputs.color=get(thiscolor,'userdata');

appendbutton=findobj(handles.gui,'tag','append');

if get(appendbutton,'value')
    outputs.method='append';
else
    outputs.method='replace';
end

selectnew=findobj(handles.gui,'tag','selectnew');
selectexisting=findobj(handles.gui,'tag','selectexisting');

outputs.selectnew=get(selectnew,'value');
outputs.selectexisting=get(selectexisting,'value');

set(0,'userdata',outputs);

close(handles.gui);

function cancelbutton_callback(src,ev)
thisfig=get(src,'parent');
close(thisfig);


function append_callback(src,ev)
handles=guidata(src);
appendbutton=findobj(handles.gui,'tag','append');
replacebutton=findobj(handles.gui,'tag','replace');

set([appendbutton replacebutton],'value',0)
set(src,'value',1)




