function drawlistboxkeypressfcn(pane,ev)
%Keypressfcn for drawlistbox. Allows scrolling.
%NOTE: this function only runs for listboxes whose pane is stored
%in handles.fakerax.


switch ev.Key
    case 'a'
        if ismember(ev.Modifier,'control')
            drawlistboxselectall(pane,ev);
        end
    case 'uparrow'
        drawlistboxselectup(pane,ev);
    case 'downarrow'
        drawlistboxselectdown(pane,ev);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function drawlistboxselectup(pane,ev)
%Select all elements of this listbox.

%How many entries does this listbox have?
tax = findobj(pane,'tag','tax','-depth',1);
ttax = findobj(pane,'tag','ttax','-depth',1);

img = get(tax,'children');
texthandles = findobj(ttax,'tag','texthandles');

selectedrows = get(img,'userdata');
firstselectedrow = find(selectedrows,1);

if firstselectedrow>2
    newselectedrow = firstselectedrow-1;
else
    newselectedrow = 1;
end

if ismember('shift',ev.Modifier) || ismember('control',ev.Modifier') %add to selection
    selectedrows(newselectedrow)=true;
else %start a new selection
    selectedrows = false(size(selectedrows));
    selectedrows(newselectedrow)=true;
end

setlistboxselection(pane,ev,selectedrows)
runlistboxcallback(pane,ev)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function drawlistboxselectdown(pane,ev)
%Select all elements of this listbox.

%How many entries does this listbox have?
tax = findobj(pane,'tag','tax','-depth',1);
ttax = findobj(pane,'tag','ttax','-depth',1);

img = get(tax,'children');
texthandles = findobj(ttax,'tag','texthandles');

selectedrows = get(img,'userdata');
lastselectedrow = find(selectedrows,1,'last');

if lastselectedrow<numel(selectedrows);
    newselectedrow = lastselectedrow+1;
else
    newselectedrow = numel(selectedrows);
end

if ismember('shift',ev.Modifier) || ismember('control',ev.Modifier') %add to selection
    selectedrows(newselectedrow)=true;
else %start a new selection
    selectedrows = false(size(selectedrows));
    selectedrows(newselectedrow)=true;
end

setlistboxselection(pane,ev,selectedrows)
runlistboxcallback(pane,ev)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function drawlistboxselectall(pane,ev)
%Select all elements of this listbox.

%How many entries does this listbox have?
tax = findobj(pane,'tag','tax','-depth',1);
ttax = findobj(pane,'tag','ttax','-depth',1);

img = get(tax,'children');
texthandles = findobj(ttax,'tag','texthandles');

selectedrows = true(size(texthandles));

setlistboxselection(pane,ev,selectedrows)
runlistboxcallback(pane,ev)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function setlistboxselection(pane,ev,selectedrows)
%Sets the listbox whose pane is src to have the indicated rows selected
handles=guidata(pane);

ai = find(handles.fakerax==pane);

if isempty(ai);return;end

udata=get(handles.fakeax(ai),'userdata');

%Get handles:
tax = findobj(pane,'tag','tax','-depth',1);
ttax = findobj(pane,'tag','ttax','-depth',1);

img = findobj(tax,'type','image');
texthandles = findobj(ttax,'tag','texthandles');

%Get row height:
[style,javastyle]=ravegetfontstyle(udata.format.fontstyle);
axiscanvas=ravegetaxiscanvas(handles.fig);
FontMetrics=RaveFontMetrics(axiscanvas.getGraphics,udata.format.fontname,ceil(udata.format.fontsize*4/3),javastyle);
[w,fontheight]=ravecalculatestringlengthfast('A',FontMetrics);
rowheight = fontheight;

%Get colors:
axescolors=handles.colors.axes{udata.format.bg};
bgcolor=axescolors(2,:);
txt=axescolors(5,:);
unselectedcolor=bgcolor;
selectedcolor=udata.format.selectedcolor;

numrows=numel(selectedrows);

t=ones(numrows*rowheight,1,3);
t(:,:,1)=unselectedcolor(1);
t(:,:,2)=unselectedcolor(1);
t(:,:,3)=unselectedcolor(1);

set(img,'userdata',selectedrows);

%color according to row state:
selectedrows=find(selectedrows);
adders=repmat(0:rowheight-1,size(selectedrows,1),1);
pixes=selectedrows(:).*rowheight-rowheight;
pixes=bsxfun(@plus,pixes,adders);
pixes=pixes(:)+1;
t(pixes,:,1)=selectedcolor(1);
t(pixes,:,2)=selectedcolor(2);
t(pixes,:,3)=selectedcolor(3);

%color text:
txts=get(tax,'userdata');
set(txts,'color',txt);

a=[0.299 0.587 0.114]'*selectedcolor;

if a<0.5
    set(txts(selectedrows),'color',[1 1 1])
else
    set(txts(selectedrows),'color',[0 0 0])
end

set(img,'cdata',t);

% set(src,'windowbuttonupfcn',{@listboxdragdone,selectedrows,ai,callback})


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function runlistboxcallback(pane,ev)

%Dig up the callback for this listbox, and run it:
handles=guidata(pane);
ai = find(handles.fakerax==pane);
if isempty(ai);return;end


%Get handles:
tax = findobj(pane,'tag','tax','-depth',1);
img = get(tax,'children');

selectedrows = get(img,'userdata');
buttondownfcn = get(img,'buttondownfcn');
callback = buttondownfcn{3};

if ~iscell(callback)
    callback(img,ai,selectedrows);
else
    callback{1}(img,ai,selectedrows,callback{2:end});
end

%
% image(t,'parent',tax,'tag','mainimage','buttondownfcn',{@selectlistbox,handles.activeindex,callback},
%
%
%
% function listboxdragdone(src,ev,selectedrows,ai,callback)
% fig=ancestor(src,'figure');
% set(fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
% set(fig,'windowbuttonupfcn','')
%
% if ~iscell(callback)
%     callback(src,ai,selectedrows);
% else
%     callback{1}(src,ai,selectedrows,callback{2:end});
% end
