function [axhandle,javahandle]=drawpopupmenu(parent,pos,string,value,callback,backgroundcolor,jobj)
% %DRAWPOPUPMENU draw a popupmenu using non-java objects.
%
% ALL INPUTS ARE REQUIRED (except the last one, background color, is optional)
% parent is the figure or uipanel on which to place the button.
% pos is the position of the button in pixels 
% string is a cell array of strings, containing the options for the popup
%     menu
% value is an integer value indexing into string, indicating the initial
%     entry to be selected in this popup menu
% callback is a function (handle) that runs whenever the pushbutton is clicked.
%     This function must take 2 inputs: the first is the handle of the
%     figure containing the pushbutton, the second is the new value of the popup menu. 
% backgroundcolor is the color of the next visible object behind this,
%     used to recolor the border (if empty, it will be white). 
% jobj is an optional jComboBox (or equivalent) to use as the java popup.
%     If empty, a new one will be created.
% The first output, axhandles, is the axes handle that comprises the
%     popupmenu.
% The second output, javahandle, is an invisible JComboBox that is used to
%     display the popup portion of the menu when it is clicked.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%this function achieves the popupness by using an invisible JComboBox that
%is co-located with the fake popupmenu. Clicking the fake popupmenu makes
%the editor of the JComboBox appear. This seems to achieve the
%desired effect with no nasty side effects...

pos=ceil(pos); %ensure integers
pos(4)=22; %height must be 22 pixels

if numel(value)>1;
    error('drawpopupmenu can only be used with a scalar value. use a listbox for multiple values')
end


%make java box:

%I guess we'll do this the hard way:
j=javax.swing.JComboBox(string);
javaObjectEDT(j);
j.setSelectedIndex(value-1);

[m,mh]=javacomponent(j,[pos(1:3) 0.1],parent);

if ~exist('backgroundcolor','var')
    backgroundcolor=[1 1 1];
end


%make main axes:
a=uipanel('parent',parent,'units','pixels','position',pos,'visible','on',...
    'bordertype','line','highlightcolor',[.5 .5 .5],'backgroundcolor',backgroundcolor,...
    'hittest','on','userdata',callback);

axhandle=a;

ax=axes('parent',a,'units','pixels','position',[0 0 pos(3) pos(4)],'xlim',[0 pos(3)],'ylim',[0 pos(4)],'visible','off',...
    'ydir','reverse');

set(a,'buttondownfcn',{@popupmenu_callback,j})

javahandle=mh;
%set(javahandle,'deletefcn',''); %remove deletefcns that are used in newer matlab versions and dont work in older versions.

if pos(3)<26 
    warning('Size of drawpopupmenu must be at least 26x10 pixels. Not rendering menu.')
    set(a,'visible','off')
    return
end



%Make arrow image:
imdata= imread('popuparrow.png');

i = image('xdata',[pos(3)-16 pos(3)-2],'ydata',[1 20],'cdata',imdata,'parent',ax,'hittest','off');
t = text(3,20,string{value},'parent',ax,'hor','left','vert','bottom','hittest','off','fontsize',8,...
    'fontname','arial');


%make text:

set(m,'ActionPerformedCallback',{@popupactivation,ax});



function popupmenu_callback(src,ev,j)
%This doesnt work, apparently because clicking the matlab image a second
%time hides the popup immediately before this function is called, so
%isPopupVisible is always false
if ~j.isPopupVisible
    j.setPopupVisible(true);
else
    j.setPopupVisible(false);
end


function popupactivation(jsrc,jev,ax)
%i is image. update its cdata...
string=jsrc.getSelectedItem;
value=jsrc.getSelectedIndex+1;

t=findobj(ax,'type','text');

set(t,'string',string);

panel=get(ax,'parent');
callback=get(panel,'userdata');

handles=guidata(panel);

if iscell(callback)
    c=callback{1};
    c(handles.fig,value,callback{2:end});
else
    callback(handles.fig,value);
end
    

% 
% 
% function imdata=maketextimage(string,pos)
% %make imagedata of text in string limited to position pos
% pos=pos-[0 0 22 3];
% f=figure('visible','off','position',[100 100 pos(3) pos(4)],'toolbar','none','menubar','none','color',[1 1 1]);
% a=axes('units','pixels','position',[0 0 pos(3:4)],'ylimmode','manual','xlimmode','manual','visible','off');
% text(0,0.5,string,'fontsize',9,'interpreter','none')
% resolution=get(0,'ScreenPixelsPerInch');
% set(f,'papersize',pos(3:4)/resolution,'paperposition',[0 0 pos(3:4)/resolution])
% imdata=hardcopy(f,'-dzbuffer','-r96');
% imdata=double(imdata)./255;
% delete(f);


