function controls=makeuicontrolsuserfcn(src,ev)
%Run User-Supplied Function
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2015 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
origin=get(handles.tabaxes,'position');

%Initialize properties:
if ~isfield(udata.explore,'userfcn')
    udata.explore.userfcn.functions={''};
    udata.explore.userfcn.selectedfunctions=[];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

ctext=text(.10,.87,'Select functions to run:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
    'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

if isempty(udata.explore.userfcn.functions)
    funstring='No functions have been added yet.';
else
    funstring=[udata.explore.userfcn.functions(:)];
end

functionmenu=uicontrol('position',[origin(1)+30 origin(2)+320 190 220],...
    'style','listbox','string',funstring,...
    'callback',@functionmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','functionmenu','max',100,'value',udata.explore.userfcn.selectedfunctions+1);


upbutton=uicontrol('position',[origin(1)+222 origin(2)+477 35 59],...
    'style','pushbutton','string','^','backgroundcolor',handles.colors.buttonbg,...
    'callback',@upbutton_callback);
downbutton=uicontrol('position',[origin(1)+222 origin(2)+416 35 59],...
    'style','pushbutton','string','v','backgroundcolor',handles.colors.buttonbg,...
    'callback',@downbutton_callback);

addbutton=uicontrol('position',[origin(1)+40 origin(2)+290 80 24],...
    'style','pushbutton','string','Add...','backgroundcolor',handles.colors.buttonbg,...
    'callback',@addbutton_callback);
removebutton=uicontrol('position',[origin(1)+130 origin(2)+290 80 24],...
    'style','pushbutton','string','Remove','backgroundcolor',handles.colors.buttonbg,...
    'callback',@removebutton_callback);

%update tabobjects:
controls=[ctext functionmenu upbutton downbutton addbutton removebutton];

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function functionmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.explore.userfcn.selectedfunctions=get(src,'value')-1; %indicies of constraints drawn on this graph.

if any(udata.explore.userfcn.selectedfunctions>0)
    udata.explore.method=union(udata.explore.method,{'userfcn'});  
else
     udata.explore.method=setdiff(udata.explore.method,{'userfcn'}); 
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function upbutton_callback(src,ev)
%move all selected variables up one spot.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
functionmenu = findobj(handles.tabobjects,'tag','functionmenu');

selected = get(functionmenu,'value');
selected = setdiff(selected,1);
if isempty(selected);return;end


varorder=1:numel(udata.explore.userfcn.functions);

%First selected entry:
topvar=selected(1);

if topvar>2
    othervars=varorder(topvar-1:end);
    newvarorder=[varorder(1:topvar-2) selected othervars(~ismember(othervars,selected))];
else
    othervars=varorder;
    newvarorder=[selected othervars(~ismember(othervars,selected))];
end

udata.explore.userfcn.functions=udata.explore.userfcn.functions(newvarorder);
set(functionmenu,'string',udata.explore.userfcn.functions,'value',find(ismember(newvarorder,selected)))
set(handles.fakeax(handles.activeindex),'userdata',udata);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function downbutton_callback(src,ev)
%move all selected variables up one spot.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
functionmenu = findobj(handles.tabobjects,'tag','functionmenu');

selected = get(functionmenu,'value');
selected = setdiff(selected,1);
if isempty(selected);return;end

varorder=1:numel(udata.explore.userfcn.functions);

%First selected entry:
botvar=selected(end);

if botvar<(numel(varorder)-1)
    othervars=varorder(1:botvar+1);
    newvarorder=[othervars(~ismember(othervars,selected)) selected varorder(botvar+2:end)];
else
    othervars=varorder;
    newvarorder=[othervars(~ismember(othervars,selected)) selected];
end

udata.explore.userfcn.functions=udata.explore.userfcn.functions(newvarorder);
set(functionmenu,'string',udata.explore.userfcn.functions,'value',find(ismember(newvarorder,selected)))
set(handles.fakeax(handles.activeindex),'userdata',udata);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function downbutton2_callback(src,ev)
%move all selected variables down one spot.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
varorder=udata.view.varorder{d};
botvar=find(ismember(varorder,udata.view.x),1,'last');

if botvar<(numel(varorder)-1)
    othervars=varorder(1:botvar+1);
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x varorder(botvar+2:end)];
else
    othervars=varorder;
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x];
end

udata.view.varorder{d}=newvarorder;
set(handles.fakeax(handles.activeindex),'userdata',udata);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function addbutton_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

[name,path]=uigetfile('.m');
if ~ischar(path)
    return
end

if isempty(which(name))
    addpath(path);    
end

if ~strcmp(fullfile(path,name),which(name))
    errordlg('Could not add this function because there is another function with the same name higher on your Matlab search path. Consequently there is no way for Matlab to access the selected function.')    
    return
end

udata.explore.userfcn.functions{end+1} = name;


functionmenu = findobj(handles.tabobjects,'tag','functionmenu');
set(functionmenu,'string',udata.explore.userfcn.functions);
set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function removebutton_callback(src,ev)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

functionmenu = findobj(handles.tabobjects,'tag','functionmenu');

val = get(functionmenu,'value');
val = setdiff(val,1);

str = get(functionmenu,'string');

keep = setdiff(1:numel(str),val);

str = str(keep);
set(functionmenu,'string',str,'value',1);
udata.explore.userfcn.functions = str;

set(handles.fakeax(handles.activeindex),'userdata',udata);


