function ravecreatescrollabletext(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;
xcol=udata.view.x;


set(handles.activegraph,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')

%process expression:
indata=udata.view.expression;

%look for file references:
matches=parenthesesmatch(indata,'{','}');
linenumbers=parenthesesmatch(indata,'[',']');

txts={};

for f=1:size(matches,1)
    fid=fopen(indata(matches(f,1)+1:matches(f,2)-1));
    if fid>0
        
        %look for line numbers:
        idx=linenumbers(:,1)==matches(f,2)+1;
        idx=linenumbers(idx,:);
        if ~isempty(idx)
            theselines=indata(idx(1)+1:idx(2)-1);
            theselines=strrep(theselines,'-',':');
            theselines=eval(theselines);
            start=theselines(1);
            finish=theselines(end);
        else
            start=1;
            finish=inf;
        end
        
        tline=fgetl(fid);
        txts{f}={};
        count=1;
        while ischar(tline) && count<=finish;
            txts{f}{end+1}=tline;
            tline=fgetl(fid);
            count=count+1;
        end
        
        txts{f}=txts{f}(start:end);
        
        
    else
        txts{f}={'(File Not Found)'};
    end
end

%allocate strings for txts:
strs=cell(numel(txts),1);

for f=1:numel(strs)
    
    bits=cellfun(@length,txts{f});
    bitz=[0 cumsum(bits+1)];
    strs{f}=char(zeros(1,sum(bits)+numel(txts{f})));
    for s=1:numel(txts{f});
        strs{f}(bitz(s)+1:bitz(s+1))=[txts{f}{s},char(10)];
    end
end

%insert file text into indata:
if numel(strs)==0
    fulldata=indata;
else
    fulldata=indata(1:matches(1)-1);
    for s=1:numel(strs)
        %determine whether to stop after the filename or after the line
        %numbers:
        %look for line numbers:
        idx=linenumbers(:,1)==matches(s,2)+1;
        if isempty(idx)
            startat=matches(s,2)+1;
        else
            startat=linenumbers(s,2)+1;
        end
        
        if s<numel(strs)
            stopat=matches(s+1,1)-1;
        else
            stopat=length(indata);
        end
        fulldata=[fulldata,strs{s}(1:end-1),indata(startat:stopat)];
    end
    
end

[style,javastyle]=ravegetfontstyle(udata.format.fontstyle);

%replace tabs with spaces since tabs arent directly supported
%TO DO: smartly insert a variable number of spaces to make tabs works like tabs.
fulldata=strrep(fulldata,char(9),'    ');



%Font size interpolation info
axiscanvas=ravegetaxiscanvas(handles.fig);
FontMetrics=RaveFontMetrics(axiscanvas.getGraphics,udata.format.fontname,ceil(udata.format.fontsize*4/3),javastyle);
[w,fontheight]=ravecalculatestringlengthfast(fulldata,FontMetrics);

%do wrapping:

if udata.format.wordwrap
    axpos=get(handles.fakerax(handles.activeindex),'position');
    maxwidth=axpos(3)-5;
    %from raveresizetable:
    bpos=get(handles.allblockers(handles.activeindex),'position');
    
    if isempty(udata.format.title)
        maxpos=bpos+[24 26 -46 -41];
    else
        maxpos=bpos+[24 26 -46 -61];
    end
    
    maxwidth=maxpos(3)-5;
    
    
    [fulldata,status,nrows,finalwidth]=superiorlinewrapfast(fulldata,maxwidth,FontMetrics);
else
    fulldata={fulldata};
end

fulldata=regexp(fulldata,['\',char(10)],'split'); %also wrap at new lines
if iscell(fulldata{1})
    fulldata=[fulldata{:}]; %peel off outermost cell array if regexp added a layer of cells
end

%ensure column:
fulldata=fulldata(:);

%determine how tall to make the table:
nrows=numel(fulldata);
height=fontheight*nrows+5;

%determine how wide to make table:
linelengths=ravecalculatestringlengthfast(fulldata,FontMetrics);
maxline=max(linelengths);

width=maxline+10;

if width<98;width=98;end

%                                                   data,rownames,rowheight,colnames,colwidths,headerwidth,headerheight,rowcolors,rhcolors);
ravemaketablecore(handles.fig,[],handles.activeindex,fulldata,{},fontheight,{},width,0.1,0.1,udata.format.bgcolor,[1 1 1]);

udata=get(handles.fakeax(handles.activeindex),'userdata');

%realign text to top left corner:
t=udata.view.texthandles;
%set(t,'hor','left','ver','top')
%set(t,'position',[4 0 0]);

%apply font style:
set(t,style{:});

set(handles.fakeax(handles.activeindex),'hittest','on','buttondownfcn','')
set(handles.fakerax(handles.activeindex),'backgroundcolor',udata.format.bgcolor);

raveresizetable(handles.fig,[],handles.activeindex);
%fix scrolling
verttablescroll(handles.activegraph,[],0)



