function h=ravedrawconstraintderivativeprofiler(src,ai,selectedconstraints) %#ok<INUSD>
%note: src MUST be the handle of the graph in which to draw the constraint.
% ai is the index of the graph containing the axes in src (as the main axes or a subax)
% thisconstraint is an index into the constraint list.
% h is the handles of all lines created by this function.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

h=[];

handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;


for q=1:length(selectedconstraints)
    thisconstraint=selectedconstraints(q);
    if thisconstraint==0;continue;end
    
    % xcol=udata.view.x;
    % ycol=udata.view.y;
    
    % h=ravedrawconstraint1D(src,xcol,ycol,d,a,thisconstraint,ai);
    
    %delete old constraints:
    oldconstraints=findobj(handles.subax{ai},'tag','constraint');
    delete(oldconstraints);
    
    xcol=udata.view.x;
    numx=numel(xcol);
    
    datatypes=handles.datatype{d}(xcol);
    numpts=udata.view.numpts;
    xdials=handles.xdials{d}{a};
    
    [xsteps totalrows]=ravecalculatexsteps(handles.fig,d,a,xcol,numpts,1);
    
    rowz=sum(totalrows);
    xfixed=repmat(xdials,[rowz,1]);
    
    %convert to cell if strings are present:
    if any(strcmp('string',datatypes))
        xfixed=num2cell(xfixed);
        for k=1:numx
            [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
        end
        
    else
        for k=1:numx
            xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
        end
    end
    
    
    Z=raveevaluateconstraint(handles,d,a,thisconstraint,xfixed,false(1,size(xfixed,2)));
    
    %make cell array for each x:
    Z=mat2cell(Z,totalrows,1);
    xfixed=mat2cell(xfixed,totalrows,size(xfixed,2));
    
    
    %draw constraint:
    for c=1:numx
        thisx=xcol(c);
        xdata=xfixed{c}(:,thisx);
        zdata=Z{c};
        
        %determine number of flips from feasible to infeasible (or infeas to feas)
        fliplocations=find(diff(sign(zdata')));
        thislocation=zeros(size(fliplocations));
        
        % fliplocations=sort([fliplocations fliplocations+1]);
        set(handles.subfakeax{ai}(:,c),'color','none')
        if isempty(fliplocations)
            fliplocations=[];%this avoids a warning below...
            
            if zdata(1)>0 %all points are infeasible
                set(handles.subfakeax{ai}(:,c),'color',handles.constraintcolors{d}{a}{thisconstraint})
            end
            
            
        else
            %interpolate to find actual location:
            for b=1:numel(fliplocations)
                thislocation(b)=interp1(zdata([fliplocations(b) fliplocations(b)+1]),xdata([fliplocations(b) fliplocations(b)+1]),0);
            end
        end
        
        %do stuff related to feasibility at end poitns:
        xlim=get(handles.subax{ai}(1,c),'xlim');
        startfeas=true;
        if zdata(1)>0
            startfeas=false;
            thislocation=[xlim(1) thislocation]; %#ok<AGROW>
        end
        
        if zdata(end)>0
            thislocation=[thislocation xlim(2)]; %#ok<AGROW>
        end
        
        %draw the constraints:
        
        
        
        for r=1:size(handles.subax{ai},1)
            index=1;
            
            
            while index<numel(thislocation)
                
                
                ylim=get(handles.subax{ai}(r,c),'ylim');
                p=patch(thislocation([index index+1 index+1 index]),ylim([1 1 2 2]),...
                    handles.constraintcolors{udata.view.dataset}{udata.view.analysis}{thisconstraint},...
                    'parent',handles.subax{ai}(r,c),'tag','constraint','hittest','off','edgecolor','none');
                uistack(p,'bottom');
                index=index+2;
            end
            
        end
        
    end
    
    
end

