function ravecreatecarpet2d(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z; %"sweep through values of"
ccol=udata.view.c; %"draw a line for each value of"

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontsize',14,'tag','title')

sweepvar1=zcol;
[sweepvalues1,totalrows]=ravecalculatexsteps(handles.fig,d,a,sweepvar1,udata.view.numpts1,0);
sweepvalues1=sweepvalues1{1};

sweepvar2=ccol;
if ccol==0
    sweepvalues2=[1];
else
    [sweepvalues2,totalrows]=ravecalculatexsteps(handles.fig,d,a,sweepvar2,udata.view.numpts2,0);
    sweepvalues2=sweepvalues2{1};
end

if numel(sweepvalues1)==1;
    sweepvalues1=[sweepvalues1 sweepvalues1];
end

if numel(sweepvalues2)==1;
    sweepvalues2=[sweepvalues2 sweepvalues2];
end
% 
% if ccol~=zcol && ccol~=0 && ~isempty(udata.view.steps)
%     zcalldata=repmat(sweepvalues1(:),numel(udata.view.steps),1);
%     stepdata=repmat(udata.view.steps(:)',numel(sweepvalues1),1);
%     stepdata=stepdata(:);
%     
% else
%     zcalldata=sweepvalues1(:);
%     stepdata=[];
% end

[S1,S2]=meshgrid(sweepvalues1,sweepvalues2);

S1=S1(:);
S2=S2(:);

constants=repmat(handles.xdials{d}{a},[length(S1),1]);
constants(:,zcol)=S1;
if ccol~=0
    constants(:,ccol)=S2;
end

%evaluate the functions:

outvalues=raveevaluatefunction(src,d,[udata.view.x udata.view.y],constants,handles);
xvalues=outvalues(:,1);
yvalues=outvalues(:,2);

[S1,S2]=meshgrid(sweepvalues1,sweepvalues2);

Xvalues=reshape(xvalues,size(S1));
Yvalues=reshape(yvalues,size(S2));

Zvalues=zeros(size(S1));

% constants(:,xcol)=xvalues;
% constants(:,ycol)=yvalues;
%
% ncurves=numel(zcalldata)./numel(sweepvalues1);
% npts=numel(sweepvalues1);

%if ccol~=0

%normalize cdata to desired colormap:
if ~udata.format.reversecolors1
    C1=(S1-min(S1(:)))/(max(S1(:))-min(S1(:))).*127+1+128.*(udata.format.mcmap1-1);
else
    C1=-(S1-min(S1(:)))/(max(S1(:))-min(S1(:)))*127+128*(udata.format.mcmap1);
end


%normalize cdata to desired colormap:
if ~udata.format.reversecolors2
    C2=(S2-min(S2(:)))/(max(S2(:))-min(S2(:))).*127+1+128.*(udata.format.mcmap2-1);
else
    C2=-(S2-min(S2(:)))/(max(S2(:))-min(S2(:)))*127+128*(udata.format.mcmap2);
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

surf1=surface(Xvalues,Yvalues,Zvalues,'parent',targetaxes,'facecolor','none','edgecolor','flat','tag','main1','meshstyle','column','cdata',C1,'linewidth',udata.format.linesize);
surf2=surface(Xvalues,Yvalues,Zvalues,'parent',targetaxes,'facecolor','none','edgecolor','flat','tag','main2','meshstyle','row','cdata',C2,'linewidth',udata.format.linesize);

udata.view.surfacehandles=[surf1 surf2];

if ~udata.view.isvisible1;set(surf2,'visible','off');end
if ~udata.view.isvisible2;set(surf1,'visible','off');end

%make current point:
cp=handles.xdials{d}{a};
cpz=raveevaluatefunction(src,d,[xcol ycol],cp,handles);
udata.view.currentpointhandle=line(cpz(1),cpz(2),10,'marker','o','markeredgecolor',[0 0 0],'markerfacecolor',[1 1 1],...
    'parent',targetaxes,'markersize',6);

if ~udata.view.showcurrentpoint
    set(udata.view.currentpointhandle,'visible','off')
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

xlim=[handles.minlimit{d}{a}(udata.view.x) handles.maxlimit{d}{a}(udata.view.x)];
ylim=[handles.minlimit{d}{a}(udata.view.y) handles.maxlimit{d}{a}(udata.view.y)];
set(targetaxes,'xlim',[xlim(1)-0.02*diff(xlim) xlim(2)+0.02*diff(xlim)],'ylim',[ylim(1)-0.02*diff(ylim) ylim(2)+0.02*diff(ylim)]);

xlim = get(targetaxes,'xlim');
ylim = get(targetaxes,'ylim');

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',ylim);
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',ylim);

%% make tick marks:

fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},...
    'fontweight',fweight,'interpreter','none','fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontweight',fweight,'interpreter','none','fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','',...
        'fontweight',fweight,'interpreter','none','fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)
ravemakedoublecolorbar(handles.fig,[]);
ravemaketextticks(targetaxes,[],xcol(1),ycol,[],[])
raveresizeaxes(src,[],targetindex)

%draw constraints:

constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects





