function ravecreatecontour(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

%set(targetaxes,'drawmode','fast');

xcol=udata.view.x;
ycol=udata.view.y;

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

%prepare data for gridding:
if isempty(xcol);
    return %no functions loaded.
end

xcol=[xcol ycol];

if ~udata.view.doadaptive %uniform grid sampling
    numpts=udata.view.smooth;
else
    numpts=udata.view.asmooth;
end

%Make initial grid:
xdials=handles.xdials{d}{a};
[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(3,1);
thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,xcol(2))=ycalldata;

%evaluate the functions:
zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata(3);
zdataz=zdataz(:);
zdataz=cell2mat(zdataz);
zdataz=zdataz(:);

doupdate=0;
if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end
if doupdate;guidata(handles.fig,handles);end;

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);

udata.view.clim=[lowestvalue highestvalue];
set(handles.fakeax(handles.activeindex),'userdata',udata);

zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);


if udata.view.doadaptive
    x=thisxdata(:);
    y=thisydata(:);

    for q=1:udata.view.adaptivelevels+1
        [xgrid,ygrid]=meshgrid(x,y);
        
        C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
        if q==udata.view.adaptivelevels+1;break;end
        
        %Divide contour matrix into individual contours:
        conts=cell(0);
        I=size(C,2);
        i=1;
        while i<I
            level=C(1,i);
            ncols=C(2,i);
            thisx=C(1,i+1:i+ncols);
            thisy=C(2,i+1:i+ncols);
            conts{end+1}=[thisx;thisy]';
            i=i+ncols+1;
        end
        
        %% Determine which edges are crossed by contours:
        
        %make a grid to represent the edges:
        horizedges=false(numel(y),numel(x)-1); %note reversal of x/y to match plot axes
        vertedges=false(numel(y)-1,numel(x));
        
        % The points in conts{} are intersections where the contour crosses an
        % edge. Since each edge has a constant x or constant y value, we can check
        % which edge was crossed by looking for the x or y value of each vertex in
        % conts that is approximately == to one of the discrete levels of x/y:
        for c=1:numel(conts) %for each contour...
            for v=1:size(conts{c},1) %for each vertex...
                %which x/y value is most similar to this vertex?
                vtx=conts{c}(v,:);
                diffsx=x-vtx(1);
                diffsy=y-vtx(2);
                
                [minx,idxx]=min(abs(diffsx));
                [miny,idxy]=min(abs(diffsy));
                
                if minx<miny %x is constant
                    %find largest y that is smaller than vtx(2)
                    idxy=find(y<=vtx(2),1,'last');
                    vertedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
                else %y is constant
                    idxx=find(x<=vtx(1),1,'last');
                    horizedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
                end
                
            end
        end
        
        %Note that horizedges and vertedges are flipud compared to the plotted
        %contours (i.e. y increases bottom to top)
        
        %Loop through all grid squares. If any edge of the grid square is
        %intersected by the contour, refine that square:
        
        dorefine=false(numel(y)-1,numel(x)-1);
        
        for r=1:numel(y)-1
            for c=1:numel(x)-1
                if vertedges(r,c) || vertedges(r,c+1) || horizedges(r,c) || horizedges(r+1,c)
                    dorefine(r,c)=true;
                end
            end
        end
        
        %Squares to refine:
        [R,C]=find(dorefine);
        
        %Determine where to place new refinement points:
        R=2*R-1;
        C=2*C-1;
        
        newpts=[R+1 C;R C+1; R+1 C+2; R+2 C+1; R+1 C+1];
        idx=sub2ind([2*numel(y)-1,2*numel(x)-1],newpts(:,1),newpts(:,2));
        
        %Refine grid
        newx=[x x+[diff(x)/2;0]]';
        newy=[y y+[diff(y)/2;0]]';
        
        newx=newx(:);
        newy=newy(:);
        
        [newX,newY]=meshgrid(newx(1:end-1),newy(1:end-1));
        
        %Use linear interpolation to expand original Z matrix (no new information added)
        Z=interp2(xgrid,ygrid,allthedata{3},newX,newY);
        
        %Resample space at the new refinement points (new information added)
        constants=repmat(xdials,[length(idx),1]);
        constants(:,xcol(1))=newX(idx);
        constants(:,xcol(2))=newY(idx);
        
        %evaluate the functions:
        zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);
        Z(idx)=zvalues;

        allthedata{1}=newX;
        allthedata{2}=newY;
        allthedata{3}=Z;
        
        zdataz=allthedata(3);
        zdataz=zdataz(:);
        zdataz=cell2mat(zdataz);
        zdataz=zdataz(:);
        
        doupdate=0;
        if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
        if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end
        if doupdate;guidata(handles.fig,handles);end;
        
        lowestvalue=handles.minlimit{d}{a}(udata.view.z);
        highestvalue=handles.maxlimit{d}{a}(udata.view.z);
        
        levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
        
        udata.view.clim=[lowestvalue highestvalue];
        set(handles.fakeax(handles.activeindex),'userdata',udata);
        
        zplotdata=allthedata{3};
        if ~udata.format.reversecolors
            zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        else
            zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        end
        
        
        %Go to next iteration
        x=newx(1:end-1);
        y=newy(1:end-1);
        X=newX;
        Y=newY;
        
    end

end


udata.view.funcalls=numel(zdataz);
set(handles.infobar,'string',['Total number of sampled points = ',num2str(numel(zdataz))]);


%initialize empty lines:
for i=numel(levelz):-1:1
    mainhandles(i)=line(nan,nan,'parent',targetaxes,'linewidth',udata.format.linesize,'hittest','off');
end

%fill with actual data:
contourplot(C,mainhandles,'update');
clear contourplot %clear persistent vars;

%use single precision axes limits for correct positioning on screen:
xlimz=single([allthedata{1}(1,1) allthedata{1}(1,end)]);
ylimz=single([allthedata{2}(1,1) allthedata{2}(end,1)]);

set(targetaxes,'xlim',xlimz,'ylim',ylimz,'xlimmode','manual','ylimmode','manual','xgrid','off','ygrid','off','layer','top')


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};


%make tracers
if udata.format.tracersize>0
    w=udata.format.tracersize;
else
    w=1;
end
L1=line([xdials(xcol(1)) xdials(xcol(1))],[xmin(xcol(2)) xmax(xcol(2))],'parent',targetaxes,'hittest','off',...
    'linestyle',udata.format.linestyle,'linewidth',w,'color',udata.format.tracercolor);
L2=line([xmin(xcol(1)) xmax(xcol(1))],[xdials(xcol(2)) xdials(xcol(2))],'parent',targetaxes,'hittest','off',...
    'linestyle',udata.format.linestyle,'linewidth',w,'color',udata.format.tracercolor);
if udata.format.tracersize==0
    set([L1,L2],'visible','off')
end

xtracerhandles=L1;
ytracerhandles=L2;

if ~udata.view.showcurrentpoint
   set([xtracerhandles ytracerhandles],'visible','off'); 
end


xlim = get(targetaxes,'xlim');
ylim = get(targetaxes,'ylim');

udata.format.xlim=get(targetaxes,'xlim');
udata.format.ylim=get(targetaxes,'ylim');


udata.view.mainhandles=mainhandles;
udata.view.xhair=xtracerhandles;
udata.view.yhair=ytracerhandles;

set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',ylim);
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',ylim);


%% make tick marks:

labelsize=12;
fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',labelsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
end


%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)

set(targetaxes,'hittest','on','buttondownfcn',@buttondownfcncontour);

%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.

ravemakecolorbar(handles.fig,[]);


%draw constraints:
constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end

ravemaketextticks(targetaxes,[],udata.view.x,udata.view.y,[],[])
raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




