function handles=raveupdatecontourxdials(src,ev,handles,thesevars)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%handles=guidata(src);
getudata;
ai=handles.activeindex;
targetaxes=handles.allgraphs(ai);
xcol=udata.view.x;
ycol=udata.view.y;

d=udata.view.dataset;
a=udata.view.analysis;



xdials=handles.xdials{d}{a};
set(udata.view.xhair,'xdata',[xdials(xcol) xdials(xcol)])
set(udata.view.yhair,'ydata',[xdials(ycol) xdials(ycol)])

xcol=[xcol ycol];

%prepare data for gridding:
if ~udata.view.doadaptive %uniform grid sampling
    numpts=udata.view.smooth;
else
    numpts=udata.view.asmooth;
end

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

%precompute data so contour levels can be shared among all plots:
allthedata=cell(1,3);
thisxdata=xsteps{1};
thisydata=xsteps{2};

[xgrid,ygrid]=meshgrid(thisxdata,thisydata);

xcalldata=xgrid(:);
ycalldata=ygrid(:);

constants=repmat(xdials,[length(xcalldata),1]);
constants(:,xcol(1))=xcalldata;
constants(:,ycol)=ycalldata;

%evaluate the functions:
zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);

allthedata{1}=reshape(xcalldata,numel(thisydata),numel(thisxdata));
allthedata{2}=reshape(ycalldata,numel(thisydata),numel(thisxdata));
allthedata{3}=reshape(zvalues,numel(thisydata),numel(thisxdata));

zdataz=allthedata{3};
zdataz=zdataz(:);

doupdate=0;
if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end

lowestvalue=handles.minlimit{d}{a}(udata.view.z);
highestvalue=handles.maxlimit{d}{a}(udata.view.z);

levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
udata.view.levels=levelz;

zplotdata=allthedata{3};
if ~udata.format.reversecolors
    zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
else
    zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
end

C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);

if udata.view.doadaptive
    x=thisxdata(:);
    y=thisydata(:);
    
    for q=1:udata.view.adaptivelevels+1
        [xgrid,ygrid]=meshgrid(x,y);
        
        C=contourc(allthedata{1}(1,:),allthedata{2}(:,1),zplotdata,levelz);
        if q==udata.view.adaptivelevels+1;break;end
        
        %Divide contour matrix into individual contours:
        conts=cell(0);
        I=size(C,2);
        i=1;
        while i<I
            level=C(1,i);
            ncols=C(2,i);
            thisx=C(1,i+1:i+ncols);
            thisy=C(2,i+1:i+ncols);
            conts{end+1}=[thisx;thisy]';
            i=i+ncols+1;
        end
        
        %% Determine which edges are crossed by contours:
        
        %make a grid to represent the edges:
        horizedges=false(numel(y),numel(x)-1); %note reversal of x/y to match plot axes
        vertedges=false(numel(y)-1,numel(x));
        
        % The points in conts{} are intersections where the contour crosses an
        % edge. Since each edge has a constant x or constant y value, we can check
        % which edge was crossed by looking for the x or y value of each vertex in
        % conts that is approximately == to one of the discrete levels of x/y:
        for c=1:numel(conts) %for each contour...
            for v=1:size(conts{c},1) %for each vertex...
                %which x/y value is most similar to this vertex?
                vtx=conts{c}(v,:);
                diffsx=x-vtx(1);
                diffsy=y-vtx(2);
                
                [minx,idxx]=min(abs(diffsx));
                [miny,idxy]=min(abs(diffsy));
                
                if minx<miny %x is constant
                    %find largest y that is smaller than vtx(2)
                    idxy=find(y<=vtx(2),1,'last');
                    vertedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
                else %y is constant
                    idxx=find(x<=vtx(1),1,'last');
                    horizedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
                end
                
            end
        end
        
        %Note that horizedges and vertedges are flipud compared to the plotted
        %contours (i.e. y increases bottom to top)
        
        %Loop through all grid squares. If any edge of the grid square is
        %intersected by the contour, refine that square:
        
        dorefine=false(numel(y)-1,numel(x)-1);
        
        for r=1:numel(y)-1
            for c=1:numel(x)-1
                if vertedges(r,c) || vertedges(r,c+1) || horizedges(r,c) || horizedges(r+1,c)
                    dorefine(r,c)=true;
                end
            end
        end
        
        %Squares to refine:
        [R,C]=find(dorefine);
        
        %Determine where to place new refinement points:
        R=2*R-1;
        C=2*C-1;
        
        newpts=[R+1 C;R C+1; R+1 C+2; R+2 C+1; R+1 C+1];
        idx=sub2ind([2*numel(y)-1,2*numel(x)-1],newpts(:,1),newpts(:,2));
        
        %Refine grid
        newx=[x x+[diff(x)/2;0]]';
        newy=[y y+[diff(y)/2;0]]';
        
        newx=newx(:);
        newy=newy(:);
        
        [newX,newY]=meshgrid(newx(1:end-1),newy(1:end-1));
        
        %Use linear interpolation to expand original Z matrix (no new information added)
        Z=interp2(xgrid,ygrid,allthedata{3},newX,newY);
        
        %Resample space at the new refinement points (new information added)
        constants=repmat(xdials,[length(idx),1]);
        constants(:,xcol(1))=newX(idx);
        constants(:,xcol(2))=newY(idx);
        
        %evaluate the functions:
        zvalues=raveevaluatefunction(src,d,udata.view.z,constants,handles);
        Z(idx)=zvalues;
        
        allthedata{1}=newX;
        allthedata{2}=newY;
        allthedata{3}=Z;
        
        zdataz=allthedata(3);
        zdataz=zdataz(:);
        zdataz=cell2mat(zdataz);
        zdataz=zdataz(:);
        
        doupdate=0;
        if min(zdataz)<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=min(zdataz);doupdate=1;end
        if max(zdataz)>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=max(zdataz);doupdate=1;end
        if doupdate;guidata(handles.fig,handles);end;
        
        lowestvalue=handles.minlimit{d}{a}(udata.view.z);
        highestvalue=handles.maxlimit{d}{a}(udata.view.z);
        
        levelz=linspace(1,128,udata.view.ncont)+128*(udata.format.mcmap-1);
        
        udata.view.clim=[lowestvalue highestvalue];
        set(handles.fakeax(handles.activeindex),'userdata',udata);
        
        zplotdata=allthedata{3};
        if ~udata.format.reversecolors
            zplotdata=(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        else
            zplotdata=128-(zplotdata-lowestvalue)/(highestvalue-lowestvalue)*127+128*(udata.format.mcmap-1)+1;
        end
        
        
        %Go to next iteration
        x=newx(1:end-1);
        y=newy(1:end-1);
        X=newX;
        Y=newY;
        
    end
    
end


%Update the plot:
contourplot(C,udata.view.mainhandles,'update');

if doupdate
    guidata(src,handles); %save now in case an error happens later.
end

%udpate colorbar:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[lowestvalue highestvalue];
    udata.view.clim=newlim;
    ylim=get(cbar,'ylim');
    if ~all(oldlim==newlim)
        raveupdatecolorbar(handles.fig,[],ai);
        legdata=get(handles.fakerax(ai),'userdata');
        cbar=findobj(legdata,'tag','Colorbar');
    end
    
    %determine current c value:
    thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis},handles);
    thiscoord=interp1(newlim,ylim,thisvalue);
    
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    if ~isempty(cvtext)
        pos=get(cvtext,'pos');
        set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
    end
end

%% draw constraints:
constraints=udata.view.constraints(udata.view.constraints~=0);

oldconstraints=findobj(handles.allgraphs(ai),'tag','constraint');
for i=1:numel(oldconstraints)
    ldata=get(oldconstraints(i),'userdata');
    delete(ldata.handles)
end

for i=1:numel(constraints)
    thisconstraint=constraints(i);
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end

