function viewtabcontour(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;


%% text headers

projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);


%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists
xvars=handles.funvars{udata.view.dataset}{udata.view.z(1)};
for yy=1:length(udata.view.z)
    xvars=union(xvars,handles.funvars{udata.view.dataset}{udata.view.z(yy)});
end

xvars=find(~handles.isconstant{d} & ~handles.isfunction{d});

allnames=handles.allnames{d};
xvarnames=allnames(xvars); 

xaxistext=text(.25-.10317,.72*1.0317-.0317,'X-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
yaxistext=text(.25-.10317,.65*1.0317-.0317,'Y-Axis Variable:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
zaxistext=text(.25-.10317,.58*1.0317-.0317,'Draw contours of:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+405 180 30],...
    'style','popupmenu','string',xvarnames,'value',find(ismember(xvarnames,allnames(udata.view.x))),...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',1,'callback',@xaxismenu_callback);

yaxismenu=uicontrol('position',[origin(1)+55 origin(2)+360 180 30],...
    'style','popupmenu','string',xvarnames,'value',find(ismember(xvarnames,allnames(udata.view.y))),...
    'backgroundcolor',[1 1 1],'tag','yaxismenu','max',1,'callback',@yaxismenu_callback);

zaxismenu=uicontrol('position',[origin(1)+55 origin(2)+315 180 30],...
    'style','popupmenu','string',handles.allnames{d}(logical(handles.isfunction{d})),...
    'backgroundcolor',[1 1 1],'tag','zaxismenu','callback',@zaxismenu_callback,'value',find(ismember(find(handles.isfunction{d}),udata.view.z)));

layouthandles=[xaxistext xaxismenu zaxistext zaxismenu yaxistext yaxismenu];

if udata.view.ncont>20
    value=(udata.view.ncont-20)/2+20;
else
    value=udata.view.ncont;
end

nconttext=uicontrol('position',[origin(1)+37 origin(2)+280-85+90 150 20],...
    'style','text','string','Number of contours:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ncont=uicontrol('position',[107+3.6*value origin(2)+235-85+90 50 20],...
    'style','text','string',num2str(udata.view.ncont),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');


[jslider1,hslider1]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+255-85+90 190 30]);
jslider1.setPaintTicks(1)

jslider1.setSnapToTicks(0)
jslider1.setMinimum(2)
jslider1.setMaximum(50)
jslider1.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider1.setValue(value);
set(jslider1,'background',java.awt.Color.white)
set(jslider1,'StateChangedCallback',{@ncont_Callback,ncont});

if udata.view.smooth>20
    value=(udata.view.smooth-20)/2+20;
else
    value=udata.view.smooth;
end


adaptivecheck=uicontrol('position',[origin(1)+15 origin(2)+215 200 20],...
    'style','radio','string','Adaptive Sampling','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold',...
    'tag','adaptivecheck','callback',@adaptivecheck_Callback);
asmoothtext=uicontrol('position',[origin(1)+33 origin(2)+190 150 20],...
    'style','text','string','Initial resolution:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','normal');
asmooth=uicontrol('position',[86+10.8*udata.view.asmooth origin(2)+140 50 20],...
    'style','text','string',sprintf('%3.0f',udata.view.asmooth),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');
refinetext=uicontrol('position',[origin(1)+150 origin(2)+190 110 20],...
    'style','text','string','Refinement levels:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','normal');
refineedit=uicontrol('position',[250 origin(2)+170 50 20],...
    'style','edit','string',num2str(udata.view.adaptivelevels),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','center','fontweight','normal',...
    'callback',@refineedit_Callback);



%scale slider:
[jslider3,hslider3]=javacomponent('javax.swing.JSlider',[origin(1)+33 origin(2)+160 100 30]);
jslider3.setPaintTicks(1)

jslider3.setSnapToTicks(true)
jslider3.setMinimum(2)
jslider3.setMaximum(10)
jslider3.setMajorTickSpacing(1);
%jslider.setMinorTickSpacing(1);
jslider3.setValue(udata.view.asmooth);
set(jslider3,'background',java.awt.Color.white)
set(jslider3,'StateChangedCallback',{@asmooth_Callback,asmooth});


uniformcheck=uicontrol('position',[origin(1)+15 origin(2)+115 200 20],...
    'style','radio','string','Uniformly Sampled Grid','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','tag','uniformcheck',...
    'callback',@uniformcheck_Callback);
smoothtext=uicontrol('position',[origin(1)+37 origin(2)+92 150 20],...
    'style','text','string','Grid resolution:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','normal');
smooth=uicontrol('position',[107+3.6*value origin(2)+47 50 20],...
    'style','text','string',num2str(udata.view.smooth+2),'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold');

%scale slider:
[jslider2,hslider2]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+67 190 30]);
jslider2.setPaintTicks(1)

jslider2.setSnapToTicks(0)
jslider2.setMinimum(2)
jslider2.setMaximum(50)
jslider2.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider2.setValue(value);
set(jslider2,'background',java.awt.Color.white)
set(jslider2,'StateChangedCallback',{@smooth_Callback,smooth});


if udata.view.doadaptive
    set(adaptivecheck,'value',1)
else
    set(uniformcheck,'value',1)
end

%draw this on top:
optionstext=text(.15-.10317,.15*1.0317-.0317,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title,'visible','off');

%% variable lists

analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_callback,'value',a);

datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback,'value',d);


%% buttons
scalebutton=uicontrol('position',[origin(1)+55 origin(2)+10 180 30],...
    'style','pushbutton','string','Scale/Transform Variables','backgroundcolor',handles.colors.buttonbg,'visible','off');

showpointscheck=uicontrol('position',[origin(1)+55 origin(2)+45 200 26],...
    'style','checkbox','string','Show Data Set',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.format.showpoints,'callback',@showpoints_callback,'visible','off');

showcurrentpointcheck=uicontrol('position',[origin(1)+20 origin(2)+15 200 26],...
    'style','checkbox','string','Show Current Point',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.showcurrentpoint,'callback',@showcurrentpoint_Callback,...
    'visible','on','fontweight','bold','fontsize',10);




%% group items and finish up
handles.tabobjects=[datasettext datasetmenu layouthandles,...
    scalebutton analysistext projecttext analysismenu dataseriestext optionstext showpointscheck,...
    smoothtext smooth hslider1 hslider2 ncont nconttext hslider3 asmooth asmoothtext,...
    adaptivecheck uniformcheck refinetext refineedit showcurrentpointcheck];

handles.activetab='view';

guidata(src,handles);

%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata)

handles=guidata(src);
%update other menus to show correct variables

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

d=get(datasetmenu,'value');

udata=get(handles.fakeax(handles.activeindex),'userdata');

z=find(handles.isfunction{d},1);
if ~isempty(z)
    inputs.z=z;
    inputs.dataset=d;
    X=handles.funvars{d}{z};
    X=X(~handles.isconstant{d}(X));
    inputs.x=X(1);
    if numel(X)>1
        inputs.y=X(2);
    else
        inputs.y=X(1);
    end
    udata.view.isvalid=1;
end

if isempty(z) || isempty(inputs.x);
    inputs.x=[];
    inputs.y=[];
    inputs.z=[];
    inputs.dataset=1;
    udata.view.isvalid=0;
end

udata.view.dataset=get(datasetmenu,'value');
udata.view.y=inputs.y;
udata.view.x=inputs.x;
udata.view.z=inputs.z;


set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata)
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);

udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatecontour(src,[]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

xvars=find(~handles.isconstant{udata.view.dataset} & ~handles.isfunction{udata.view.dataset});
udata.view.x=xvars(get(src,'value'));
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yaxismenu_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

xvars=find(~handles.isconstant{udata.view.dataset} & ~handles.isfunction{udata.view.dataset});
udata.view.y=xvars(get(src,'value'));
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zaxismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
zvars=find(handles.isfunction{udata.view.dataset});
udata.view.z=zvars(get(src,'value'));
udata.view.c=udata.view.z;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ncont_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars),'position',[49+58+3.6*src.getValue pos(2:4)]);
if src.getValueIsAdjusting;return;end
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.ncont=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);

ravecreatecontour(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function smooth_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars
numbars=src.getValue;
if numbars>20;numbars=20+(numbars-20)*2;end
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',numbars+2),'position',[49+58+3.6*src.getValue pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.smooth=numbars;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);

ravecreatecontour(handles.fig,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function asmooth_Callback(src,ev,hObject) %#ok<INUSL>
%hObject = nbars
value=src.getValue;
pos=get(hObject,'position');
set(hObject,'string',sprintf('%3.0f',value),'position',[86+10.8*value pos(2:4)]);

if src.getValueIsAdjusting;return;end

handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.asmooth=value;
set(handles.fakeax(handles.activeindex),'userdata',udata)
guidata(hObject,handles);

ravecreatecontour(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function refineedit_Callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

val=str2double(get(src,'string'));
if isnan(val) || isinf(val) || val<0 || val>12
    errordlg('You must enter a positive integer number less than 12 for the number of refinement levels.')
    return
end

udata.view.adaptivelevels=ceil(val);
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function uniformcheck_Callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
set(src,'value',1)
set(findobj(handles.tabobjects,'tag','adaptivecheck'),'value',0);
udata.view.doadaptive=0;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function adaptivecheck_Callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
set(src,'value',1)
set(findobj(handles.tabobjects,'tag','uniformcheck'),'value',0);
udata.view.doadaptive=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function showcurrentpoint_Callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.showcurrentpoint=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreatecontour(src,[]);


