function ravecreateplaneprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

%inputs=varargin{3};

targetaxes=handles.activegraph; %=handles.activegraph

%set(targetaxes,'drawmode','fast');

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

leg=get(handles.fakerax(handles.activeindex),'userdata');
leg = leg(ishandle(leg));
if ~isempty(leg)
    legpos = get(leg,'position');
    legpos=[legpos(1) legpos(2)+legpos(4)];
    delete(leg);
else
    legpos=[];
end


%prepare data for gridding:
if ~udata.view.isvalid
    return %no functions loaded.
end

datatypes=handles.datatype{d}(zcol);
numpts=udata.view.npoints;
xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};
step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));
xdials=handles.xdials{d}{a};
nvars=numel(zcol);

[xsteps totalrows]=ravecalculatexsteps(src,d,a,zcol,numpts,1);


%% MAKE PATHS:
udata.view.linepaths=zeros(1,nvars);
udata.view.lineplus=zeros(1,nvars);
udata.view.tickpaths=zeros(1,nvars);

colorz=hsv(nvars);

for j=1:nvars
    thisvar=zcol(j);
    thiscolor=handles.variablecolors{d}(thisvar,:);
    thiscolor=colorz(j,:);
    
    startpoint=xdials;

    npoints=numel(xsteps{j});
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=xsteps{j};
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata,handles);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    udata.view.linepaths(j)=line(xvalues,yvalues,'linewidth',1,'marker','none','color',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','on','tag','pprof','buttondownfcn','',...
        'linewidth',udata.format.linesize);
    udata.view.lineplus(j)=line(xvalues(end),yvalues(end),'marker','+','color',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','on','tag','pprof','buttondownfcn','',...
        'linewidth',udata.format.linesize); %#ok<COLND,COLND>
end


%% MAKE TICK MARKS:
xdials=handles.xdials{d}{a};
nvars=numel(zcol);

[xsteps totalrows]=ravecalculatexsteps(src,d,a,zcol,5,1);

for j=1:nvars
    thisvar=zcol(j);
    thiscolor=handles.variablecolors{d}(thisvar,:);
    thiscolor=colorz(j,:);
    
    startpoint=xdials;

    npoints=numel(xsteps{j});
    xdata=repmat(startpoint,[npoints 1]);
    xdata(:,thisvar)=xsteps{j};
    
    outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdata,handles);
    
    xvalues=outputs(:,1);
    yvalues=outputs(:,2);
    %make paths:
    udata.view.tickpaths(j)=line(xvalues,yvalues,'linestyle','none','marker','.',...
        'markerfacecolor',thiscolor,'markeredgecolor',thiscolor,...
        'parent',handles.allgraphs(handles.activeindex),'hittest','off','visible','on');
end

if ~udata.format.showticks
    set(udata.view.tickpaths,'visible','off')
end


if udata.format.showpoints
    nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};
    xdata=datasets.alldata{udata.view.dataset}(nothidden,udata.view.x);
    ydata=datasets.alldata{udata.view.dataset}(nothidden,udata.view.y);
    
    points=line(xdata,ydata,-ones(size(ydata)),'parent',targetaxes,'tag','points','linestyle','none','markeredgecolor','none','markerfacecolor',udata.format.pointcolor,...
        'marker','o','markersize',2,'hittest','off');
    uistack(points,'bottom');
end


outputs=raveevaluatefunction(handles.fig,d,[udata.view.x udata.view.y],xdials,handles);

xvalues=outputs(:,1);
yvalues=outputs(:,2);

%make the white dot at the center:
udata.view.pspot=line(xvalues,yvalues,'marker','o','markeredgecolor',[0 0 0],...
    'markerfacecolor',[1 1 1],'markersize',6,'parent',handles.allgraphs(handles.activeindex),'tag','pprof','linestyle','none');

%handles.tabobjects(end+1)=handles.pprof.responsevalues;

%record presence of profiler:
%udata.explore.method=union(udata.explore.method,'responseprofiler');
set(handles.fakeax(handles.activeindex),'userdata',udata);

guidata(src,handles);

xrange=handles.maxlimit{d}{a}(xcol)-handles.minlimit{d}{a}(xcol);
yrange=handles.maxlimit{d}{a}(ycol)-handles.minlimit{d}{a}(ycol);

set(targetaxes,'xlim',[handles.minlimit{d}{a}(xcol)-0.02*xrange,handles.maxlimit{d}{a}(xcol)+0.02*xrange],...
    'ylim',[handles.minlimit{d}{a}(ycol)-0.02*yrange,handles.maxlimit{d}{a}(ycol)+0.02*yrange])

%% make tick marks:
labelsize=12;
fweight='normal';

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol(1)},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,...
        'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','',...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,'fontname',handles.fonts.graph)
end

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end


%update da handles
guidata(src,handles)

ravemaketextticks(targetaxes,[],xcol(1),ycol,[],[])
raveresizeaxes(src,[],targetindex)


%draw constraints:
constraints=udata.view.constraints(udata.view.constraints~=0);
for i=1:numel(constraints)
    thisconstraint=constraints(i);   
    h=ravedrawconstraint(targetaxes,handles.activeindex,thisconstraint);
end

% L=legend(udata.view.linepaths,handles.allnames{d}(zcol));
L=ravemakelegend(src,[],handles.activeindex,handles.allnames{d}(zcol),udata.view.linepaths,[],'',legpos);
legdata=get(handles.fakerax(handles.activeindex),'userdata');
legdata=union(legdata,L);
set(handles.fakerax(handles.activeindex),'userdata',legdata);
% rmappdata(handles.activegraph,'LegendPeerHandle');
%try rmappdata(handles.activegraph,'LegendColorbarText');end
%try rmappdata(handles.activegraph,'LegendColorbarInnerList');end
%try rmappdata(handles.activegraph,'LegendColorbarOuterList');end

% set(findobj(L,'type','text'),'interpreter',handles.prefs.interpreter);
% set(L,'color',get(handles.fakerax(handles.activeindex),'color'))

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
