function ravecreatesurfaceprofiler(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x; %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


d=udata.view.dataset;
a=udata.view.analysis;

xdials=handles.xdials{d}{a};
funvars=udata.view.funvars;
funvarnames=handles.allnames{d}(funvars);

if isempty(zcol)
    set(handles.infobar,'string','Error: No variables in this dataset are modeled by Equations.')
    ravedeletegraph(src,[],0)
    return
end

%clear existing graph
cla(targetaxes);
%legend(targetaxes,'off');

% if ~handles.ishg2
%     set(targetaxes,'drawmode','fast');
% else
%     set(targetaxes,'sortmethod','childorder');
% end


set(targetaxes,'buttondownfcn',@buttondownfcnsurfaceprofiler,'hittest','on');

if udata.format.boxon
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','on')
else
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','off')
end



%font formatting
fweight='normal';
labelsize=handles.fonts.graphsize;

%get data


%get type of each variable:
datatypes=handles.datatype{d}([xcol ycol]);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.


xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

numpts=udata.view.numpts;

colz=[xcol ycol];

[xsteps totalrows]=ravecalculatexsteps(src,d,a,colz,numpts,0);

%evaluation grid will be numel(xsteps{2}) by numel(xsteps{1}) elements.
yevals=repmat(xsteps{2}(:),[numel(xsteps{1}) 1]);
index=repmat(1:numel(xsteps{1}),[numel(xsteps{2}) 1]);
xevals=xsteps{1}(index(:));

%make whole matrix of poitns to eval. this is fixed array of xdials, with
%xeval/yeval subbed in:
indata=repmat(xdials,[numel(xevals) 1]);
indata(:,xcol)=xevals;
indata(:,ycol)=yevals;

%eval z at indata:
zvalues=raveevaluatefunction(src,d,zcol,indata,handles);

%update axis limits only if needed:
if min(zvalues)<handles.minlimit{d}{a}(zcol);handles.minlimit{d}{a}(zcol)=min(zvalues);doupdate=1;end
if max(zvalues)>handles.maxlimit{d}{a}(zcol);handles.maxlimit{d}{a}(zcol)=max(zvalues);doupdate=1;end





%reshape to square matrices:
xdata=reshape(xevals,[numel(xsteps{2}) numel(xsteps{1})]);
ydata=reshape(yevals,[numel(xsteps{2}) numel(xsteps{1})]);
zdata=reshape(zvalues,[numel(xsteps{2}) numel(xsteps{1})]);

if ccol~=0
    c=raveevaluatefunction(src,d,ccol,indata,handles);
    if min(c)<handles.minlimit{d}{a}(ccol);handles.minlimit{d}{a}(ccol)=min(c);doupdate=1;end
    if max(c)>handles.maxlimit{d}{a}(ccol);handles.maxlimit{d}{a}(ccol)=max(c);doupdate=1;end
    %normalize cdata to desired colormap:
    
    if strcmp(udata.format.colormapping,'local')
        minc=min(c);
        maxc=max(c);
    else
        minc=handles.minlimit{d}{a}(ccol);
        maxc=handles.maxlimit{d}{a}(ccol);
    end
    c=(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap-1)+1;
    udata.view.clim=[minc maxc];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    cdata=reshape(c,[numel(xsteps{2}) numel(xsteps{1})]);
else
    cdata=[0 .5 1]; %non-noisy colors cdata=0.5
end

set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

%fold if xvar is discrete:
if  ~strcmp(datatypes{1},'continuous') %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{xcol};
    endind=numel(allowed);

    xvals=xsteps{1}(:); %force to be column.
    xintermediates=xvals+[diff(xvals)./2;0];
    xvals=sort([xvals;xintermediates(1:end-1)]);
    
    xmap=repmat(2:2:2*(endind-1),2,1);
    xmap=[1 xmap(:)' xmap(end)+1];
    
    zmap=repmat(1:numel(xmap)/2,2,1);
    zmap=zmap(:)';
    
    xdata=repmat(xvals(xmap)',size(xdata,1),1);
    ydata=repmat(ydata(:,1),1,numel(xmap));
    zdata=zdata(:,zmap);
    cdata=cdata(:,zmap);   
end

%fold if yvar is discrete:
if  ~strcmp(datatypes{2},'continuous') %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{ycol};
    endind=numel(allowed);

    yvals=xsteps{2}(:); %force to be column.
    yintermediates=yvals+[diff(yvals)./2;0];
    yvals=sort([yvals;yintermediates(1:end-1)]);

    ymap=repmat(2:2:2*(endind-1),2,1);
    ymap=[1 ymap(:)' ymap(end)+1];
    
    zmap=repmat(1:numel(ymap)/2,2,1);
    zmap=zmap(:)';
    
    ydata=repmat(yvals(ymap),1,size(ydata,2));
    xdata=repmat(xdata(1,:),numel(ymap),1);
    zdata=zdata(zmap,:);
    cdata=cdata(zmap,:);   
end

facestyles={udata.format.facecolor,'flat','interp','none'};
edgestyles={udata.format.edgecolor,'flat','interp','none'};

if isscalar(zdata) %in case both x and y are constants:
    xdata=[xdata xdata;xdata+eps(xdata) xdata+eps];
    ydata=[ydata ydata+eps(ydata);ydata ydata+eps(ydata)];
    zdata=[zdata zdata;zdata zdata];
    cdata=[cdata cdata;cdata cdata];
end

H=surface(xdata,ydata,zdata,cdata,'parent',targetaxes,'tag','main','linewidth',udata.format.linesize,...
    'edgecolor',edgestyles{udata.format.edgestyle},'facecolor',facestyles{udata.format.facestyle},...
    'cdatamapping','direct','hittest','off','buttondownfcn',@pointselectsurfaceprofiler);

if xcol==ycol
    set(H,'visible','off')
end



%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
    
    
set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)  
    
set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
        'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)   

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
ravesetgrid3D;

%make reasonable axes limits:

zlims=[handles.minlimit{d}{a}(zcol) handles.maxlimit{d}{a}(zcol)];
xlims=[handles.minlimit{d}{a}(xcol) handles.maxlimit{d}{a}(xcol)];
ylims=[handles.minlimit{d}{a}(ycol) handles.maxlimit{d}{a}(ycol)];

try %#ok<TRYNC> %gives error if x/y is constant. just let it use auto lims in that case. 
    set(targetaxes,'xlim',xlims,'ylim',ylims,'zlim',zlims);
end

view(targetaxes,[udata.view.az udata.view.el]);
view(handles.fakeax(handles.activeindex),[udata.view.az udata.view.el]);
view(handles.fakerax(handles.activeindex),[udata.view.az udata.view.el]);

%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];
% 
% set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
%     'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
%     'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
%     'ztickmode','manual','ztick',get(targetaxes,'ztick'));
% 
% set(Q,...
%     'xticklabel',get(targetaxes,'xticklabel'),...
%     'yticklabel',get(targetaxes,'yticklabel'),...
%     'zticklabel',get(targetaxes,'zticklabel'));
% 
% set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
% set(targetaxes,'nextplot','add')
% 
% set(Q,'ticklength',[0 0]);

%make current point:
if udata.view.showcurrentpoint
    cp=handles.xdials{d}{a};
    cpz=raveevaluatefunction(src,d,zcol,cp,handles);
    
    %nudge up 4 pixels so it sits on top of the point instaed of centered
    %on the point:
    vvv=get(targetaxes,'view');
    el=vvv(2);
    pos=get(targetaxes,'position');
    zpix=pos(4)*cos(el*2*pi/180);
    zrange=diff(get(targetaxes,'zlim'));
    
    zshift=5*zrange/zpix;
    
    udata.view.currentpointhandle=line(cp(xcol),cp(ycol),cpz+zshift,'marker','v','markeredgecolor',[0 0 0],'markerfacecolor',handles.colors.text,...
        'parent',targetaxes,'markersize',8);
else
    udata.view.currentpointhandle=[];
end

set(handles.fakeax(handles.activeindex),'userdata',udata);



%update handles
guidata(handles.fig,handles)

ravemakecolorbar(handles.fig,[]);     
ravemaketextticks(targetaxes,[],xcol,ycol,zcol,[])
raveresizeaxes(src,[],targetindex)
clear ravearrangeobjects