function ravecreategraphaxes(src,ev)
%create generic view tab controls on workspace.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.allgraphs(handles.activeindex); %=handles.activegraph
a=udata.view.analysis;
d=udata.view.dataset; %data set index

set(targetaxes,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')
set(handles.fakerax(handles.activeindex),'visible','off')

origin=getpixelposition(targetaxes);

%delete old objects:
cla(targetaxes);
delete(udata.view.X.menuobj);
delete(udata.view.X.menujobj);
delete(udata.view.Y.menuobj);
delete(udata.view.Y.menujobj);
delete(udata.view.Z.menuobj);
delete(udata.view.Z.menujobj);
delete(udata.view.C.menuobj);
delete(udata.view.C.menujobj);
delete(udata.view.S.menuobj);
delete(udata.view.S.menujobj);

%make pixel coordinates:
set(targetaxes,'xlim',[1 origin(3)],'ylim',[1 origin(4)],'xticklabel','','yticklabel','');

if udata.view.linkedindex==0
    return
end


%make new objects:

%start at the top and work way down:
off=10; %offset in pixels from top of control:

pos=getpixelposition(targetaxes);
set(targetaxes,'xlim',[1 pos(3)],'ylim',[1 pos(4)],'xticklabel','','yticklabel','','visible','off');


strings={'X';'Y';'Z';'C';'S'};

%
% callbacks={{@xmenu_callback,handles.activeindex},{@ymenu_callback,handles.activeindex},...
%     {@zmenu_callback,handles.activeindex},{@cmenu_callback,handles.activeindex},...
%     {@smenu_callback,handles.activeindex}};

%clear existing handle values to avoid invalid handles later:
udata.view.X.menuobj=[];
udata.view.X.menujobj=[];
udata.view.Y.menuobj=[];
udata.view.Y.menujobj=[];
udata.view.Z.menuobj=[];
udata.view.Z.menujobj=[];
udata.view.C.menuobj=[];
udata.view.C.menujobj=[];
udata.view.S.menuobj=[];
udata.view.S.menujobj=[];

for r=1:5
    q=strings{r};
    if udata.view.ispresent(r) && udata.view.isvisible(r);
        udata.view.(q).menutext=text(0,pos(4)-off,udata.view.(q).label,'fontname','arial','fontsize',10,'fontweight','bold','verticalalign','baseline','parent',targetaxes);
        off=off+22;
        
        val=udata.view.(q).value;
        if max(val)>numel(udata.view.(q).variables);val=1;end
        
        if numel(val)<2
            [udata.view.(q).menuobj,udata.view.(q).menujobj]=drawpopupmenu(handles.wspanel,[pos(1)+10,pos(2)+pos(4)-off 200 18],...
                udata.view.(q).string(udata.view.(q).variables),val,{@menu_callback,handles.activeindex,q},handles.colors.wsbg);
            udata.view.(q).listobj=[];
            udata.view.(q).listscroll=[];
            
            %set(udata.view.(q).menujobj,'deletefcn',''); %avoid error when
            %creating controls under 2011a or newer and then reusing in older versions.
            off=off+25;
        else %draw a listbox:
            [udata.view.(q).listobj,udata.view.(q).listscroll]=drawlistbox(handles.wspanel,[pos(1)+10,pos(2)+pos(4)-off-100 200 118],...
                udata.view.(q).string(udata.view.(q).variables),100,true,val,1,'line',[.5 .5 .5],18,{@list_callback,handles.activeindex,q});
            udata.view.(q).menuobj=[];
            udata.view.(q).menujobj=[];
            off=off+125;
        end
    end
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
%ravearrangeobjectsgraphaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects


function menu_callback(src,value,ai,thisaxis)
%ai is index into handles.allgraphs of this control
%thisaxis is a member of {'X';'Y';'Z';'C';'S'} indicating which axis was
%changed.

handles=guidata(src);

status=validategraphaxes(src,[],ai,thisaxis);
if ~status;return;end

udata=get(handles.fakeax(ai),'userdata'); %usedata of this control;
idx=udata.view.linkedindex;

switch thisaxis
    case 'X'
        thismenu=findobj(udata.view.tabobjects,'tag','xaxismenu');
    case 'Y'
        thismenu=findobj(udata.view.tabobjects,'tag','yaxismenu');
    case 'Z'
        thismenu=findobj(udata.view.tabobjects,'tag','zaxismenu');
    case 'C'
        thismenu=findobj(udata.view.tabobjects,'tag','colormenu','-or','tag','cmenu','-or','tag','caxismenu');
    case 'S'
        thismenu=findobj(udata.view.tabobjects,'tag','sizemenu','-or','tag','smenu','-or','tag','saxismenu');
end

%value is the index in the list that was clicked. Need to convert this to
%the target graph's index.


udata.view.(thisaxis).value=value;
set(handles.fakeax(ai),'userdata',udata);


value=udata.view.(thisaxis).variables(value);

set(thismenu,'value',value)
cb=get(thismenu,'callback');

tabobjects=handles.tabobjects;
ai=handles.activeindex;
ag=handles.activegraph;
at=handles.activetype;

handles.tabobjects=udata.view.tabobjects;
handles.activeindex=idx;
handles.activegraph=handles.allgraphs(idx);
handles.activetype=get(handles.allgraphs(idx),'userdata');

guidata(handles.fig,handles)
cb(thismenu,[]);
handles=guidata(handles.fig);

handles.activeindex=ai;
handles.activegraph=ag;
handles.activetype=at;
handles.tabobjects=tabobjects;

guidata(handles.fig,handles);



function list_callback(src,ev,value,ai,thisaxis)
%ai is index into handles.allgraphs of this control
%thisaxis is a member of {'X';'Y';'Z';'C';'S'} indicating which axis was
%changed.

handles=guidata(src);

status=validategraphaxes(src,[],ai,thisaxis);
if ~status;return;end

udata=get(handles.fakeax(ai),'userdata'); %usedata of this control;
idx=udata.view.linkedindex;

switch thisaxis
    case 'X'
        thismenu=findobj(udata.view.tabobjects,'tag','xaxismenu');
    case 'Y'
        thismenu=findobj(udata.view.tabobjects,'tag','yaxismenu');
    case 'Z'
        thismenu=findobj(udata.view.tabobjects,'tag','zaxismenu');
    case 'C'
        thismenu=findobj(udata.view.tabobjects,'tag','colormenu','-or','tag','cmenu','-or','tag','caxismenu');
    case 'S'
        thismenu=findobj(udata.view.tabobjects,'tag','sizemenu','-or','tag','smenu','-or','tag','saxismenu');
end

%value is the index in the list that was clicked. Need to convert this to
%the target graph's index.


udata.view.(thisaxis).value=value;
set(handles.fakeax(ai),'userdata',udata);


value=udata.view.(thisaxis).variables(value);

set(thismenu,'value',value)
cb=get(thismenu,'callback');

tabobjects=handles.tabobjects;
ai=handles.activeindex;
ag=handles.activegraph;
at=handles.activetype;

handles.tabobjects=udata.view.tabobjects;
handles.activeindex=idx;
handles.activegraph=handles.allgraphs(idx);
handles.activetype=get(handles.allgraphs(idx),'userdata');

guidata(handles.fig,handles)
cb(thismenu,[]);
handles=guidata(handles.fig);

handles.activeindex=ai;
handles.activegraph=ag;
handles.activetype=at;
handles.tabobjects=tabobjects;

guidata(handles.fig,handles);









