function viewtabgraphaxes(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0);
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1);
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% clear existing tab
delete(handles.tabobjects)

d=udata.view.dataset;
a=udata.view.analysis;


%determine legal graphs:
idxs=find(ishandle(handles.allgraphs));
types=get(handles.allgraphs(idxs),'userdata');

if ~iscell(types)
    types={types};
end

keeperz=[];
for i=1:numel(idxs)
    if ismember(handles.graphinfo.(types{i}).class,{'discrete','continuous','special','table'})
        keeperz(end+1)=idxs(i);
    end
end

if isempty(keeperz);
    idxtext=uicontrol('style','text','position',[origin(1)+15 origin(2)+500 240 100],...
        'string',{'No graphs with controllable axes are on the workspace. Create a graph with axes, and then you can link this control to it.'},...
        'backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');
    handles.tabobjects=idxtext;
    guidata(handles.fig,handles);
    return
end

namez=cell(numel(keeperz),1);
for g=1:numel(keeperz)
    if ~isempty(handles.graphnames{keeperz(g)});
        namez{g}=handles.graphnames{keeperz(g)};
    else
        namez{g}=ravenamegraph(handles.fig,[],keeperz(g));
    end
end

keeperz=[0 keeperz];

namez=['<Not Linked>';namez];

idxtext=uicontrol('style','text','position',[origin(1)+15 origin(2)+600 240 24],...
    'string','Select Linked Graph:',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

idxmenu=uicontrol('style','popupmenu','position',[origin(1)+15 origin(2)+580 240 22],...
    'string',namez,'backgroundcolor',handles.colors.tabbg,'value',find(keeperz==udata.view.linkedindex),'callback',@linkedindex_callback);

vartext=uicontrol('style','text','position',[origin(1)+15 origin(2)+550 240 24],...
    'string','Select axis to customize:',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');

varmenu=uicontrol('style','popupmenu','position',[origin(1)+15 origin(2)+530 240 22],...
    'string',{'X';'Y';'Z';'C';'S'},'backgroundcolor',handles.colors.tabbg,'value',1,'tag','varmenu','callback',@displaysettings);

settingtext=text(.15-.10317,.80*1.0317-.0317,'Axis Settings','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

%% group items and finish up
handles.tabobjects=[idxtext,idxmenu,vartext,varmenu,settingtext];
handles.activetab='view';
guidata(src,handles);

displaysettings(handles.fig,[]);


function displaysettings(src,ev)
handles=guidata(src);
delete(handles.tabobjects(6:end));
handles.tabobjects=handles.tabobjects(1:5);
guidata(handles.fig,handles);
udata=get(handles.fakeax(handles.activeindex),'userdata');

origin=get(handles.tabaxes,'position');

varmenu=findobj(handles.tabobjects,'tag','varmenu');
strings={'X','Y','Z','C','S'};
q=strings{get(varmenu,'value')};

thisvarcheck=uicontrol('style','checkbox','position',[origin(1)+15 origin(2)+460 240 22],...
    'string','Show this axis in Axis Controls','backgroundcolor',handles.colors.tabbg,'value',udata.view.isvisible(get(varmenu,'value')),'tag','thisvarcheck',...
    'fontsize',10,'fontweight','bold','callback',@axisfilter_callback);

if ~udata.view.ispresent(get(varmenu,'value'))
    set(thisvarcheck,'enable','off')
end

varlabeltext=uicontrol('style','text','position',[origin(1)+15 origin(2)+430 240 22],...
    'string','Label shown in Axis Controls:','backgroundcolor',handles.colors.tabbg,'value',1,'tag','thisvarcheck',...
    'hor','left','fontweight','bold','fontsize',10);

varlabeledit=uicontrol('style','edit','position',[origin(1)+35 origin(2)+410 200 22],...
    'string',udata.view.(q).label,'backgroundcolor',handles.colors.tabbg,'value',1,'tag','thisvarcheck','hor','left',...
    'callback',@labeledit_callback);

filtertext=uicontrol('style','text','position',[origin(1)+15 origin(2)+380 240 22],...
    'string','Variables to be listed in menu:','backgroundcolor',handles.colors.tabbg,'value',1,'tag','thisvarcheck',...
    'hor','left','fontweight','bold','fontsize',10);

filtermenu=uicontrol('style','listbox','position',[origin(1)+35 origin(2)+100 200 280],...
    'string',udata.view.(q).string,'backgroundcolor',handles.colors.tabbg,'value',udata.view.(q).variables,...
    'tag','thisvarcheck','hor','left','max',100,'callback',@filter_callback,'enable','on');

handles.tabobjects=[handles.tabobjects,thisvarcheck varlabeltext varlabeledit filtertext filtermenu];

guidata(handles.fig,handles);

function linkedindex_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

idxs=find(ishandle(handles.allgraphs));
types=get(handles.allgraphs(idxs),'userdata');

if ~iscell(types)
    types={types};
end

keeperz=[];
for i=1:numel(idxs)
    if ismember(handles.graphinfo.(types{i}).class,{'discrete','continuous','special','table'})
        keeperz(end+1)=idxs(i);
    end
end

keeperz=[0 keeperz];
udata.view.linkedindex=keeperz(get(src,'value'));
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravelinkgraphaxes(handles.fig,[],handles.activeindex,udata.view.linkedindex);
ravecreategraphaxes(handles.fig,[]);
viewtab(handles.fig,[])


function axisfilter_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

varmenu=findobj(handles.tabobjects,'tag','varmenu');
i=get(varmenu,'value');

udata.view.isvisible(i)=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreategraphaxes(handles.fig,[]);

function filter_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

varmenu=findobj(handles.tabobjects,'tag','varmenu');
strings={'X','Y','Z','C','S'};
q=strings{get(varmenu,'value')};

value=get(src,'value');

if isempty(value)
    set(handles.infobar,'string','You must select at least one variable to be listed in the menu.');
    set(src,'value',udata.view.(q).variables);
    return
else
    set(handles.infobar,'string','');
end

udata.view.(q).variables=value;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreategraphaxes(handles.fig,[]);




function labeledit_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

varmenu=findobj(handles.tabobjects,'tag','varmenu');
strings={'X','Y','Z','C','S'};
q=strings{get(varmenu,'value')};

newlabel=get(src,'string');
udata.view.(q).label=newlabel;
set(handles.fakeax(handles.activeindex),'userdata',udata);
set(udata.view.(q).menutext,'string',newlabel);

