function ravecreateline(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
constcol=udata.view.c;
a=udata.view.analysis;
d=udata.view.dataset;

%clear existing graph
labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);
cla(targetaxes);
leg=get(handles.fakerax(handles.activeindex),'userdata');
leg = leg(ishandle(leg));
if ~isempty(leg)
    legpos = get(leg,'position');
    legpos=[legpos(1) legpos(2)+legpos(4)];
    delete(leg);
else
    legpos=[];
end

if ~udata.format.persistentlimits
    set(targetaxes,'xlimmode','auto','ylimmode','auto','xtickmode','auto','ytickmode','auto')
else
    set(targetaxes,'xtickmode','auto','ytickmode','auto')
end


%font formatting
fweight='normal';

%get data
if xcol>0
    x=datasets.alldata{d}(:,xcol);
else
    x=(1:size(handles.isselected{d}{1},1))';
end
y=datasets.alldata{d}(:,ycol);

if isempty(x) %empty data set
    return
end

isselected=handles.isselected{d}{a};
ishidden=~handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

%do scaling on y1 if necessary:
legendmultipliertext=cell(1,size(y,2));

if udata.view.autoscale==1 && numel(ycol)>1 %scale to same order of magnitude
    %determine order of magnitude of each response:
    ordersofmagnitude=10.^(round(log10(abs(mean(y)))));
    y=bsxfun(@rdivide,y,ordersofmagnitude);
    legendmultipliers=log10(ordersofmagnitude);
    
    for ggg=1:length(legendmultipliers)
        if legendmultipliers(ggg)==-1
            legendmultipliertext{ggg} = ' x10';
        else
            if strcmp(handles.prefs.interpreter,'tex')
                legendmultipliertext{ggg} = [' x10^{',num2str(-legendmultipliers(ggg)),'}'];
            else
                legendmultipliertext{ggg} = [' x10^',num2str(-legendmultipliers(ggg))];
            end
        end
    end
    [legendmultipliertext{legendmultipliers==0}]=deal('');
elseif udata.view.autoscale==2 %normalize 0-1
    minz=handles.minlimit{d}{a}(ycol);
    maxz=handles.maxlimit{d}{a}(ycol);
    y=bsxfun(@minus,y,minz);
    y=bsxfun(@rdivide,y,maxz-minz);
    [legendmultipliertext{:}]=deal('');
else
    [legendmultipliertext{:}]=deal('');
end

%sort data:
if udata.view.s==0 || (udata.view.s==-1 && udata.view.x==0);
    %no sorting, do nothing
    idx=1:size(handles.isselected{d}{1},1);
elseif udata.view.s==-1
    s=udata.view.x;
    [junk,idx]=sort(datasets.alldata{d}(:,s)); %#ok<ASGLU>
else
    s=udata.view.s;
    [junk,idx]=sort(datasets.alldata{d}(:,s)); %#ok<ASGLU>
end

if xcol>0
    x=x(idx);
end

y=y(idx,:);
isselected=isselected(idx);
ishidden=ishidden(idx);
isfiltered=isfiltered(idx);


%% Assign data to lines:
%assign ALL data before considering hidden/filtered state. THis will ensure
%that line colors dont change due to subsequent hiding/filtering.

if numel(udata.view.y)==1 && udata.view.c==0 %only drawing one line
    lineidx={true(length(x),1)};
    unz=[];
elseif numel(udata.view.y)>1 %drawing one line for each response variable, with all data points on each line.
    lineidx=true(length(x),numel(ycol));
    lineidx=mat2cell(lineidx,length(x),ones(1,numel(ycol)));
else %drawing one response with data divided into lines according to udata.view.c
    constdata=datasets.alldata{d}(:,udata.view.c);
    constdata=constdata(idx);
    unz=unique(constdata);
    unz=unz(~isnan(unz));
    
    %comment out the line below to draw line graphs with more than 1000 lines:
    if numel(unz)>1000;errordlg('I won''t draw a line plot that contains more than 1000 lines because presumably that''s not what you really wanted me to do in the first place. If you really want to, find this line in ravecreateline.m and comment it out.');return;end
    
    lineidx=cell(1,numel(unz));
    for l=1:numel(lineidx)
        lineidx{l}=constdata==unz(l);
    end
end

V=numel(lineidx); %total number of lines being plotted


%run colormap creation script
if udata.view.usevariablecolors && udata.view.c==0; %if not drawing lines of constant X...
    %linecolors=ravemakelinecolors(handles.fig,[],V,2*J); %still need these markercolors
    linecolors=handles.variablecolors{d}(ycol,:);
else
    linecolors=ravemakelinecolors(handles.fig,[],V,udata.format.lcmap);
end

% set(handles.activegraph,'drawmode','fast');

linehandles=ones(V,1);
markerhandles=ones(V,1);
selectedhandles=ones(V,1);
filteredhandles=ones(V,1);

set(targetaxes,'nextplot','add');

for v=1:V %v is counter for number of responses being plotted
    inline=lineidx{v};
    
    thisxall=x(inline);
    if numel(ycol)>1;
        thisyall=y(inline,v);
    else
        thisyall=y(inline);
    end
    
    thisx=thisxall;
    thisy=thisyall;
    
    %remove hidden/filtered points
    if udata.format.filtermode  && ( any(ishidden(inline) | isfiltered(inline))  )
        thisx=nan(size(thisx));
        thisy=nan(size(thisy));
    else
        thisx(ishidden(inline) | isfiltered(inline))=nan;
        thisy(ishidden(inline) | isfiltered(inline))=nan;
    end
    
    %make filtered lines:
    if ~udata.format.filtermode
        thisxall(ishidden(inline))=nan;
        thisyall(ishidden(inline))=nan;
    elseif any(ishidden(inline))
        thisxall=nan(size(thisxall));
        thisyall=nan(size(thisyall));
    end
    
    filteredhandles(v)=line(thisxall,thisyall,-ones(size(thisxall)),'linewidth',udata.format.linesize,...
        'parent',targetaxes,'color',handles.colors.filtered,'hittest','on',...
        'linestyle',udata.format.linestyle,'marker',udata.format.marker,'tag','filtered',...
        'markeredgecolor',handles.colors.filtered,'markerfacecolor',handles.colors.filtered,...
        'markersize',udata.format.markersize,...
        'buttondownfcn',@buttondownfcnline);
    
    if numel(ycol)>1
        set(filteredhandles(v),'userdata',ycol(v));
    elseif udata.view.c>0
        set(filteredhandles(v),'userdata',unz(v));
    end
    
    
    
    %make main series lines:
    linehandles(v)=line(thisx,thisy,'linewidth',udata.format.linesize,...
        'parent',targetaxes,'color',linecolors(v,:),'hittest','off',...
        'linestyle',udata.format.linestyle,'marker','none','buttondownfcn',@pointselectline,'tag','line');
    %NOTE: markersize property is essentially being used as another userdata property. can be used to
    %determine which varible this line is.
    
    if numel(ycol)>1
        set(linehandles(v),'userdata',ycol(v));
    elseif udata.view.c>0
        set(linehandles(v),'userdata',unz(v));
    end
    
    %make marker lines:
    markerhandles(v)=line(thisx,thisy,'markersize',udata.format.markersize,...
        'parent',targetaxes,'hittest','off',...
        'linestyle','none','marker',udata.format.marker,'markerfacecolor',linecolors(v,:),...
        'markeredgecolor',[0 0 0],'buttondownfcn',@pointselectline,'tag','marker');
    
    if numel(ycol)>1
        set(markerhandles(v),'userdata',ycol(v));
    elseif udata.view.c>0
        set(markerhandles(v),'userdata',unz(v));
    end
    
    %make selected lines:
    thisxsel=x(inline & isselected);
    if numel(ycol)>1;
        thisysel=y(inline & isselected,v);
    else
        thisysel=y(inline & isselected);
    end
    
    if ~isempty(thisxsel)
        selectedhandles(v)=line(thisxsel,thisysel,'markersize',udata.format.markersize,...
            'userdata',ycol(v),'parent',targetaxes,'hittest','off',...
            'linestyle','none','marker',udata.format.marker,'markerfacecolor','none',...
            'markeredgecolor',handles.colors.selectionhighlight,'buttondownfcn','','tag','selected','linewidth',2);
    else
        selectedhandles(v)=line(nan,nan,'markersize',udata.format.markersize,...
            'userdata',0,'parent',targetaxes,'hittest','off',...
            'linestyle','none','marker',udata.format.marker,'markerfacecolor','none',...
            'markeredgecolor',handles.colors.selectionhighlight,'buttondownfcn','','tag','selected','linewidth',2,'visible','off');
    end
    
end

if ~udata.format.mcheck
    set(filteredhandles,'marker','none')
end

if ~udata.format.lcheck
    set(filteredhandles,'linestyle','none')
end
    

if handles.ishg2
    set(linehandles,'pickableparts','none')
    set(selectedhandles,'pickableparts','none')
end


udata.view.linehandles=linehandles;
udata.view.markerhandles=markerhandles;
udata.view.filteredhandles=filteredhandles;
udata.view.selectedhandles=selectedhandles;

% %make selected data series:
uistack(selectedhandles(:),'top')

if ~udata.format.mcheck
    set(markerhandles,'visible','off')
end

%make all axes have same limits
set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))

udata.format.xlim=get(targetaxes,'xlim');
udata.format.ylim=get(targetaxes,'ylim');
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(targetaxes,'xlimmode','manual','ylimmode','manual');

%set axes labels

if xcol>0
    set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.fonts.graphsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graph)
else
    set(get(targetaxes,'XLabel'),'String','Index','fontsize',handles.fonts.graphsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graph)
end

if numel(ycol)==1 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.fonts.graphsize,'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graph)
elseif udata.view.autoscale==2
    set(get(targetaxes,'YLabel'),'String','Normalized Value','fontsize',handles.fonts.graphsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','','fontsize',handles.fonts.graphsize,...
        'fontweight',fweight,'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graph)
end

%set line series visibility depending on format settings:
if ~udata.format.mcheck
    set(markerhandles,'visible','off')
end

if ~udata.format.lcheck
    set(linehandles,'visible','off')
end


%adjust gridlines to match settings:
ravesetgridsingle;

%do mouseovers
if udata.explore.mouseover
    l1=line(nan,nan,'marker','none','color',handles.colors.mouseoverhighlight,'linewidth',udata.format.linesize,...
        'visible','off','parent',targetaxes,'hittest','off','linestyle','-');
    t1=text(0,0,1000,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t2=text(0,0,1000,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t3=text(0,0,1000,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t4=text(0,0,1000,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t5=text(0,0,1000,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    
    udata.explore.mouseoverhandles=[l1 t1 t2 t3 t4 t5];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[];
    
    handles.mouseoverhandles=[handles.mouseoverhandles;targetaxes;filteredhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverlineaxes,handles.activeindex}};repmat({{@ravemouseoverlinedata,handles.activeindex}},numel(filteredhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(filteredhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverlineaxes ravemouseoverlinedata
    
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

%if more than one response variable, make legend:
set(handles.fakerax(handles.activeindex),'userdata',[]) %clear existing legdata

leghandles=linehandles;
if ~udata.format.lcheck
    leghandles=markerhandles;
end

leg1=[];
if V>0
    if udata.format.legend && numel(ycol)>1
        leg1=ravemakelegend(src,[],handles.activeindex,strcat(handles.allnames{d}(ycol),legendmultipliertext),leghandles,[],'',legpos);
    elseif udata.format.legend && udata.view.c>0 && numel(unz)<20
        if isnumeric(unz)
            legendvalues=num2cell(unz);
            legendvalues=cellfun(@num2str,legendvalues,'uniformoutput',0);
        else
            legendvalues=unz;
        end
        
        leg1=ravemakelegend(src,[],handles.activeindex,legendvalues,leghandles,[],'',legpos);
        
        
    end
end

set(handles.fakerax(handles.activeindex),'userdata',leg1);
guidata(src,handles)


if udata.view.x>0
    ravemaketextticks(targetaxes,[],udata.view.x,[],[]);
else
%     xticks=get(targetaxes,'xtick');
%     xticklabels=num2cell(xticks);
%     xticklabels=cellfun(@num2str,xticklabels,'UniformOutput',false);
    ravemaketextticks(targetaxes,[],-1,[],[],[])
end

if numel(ycol)==1
    ravemaketextticks(targetaxes,[],[],ycol,[],[])
else
%     yticks=get(targetaxes,'ytick');
%     yticklabels=num2cell(yticks);
%     yticklabels=cellfun(@num2str,yticklabels,'UniformOutput',false);
    ravemaketextticks(targetaxes,[],[],-1,[],[],[])
end

set(targetaxes,'hittest','on','buttondownfcn',@buttondownfcnline)

set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xlim',get(handles.activegraph,'xlim'),...
    'ylim',get(handles.activegraph,'ylim'),'xtick',get(handles.activegraph,'xtick'),'ytick',get(handles.activegraph,'ytick'))

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects










