function handles=raveupdatelineisselected(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
a=udata.view.analysis;
d=udata.view.dataset;

%get data
if xcol>0
    x=datasets.alldata{d}(:,xcol);
else
    x=(1:size(handles.isselected{d}{1},1))';
end

y=datasets.alldata{d}(:,ycol);

if isempty(x) %empty data set
    return
end

if udata.view.autoscale==1 && numel(ycol)>1
    ordersofmagnitude=10.^(round(log10(abs(mean(y)))));
    y=bsxfun(@rdivide,y,ordersofmagnitude);
elseif udata.view.autoscale==2
    minz=handles.minlimit{d}{a}(ycol);
    maxz=handles.maxlimit{d}{a}(ycol);
    y=bsxfun(@minus,y,minz);
    y=bsxfun(@rdivide,y,maxz-minz);
    
end

isselected=handles.isselected{d}{a};

%sort data:
if udata.view.s==0 || (udata.view.s==-1 && udata.view.x==0);
    %no sorting, do nothing
    idx=1:size(handles.isselected{d}{1},1);
elseif udata.view.s==-1
    s=udata.view.x;
    [junk,idx]=sort(datasets.alldata{d}(:,s)); %#ok<ASGLU>
else
    s=udata.view.s;
    [junk,idx]=sort(datasets.alldata{d}(:,s)); %#ok<ASGLU>
end

if udata.view.x>0
x=x(idx);
end

y=y(idx,:);
isselected=isselected(idx);

%% Assign data to lines:
%assign ALL data before considering hidden/filtered state. THis will ensure
%that line colors dont change due to subsequent hiding/filtering.

if numel(udata.view.y)==1 && udata.view.c==0 %only drawing one line
    lineidx={true(length(x),1)};
    unz=[];
elseif numel(udata.view.y)>1 %drawing one line for each response variable, with all data points on each line.
    lineidx=true(length(x),numel(ycol));
    lineidx=mat2cell(lineidx,length(x),ones(1,numel(ycol)));
    unz=[];
else %drawing one response with data divided into lines according to udata.view.c
    constdata=datasets.alldata{d}(:,udata.view.c);
    constdata=constdata(idx);
    unz=unique(constdata);
    
    %comment out the line below to draw line graphs with more than 1000 lines:
    if numel(unz)>1000;errordlg('I won''t draw a line plot that contains more than 1000 lines because presumably that''s not what you wanted me to do in the first place. If you really want to, find this line in ravecreateline.m and comment it out.');return;end
    
    lineidx=cell(1,numel(unz));
    for l=1:numel(lineidx)
        lineidx{l}=constdata==unz(l);
    end
end

V=numel(lineidx); %total number of lines being plotted


set(targetaxes,'nextplot','add');

for v=1:V %v is counter for number of responses being plotted
    inline=lineidx{v};
    
    thisxsel=x(inline & isselected);
    if numel(ycol)>1;
        thisysel=y(inline & isselected,v);
    else
        thisysel=y(inline & isselected);
    end
    
    if ~isempty(thisxsel)
        set(udata.view.selectedhandles(v),'xdata',thisxsel,'ydata',thisysel,'visible','on');
    else
        set(udata.view.selectedhandles(v),'visible','off');
    end
end


