function handles=raveupdatescatterdatachanged(src,ev,handles,thesevars)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

datasets=raveguidata(handles.fig,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%get data
x=datasets.alldata{d}(:,xcol);
y=datasets.alldata{d}(:,ycol);

[mainrows,selectedrows,filteredrows]=ravemakeplotdataseries(handles,udata);
filteredrows=mainrows|filteredrows;


%% get colors:
c=ravemakerowcdata(handles,datasets,udata,ccol,udata.format.mcmap);

%% normalize sdata to fit a reasonable range of marker sizes:
if scol~=0
    s=datasets.alldata{d}(:,scol);
    minsize=udata.format.minmarkersize;
    maxsize=udata.format.maxmarkersize;
    if min(s)~=max(s) %avoid divide by zero
        s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize + rand(length(x),1)/100;
        s=round(s)+1;
    else
        s=minsize.*ones(size(s));
    end
else
    s=ones(length(x),1)*udata.format.minmarkersize;
end

s=sqrt(s); %for consistency with other matlab functions



%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);

%if patches are overkill, use lines:
patchidx=zeros(size(s,1),1);

if npatches==1 && ccol==0
    npatches=10;
    patchidx=handles.rowcolors{d}{a};
    uselines=true;
else
    uselines=false;
    for g=1:size(s,1)
        patchidx(g)=find(usdata==s(g),1,'first');
    end
end

udata.view.patchidx=patchidx;


%If scol~=0, must recreate patch objects because the number of patches has
%potentially changed. Otherwise, can just set x/y/c data on existing
%patches/lines:
if scol==0
    %make z data so that point overlapping looks better:
    z=1:numel(x);
    
    %make the patches:
    filtpatchhandles=udata.view.filthandles;
    mainpatchhandles=udata.view.mainhandles;
    selpatchhandles=udata.view.selhandles;
    %filtered handles
    if ~uselines
        for j=1:npatches
            filtidx=patchidx==j & filteredrows;
            if any(filtidx)
                set(filtpatchhandles(j),'xdata',x(filtidx),'ydata',y(filtidx),'markersize',usdata(j),'visible','on');
            else
                set(filtpatchhandles(j),'xdata',x(1),'ydata',y(1),'markersize',usdata(j),'visible','off');
            end
        end
    else
        if any(filteredrows)
            set(filtpatchhandles,'xdata',x(filteredrows),'ydata',y(filteredrows),'markersize',usdata(1),'visible','on');
        else
            set(filtpatchhandles,'xdata',x(1),'ydata',y(1),'markersize',usdata(1),'visible','off');
        end
    end
    
    %main handles
    if ~uselines
        for j=1:npatches
            mainidx=patchidx==j & mainrows;
            if any(mainidx)
                set(mainpatchhandles(j),'xdata',x(mainidx),'ydata',y(mainidx),'cdata',c(mainidx),'markersize',usdata(j),'visible','on');
            else
                set(mainpatchhandles(j),'xdata',x(1),'ydata',y(1),'cdata',c(1),'markersize',usdata(j),'visible','off');
            end
        end
    else
        for j=1:npatches
            mainidx=patchidx==j & mainrows;
            if any(mainidx)
                set(mainpatchhandles(j),'xdata',x(mainidx),'ydata',y(mainidx),'markersize',usdata(1),'visible','on')
                
            else
                set(mainpatchhandles(j),'xdata',x(1),'ydata',y(1),'markersize',usdata(1),'visible','off');
            end
        end
    end
    
    %selected handles
    if ~uselines
        for j=1:npatches
            selidx=patchidx==j & selectedrows;
            if any(selidx)
                set(selpatchhandles(j),'xdata',x(selidx),'ydata',y(selidx),'markersize',usdata(j),'visible','on');
            else
                set(selpatchhandles(j),'xdata',x(1),'ydata',y(1),'markersize',usdata(j),'visible','off');
            end
        end
    else
        if any(selectedrows)
            set(selpatchhandles,'xdata',x(selectedrows),'ydata',y(selectedrows),'markersize',usdata(1),'visible','on');
        else
            set(selpatchhandles,'xdata',x(1),'ydata',y(1),'markersize',usdata(1),'visible','off');
        end
    end
    
    
else %Scol ~=0, recreate patches:
    
    delete(udata.view.filthandles);
    delete(udata.view.mainhandles);
    delete(udata.view.selhandles);
    
    %make z data so that point overlapping looks better:
    z=1:numel(x);
    
    %make the patches:
    filtpatchhandles=zeros(npatches,1);
    mainpatchhandles=zeros(npatches,1);
    selpatchhandles=zeros(npatches,1);
    %filtered handles
    if ~uselines
        for j=1:npatches
            filtidx=patchidx==j & filteredrows;
            if any(filtidx)
                filtpatchhandles(j)=line(x(filtidx),y(filtidx),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                    'markeredgecolor','none','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
            else
                filtpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                    'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
            end
        end
    else
        if any(filteredrows)
            filtpatchhandles=line(x(filteredrows),y(filteredrows),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','visible','on','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
        else
            filtpatchhandles=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter);
        end
    end
    
    %main handles
    if ~uselines
        for j=1:npatches
            mainidx=patchidx==j & mainrows;
            if any(mainidx)
                mainpatchhandles(j)=patch(x(mainidx),y(mainidx),c(mainidx),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                    'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
                    'buttondownfcn',@pointselectscatter,'linestyle','none');
            else
                mainpatchhandles(j)=patch(x(1),y(1),c(1),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                    'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
                    'buttondownfcn',@pointselectscatter,'linestyle','none');
            end
        end
    else
        colors=handles.prefs.rowcolors;
        for j=1:npatches
            mainidx=patchidx==j & mainrows;
            if any(mainidx)
                mainpatchhandles(j)=line(x(mainidx),y(mainidx),'markersize',usdata(1),'marker',udata.format.marker,...
                    'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                    'markeredgecolor','none','hittest','off',...
                    'buttondownfcn',@pointselectscatter,'linestyle','none');
            else
                mainpatchhandles(j)=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
                    'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                    'markeredgecolor','none','visible','off','hittest','off',...
                    'buttondownfcn',@pointselectscatter,'linestyle','none');
            end
        end
    end
    
    %selected handles
    if ~uselines
        for j=1:npatches
            selidx=patchidx==j & selectedrows;
            if any(selidx)
                selpatchhandles(j)=line(x(selidx),y(selidx),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                    'markeredgecolor',handles.colors.selectionhighlight,...
                    'visible','on','tag','selected','linewidth',2,'hittest','off');
            else
                selpatchhandles(j)=line(x(1),y(1),'markersize',usdata(j),'marker',udata.format.marker,...
                    'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                    'markeredgecolor',handles.colors.selectionhighlight,...
                    'visible','off','tag','selected','linewidth',2,'hittest','off');
            end
        end
    else
        if any(selectedrows)
            selpatchhandles=line(x(selectedrows),y(selectedrows),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','on','tag','selected','linewidth',2,'hittest','off');
        else
            selpatchhandles=line(x(1),y(1),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','off','tag','selected','linewidth',2,'hittest','off');
        end
        
    end
    udata.view.filthandles=filtpatchhandles;
    udata.view.mainhandles=mainpatchhandles;
    udata.view.selhandles=selpatchhandles;
    udata.view.patchidx=patchidx;
    udata.view.uselines=uselines;
    
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

raveupdateexploremethods(handles.fig,handles,handles.activeindex,'datachanged');



