function formattabscatter3D(src,eventdata)
%rave format tab for line graphs.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.40*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.085*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblob_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblob_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblob_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblob_callback,'userdata',4,'tag','bgblob');


gridindex=eye(3);
gridin=udata.format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activegraph,'xlim');
ylim=get(handles.activegraph,'ylim');
zlim=get(handles.activegraph,'zlim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activegraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback);
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+185 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410+4 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386+4 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+390 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+416 160 20],...
    'style','edit','string',get(get(handles.activegraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback);
yminbox=uicontrol('position',[origin(1)+85 origin(2)+392 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+392 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+368 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);

zlabeltext=uicontrol('position',[origin(1)+25 origin(2)+330 180 20],...
    'style','text','string','Z Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zlimtext=uicontrol('position',[origin(1)+35 origin(2)+306 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zto=uicontrol('position',[origin(1)+158 origin(2)+306 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zlabelbox=uicontrol('position',[origin(1)+85 origin(2)+332 160 20],...
    'style','edit','string',get(get(handles.activegraph,'zlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@zlabel_callback);
zminbox=uicontrol('position',[origin(1)+85 origin(2)+308 70 20],...
    'style','edit','string',zlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@zmin_callback,...
    'tag','zminbox');
zmaxbox=uicontrol('position',[origin(1)+175 origin(2)+308 70 20],...
    'style','edit','string',zlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@zmax_callback,...
    'tag','zmaxbox');
zresetbutton=uicontrol('position',[origin(1)+85 origin(2)+284 160 20],...
    'style','pushbutton','string','Reset Z Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@zresetbutton_callback);

titletext=uicontrol('position',[origin(1)+25 origin(2)+252 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
titlebox=uicontrol('position',[origin(1)+65 origin(2)+253 180 20],...
    'style','edit','string',get(get(handles.activegraph,'title'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%%  marker DATA SERIES UICONTROLS

markerstyletext=uicontrol('position',[origin(1)+50 origin(2)+205 49 20],...
    'style','text','string','Marker:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markersizetext=uicontrol('position',[origin(1)+66 origin(2)+175 36 20],...
    'style','text','string','Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markercolortext=uicontrol('position',[origin(1)+60 origin(2)+120 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
% selecttext=uicontrol('position',[origin(1)+22 origin(2)+88 180 20],...
%     'style','text','string','Select Data:','backgroundcolor',handles.colors.tabbg,...
%     'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


markerstyleaxes=axes('units','pixels','position',[origin(1)+102 origin(2)+207 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
gap=.9/7;
%markerstyleoptions:
offcolor=[.9 .9 .9];
oncolor=handles.colors.text;

line(.05,.5,'marker','s','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+gap,.5,'marker','o','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+2*gap,.5,'marker','d','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+3*gap,.5,'marker','^','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+4*gap,.5,'marker','h','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+5*gap,.5,'marker','p','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+6*gap,.5,'marker','*','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.95,.5,'marker','+','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback,'tag','styleblob')

%handles.styleblobs=get(markerstyleaxes,'children');
%color selected style:

hhh=findobj(markerstyleaxes,'marker',udata.format.marker);
set(hhh,'markerfacecolor',oncolor);


markersizeaxes=axes('units','pixels','position',[origin(1)+100 origin(2)+177 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
% gap=.9/7;
%markersizeoptions:
line(.05,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',1,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.12,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',4,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.22,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',6,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.33,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.45,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',10,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.59,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.75,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',14,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.95,.5,'marker',udata.format.marker,'parent',markersizeaxes,'markersize',18,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')

%handles.sizeblobs=get(markersizeaxes,'children');

%color the selected blob
hhh=findobj(markersizeaxes,'markersize',sqrt(udata.format.minmarkersize));
set(hhh,'markerfacecolor',oncolor);

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});

markercolor=uicontrol('position',[origin(1)+115 origin(2)+122 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);
%
% noisycolorcheck=uicontrol('position',[origin(1)+145 origin(2)+172-60 120 20],...
%     'style','checkbox','string','Use Noisy Colors','value',udata.format.ncolor,'backgroundcolor',handles.colors.tabbg,...
%     'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@ncolor_callback);

boxoncheck=uicontrol('position',[origin(1)+135 origin(2)+25 120 20],...
    'style','checkbox','string','Show Cube Edges','value',udata.format.boxon,'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial narrow','fontsize',10,'horizontalalignment','left','callback',@boxon_callback);



%% marker fill uicontrols:
markerfilltext=uicontrol('position',[origin(1)+74 origin(2)+147 29 20],...
    'style','text','string','Fill:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

markerfillaxes=axes('units','pixels','position',[origin(1)+132 origin(2)+149 80 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
gap=.9/2;
%markerstyleoptions:
offcolor=[.6 .6 .6];
oncolor=handles.colors.text;

f1=line(.05,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',[1 1 1],...
    'markeredgecolor',offcolor,'buttondownfcn',@fillblob_callback,'userdata',1,'tag','fillblob');
f2=line(.05+gap,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',offcolor,...
    'markeredgecolor',offcolor,'buttondownfcn',@fillblob_callback,'userdata',2,'tag','fillblob');
f3=line(.95,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',offcolor,...
    'markeredgecolor',[0 0 0],'buttondownfcn',@fillblob_callback,'userdata',3,'tag','fillblob');

%handles.fillblobs=[f1 f2 f3];
%color selected style:
fillstyle=udata.format.markerfill;
if fillstyle==1
    set(f1,'markeredgecolor',oncolor)
elseif fillstyle==2
    set(f2,'markerfacecolor',oncolor,'markeredgecolor',oncolor)
else %fillstyle=3
    set(f3,'markerfacecolor',oncolor)
end

%% legend uicontrols

sizelegendcheck=uicontrol('position',[origin(1)+135 origin(2)+02 110 20],...
    'style','checkbox','string','Show Size Legend','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback','',...
    'fontweight','normal','horizontalalignment','left','value',0);

colorbarcheck=uicontrol('position',[origin(1)+25 origin(2)+02 100 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);



%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton markerstyletext markersizetext markercolortext...
    markerstyleaxes markersizeaxes markercolor markerfilltext markerfillaxes...
    legendtext titletext titlebox boxoncheck colorbarcheck...
    zlimtext zlabelbox zminbox zmaxbox zto zresetbutton zlabeltext sizelegendcheck];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function styleblob_callback(src,eventdata)

handles=guidata(src);
offcolor=[.9 .9 .9];

styleblobs=findobj(handles.tabobjects,'tag','styleblob');
set(styleblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.marker=get(src,'marker');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=[udata.view.filthandles(:);udata.view.mainhandles(:);udata.view.selhandles(:)];

%update graph
set(targets,'marker',get(src,'marker'));
%update sizeblobs
sizeblobs=findobj(handles.tabobjects,'tag','sizeblob');
set(sizeblobs,'marker',get(src,'marker'));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fillblob_callback(src,eventdata)

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

offcolor=[.6 .6 .6];
oncolor=handles.colors.text;

f1=findobj(handles.tabobjects,'tag','fillblob','userdata',1);
f2=findobj(handles.tabobjects,'tag','fillblob','userdata',2);
f3=findobj(handles.tabobjects,'tag','fillblob','userdata',3);

%fillblobs=[f1 f2 f3];

set(f1,'markeredgecolor',offcolor)
set(f2,'markerfacecolor',offcolor,'markeredgecolor',offcolor)
set(f3,'markerfacecolor',offcolor)

%get target data series
fillstyle=get(src,'userdata');
if ~udata.view.uselines
    if fillstyle==1
        set(udata.view.mainhandles,'markerfacecolor','none','markeredgecolor','flat');
        set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
    elseif fillstyle==2
        set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor','none');
        set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
    else %fillstyle=3
        set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
        set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
    end
else
    colors=handles.prefs.rowcolors;
    for j=1:10
        if fillstyle==1
            set(udata.view.mainhandles(j),'markerfacecolor','none','markeredgecolor',colors(j,:));
            set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
        elseif fillstyle==2
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor','none');
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        else %fillstyle=3
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor',[0 0 0]);
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        end
    end
end

if fillstyle==1
    set(f1,'markeredgecolor',oncolor)
elseif fillstyle==2
    set(f2,'markerfacecolor',oncolor,'markeredgecolor',oncolor)
else %fillstyle=3
    set(f3,'markerfacecolor',oncolor)
end

udata.format.markerfill=fillstyle;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeblob_callback(src,eventdata)
%NOTE: for scatterplot, remake this to determine the size of the smallest
%marker.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

sizeblobs=findobj(handles.tabobjects,'tag','sizeblob');

offcolor=[.9 .9 .9];
oncolor=handles.colors.text;

%get new size:
newsize=get(src,'markersize')^2;

if strcmp(get(gcf,'selectiontype'),'alt')
    if newsize>udata.format.minmarkersize
        udata.format.maxmarkersize=newsize;
    else
        return
    end
else
    udata.format.minmarkersize=newsize;
    set(sizeblobs,'markerfacecolor',offcolor)
    set(src,'markerfacecolor',oncolor)
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter3D(handles.fig,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')
set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on','ZminorGrid','on')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','ZGrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on','zgrid','on')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off','zminorgrid','off')
set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')
set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off','zgrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(1)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activegraph,'xlim');
xlim(2)=newval;
set(handles.activegraph,'xlim',xlim)
set(handles.fakeax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'xlim',xlim,'ylim',get(handles.activegraph,'ylim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ymin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(1)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ymax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activegraph,'ylim');
ylim(2)=newval;
set(handles.activegraph,'ylim',ylim)
set(handles.fakeax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))
set(handles.fakerax(handles.activeindex),'ylim',ylim,'xlim',get(handles.activegraph,'xlim'),'zlim',get(handles.activegraph,'zlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
zlim=get(handles.activegraph,'zlim');
zlim(1)=newval;
set(handles.activegraph,'zlim',zlim)
set(handles.fakeax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
zlim=get(handles.activegraph,'zlim');
zlim(2)=newval;
set(handles.activegraph,'zlim',zlim)
set(handles.fakeax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))
set(handles.fakerax(handles.activeindex),'zlim',zlim,'ylim',get(handles.activegraph,'ylim'),'xlim',get(handles.activegraph,'xlim'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'xlimmode','auto')
xlim=get(handles.activegraph,'xlim');

set(handles.fakeax(handles.activeindex),'xlim',xlim)
set(handles.fakerax(handles.activeindex),'xlim',xlim)

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'ylimmode','auto')
ylim=get(handles.activegraph,'ylim');

set(handles.fakeax(handles.activeindex),'ylim',ylim)
set(handles.fakerax(handles.activeindex),'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activegraph,'zlimmode','auto')
zlim=get(handles.activegraph,'zlim');

set(handles.fakeax(handles.activeindex),'zlim',zlim)
set(handles.fakerax(handles.activeindex),'zlim',zlim)

zminbox=findobj(handles.tabobjects,'tag','zminbox');
zmaxbox=findobj(handles.tabobjects,'tag','zmaxbox');

set(zminbox,'string',num2str(zlim(1)))
set(zmaxbox,'string',num2str(zlim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function zlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'zlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,eventdata)
handles=guidata(src);
datasets=raveguidata(src,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

newcmap=ravechoosecolormap(src,[],udata.format.mcmap);


if newcmap==udata.format.mcmap;return;end

udata.format.mcmap=newcmap;


colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);


main=findobj(handles.activegraph,'tag','main');
selected=findobj(handles.activegraph,'tag','selected');

if udata.view.c~=0
    c=datasets.alldata{udata.view.dataset}(:,udata.view.c);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+1+128*(newcmap-1);
    
    set(main,'cdata',c);
end


minsize=udata.format.minmarkersize;
if udata.view.s==0
    set(get(main,'children'),'markersize',sqrt(minsize));
    set(get(selected,'children'),'markersize',sqrt(minsize));
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

if udata.format.colorbar;
    if udata.view.c
        ravemakecolorbar(handles.fig,[]);
    end
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcheck_callback(src,eventdata) %#ok<DEFNU>
handles=guidata(src);

%get target data series
targets=get(handles.activegraph,'children');
%filter out line series with no marker
nottargets=findobj(targets,'marker','none');
targets=setdiff(targets,nottargets);

if get(src,'value') %turn markers on
    set(targets,'visible','on');
else
    set(targets,'visible','off');
end

%update userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.mcheck=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function formatlegend_callback(src,eventdata)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ncolor_callback(src,eventdata)
handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.ncolor=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series
targets=findobj(get(handles.activegraph,'children'),'type','hggroup');
x=get(targets,'cdata');

if udata.view.c==0
    if udata.format.ncolor
        c=rand(length(x),1)-.5; %noisy colors
        set(handles.activegraph,'climmode','manual','clim',[-7 7]);
        set(targets,'cdata',c);
    else
        c=zeros(length(x),1); %non-noisy colors
        %set(handles.activegraph,'climmode','auto');
        set(targets,'cdata',c);
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function boxon_callback(src,eventdata)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.boxon=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
targets=[handles.activegraph handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)];

if udata.format.boxon
    set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','on')
else
    set(targets,'box','off')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorbarcheck_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.colorbar=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescatter3D(src,[]);






