function ravecreatescatter3D(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

%set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'vis','off');

%inputs=varargin{3};
targetaxes=handles.activegraph;
xcol=udata.view.x(1); %ones are used to take first var in case 2 vars have same name.
ycol=udata.view.y(1);
zcol=udata.view.z(1);
ccol=udata.view.c(1);
scol=udata.view.s(1);
%pscales=udata.view.pscale;
%gscales=udata.view.gscale;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

if handles.ishg2
   set(targetaxes,'sortmethod','childorder'); 
else
    set(targetaxes,'drawmode','fast');
end


%clear existing graph
cla(targetaxes);
delete(findall(targetaxes,'type','text')); %delete hidden tick labels
% legend(targetaxes,'off');

set(targetaxes,'xlimmode','auto','ylimmode','auto','zlimmode','auto');
set(targetaxes,'xtickmode','auto','ytickmode','auto','ztickmode','auto');
set(targetaxes,'xticklabelmode','auto','yticklabelmode','auto','zticklabelmode','auto');

if udata.format.boxon
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','on')
else
    set([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'box','off')
end

%font formatting
fweight='normal';
labelsize=handles.fonts.graphsize;

%get data
x=datasets.alldata{d}(:,xcol);
y=datasets.alldata{d}(:,ycol);
z=datasets.alldata{d}(:,zcol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

% 
% if ccol~=0
%     c=datasets.alldata{d}(:,ccol);
%     %normalize cdata to desired colormap:
%     if ~udata.format.reversecolors
%         c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
%     else
%         c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
%     end
% else
%     %c=zeros(length(x),1);
%     if udata.format.ncolor
%         c=1920+(handles.rowcolors{d}{a})+ rand -.5; %noisy colors
%     else
%         c=1920+(handles.rowcolors{d}{a}); %non-noisy colors cdata=0.5
%     end
% end

c=ravemakerowcdata(handles,datasets,udata,ccol,udata.format.mcmap);


set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);

if scol~=0 && ~isempty(x)
    s=datasets.alldata{d}(:,scol);
    minsize=udata.format.minmarkersize;
    maxsize=udata.format.maxmarkersize;
    if min(s)~=max(s) %avoid divide by zero
        s=(((s-min(s))/(max(s)-min(s))).^2)*maxsize + minsize;% + rand(length(x),1)/100;
        s=round(s)+1;
    else
        s=minsize.*ones(size(s));
    end
else
    s=ones(length(x),1)*udata.format.minmarkersize;
end

s=sqrt(s); %for consistency with other matlab functions


%% normalize sdata to fit a reasonable range of marker sizes:

%% MAKE FILTERED DATA SERIES
usdata=unique(s);

%determine which patch each row belongs to (based on s)
npatches=numel(usdata);
patchidx=zeros(size(s,1),1);

%assign points to patches
if npatches==1 && ccol==0
    npatches=10;
    uselines=true;
    patchidx=handles.rowcolors{d}{a};
else
    uselines=false;
    for g=1:size(s,1)
        patchidx(g)=find(usdata==s(g),1,'first');
    end
end
%make the patches:

filtpatchhandles=zeros(npatches,1);
mainpatchhandles=zeros(npatches,1);
selpatchhandles=zeros(npatches,1);
%filtered handles
if ~uselines
    for j=1:npatches
        filtidx=patchidx==j & nothidden;
        if any(filtidx)
            filtpatchhandles(j)=line(x(filtidx),y(filtidx),z(filtidx),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter3D);
        else
            filtpatchhandles(j)=line(x(1),y(1),z(1),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
                'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter3D);
        end
    end
else
    if any(nothidden)
        filtpatchhandles=line(x(nothidden),y(nothidden),z(nothidden),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','on','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter3D);
    else
        filtpatchhandles=line(x(1),y(1),z(1),'markersize',usdata(1),'marker',udata.format.marker,...
            'markerfacecolor',handles.colors.filtered,'parent',targetaxes,'tag','filtered',...
            'markeredgecolor','none','visible','off','hittest','on','linestyle','none','buttondownfcn',@buttondownfcnscatter3D);
    end
end

%main handles
if ~uselines
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            mainpatchhandles(j)=patch(x(mainidx),y(mainidx),z(mainidx),c(mainidx),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
                'buttondownfcn',@pointselectscatter3D);
        else
            mainpatchhandles(j)=patch(x(1),y(1),z(1),c(1),'markersize',usdata(j),'marker',udata.format.marker,...
                'markerfacecolor','flat','parent',targetaxes,'cdatamapping','direct','tag','main',...
                'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
                'buttondownfcn',@pointselectscatter3D);
        end
    end
else
    colors=handles.prefs.rowcolors;
    for j=1:npatches
        mainidx=patchidx==j & ~isfiltered & nothidden;
        if any(mainidx)
            mainpatchhandles(j)=line(x(mainidx),y(mainidx),z(mainidx),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                'markeredgecolor','none','hittest','off',...
                'buttondownfcn',@pointselectscatter3D,'linestyle','none');
        else
            mainpatchhandles(j)=line(x(1),y(1),z(1),'markersize',usdata(1),'marker',udata.format.marker,...
                'markerfacecolor',colors(j,:),'parent',targetaxes,'tag','main',...
                'markeredgecolor','none','visible','off','hittest','off',...
                'buttondownfcn',@pointselectscatter3D,'linestyle','none');
        end
    end
end


if strcmp(handles.selectionmode,'point')
    set(mainpatchhandles,'hittest','on');
end

%selected handles
if ~uselines
    for j=1:npatches
        selidx=patchidx==j & isselected & nothidden;
        if any(selidx)
            selpatchhandles(j)=line(x(selidx),y(selidx),z(selidx),'markersize',usdata(j)+1,'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','on','tag','selected','linewidth',2,'hittest','off');
        else
            selpatchhandles(j)=line(x(1),y(1),z(1),'markersize',usdata(j)+1,'marker',udata.format.marker,...
                'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
                'markeredgecolor',handles.colors.selectionhighlight,...
                'visible','off','tag','selected','linewidth',2,'hittest','off');
        end
    end
else
    selidx=isselected & nothidden;
    if any(selidx)
        selpatchhandles=line(x(selidx),y(selidx),z(selidx),'markersize',usdata(1)+1,'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','on','tag','selected','linewidth',2,'hittest','off');
    else
        selpatchhandles=line(x(1),y(1),z(1),'markersize',usdata(1)+1,'marker',udata.format.marker,...
            'markerfacecolor','none','parent',targetaxes,'linestyle','none',...
            'markeredgecolor',handles.colors.selectionhighlight,...
            'visible','off','tag','selected','linewidth',2,'hittest','off');
    end
end

if handles.ishg2
    set(mainpatchhandles,'pickableparts','none')
    set(selpatchhandles,'pickableparts','none')
end


udata.view.filthandles=filtpatchhandles;
udata.view.mainhandles=mainpatchhandles;
udata.view.selhandles=selpatchhandles;
udata.view.patchidx=patchidx;
udata.view.uselines=uselines;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%set buttondownfcns:
set(targetaxes,'hittest','on','buttondownfcn',@buttondownfcnscatter3D)
%ravesetselectionmodescatter3D(handles.fig,handles.activeindex);

%format for fillstyle:
fillstyle=udata.format.markerfill;
if ~uselines
if fillstyle==1
    set(udata.view.mainhandles,'markerfacecolor','none','markeredgecolor','flat');
    set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
elseif fillstyle==2
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor','none');
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
else %fillstyle=3
    set(udata.view.mainhandles,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
    set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
end
else
    for j=1:10
        if fillstyle==1
            set(udata.view.mainhandles(j),'markerfacecolor','none','markeredgecolor',colors(j,:));
            set(udata.view.filthandles,'markerfacecolor','none','markeredgecolor',handles.colors.filtered);
        elseif fillstyle==2
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor','none');
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        else %fillstyle=3
            set(udata.view.mainhandles(j),'markerfacecolor',colors(j,:),'markeredgecolor',[0 0 0]);
            set(udata.view.filthandles,'markerfacecolor',handles.colors.filtered,'markeredgecolor','none');
        end
    end
    
end

set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))


%link axes so subsequent limit changes affect all axes:
% linkaxes([targetaxes handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xy');
set(targetaxes,'xlimmode','manual','ylimmode','manual','zlimmode','manual');

%set axes labels

set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(targetaxes,'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)

set(get(handles.fakerax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakerax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)

set(get(handles.fakeax(handles.activeindex),'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',labelsize,...
    'fontweight',fweight,'interpreter','none','fontname','arial')
set(get(handles.fakeax(handles.activeindex),'YLabel'),'String',handles.allnames{d}{ycol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)
set(get(handles.fakeax(handles.activeindex),'ZLabel'),'String',handles.allnames{d}{zcol},...
    'fontsize',labelsize,'fontweight',fweight,'interpreter','none','fontname',handles.fonts.graph)

%resize axes so axes labels/ticks fit inside the bounding box.
targetindex=find(handles.allgraphs==targetaxes);

%adjust gridlines to match settings:
ravesetgrid3D;


%make all axes have same limits
Q=[handles.fakerax(handles.activeindex) handles.fakeax(handles.activeindex)];

set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'),...
    'xtickmode','manual','xtick',get(targetaxes,'xtick'),...
    'ytickmode','manual','ytick',get(targetaxes,'ytick'),...
    'ztickmode','manual','ztick',get(targetaxes,'ztick'));
set(Q,...
    'xticklabel',get(targetaxes,'xticklabel'),...
    'yticklabel',get(targetaxes,'yticklabel'),...
    'zticklabel',get(targetaxes,'zticklabel'));
set(Q,'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'),'zlim',get(targetaxes,'zlim'))
set(targetaxes,'nextplot','add')
set(Q,'ticklength',[0 0]);

%do mouseovers
if udata.explore.mouseover
    l1=line(nan,nan,nan,'marker',udata.format.marker,'markerfacecolor','none','markeredgecolor',handles.colors.mouseoverhighlight,'linewidth',1.5,...
        'visible','off','parent',targetaxes,'hittest','off','linestyle','none');
    t1=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t2=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t3=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t4=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t5=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    udata.explore.mouseoverhandles=[l1 t1 t2 t3 t4 t5];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    handles.mouseoverhandles=[handles.mouseoverhandles;targetaxes;filtpatchhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverscatter3Daxes,handles.activeindex}};repmat({{@ravemouseoverscatter3Ddata,handles.activeindex}},numel(filtpatchhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(filtpatchhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverscatter3Daxes ravemouseoverscatter3Ddata
end

%update handles
guidata(handles.fig,handles)


%if there is a colorbar, fix it:
%NOTE: shrinking graph to make room for colorbar is handled by raveresizeaxes.

ravemakecolorbar(handles.fig,[]);

%fix rotation if necessary:
view(handles.activegraph,udata.view.az,udata.view.el)
view(handles.fakeax(handles.activeindex),udata.view.az,udata.view.el)
view(handles.fakerax(handles.activeindex),udata.view.az,udata.view.el)

set([handles.activegraph handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ticklength',[0.02 0.0025]);

%note: for 3d graphs, set view before making text ticks, because zticks
%dont get made in a 2d view.

ravemaketextticks(targetaxes,[],udata.view.x,udata.view.y,udata.view.z);
raveresizeaxes(src,[],targetindex)
clear ravearrangeobjects



