function formattabscattermatrix(src,eventdata)
%rave format tab for a scatterplot matrix graph.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %move swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

%% highlight the selected subplot... this only gets highlighted on the format tab, and goes away when you click another tab.
set(handles.activesubgraph,'xcolor',[1 .5 0],'ycolor',[1 .5 0]);


%% clear existing tab
delete(handles.tabobjects)

%% text headers
graphtext=text(.15-.10317,.97*1.0317-.0317,'Graph','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
axestext=text(.15-.10317,.85*1.0317-.0317,'Axes','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.515*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
legendtext=text(.15-.10317,.16*1.0317-.0317,'Legends','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

%% GRAPH UICONTROLS

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
gridtext=uicontrol('position',[origin(1)+25 origin(2)+550 180 20],...
    'style','text','string','Grid Lines:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblobsubax_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblobsubax_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblobsubax_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblobsubax_callback,'userdata',4,'tag','bgblob');

gridindex=eye(3);
gridin=udata.format.grid;


%grid radios
nogrid=uicontrol('position',[origin(1)+100 origin(2)+551 60 20],...
    'style','radiobutton','string','None','backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(1,gridin),'callback',@nogrid_callback);
magrid=uicontrol('position',[origin(1)+152 origin(2)+551 60 20],...
    'style','radiobutton','string','Major','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(2,gridin),'callback',@magrid_callback);
migrid=uicontrol('position',[origin(1)+204 origin(2)+551 55 20],...
    'style','radiobutton','string','Minor','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',gridindex(3,gridin),'callback',@migrid_callback);

handles.gridchecks=[nogrid magrid migrid];

%% Axes UICONTROLS

xlim=get(handles.activesubgraph,'xlim'); %for scatterplot matrix, can select individual subplots and change their properities from here.
ylim=get(handles.activesubgraph,'ylim');

xlabeltext=uicontrol('position',[origin(1)+25 origin(2)+498 180 20],...
    'style','text','string','X Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlimtext=uicontrol('position',[origin(1)+35 origin(2)+474 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xto=uicontrol('position',[origin(1)+158 origin(2)+474 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
xlabelbox=uicontrol('position',[origin(1)+85 origin(2)+500 160 20],...
    'style','edit','string',get(get(handles.activesubgraph,'xlabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@xlabel_callback,'tag','xlabelbox');
xminbox=uicontrol('position',[origin(1)+85 origin(2)+476 70 20],...
    'style','edit','string',xlim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmin_callback,...
    'tag','xminbox');
xmaxbox=uicontrol('position',[origin(1)+175 origin(2)+476 70 20],...
    'style','edit','string',xlim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@xmax_callback,...
    'tag','xmaxbox');
xresetbutton=uicontrol('position',[origin(1)+85 origin(2)+452 160 20],...
    'style','pushbutton','string','Reset X Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@xresetbutton_callback);

ylabeltext=uicontrol('position',[origin(1)+25 origin(2)+410 180 20],...
    'style','text','string','Y Label:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylimtext=uicontrol('position',[origin(1)+35 origin(2)+386 180 20],...
    'style','text','string','Limits:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
yto=uicontrol('position',[origin(1)+158 origin(2)+386 20 20],...
    'style','text','string','to','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
ylabelbox=uicontrol('position',[origin(1)+85 origin(2)+412 160 20],...
    'style','edit','string',get(get(handles.activesubgraph,'ylabel'),'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@ylabel_callback,'tag','ylabelbox');
yminbox=uicontrol('position',[origin(1)+85 origin(2)+388 70 20],...
    'style','edit','string',ylim(1),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymin_callback,...
    'tag','yminbox');
ymaxbox=uicontrol('position',[origin(1)+175 origin(2)+388 70 20],...
    'style','edit','string',ylim(2),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','center','callback',@ymax_callback,...
    'tag','ymaxbox');
yresetbutton=uicontrol('position',[origin(1)+85 origin(2)+364 160 20],...
    'style','pushbutton','string','Reset Y Axis Limits','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold','callback',@yresetbutton_callback);


titletext=uicontrol('position',[origin(1)+25 origin(2)+332 180 20],...
    'style','text','string','Title:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


titel=findobj(handles.activegraph,'tag','title');

titlebox=uicontrol('position',[origin(1)+65 origin(2)+333 180 20],...
    'style','edit','string',get(titel,'string'),'backgroundcolor',[1 1 1],...
    'fontname','arial','fontsize',10,'horizontalalignment','left','callback',@title_callback);

%%  marker DATA SERIES UICONTROLS
markerstyletext=uicontrol('position',[origin(1)+50 origin(2)+280 49 20],...
    'style','text','string','Marker:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markersizetext=uicontrol('position',[origin(1)+66 origin(2)+250 36 20],...
    'style','text','string','Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
markercolortext=uicontrol('position',[origin(1)+60 origin(2)+180 180 20],...
    'style','text','string','Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

markerstyleaxes=axes('units','pixels','position',[origin(1)+102 origin(2)+282 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
gap=.9/7;
%markerstyleoptions:
offcolor=[.9 .9 .9];
oncolor=handles.colors.text;

line(.05,.5,'marker','s','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+gap,.5,'marker','o','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+2*gap,.5,'marker','d','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+3*gap,.5,'marker','^','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+4*gap,.5,'marker','p','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+5*gap,.5,'marker','*','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.05+6*gap,.5,'marker','+','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)
line(.95,.5,'marker','.','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@styleblob_callback)

handles.styleblobs=findobj(markerstyleaxes,'type','line');
%color selected style:

hhh=findobj(handles.styleblobs,'marker',udata.format.marker);
set(hhh,'markerfacecolor',oncolor);


markersizeaxes=axes('units','pixels','position',[origin(1)+100 origin(2)+277-25 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1],'parent',handles.fig);
%gap=.9/7;
%markersizeoptions:
line(.05,.5,'marker','s','parent',markersizeaxes,'markersize',0.5,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.12,.5,'marker','s','parent',markersizeaxes,'markersize',1,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.22,.5,'marker','s','parent',markersizeaxes,'markersize',2,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.33,.5,'marker','s','parent',markersizeaxes,'markersize',3,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.45,.5,'marker','s','parent',markersizeaxes,'markersize',5,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.59,.5,'marker','s','parent',markersizeaxes,'markersize',6,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.75,.5,'marker','s','parent',markersizeaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)
line(.95,.5,'marker','s','parent',markersizeaxes,'markersize',10,'markerfacecolor',offcolor,'markeredgecolor',[0 0 0],'buttondownfcn',@sizeblob_callback)

handles.sizeblobs=findobj(markersizeaxes,'type','line');

%make size blobs the right style
set(handles.sizeblobs,'marker',udata.format.marker)

%color the selected blob
hhh=findobj(handles.sizeblobs,'markersize',udata.format.markersize);
set(hhh,'markerfacecolor',oncolor);

%cdata=ravemakecdata(handles.fig,[],'hsv');

cdata=ravemakecdata(handles.fig,[],handles.cmapnames{udata.format.mcmap});


markercolor=uicontrol('position',[origin(1)+115 origin(2)+182 120 20],...
    'style','pushbutton','string','','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','cdata',cdata,'callback',@mcolor_callback);

%% marker fill uicontrols:
markerfilltext=uicontrol('position',[origin(1)+74 origin(2)+215 29 20],...
    'style','text','string','Fill:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

markerfillaxes=axes('units','pixels','position',[origin(1)+132 origin(2)+217 80 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1]);
gap=.9/2;
%markerstyleoptions:
offcolor=[.6 .6 .6];
oncolor=handles.colors.text;

f1=line(.05,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',[1 1 1],...
    'markeredgecolor',offcolor,'buttondownfcn',@fillblob_callback,'userdata',1,'tag','fillblob');
f2=line(.05+gap,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',offcolor,...
    'markeredgecolor',offcolor,'buttondownfcn',@fillblob_callback,'userdata',2,'tag','fillblob');
f3=line(.95,.5,'marker','p','parent',markerfillaxes,'markersize',16,'markerfacecolor',offcolor,...
    'markeredgecolor',[0 0 0],'buttondownfcn',@fillblob_callback,'userdata',3,'tag','fillblob');

%handles.fillblobs=[f1 f2 f3];
%color selected style:
fillstyle=udata.format.markerfill;
if fillstyle==1
    set(f1,'markeredgecolor',oncolor)
elseif fillstyle==2
    set(f2,'markerfacecolor',oncolor,'markeredgecolor',oncolor)
else %fillstyle=3
    set(f3,'markerfacecolor',oncolor)
end


%% legend uicontrols
legendbutton=uicontrol('position',[origin(1)+85 origin(2)+48 160 20],...
    'style','pushbutton','string','Format Legend','fontname','arial','fontsize',10,...
    'backgroundcolor',handles.colors.buttonbg,'callback',@formatlegend_callback,...
    'fontweight','bold','visible','off');

colorbarcheck=uicontrol('position',[origin(1)+35 origin(2)+46 160 20],...
    'style','checkbox','string','Show Colorbar','fontname','arial narrow','fontsize',10,...
    'backgroundcolor',handles.colors.tabbg,'callback',@colorbarcheck_callback,...
    'fontweight','normal','horizontalalignment','left','value',udata.format.colorbar);


%% group items and finish up
handles.tabobjects=[graphtext axestext dataseriestext bgtext gridtext bgwhite...
    bglight bgdark bgblack nogrid magrid migrid xlabeltext ylabeltext...
    xlimtext ylimtext xlabelbox ylabelbox xminbox yminbox xto xmaxbox...
    xresetbutton yto ymaxbox yresetbutton markerstyletext markersizetext markercolortext...
    markerstyleaxes markersizeaxes markercolor titletext titlebox colorbarcheck...
    legendbutton markerfillaxes markerfilltext legendtext];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);

%% HERE BE CALLBACKS
%
%
%
%
%
%
%
%
%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function styleblob_callback(src,eventdata)

handles=guidata(src);

offcolor=[.9 .9 .9];
set(handles.styleblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.marker=get(src,'marker');
set(handles.fakeax(handles.activeindex),'userdata',udata);


%get target data series

allkidz=get(handles.subax{handles.activeindex},'children');
if iscell(allkidz)
    allkidz=[allkidz{:}];
end

targets=findobj(allkidz,'tag','main','-or','tag','selected');
%filter out line series with no marker
%nottargets=findobj(targets,'marker','none');
%targets=setdiff(targets,nottargets);
%update graph
set(targets,'marker',get(src,'marker'));
%update sizeblobs
set(handles.sizeblobs,'marker',get(src,'marker'));


if strcmp(udata.format.marker,'.') || strcmp(udata.format.marker,'+') || strcmp(udata.format.marker,'*')
    
    offcolor=[.6 .6 .6];
    oncolor=handles.colors.text;
    
    fillblobs=sort(findobj(handles.tabobjects,'tag','fillblob'));
    
    set(fillblobs(1),'markeredgecolor',offcolor)
    set(fillblobs(2),'markerfacecolor',offcolor,'markeredgecolor',offcolor)
    set(fillblobs(3),'markerfacecolor',offcolor)
    
    allkidz=get(handles.subax{handles.activeindex},'children');
    allkidz=[allkidz{:}]
    
    %these only have edges, so must render as an edge:
    set(fillblobs(1),'markeredgecolor',oncolor)
    if udata.view.c
        targets=findobj(allkidz,'type','hggroup','tag','main');
        set(targets,'markerfacecolor','none','markeredgecolor','flat');
        
    else %line mode
        numcolors=10;
        for q=1:numcolors
            targets=findobj(allkidz,'type','line','tag','main','userdata',q);
            set(targets,'markerfacecolor','none','markeredgecolor',handles.prefs.rowcolors(q,:));
        end
    end
end


%{
if udata.view.c
    ravecreatescattermatrix(src,[]);
end
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeblob_callback(src,eventdata)

handles=guidata(src);

offcolor=[.9 .9 .9];
set(handles.sizeblobs,'markerfacecolor',offcolor)
set(src,'markerfacecolor',handles.colors.text)
%update fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.markersize=get(src,'markersize');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%get target data series

allkidz=get(handles.subax{handles.activeindex},'children');
if iscell(allkidz)
    allkidz=[allkidz{:}];
end



targets=findobj(allkidz,'tag','main','-or','tag','selected','-or','tag','filtered');

%update graph
set(targets,'markersize',get(src,'markersize'));

%for scatter mode:
targets=findobj(allkidz,'type','hggroup','tag','main');

targs = get(targets,'children');
set([targs{:}],'markersize',get(src,'markersize'));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function magrid_callback(src, eventdata)
handles=guidata(src);
%inputs=get(handles.fakerax{handles.activeindex},'userdata');

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=2;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

function migrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)


%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=3;
set(handles.fakeax(handles.activeindex),'userdata',udata);

set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')

function nogrid_callback(src, eventdata)
handles=guidata(src);

set(handles.gridchecks,'value',0)
set(src,'value',1)

%updata fakerax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.grid=1;
set(handles.fakeax(handles.activeindex),'userdata',udata);


set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function xmin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(1)=newval;

%get coords of this subgraph:
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function xmax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
xlim=get(handles.activesubgraph,'xlim');
xlim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);

set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

function ymin_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(1)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function ymax_callback(src, eventdata)

handles=guidata(src);
newval=str2double(get(src,'string'));
ylim=get(handles.activesubgraph,'ylim');
ylim(2)=newval;
[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)


function xresetbutton_callback(src,eventdata)
handles=guidata(src);
set(handles.activesubgraph,'xlimmode','auto')
xlim=get(handles.activesubgraph,'xlim');

[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph);
set(handles.subax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakeax{handles.activeindex}(:,c),'xlim',xlim)
set(handles.subfakerax{handles.activeindex}(:,c),'xlim',xlim)

xminbox=findobj(handles.tabobjects,'tag','xminbox');
xmaxbox=findobj(handles.tabobjects,'tag','xmaxbox');

set(xminbox,'string',num2str(xlim(1)))
set(xmaxbox,'string',num2str(xlim(2)))

function yresetbutton_callback(src,eventdata)

handles=guidata(src);
set(handles.activesubgraph,'ylimmode','auto')
ylim=get(handles.activesubgraph,'ylim');


[r,c]=find(handles.subax{handles.activeindex}==handles.activesubgraph); %#ok<NASGU>
set(handles.subax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakeax{handles.activeindex}(r,:),'ylim',ylim)
set(handles.subfakerax{handles.activeindex}(r,:),'ylim',ylim)

%set(handles.fakeax{handles.activeindex},'ylim',ylim)
%set(handles.fakerax{handles.activeindex},'ylim',ylim)

yminbox=findobj(handles.tabobjects,'tag','yminbox');
ymaxbox=findobj(handles.tabobjects,'tag','ymaxbox');

set(yminbox,'string',num2str(ylim(1)))
set(ymaxbox,'string',num2str(ylim(2)))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xlabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'xlabel'),'string',get(src,'string'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ylabel_callback(src,eventdata)

handles=guidata(src);
set(get(handles.activegraph,'ylabel'),'string',get(src,'string'))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mcolor_callback(src,eventdata)
handles=guidata(src);
datasets=raveguidata(src,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

newcmap=ravechoosecolormap(src,[],udata.format.mcmap);


if newcmap==udata.format.mcmap;return;end

udata.format.mcmap=newcmap;


colorz=handles.cmapnames;
newcolormap=colorz{newcmap};

cdata=ravemakecdata(handles.fig,[],newcolormap);
set(src,'cdata',cdata);

set(handles.fakeax(handles.activeindex),'userdata',udata);

if ~udata.view.c;
    set(handles.infobar,'string','Changing colormap has no effect when coloring by row colors.');
    return;
    
end


numgraphs=numel(handles.subax{handles.activeindex});

for q=1:numgraphs
    thisgraph=handles.subax{handles.activeindex}(q);
    
    main=findobj(thisgraph,'tag','main');
    %selected=findobj(thisgraph,'tag','selected');
    
    if udata.view.c~=0
        c=datasets.alldata{udata.view.dataset}(:,udata.view.c);
        %normalize cdata to desired colormap:
        c=(c-min(c))/(max(c)-min(c))*127+1+128*(newcmap-1);
        
        set(main,'cdata',c);
    end
    
    
    %minsize=udata.format.markersize;
    %if udata.view.s==0
    %    set(get(main,'children'),'markersize',sqrt(minsize));
    
    %set(get(selected,'children'),'markersize',sqrt(minsize));
    %end
    allkidz = get(handles.subax{handles.activeindex},'children');
    allkidz=[allkidz{:}];
    
    %for scatter mode:
    targets=findobj(allkidz,'type','hggroup');
    
    %if udata.format.marker=='.'
    targs = get(targets,'children');
    set([targs{:}],'markersize',udata.format.markersize);
    
end

if udata.format.colorbar;
    if udata.view.c
        ravemakecolorbar(handles.fig,[]);
    end
end




%uicontrol(handles.dummyobj)

function fillblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

offcolor=[.6 .6 .6];
oncolor=handles.colors.text;

fillblobs=sort(findobj(handles.tabobjects,'tag','fillblob'));

set(fillblobs(1),'markeredgecolor',offcolor)
set(fillblobs(2),'markerfacecolor',offcolor,'markeredgecolor',offcolor)
set(fillblobs(3),'markerfacecolor',offcolor)

fillstyle=get(src,'userdata');
allkidz=get(handles.subax{handles.activeindex},'children');
allkidz=[allkidz{:}];

if strcmp(udata.format.marker,'.') || strcmp(udata.format.marker,'+') || strcmp(udata.format.marker,'*')
    %these only have edges, so must render as an edge:
    set(fillblobs(1),'markeredgecolor',oncolor)
    if udata.view.c
        targets=findobj(allkidz,'type','hggroup','tag','main');
        set(targets,'markerfacecolor','none','markeredgecolor','flat');
        
    else %line mode
        numcolors=10;
        for q=1:numcolors
            targets=findobj(allkidz,'type','line','tag','main','userdata',q);
            set(targets,'markerfacecolor','none','markeredgecolor',handles.prefs.rowcolors(q,:));
        end
    end
    
else
    if udata.view.c %SCATTER MODE
        targets=findobj(allkidz,'type','hggroup','tag','main');
        if fillstyle==1
            set(targets,'markerfacecolor','none','markeredgecolor','flat');
            set(fillblobs(1),'markeredgecolor',oncolor)
        elseif fillstyle==2
            set(targets,'markerfacecolor','flat','markeredgecolor','none');
            set(fillblobs(2),'markerfacecolor',oncolor,'markeredgecolor',oncolor)
        else %fillstyle=3
            set(targets,'markerfacecolor','flat','markeredgecolor',[0 0 0]);
            set(fillblobs(3),'markerfacecolor',oncolor)
        end
        
    else %LINE MODE
        numcolors=10;
        if fillstyle==1
            for q=1:numcolors
                targets=findobj(allkidz,'type','line','tag','main','userdata',q);
                set(targets,'markerfacecolor','none','markeredgecolor',handles.prefs.rowcolors(q,:));
                set(fillblobs(1),'markeredgecolor',oncolor)
            end
        elseif fillstyle==2
            for q=1:numcolors
                targets=findobj(allkidz,'type','line','tag','main','userdata',q);
                set(targets,'markerfacecolor',handles.prefs.rowcolors(q,:),'markeredgecolor','none');
                set(fillblobs(2),'markerfacecolor',oncolor,'markeredgecolor',oncolor)
            end
        else %fillstyle=3
            for q=1:numcolors
                targets=findobj(allkidz,'type','line','tag','main','userdata',q);
                set(targets,'markerfacecolor',handles.prefs.rowcolors(q,:),'markeredgecolor',[0 0 0]);
                set(fillblobs(3),'markerfacecolor',oncolor)
            end
        end
        
    end
end

%update fakerax userdata

udata.format.markerfill=fillstyle;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function title_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

titel=findobj(handles.activegraph,'tag','title');
titlestring=get(src,'string');
set(titel,'string',titlestring)

udata.format.title=titlestring;
set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorbarcheck_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.format.colorbar=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatescattermatrix(src,[]);





