function ravecreatescattermatrix(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
targetaxes=handles.activegraph; %=handles.activegraph

xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;

a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);

delete(handles.subax{handles.activeindex})
delete(handles.subfakeax{handles.activeindex})
delete(handles.subfakerax{handles.activeindex})

%matrix graph does not use axeslisteners:
% delete(handles.axeslisteners{handles.activeindex});
% handles.axeslisteners{handles.activeindex}=[];

leg=get(handles.fakerax(handles.activeindex),'userdata');
leg = leg(ishandle(leg));
trueleg = findobj(leg,'flat','-not','tag','Colorbar');
if ~isempty(trueleg)
    legpos = get(trueleg,'position');
    legpos=[legpos(1) legpos(2)+legpos(4)];
else
    legpos=[];
end
delete(leg);

%get data
xdata=datasets.alldata{d}(:,xcol);
xnames=handles.allnames{d}(xcol);

ydata=datasets.alldata{d}(:,ycol);
ynames=handles.allnames{d}(ycol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

set(targetaxes,'Visible','off','color','none')

titlestring=udata.format.title;
%make overall title
text(.5,1,titlestring,'parent',targetaxes,'horizontalalignment','center','verticalalignment','bottom',...
    'interpreter',handles.prefs.interpreter,'fontname',handles.fonts.graphtitle,'fontsize',handles.fonts.graphtitlesize,'tag','title')

if isempty(titlestring)
    titoffset=0;
else
    titoffset=10;
end

% Create and plot into small axes
ax = zeros(numel(ycol),numel(xcol));
fakeax = zeros(numel(ycol),numel(xcol));
fakerax = zeros(numel(ycol),numel(xcol));
selected= zeros(numel(ycol),numel(xcol));

rows=numel(ycol);
cols=numel(xcol);

pos = get(targetaxes,'Position');%+[30 30 -45 -30]; %shrink a little to fit tick labels. (now handled by raveresizeaxes)
space = handles.prefs.matrixspacing; % 3 percent space between subaxes

width = floor((pos(3)-numel(xcol)*space)/numel(xcol));
height = floor((pos(4)-titoffset-numel(ycol)*space)/numel(ycol));

% xlim = zeros([rows cols 2]);
% ylim = zeros([rows cols 2]);

%format for fillstyle:
fillstyle=udata.format.markerfill;

if udata.view.c
    if fillstyle==1
        pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
    elseif fillstyle==2
        pvpairs={'markerfacecolor','flat','markeredgecolor','none'};
    else %fillstyle=3
        pvpairs={'markerfacecolor','flat','markeredgecolor',[0 0 0]};
    end
    %override for dots:
    if udata.format.marker=='.' || udata.format.marker=='*' || udata.format.marker=='+'
        pvpairs={'markerfacecolor','none','markeredgecolor','flat'};
    end
else
    thecolors=handles.prefs.rowcolors;
    if fillstyle==1
        facecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        edgecolors=thecolors;
    elseif fillstyle==2
        edgecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        facecolors=thecolors;
    else %fillstyle=3
        edgecolors=zeros(10,3);
        facecolors=thecolors;
    end
    
    %override for dots:
    if strcmp(udata.format.marker,'.') || strcmp(udata.format.marker,'+') || strcmp(udata.format.marker,'*')
        facecolors=['none';'none';'none';'none';'none';'none';'none';'none';'none';'none'];
        edgecolors=thecolors;
    end
end

%initialize handles
main=zeros(rows,cols,10);
filtered=zeros(rows,cols);
selected=zeros(rows,cols);

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);


%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end

%set up grid of axes.
for r=1:1:numel(ycol)
    for c=1:1:numel(xcol)
        axPos = floor([pos(1)+(c-1)*(width+space) pos(2)+(rows-r)*(height+space) ...
            width height]);
        fakerax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,...
            'color',bgcolor);
        fakeax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid);
        ax(r,c) = axes('units','pixels','Position',axPos,'parent',parent,...
            'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnscattermatrix,...
            'userdata',handles.activeindex,'color','none',...
            'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %userdata contains the index of the bigax for this subgraph.
        set(ax(r,c),'visible','on');
        set([fakerax(r,c) fakeax(r,c)],'units','pixels','visible','on');
        %subaxvars{r,c}=[xcol(c) ycol(r)];

        
        
        %FILTERED DATA
        if ~isempty(xdata) && any(nothidden & isfiltered)
            filtered(r,c) = line(xdata(nothidden & isfiltered,c),ydata(nothidden & isfiltered,r),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'markeredgecolor','none','linestyle','none',...
                'visible','on','markersize',udata.format.markersize,'hittest','off',...
                'tag','filtered','userdata',isselected,'linewidth',max(handles.prefs.selectionthickness-1,1),'parent',ax(r,c));
        elseif ~isempty(xdata)
            filtered(r,c) = line(xdata(1,c),ydata(1,r),'marker',udata.format.marker,...
                'markerfacecolor',handles.colors.filtered,'markeredgecolor','none','linestyle','none',...
                'visible','off','markersize',udata.format.markersize,'hittest','off',...
                'tag','filtered','userdata',false(length(xdata(:,c)),1),'linewidth',max(handles.prefs.selectionthickness-1,1),'parent',ax(r,c));
        else
            filtered(r,c)=nan;
        end
        
        %MAIN DATA
        if ~udata.view.c %make as lines:
            rowcolors=handles.rowcolors{udata.view.dataset}{udata.view.analysis};
            %main data series:
            for s=1:10;
                if any(rowcolors==s & nothidden & ~isfiltered)
                    main(r,c,s)=line(xdata(rowcolors==s & nothidden & ~isfiltered,c),ydata(rowcolors==s & nothidden & ~isfiltered,r),'marker',udata.format.marker,...
                        'markersize',udata.format.markersize,'markerfacecolor',facecolors(s,:),'markeredgecolor',edgecolors(s,:),'hittest','off',...
                        'tag','main','userdata',s,'linestyle','none','buttondownfcn',@pointselectscattermatrix,'parent',ax(r,c));
                elseif ~isempty(xdata)
                    main(r,c,s)=line(xdata(1,c),ydata(1,r),'marker',udata.format.marker,'markersize',udata.format.markersize,...
                        'markerfacecolor',facecolors(s,:),'markeredgecolor',edgecolors(s,:),'hittest','off',...
                        'tag','main','userdata',s,'visible','off','linestyle','none','buttondownfcn',@pointselectscattermatrix,'parent',ax(r,c));
                else
                    main(r,c,s)=nan;
                end
            end
            
        else %make as patch
            %s=ones(size(xdata,1),1);
            %make cdata:
            cdata=datasets.alldata{d}(:,ccol);
            %normalize cdata to desired colormap:
            if min(cdata)~=max(cdata)
                cdata=(cdata-min(cdata))/(max(cdata)-min(cdata))*127+128*(udata.format.mcmap-1)+1;
            else
                cdata=(128*(udata.format.mcmap-1)+64)*ones(size(xdata,1),1);
            end

            mainidx=~isfiltered & nothidden;
            if any(mainidx)
               main(r,c)=patch(xdata(mainidx,c),ydata(mainidx,r),cdata(mainidx),'markersize',udata.format.markersize,'marker',udata.format.marker,...
                    'markerfacecolor','flat','parent',ax(r,c),'cdatamapping','direct','tag','main',...
                    'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','hittest','off',...
                    'buttondownfcn',@pointselectscatter,'linestyle','none','parent',ax(r,c));
            else
                main(r,c)=patch(xdata(1,c),ydata(1,r),cdata(1),'markersize',udata.format.markersize,'marker',udata.format.marker,...
                    'markerfacecolor','flat','parent',ax(r,c),'cdatamapping','direct','tag','main',...
                    'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none','visible','off','hittest','off',...
                    'buttondownfcn',@pointselectscattermatrix,'linestyle','none','parent',ax(r,c));
            end
            
            set(ax(r,c),'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);
            
        end
        
        selectedsize=udata.format.markersize;
        
        %SELECTED DATA
        if ~isempty(xdata) && any(nothidden & isselected)
            selected(r,c) = line(xdata(nothidden & isselected,c),ydata(nothidden & isselected,r),'marker',udata.format.marker,...
                'markerfacecolor','none','markeredgecolor',handles.colors.selectionhighlight,'linestyle','none',...
                'visible','on','markersize',selectedsize,'hittest','off',...
                'tag','selected','userdata',isselected,'linewidth',max(handles.prefs.selectionthickness-1,1),'parent',ax(r,c));
        elseif ~isempty(xdata)
            selected(r,c) = line(xdata(1,c),ydata(1,r),'marker',udata.format.marker,...
                'markerfacecolor','none','markeredgecolor',handles.colors.selectionhighlight,'linestyle','none',...
                'visible','off','markersize',selectedsize,'hittest','off',...
                'tag','selected','userdata',false(length(xdata(:,c)),1),'linewidth',max(handles.prefs.selectionthickness-1,1),'parent',ax(r,c));
        else
            selected(r,c)=nan;
        end
        
%         set(ax(r,c),'xlimmode','auto','ylimmode','auto','xgrid','off','ygrid','off')
%         axis(ax(r,c),'tight')
        

    end
end

%store handles to the subplot axes in "sub" handles structure:
handles.subax{handles.activeindex}=ax;
handles.subfakeax{handles.activeindex}=fakeax;
handles.subfakerax{handles.activeindex}=fakerax;
handles.subaxvars{handles.activeindex}=[];
%select the subaxes in position (1,2) to start:
try
    handles.activesubgraph=handles.subax{handles.activeindex}(2);
catch
    handles.activesubgraph=handles.subax{handles.activeindex}(1);
end

guidata(handles.fig,handles);

%Fix axes limits:
% drawnow;
ravematrixplotautoaxeslimits(src,ev,0.05,0.05)

set(ax(1:rows-1,:),'xticklabel','')
set(ax(:,2:cols),'yticklabel','')

set(fakeax(1:rows-1,:),'xticklabel','')
set(fakeax(:,2:cols),'yticklabel','')

set(fakerax(1:rows-1,:),'xticklabel','')
set(fakerax(:,2:cols),'yticklabel','')

set(targetaxes,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'), ...
    'userdata','scattermatrix','tag','');


%% fix tick labels:

for c=1:numel(xcol)
    switch handles.datatype{d}{xcol(c)}
        case {'continuous','discrete','integer'}
            
            ticks=get(ax(end,c),'XTick');
            omag=round(log10(abs(ticks)));
            %set really small numbers=0
%             ticks(omag<-15)=0;
            omag(omag<-15)=0;
            %only reformat if ANY omag is 10^3 or greater.
            % if sum(omag>2)>=(numel(omag)-1)
            if any(omag>2)
%                 ticks=ticks/10^max(omag);
                if strcmp(handles.prefs.interpreter,'tex')
                    xlegendmultipliertext{c} = [' x10^{',num2str(-max(omag)),'}'];
                else
                    xlegendmultipliertext{c} = [' x10^',num2str(-max(omag))];
                end
            elseif  sum(omag<-2)>=(numel(omag)-1)
                if strcmp(handles.prefs.interpreter,'tex')
                    xlegendmultipliertext{c} = [' x10^{',num2str(-min(omag)),'}'];
                else
                    xlegendmultipliertext{c} = [' x10^',num2str(-min(omag))];
                end
            else %no scaling necessary
                xlegendmultipliertext{c} ='';
            end
        otherwise
            xlegendmultipliertext{c}='';
    end
    
    switch udata.view.layout
        case {'square','rectangle','lower'}
            ravemaketextticks(ax(end,c),[],xcol(c),[],[],[])
        case 'upper'
            if c>1
                ravemaketextticks(ax(c,c),[],xcol(c),[],[],[])
            end
    end
end

%shrink tick labels:
xticklabels=findall(ax,'tag','xticklabel');
if iscell(xticklabels);xticklabels=[xticklabels{:}];end
set(xticklabels,'fontsize',handles.fonts.matrixsize);

% Y TICKS:
ylegendmultipliertext=cell(1,numel(ycol)); %first row is xcol, 2nd is ycol.

for r=1:numel(ycol)
    ticks=get(ax(r,1),'YTick');
    omag=round(log10(abs(ticks)));
    %set really small numbers=0
    omag(omag<-15)=0;
    %only reformat if all omag is 10^3 or greater.
    % if sum(omag>2)>=(numel(omag)-1)%         all(omag>2)
    if any(omag>2)
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{r} = [' x10^{',num2str(-max(omag)),'}'];
        else
            ylegendmultipliertext{r} = [' x10^',num2str(-max(omag))];
        end
    elseif sum(omag<-2)>=(numel(omag)-1)%   all(omag<-2)
        if strcmp(handles.prefs.interpreter,'tex')
            ylegendmultipliertext{r} = [' x10^{',num2str(-min(omag)),'}'];
        else
            ylegendmultipliertext{r} = [' x10^',num2str(-min(omag))];
        end
    else %no scaling necessary
        ylegendmultipliertext{r} ='';
    end

    switch udata.view.layout
        case {'square';'lower';'rectangle'}
            ravemaketextticks(ax(r,1),[],[],ycol(r),[],[])
        otherwise %upper
            if r<numel(ycol)
                ravemaketextticks(ax(r,r+1),[],[],ycol(r),[],[])
            end
    end
end

%shrink tick labels:
yticklabels=findall(ax,'tag','yticklabel');
if iscell(yticklabels);yticklabels=[yticklabels{:}];end
set(yticklabels,'fontsize',handles.fonts.matrixsize);

%% make variable name labels:
switch udata.view.layout
    case {'square';'upper';'lower'}
        for k=1:numel(udata.view.x)
            %try %gives error if data set is empty
                set(main(k,k,:),'visible','off','tag','nevershow');
                set(selected(k,k),'visible','off','tag','nevershow');
                set(filtered(k,k),'visible','off','tag','nevershow');
           % end
           set(ax(k,k),'hittest','off')
           
            lims=double(get(ax(k,k),'xlim'));
            tt(k)=text(mean(lims),mean(lims),{xnames{k};xlegendmultipliertext{k}},...
                'parent',ax(k,k),...
                'horizontalAlignment','center','verticalAlignment','middle','interpreter',handles.prefs.interpreter,...
                'backgroundcolor',get(ax(1),'color'),'tag','axeslabel','fontname',handles.fonts.graph,...
                'fontsize',get(targetaxes,'fontsize'));
        end
    otherwise %rectangle
        %make x/y labels on ax(r,c)
        for k=1:numel(udata.view.x)
            set(get(ax(end,k),'xlabel'),'string',xnames(k),'interpreter',handles.prefs.interpreter,...
                'verticalalignment','top','fontname',handles.fonts.graph,'color',handles.colors.axes{udata.format.bg}(5,:));
        end
        
        for k=1:numel(udata.view.y)
            set(get(ax(k,1),'ylabel'),'string',ynames(k),'interpreter',handles.prefs.interpreter,...
                'verticalalignment','bottom','fontname',handles.fonts.graph,'color',handles.colors.axes{udata.format.bg}(5,:));
        end
end

%% Axes change listeners:
% switch udata.view.layout
%     case {'square','rectangle','lower'}
%         %Can't grow this entirely in a loop cause need to initialize it as
%         %a matrix of listeners instead of double.
%         handles.axeslisteners{handles.activeindex}=addlistener(ax(1,1),'YTick','PreSet',@subaxeschangelistener);
%         for r=2:size(ax,1)
%             try %these dont work on 2007b,
%                 handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(r,1),'YTick','PreSet',@subaxeschangelistener);
%             end
%         end
%         
%         for c=1:size(ax,2)
%             try %these dont work on 2007b,
%                 handles.axeslisteners{handles.activeindex}(end+1)=addlistener(ax(end,c),'XTick','PreSet',@subaxeschangelistener);
%             end
%         end
% end
% 


%% turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')

%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.subfakerax{handles.activeindex},'XGrid','on','YGrid','on')
    set(handles.subfakerax{handles.activeindex},'XminorGrid','off','YminorGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.subfakerax{handles.activeindex},'XminorGrid','on','YminorGrid','on')
    set(handles.subfakerax{handles.activeindex},'XGrid','off','YGrid','off')
    set(handles.subfakeax{handles.activeindex},'XGrid','on','YGrid','on')
end

switch udata.view.layout
    case 'upper'
        hidefilter=logical(tril(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
    case 'lower'
        hidefilter=logical(triu(ones(numel(xcol))));
        kidz=findall(ax(hidefilter),'-not','tag','axeslabel');
        set(kidz,'visible','off','tag','nevershow')
        set(ax(hidefilter),'visible','off')
        set(fakeax(hidefilter),'visible','off')
        set(fakerax(hidefilter),'visible','off')
end

udata.view.mainhandles=main;
udata.view.selectedhandles=selected;
udata.view.filteredhandles=filtered;

set(handles.fakeax(handles.activeindex),'userdata',udata);


%Set these at the very end so they don't run during this function (these
%are slow)
% for r=1:1:numel(ycol),
%     for c=1:1:numel(xcol),
%         addlistener(ax(r,c),'XTick','PreSet',@subaxeschangelistener);
%         addlistener(ax(r,c),'YTick','PreSet',@subaxeschangelistener);
%     end
% end
% clear subaxeschangelistener


%Make row color legend
if ccol==0 && numel(unique(handles.rowcolors{d}{a}))>1
    L = ravemakerowcolorlegend(src,ev,d,a,handles.activeindex,legpos);
    legdata = get(handles.fakerax(handles.activeindex),'userdata');
    legdata = union(legdata,L);
    legdata = legdata(ishandle(legdata));
    set(handles.fakerax(handles.activeindex),'userdata',legdata);
    
end



%update da handles
guidata(src,handles)
ravemakecolorbar(handles.fig,[]);
raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
