function ravecreateparallelcoords(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ccol=udata.view.c;
a=udata.view.analysis;
d=udata.view.dataset;

%clear existing graph
cla(targetaxes);
cla(handles.fakeax(handles.activeindex));
% legend(targetaxes,'off');

%reset handle vectors:
udata.view.mintexts=[];
udata.view.maxtexts=[];
udata.view.mindraggers=[];
udata.view.maxdraggers=[];
udata.view.lowfilter=handles.minfilter{d}{a};
udata.view.highfilter=handles.maxfilter{d}{a};
udata.view.varmins=handles.minlimit{d}{a};
udata.view.varranges=handles.maxlimit{d}{a}-handles.minlimit{d}{a};
set(handles.fakeax(handles.activeindex),'userdata',udata)

%get data
x=datasets.alldata{d}(:,xcol);
%normalize xdata:
data1=bsxfun(@minus,x,handles.minlimit{d}{a}(xcol));
data2=bsxfun(@rdivide,data1,(handles.maxlimit{d}{a}(xcol)-handles.minlimit{d}{a}(xcol)));
data2(isnan(data2))=.5;
x=data2;

%maintain selection:
isselected=handles.isselected{udata.view.dataset}{udata.view.analysis}; %in case of non-logical indexing.

%maintain hidden points:
nothidden=handles.isvisible{udata.view.dataset}{udata.view.analysis};

%filter based on max rows
if udata.view.usemaxlines
    switch udata.view.maxlinesmode
        case 'top'
            if udata.view.maxlines>=size(x,1)
                index=1:size(x,1);
            else
                index=1:udata.view.maxlines;
            end  
        case 'random'
            if udata.view.maxlines>=size(x,1)
                index=1:size(x,1);
            else
                rand1=randperm(size(x,1));
                index=rand1(1:udata.view.maxlines);
            end
    end
else
    index=1:size(x,1); 
end

index=sort(index,'ascend');
udata.view.maxlineindex=index;
setudata;

nothidden=nothidden(index);
isselected=isselected(index);
x=x(index,:);

%% get constantvalues:
% if constcol is blank, use a column of ones of same length as xdata.
% Otherwise use the actual data from the constcol column.
%append a column of ones on the end for when constcol="none selected"...
%might need to move this line somewhere else in code.

[c,cdata]=ravemakerowcdata(handles,datasets,udata,ccol,udata.format.lcmap,index);

%{
allcolorz=get(handles.fig,'colormap');
if ccol~=0
    c=datasets.alldata{d}(index,ccol);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.lcmap-1)+1;
    if udata.format.reversecolors
        c=(1-(c-min(c))/(max(c)-min(c)))*127+128*(udata.format.lcmap-1)+1;
    end
    c(isnan(c))=128*(udata.format.lcmap-1)+1;
    cdata=allcolorz(floor(c),:);
    
else %USE ROW COLORS
    c=1920+(handles.rowcolors{d}{a}(index))'; %non-noisy colors cdata=0.5
    cdata=allcolorz(c,:);
    if udata.format.ncolor %do noisy colors
        noise=0.3; %noise multiplier
        cdata=cdata+(rand(size(cdata))-.5)*noise;
        cdata(cdata<0)=0;
        cdata(cdata>1)=1; 
    end
end
%}

%important: for some reason ylim=[0 1] occasionally cuts off filters at the
%edges of the graph. set limits slightly larger to ensure this doesnt
%happen. 




set(handles.activegraph,'ylim',[-0.00001 1.00001],'xlim',[0 1],'ylimmode','manual','xlimmode','manual','visible','off');
set(handles.fakeax(handles.activeindex),'visible','off','ylim',[0 1],'xlim',[0 1],'ylimmode','manual','xlimmode','manual');
set(handles.fakerax(handles.activeindex),'visible','off');

uistack(handles.fakeax(handles.activeindex),'top')

nvars=numel(xcol);
spacing=1/(nvars-1);

%% make background:
background=rectangle('position',[0 0 1 1],'parent',handles.fakerax(handles.activeindex),'linestyle','none',...
    'facecolor',handles.colors.axes{udata.format.bg}(2,:),'tag','background','hittest','on','buttondownfcn',@buttondownfcnparallelcoords);

%% make data:
udata.view.mainhandles=zeros(numel(index),1);
udata.view.selectedhandles=zeros(numel(index),1);
udata.view.hiddenhandles=zeros(numel(index),1);

issurface=udata.view.drawassurface && (udata.view.c>0 || ~udata.format.ncolor);
issurface=false;

if issurface
    %draw as surfaces:
    xdata=repmat(spacing*((1:nvars)-1),size(x,1),1);
    ydata=x;

    udata.view.hiddenhandles=surface(xdata,ydata,-1*ones(size(x)),'parent',handles.activegraph,'tag','filtered',...
        'marker','none','edgecolor',udata.view.offcolor,'facecolor','none','linewidth',udata.format.linesize,'buttondownfcn',@buttondownfcnparallelcoords,...
        'meshstyle','row','normalmode','manual');
    
    xdata(~nothidden,:)=nan;
    ydata(~nothidden,:)=nan;
    
    
    udata.view.mainhandles=surface(xdata,ydata,repmat(double(c(:)),1,size(x,2)),'parent',handles.activegraph,'tag','filtered',...
        'marker','none','edgecolor','flat','facecolor','none','linewidth',udata.format.linesize,'buttondownfcn',@buttondownfcnparallelcoords,...
        'meshstyle','row','normalmode','manual');
    
    xdata(~isselected,:)=nan;
    ydata(~isselected,:)=nan;
    
    udata.view.selectedhandles=surface(xdata,ydata,-1*ones(size(x)),'parent',handles.activegraph,'tag','filtered',...
        'marker','none','edgecolor',handles.colors.selectionhighlight,'facecolor','none','linewidth',udata.format.linesize+handles.prefs.selectionthickness*0.5,...
        'hittest','off','meshstyle','row','normalmode','manual'); 
else
    %draw as lines:
    for r=1:numel(index);
        udata.view.hiddenhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','filtered',...
            'marker','none','color',udata.view.offcolor,'userdata',cdata(r,:),'linewidth',udata.format.linesize,'visible','off','hittest','on',...
            'buttondownfcn',@buttondownfcnparallelcoords);
    end
    
    for r=1:numel(index);
        udata.view.mainhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','main',...
            'marker','none','color',cdata(r,:),'userdata',cdata(r,:),'linewidth',udata.format.linesize,'hittest','on',...
            'buttondownfcn',@buttondownfcnparallelcoords);
    end
    
    for r=1:numel(index);
        udata.view.selectedhandles(r)=line(spacing*((1:nvars)-1),x(r,:),'parent',handles.activegraph,'tag','selected',...
            'marker','none','color',handles.colors.selectionhighlight,'linewidth',udata.format.linesize+handles.prefs.selectionthickness*0.5,...
            'visible','off','userdata',cdata(r,:),'hittest','off');
    end
end

set(handles.fakeax(handles.activeindex),'userdata',udata);

%turn on visiblity of selected lines:
if issurface
    
else
set(udata.view.hiddenhandles(~nothidden),'visible','off');
set(udata.view.mainhandles(~nothidden),'visible','off');
set(udata.view.selectedhandles(isselected & nothidden),'visible','on');    
end


%% make axis labels
set(targetaxes,'xtick',linspace(0,1,numel(xcol)));
ravemaketextticks(targetaxes,[],0,[],[],[],handles.allnames{d}(xcol));
xticklabels=findall(targetaxes,'tag','xticklabel');
posses=get(xticklabels,'position');
if ~iscell(posses);posses={posses};end
for i=1:numel(xticklabels)
    posses{i}(end)=100; %Stack axis labels above filter labels
    set(xticklabels,'handlevisibility','on','position',posses{i});
end

%make filter value labels:
if udata.format.bg==4
    axcolor=[.8 .8 .8];
else
    axcolor=[0 0 0];
end

if udata.format.showlimits
    viz='on';
else
    viz='off';
end

if handles.ishg2
   set(udata.view.selectedhandles,'pickableparts','none') 
end
    

minz=handles.minlimit{d}{a}(udata.view.x);
maxz=handles.maxlimit{d}{a}(udata.view.x);
rangez=maxz-minz;

maxfilterz=udata.view.maxfiltercoord(udata.view.x).*rangez+minz;
minfilterz=udata.view.minfiltercoord(udata.view.x).*rangez+minz;


axpos=get(targetaxes,'position');
if axpos(3)/numel(udata.view.x)>70
    ticksize=handles.fonts.ticksize;
else
    ticksize=handles.fonts.ticksize*0.8;
end

for g=nvars:-1:1 %note, MUST draw these in reverse order.
    udata.view.maxtexts(g)=text(spacing*(g-1),udata.view.maxfiltercoord(udata.view.x(g)),[' ',raveformatnumber(maxfilterz(g))],...
        'parent',handles.fakeax(handles.activeindex),'tag','maxtexter',...
        'backgroundcolor',get(background,'facecolor'),'color',axcolor,'fontsize',ticksize,'fontweight','normal','horizontalalignment','left',...
        'margin',1,'visible',viz,'hittest','on','buttondownfcn',@parallelcoordssetfilter,'fontname',handles.fonts.graph,'userdata',g,'vert','bottom');

    udata.view.mintexts(g)=text(spacing*(g-1),udata.view.minfiltercoord(udata.view.x(g)),[' ',raveformatnumber(minfilterz(g))],'parent',handles.fakeax(handles.activeindex),'tag','mintexter',...
        'backgroundcolor',get(background,'facecolor'),'color',axcolor,'fontsize',ticksize,'fontweight','normal','horizontalalignment','left',...
        'margin',1,'visible',viz,'hittest','on','buttondownfcn',@parallelcoordssetfilter,'fontname',handles.fonts.graph,'userdata',g,'vert','bottom');
end

%% set up axes
L=ones(nvars,1);
for q=1:nvars
    L(q)=line(spacing*(q-1)*ones(1,11),0:.1:1,'parent',handles.activegraph,'tag','parallelaxes',...
        'marker','+','color',handles.colors.axes{udata.format.bg}(1,:),'hittest','off');
end
set(L(1),'xdata',.001*ones(1,11)) %prevents first line getting cut off when you export/rasterize.


%% make top and bottom filters:
xdat=spacing*((1:nvars)-1);

for j=1:nvars
    udata.view.maxdraggers(j,1)=line(xdat(j),udata.view.maxfiltercoord(xcol(j)),10,'parent',handles.activegraph,'tag','maxdragger','marker','v','linestyle','none','markeredgecolor',[0 0 0],...
        'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize,'userdata',j,'clipping','off');
    udata.view.maxdraggers(j,2)=line(xdat(j),udata.view.maxfiltercoord(xcol(j)),-10,'parent',handles.activegraph,'tag','maxdragger','marker','v','linestyle','none','markeredgecolor',handles.colors.wsbg,...
        'markerfacecolor',handles.colors.wsbg,'hittest','on','buttondownfcn',@parallelcoordsfilter,...
        'markersize',udata.format.markersize+15,'userdata',j,'visible','off');
    udata.view.mindraggers(j,1)=line(xdat(j),udata.view.minfiltercoord(xcol(j)),10,'parent',handles.activegraph,'tag','mindragger','marker','^','linestyle','none','markeredgecolor',[0 0 0],...
        'markerfacecolor',udata.format.markercolor,'hittest','on','buttondownfcn',@parallelcoordsfilter,'markersize',udata.format.markersize,'userdata',j,'clipping','off');
    udata.view.mindraggers(j,2)=line(xdat(j),udata.view.minfiltercoord(xcol(j)),-10,'parent',handles.activegraph,'tag','mindragger','marker','^','linestyle','none','markeredgecolor',handles.colors.wsbg,...
        'markerfacecolor',handles.colors.wsbg,'hittest','on','buttondownfcn',@parallelcoordsfilter,...
        'markersize',udata.format.markersize+15,'userdata',j,'visible','off');
end

%% for decided variables, make slider:
for j=1:nvars
    if handles.isconstant{d}(xcol(j))
         line(xdat(j),x(1,j),2000,'parent',handles.activegraph,'tag','decideslider','marker','o','linestyle','none','markeredgecolor',[.4 .4 .4],...
        'markerfacecolor',[.9 .9 .9],'hittest','on','buttondownfcn',@parallelcoordsdecideslide,'markersize',udata.format.markersize,'userdata',j);
    end    
end

setudata;

%do mouseovers
if udata.explore.mouseover
    l1=line(xdat,nan(size(xdat)),'marker','none','color',handles.colors.mouseoverhighlight,'linewidth',udata.format.linesize+1,...
        'visible','off','parent',targetaxes,'hittest','off');
    t1=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t2=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t3=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    t4=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);    
    t5=text(0,0,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'fontsize',handles.fonts.mouseoversize,'fontname',handles.fonts.mouseover);
    
    udata.explore.mouseoverhandles=[l1 t1 t2 t3 t4 t5];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[]; 
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    handles.mouseoverhandles=[handles.mouseoverhandles;background;udata.view.hiddenhandles(:);udata.view.mainhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverparallelcoordsaxes,handles.activeindex}};...
        repmat({{@ravemouseoverparallelcoordsdata,handles.activeindex}},numel(udata.view.hiddenhandles)+numel(udata.view.mainhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};cell(numel(udata.view.hiddenhandles)+numel(udata.view.mainhandles),1)];
    clear ravedefaultwindowbuttonmotionfcn 
end

%update da handles
guidata(src,handles)
%raveresizeaxes(src,[],handles.activeindex)
ravemakecolorbar(handles.fig,[]);

%update filters:
raveupdateanalysis(src,handles,'isfiltered',setdiff(1:numel(handles.allgraphs),handles.activeindex),udata.view.dataset,udata.view.analysis); %update ONLY this graph to coordinate with global filters. 

raveresizeaxes(handles.fig,[],handles.activeindex);
%ravearrangeobjects(handles.fig,[])
%raveresizeaxes(handles.fig,[],handles.activeindex);
%ravearrangeobjects(handles.fig,[])
clear ravearrangeobjects

