function viewtabparallelcoords(src,eventdata) %#ok<INUSD>
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%RAVE: creates uicontrols on the view tab for line graphs.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

%move front tabs to front if necessary
% if handles.fronttabsfront==0 %swap fronttabs and backtabs
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);
% end
% handles.fronttabsfront=1;

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,2,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

set(handles.infobar,'string','');

origin=get(handles.tabaxes,'position');

%% clear existing tab
%delete(handles.tabobjects)

%% get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');
d=udata.view.dataset;

%% text headers
projecttext=text(.15-.10317,.95*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
dataseriestext=text(.15-.10317,.75*1.0317-.0317,'Data Series','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
optionstext=text(.15-.10317,.23,'Options','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title,'visible','on');

%% text labels
analysistext=text(.25-.10317,.920*1.0317-.0317,'Choose Parent Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.25-.10317,.85*1.0317-.0317,'Choose a Data Set:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
xaxistext=text(.25-.10317,.72*1.0317-.0317,'Display Variables:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

sorttext=text(.25-.10317,.50*1.0317-.0317,{'Use arrow buttons to change display order'},'horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',8,'fontname','arial','color',[0 0 0],'fontweight','normal');

colortext=text(.25-.10317,.46*1.0317-.0317,'Color Points by:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');

%% variable lists
analysismenu=uicontrol('position',[origin(1)+55 origin(2)+535 180 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',udata.view.analysis);
datasetmenu=uicontrol('position',[origin(1)+55 origin(2)+490 180 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',udata.view.dataset);


%check varorder... if new variables have been added since graph was
%created, varorder must be updated:

numvars=numel(handles.allnames{d});
if numvars>numel(udata.view.varorder{d})
    lastvar=max(udata.view.varorder{d});
    udata.view.varorder{d}=[udata.view.varorder{d} lastvar+1:numvars];
    udata.view.minfiltercoord=[udata.view.minfiltercoord zeros(1,numvars-lastvar)];
    udata.view.maxfiltercoord=[udata.view.maxfiltercoord ones(1,numvars-lastvar)];
    udata.view.varmins=[udata.view.varmins min(datasets.alldata{d}(:,lastvar+1:numvars))];
    udata.view.varranges=[udata.view.varranges max(datasets.alldata{d}(:,lastvar+1:numvars))-min(datasets.alldata{d}(:,lastvar+1:numvars))];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
end

xaxismenu=uicontrol('position',[origin(1)+55 origin(2)+315 160 120],...
    'style','listbox','string',handles.allnames{d}(udata.view.varorder{d}),'callback',@xaxismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','xaxismenu','max',100,'value',udata.view.x);

colormenu=uicontrol('position',[origin(1)+55 origin(2)+235 180 30],...
    'style','popupmenu','string',[cellstr('<Use Row Colors>'),handles.allnames{d}(1:end)],'callback',@colormenu_callback,...
    'backgroundcolor',[1 1 1],'tag','colormenu','value',udata.view.c+1);

% selectmenu=uicontrol('position',[origin(1)+85 origin(2)+240 180 30],...
%     'style','popupmenu','string',{'Dim infeasible rows';'Select infeasible rows';'Select feasible rows'},'callback',@selectmenu_callback,...
%     'backgroundcolor',[1 1 1],'tag','selectmenu','value',view.filtermode,'enable','off');

upbutton=uicontrol('position',[origin(1)+218 origin(2)+377 35 59],...
    'style','pushbutton','string','^','backgroundcolor',handles.colors.buttonbg,...
    'callback',@upbutton_callback);
downbutton=uicontrol('position',[origin(1)+218 origin(2)+316 35 59],...
    'style','pushbutton','string','v','backgroundcolor',handles.colors.buttonbg,...
    'callback',@downbutton_callback);


showvaluescheck=uicontrol('position',[origin(1)+41 origin(2)+215 190 20],...
    'style','checkbox','string','Always show filter values','backgroundcolor',handles.colors.tabbg,...
    'fontsize',10,'horizontalalignment','left','callback',@showvalues_callback,'fontweight','normal','fontname','arial',...
    'value',udata.format.showlimits);

usemaxlinescheck=uicontrol('position',[origin(1)+41 origin(2)+185 200 30],...
    'style','checkbox','string','Do not draw more than                lines',...
    'backgroundcolor',handles.colors.tabbg,'value',udata.view.usemaxlines,'callback',@usemaxlinescheck_callback,'tag','usemaxlinescheck');

maxlinesedit=uicontrol('position',[origin(1)+173 origin(2)+191 40 18],...
    'style','edit','string',num2str(udata.view.maxlines),...
    'backgroundcolor',handles.colors.tabbg,'callback',@maxlinesedit_callback,'tag','maxlinesedit');

maxlinesmenu=uicontrol('position',[origin(1)+55 origin(2)+156 180 30],...
    'style','popupmenu','string',{'From the top of data table';'Selected randomly'},'callback',@maxlinesmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','maxlinesmenu','value',1);

if strcmp(udata.view.maxlinesmode,'random');set(maxlinesmenu,'value',2);end


drawsurfacecheck=uicontrol('position',[origin(1)+41 origin(2)+115 200 20],...
    'style','checkbox','string','Draw as surface when possible','backgroundcolor',handles.colors.tabbg,...
    'fontsize',10,'horizontalalignment','left','callback',@showvalues_callback,'fontweight','normal','fontname','arial',...
    'value',udata.view.drawassurface);

mouseoverfiltercheck=uicontrol('position',[origin(1)+41 origin(2)+90 220 20],...
    'style','checkbox','string','Enable mouseover on filtered data','backgroundcolor',handles.colors.tabbg,...
    'fontsize',10,'horizontalalignment','left','callback',@mouseoverfiltered_callback,'fontweight','normal','fontname','arial',...
    'value',udata.view.mouseoverfiltered);


%% buttons

%% group items and finish up
handles.tabobjects=[datasettext xaxistext colortext datasetmenu xaxismenu colormenu,...
    analysistext projecttext analysismenu dataseriestext optionstext,...
    sorttext upbutton downbutton usemaxlinescheck,...
    maxlinesedit maxlinesmenu showvaluescheck drawsurfacecheck mouseoverfiltercheck];

handles.activetab='view';

guidata(src,handles);
% update menus to hide excluded vars
viewtabmenufilter(src,[])


%% Put variables in the requested order:
function viewtabmenufilter(src,eventdata)

handles=guidata(src);

%get input info from graph:
udata=get(handles.fakeax(handles.activeindex),'userdata');

% datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
constmenu=findobj(handles.tabobjects,'tag','colormenu');
% useresponsedata=findobj(handles.tabobjects,'tag','useresponsedata');
% excludecheck=findobj(handles.tabobjects,'tag','excludecheck');

%get old strings/values
oldxvalue=udata.view.x;
oldcvalue=udata.view.c;

d=udata.view.dataset;

newxstring=handles.allnames{d}(udata.view.varorder{d});
newxvalue=find(ismember(newxstring,handles.allnames{d}(oldxvalue)));
newcstring=[cellstr('<Use Row Colors>'),handles.allnames{d}];
if oldcvalue~=0
    newcvalue=find(strcmp(handles.allnames{d}(oldcvalue),newcstring));
else %'Row Colors'
    newcvalue=1;
end

%store column numbers of available cols. pulldown menus index into these:
handles.visiblexcols=1:1:length(handles.allnames{d});
handles.visibleccols=[0 find(ismember(handles.allnames{d},newcstring))];

%reset menu strings/values to match displayed data.
set(xaxismenu,'string',newxstring,'value',newxvalue,'listboxtop',newxvalue(1));

set(constmenu,'string',newcstring,'value',newcvalue);

%end

drawnow %cant update listboxtop til it renders :(
set(xaxismenu,'listboxtop',newxvalue(1));
guidata(src,handles)


%% Here be callbacks:%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_callback(src,eventdata) %#ok<INUSD>

handles=guidata(src);
%update other menus to show correct variables
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');

d=get(datasetmenu,'value');

%get initial columns for this data set:
x=[1 2];
y=0;
c=0;
s=0;

%set fakeax userdata
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.dataset=d;
udata.view.x=x;
udata.view.y=y;
udata.view.c=c;
udata.view.s=s;

if d>numel(udata.view.varorder)
    udata.view.varorder{d}=1:size(handles.funnames{d},2);
end

set(handles.fakeax(handles.activeindex),'userdata',udata);
handles.graphdataset(handles.activeindex)=get(src,'value');
guidata(src,handles);

ravecreateparallelcoords(src,[]);

viewtab(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
handles.graphanalysis(handles.activeindex)=get(src,'value');
guidata(src,handles);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.analysis=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateparallelcoords(src,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function xaxismenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
xaxismenu=findobj(handles.tabobjects,'tag','xaxismenu');
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.x=udata.view.varorder{udata.view.dataset}(get(xaxismenu,'value'));
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateparallelcoords(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colormenu_callback(src,eventdata) %#ok<INUSD>
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.c=get(src,'value')-1;
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateparallelcoords(src,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function upbutton_callback(src,ev)
%move all selected variables up one spot.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;

varorder=udata.view.varorder{d};
topvar=find(ismember(varorder,udata.view.x),1,'first');

if topvar>2
    othervars=varorder(topvar-1:end);
    newvarorder=[varorder(1:topvar-2) udata.view.x othervars(~ismember(othervars,udata.view.x))];
else
    othervars=varorder;
    newvarorder=[udata.view.x othervars(~ismember(othervars,udata.view.x))];
end

udata.view.varorder{d}=newvarorder;
set(handles.fakeax(handles.activeindex),'userdata',udata);
viewtabmenufilter(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function downbutton_callback(src,ev)
%move all selected variables down one spot.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
varorder=udata.view.varorder{d};
botvar=find(ismember(varorder,udata.view.x),1,'last');

if botvar<(numel(varorder)-1)
    othervars=varorder(1:botvar+1);
    % newvarorder=[varorder(1:topvar-2) udata.view.x othervars(~ismember(othervars,udata.view.x))];
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x varorder(botvar+2:end)];
    
else
    othervars=varorder;
    newvarorder=[othervars(~ismember(othervars,udata.view.x)) udata.view.x];
end

udata.view.varorder{d}=newvarorder;
set(handles.fakeax(handles.activeindex),'userdata',udata);
viewtabmenufilter(handles.fig,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
function usemaxlinescheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.usemaxlines=get(src,'value');

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function maxlinesedit_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

udata.view.maxlines=str2double(get(src,'string'));

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function maxlinesmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

if get(src,'value')==1
    udata.view.maxlinesmode='top';
else
    udata.view.maxlinesmode='random';
end

set(handles.fakeax(handles.activeindex),'userdata',udata)

ravecreateparallelcoords(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function showvalues_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
texters=findobj(handles.activegraph,'tag','maxtexter','-or','tag','mintexter');
udata.format.showlimits=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateparallelcoords(handles.fig,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mouseoverfiltered_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.view.mouseoverfiltered=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);
ravecreateparallelcoords(handles.fig,[]);
