function ravecreatehistogram(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');


targetaxes=handles.activegraph;
xcol=udata.view.x;
a=udata.view.analysis; 
d=udata.view.dataset; %data set index

%clear existing graph
cla(targetaxes);
% legend(targetaxes,'off');
% try
    delete(handles.subax{handles.activeindex})
    delete(handles.subfakeax{handles.activeindex})
    delete(handles.subfakerax{handles.activeindex})
% end

handles.subax{handles.activeindex}=[];
handles.subfakeax{handles.activeindex}=[];
handles.subfakerax{handles.activeindex}=[];
guidata(handles.fig,handles);

%% make the graph.

%how many graphs to draw?
numvars=length(xcol);
set(targetaxes,'visible','off','xticklabel','','yticklabel','');

%% %% %%
margin=20;
pos = get(targetaxes,'Position');%+[30 30 -30 -30]; %shrink a little to fit tick labels.

xlabelspace=20;% pixels to move up axes to make room for labels at the bottom.
ylabelspace=32;% pixels to allow between graphs to make room for axes labels. (note scattermatrix doesnt have this problem.)
%note y labelspace used for horizontal stack, xlabelspace used for vertical stack.

if  pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
    width = (pos(3)-ylabelspace*(numvars-1)-10)/numvars;
    height = pos(4)-xlabelspace-margin;
    space = .00; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];

else %vertical stacking
    width=pos(3)-ylabelspace-margin;
    height=pos(4)/numvars-xlabelspace;
    space = .03; % 3 percent space between subaxes
    pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
end

if width<3;width=3;end
if height<3;height=3;end

%determine where to put graphs:
if handles.sheet(handles.activeindex)==handles.activesheet
    parent=handles.wspanel;
else
    parent=handles.boneyard;
end

%get colors:
axescolors=handles.colors.axes{udata.format.bg};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

[mainrows,selectedrows,filteredrows]=ravemakeplotdataseries(handles,udata);

isnoisy=handles.isnoise{d}(xcol);

for c=1:numvars
    if ~isnoisy(c)
        [maincount,selectedcount,filtcount,nbars,edges]=...
            ravehistogramcount(handles,d,a,xcol(c),[],mainrows,selectedrows,filteredrows,...
            udata.view.nbars,udata.view.discreteascategory);
        udata.view.edges{c}=edges;
        if  udata.view.normalize || isnoisy(c) %normalize counts (make pdfs)
            nrows=size(handles.isselected{d}{1},1);
            maincount=maincount./nrows;
            selectedcount=selectedcount./nrows;
            filtcount=filtcount./nrows;
            nbars = udata.view.nbars;
        end
    else
        nnn=500000; %Number of samples used to draw histogram
        method.name='rand';
        nbars=udata.view.nbars;
        randdata=ravesamplefromdistribution(handles.fig,method,d,a,xcol(c),nnn);
        edges=linspace(min(randdata),max(randdata),nbars+1);
        udata.view.edges{c}=[];
        maincount=histc(randdata,edges);
        maincount(end-1)=maincount(end-1)+maincount(end);
        maincount=maincount(1:end-1);
        maincount=maincount./nnn;
        selectedcount=zeros(size(maincount));
        filtcount=zeros(size(maincount));
    end

    if  pos(3) > pos(4) % &&  ~udata.view.stack  %horizontal stacking
        axPos = [pos(1)+(c-1)*(width+ylabelspace)+ylabelspace pos(2)+margin ...
            width-ylabelspace height*(1-space)]; %subtract margin to make space for statistics
    else %vertical stacking
        axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
            width-ylabelspace height*(1-space)-20]; %subtract margin to make space for statistics
    end
    
    axPos(axPos<1)=1;
    
    fakerax(c) = axes('units','pixels','Position',axPos,'parent',parent,'color',bgcolor,...
        'hittest','off','gridlinestyle','-','minorgridlinestyle','-','xcolor',lightgrid,'ycolor',lightgrid,...
        'fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize); %#ok<AGROW>
    fakeax(c) = axes('units','pixels','Position',axPos,'parent',parent,...
        'hittest','off','gridlinestyle','-','color','none','xcolor',stronggrid,'ycolor',stronggrid,'fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize); %#ok<AGROW>
    ax(c) = axes('units','pixels','Position',axPos,'parent',parent,...
        'visible','on','nextplot','add','box','on','hittest','on','buttondownfcn',@buttondownfcnhistogram,...
        'userdata',handles.activeindex,'color','none','fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize,...
        'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor); %#ok<AGROW> %userdata contains the index of the bigax for this subgraph.
    
    %for robustness to deletion errors, save these on every iteration:
    handles.subax{handles.activeindex}=ax;
    handles.subfakeax{handles.activeindex}=fakeax;
    handles.subfakerax{handles.activeindex}=fakerax;
    guidata(handles.fig,handles);

    if strcmp(udata.format.orient,'horizontal') 
        view(ax(c),90,90);
        view(fakeax(c),90,90);
        view(fakerax(c),90,90);
    end
    
    if udata.format.drawedge
        fedgecolor=udata.format.edgecolor;
        edgecolor=udata.format.edgecolor;
        sedgecolor=udata.format.edgecolor;
    else
        fedgecolor=handles.colors.filtered;
        edgecolor=udata.format.facecolor;
        sedgecolor=handles.colors.selectionhighlight;
    end
    
    centers=sum([edges;[diff(edges)/2 0]],1);
    centers=centers(1:end-1);
    
    %filtered data series:
    filteredhandle=ravebar(ax(c),centers,filtcount,udata.view.barwidth,'stacked',false,false,handles.ishg2);
    set(filteredhandle,'facecolor',handles.colors.filtered,'hittest','on','tag','filtered','edgecolor',fedgecolor,'buttondownfcn',@pointselecthistogram);

    %main data series:
    mainhandle=ravebar(ax(c),centers,maincount,udata.view.barwidth,'stacked',false,false,handles.ishg2);
    set(mainhandle,'facecolor',udata.format.facecolor,'hittest','on','tag','main','edgecolor',edgecolor,'buttondownfcn',@pointselecthistogram);
    
    if isnoisy(c)
        set(mainhandle,'facecolor',udata.format.randomfacecolor,'edgecolor',udata.format.randomfacecolor)
    end
    
    
    %selected data series
    selectedhandle=ravebar(ax(c),centers,selectedcount,udata.view.barwidth,'stacked',false,false,handles.ishg2);
    set(selectedhandle,'facecolor',handles.colors.selectionhighlight,'hittest','off','tag','selected','edgecolor',sedgecolor);
    if handles.ishg2;set(selectedhandle,'pickableparts','none');end

    axis(ax(c),'tight');
    set(ax(c),'ylimmode','auto','tickdir','in','layer','bottom');
    xlim{c} = get(ax(c),'xlim');  %#ok<AGROW>
    ylim{c} = get(ax(c),'ylim'); %#ok<AGROW>
    
    %draw a baseline to cover the empty data series at the bottom of the
    %graph:
    
    line([xlim{c}(1) xlim{c}(2)],[ylim{c}(1) ylim{c}(1)],[10 10],'color',get(ax(c),'ycolor'),'hittest','off','parent',ax(c));
    
    
    %show filter?
    if udata.view.showfilter && ~isnoisy(c)
       [lof,hif]=raveaddfiltertoaxis(ax(c),d,a,xcol(c),'x');
       set([lof hif],'zdata',100);
    end

    %Make ticks:
%     yticks=get(ax(c),'yticklabel');
%     if ~iscell(yticks);
%        yticks=mat2cell(yticks,ones(size(yticks,1),1),size(yticks,2));
%        yticks=cellfun(@strtrim,yticks,'uniformoutput',0);
%     end

%     ravemaketextticks(ax(c),[],xcol(c),0,[],[],[],yticks) 
    ravemaketextticks(ax(c),[],xcol(c),-1,[]) 
    set([fakeax(c) fakerax(c)],'xtick',get(ax(c),'xtick'),'xticklabel','','ytick',get(ax(c),'ytick'),'yticklabel','',...
        'ztick',get(ax(c),'ztick'),'zticklabel','')
    xlabel(ax(c),handles.allnames{d}{xcol(c)},'interpreter',handles.prefs.interpreter,'fontsize',handles.fonts.graphsize,...
        'fontname',handles.fonts.graph,'vert','top')
end

% %store handles to the subplot axes in "sub" handles structure:
% handles.subax{handles.activeindex}=ax;
% handles.subfakeax{handles.activeindex}=fakeax;
% handles.subfakerax{handles.activeindex}=fakerax;
% guidata(handles.fig,handles);

set(handles.fakeax(handles.activeindex),'userdata',udata);

guidata(handles.fig,handles);

%fix axes limts:
for r=1:numvars
    set(fakerax(r),'xlim',xlim{r},'ylim',ylim{r})
    set(fakeax(r),'xlim',xlim{r},'ylim',ylim{r})
end

%turn off visibility of large axes
set(handles.fakerax(handles.activeindex),'visible','off');
set(handles.fakeax(handles.activeindex),'visible','off');
set(targetaxes,'nextplot','add')


%% %% %%
set(ax,'ylimmode','manual','xlimmode','manual');
set(handles.activegraph,'userdata','histogram');

if udata.view.sharedylim
   ylimz=cell2mat(ylim');
   set(ax,'ylim',[0 max(ylimz(:,2))]);
   set(fakeax,'ylim',[0 max(ylimz(:,2))]);
   set(fakerax,'ylim',[0 max(ylimz(:,2))]);
   
   %update ticks for all:
   for c = 1:numel(ax)
%    yticks=get(ax(c),'yticklabel');
%     if ~iscell(yticks);
%        yticks=mat2cell(yticks,ones(size(yticks,1),1),size(yticks,2));
%        yticks=cellfun(@strtrim,yticks,'uniformoutput',0);
%     end
   ravemaketextticks(ax(c),[],xcol(c),-1,[]) 
   end
end

%Set grid:
ravesetgridmatrix;

%do mouseovers
if udata.explore.mouseover
    for c=1:numvars
        L(c)=line(nan,nan,'marker','none','linewidth',0.5,...
            'visible','off','parent',ax(c),'hittest','off','color',handles.colors.mouseoverhighlight);
        
    end
    if handles.ishg2;set(L,'pickableparts','none');end
        
    udata.explore.mouseoverhandles=[L];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[]; 
    
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    
    mainbarhandles=findobj(ax,'tag','main');
    
    handles.mouseoverhandles=[handles.mouseoverhandles;ax(:);mainbarhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;repmat({{@ravemouseoverhistogramaxes,handles.activeindex}},numvars,1);repmat({{@ravemouseoverhistogramdata,handles.activeindex}},numvars,1)];
    handles.mouseoverdata=[handles.mouseoverdata;cell(numvars,1);cell(numvars,1)];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverhistogramaxes ravemouseoverhistogramdata
end


%select the subaxes in position (1) to start:
handles.activesubgraph=handles.subax{handles.activeindex}(1);

%update handles
guidata(handles.fig,handles)

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects



