function ravecreatebubbleplot(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

udata=get(handles.fakeax(handles.activeindex),'userdata');


%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;


a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph

labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);

leg=get(handles.fakerax(handles.activeindex),'userdata');
if ~isempty(leg)
    delete(leg(ishandle(leg)));
    set(handles.fakerax(handles.activeindex),'userdata',[]);
end

cla(targetaxes);
% legend(targetaxes,'off');

set(targetaxes,'xtickmode','auto','ytickmode','auto')

if xcol==0
    text(0.5,0.5,{'Data set must contain at least';'one discrete or string variable';'in order to create a bubble plot.'},'hor','center','ver','mid','parent',handles.activegraph);
    return
end

%get data
if strcmp(handles.datatype{d}{xcol},'string')
    xvalues=1:numel(handles.discretevalues{d}{a}{xcol});
else
    xvalues=handles.discretevalues{d}{a}{xcol}; %discrete
end

if zcol>0
    zvalues=unique(handles.discretevalues{d}{a}{zcol});
    if strcmp(handles.datatype{d}{zcol},'string')
        zvalues = 1:numel(zvalues);
    end
    
    
elseif zcol==0 %row color.
    zvalues=1:10;
else
    zvalues=1;
end

if ycol>0
    y=datasets.alldata{d}(:,ycol);
else
    y=ones(size(handles.isselected{d}{1},1),1);
end

x=datasets.alldata{d}(:,xcol);

%No point in grouping by the same variable as the x variable:
if zcol==xcol
    zcol=-1;
end


if zcol>0
    z=datasets.alldata{d}(:,zcol);
end



if udata.view.aggregateby==1
    aggfcn = @sum;
elseif udata.view.aggregateby==2
    aggfcn=@mean;
else
    aggfcn=@numel;
end

%make circle data:
circledata=zeros(numel(xvalues),numel(zvalues));
if zcol>0 %stack by variable
    %     for c=1:numel(zvalues)
    %         for r=1:numel(xvalues)
    %             circledata(r,c)=aggregate(y(z==zvalues(c) & x==xvalues(r)),udata.view.aggregateby);
    %         end
    %     end
    circledata= accumarray({x,z},y,[],aggfcn);
    
    
elseif zcol==0 %stack by row color
    %     for c=1:numel(zvalues)
    %         for r=1:numel(xvalues)
    %             circledata(r,c)=aggregate(y(handles.rowcolors{d}{a}==c & x==xvalues(r)),udata.view.aggregateby);
    %         end
    %     end
    circledata= accumarray({x,handles.rowcolors{d}{a}},y,[],aggfcn);
    
    
elseif zcol==-1 %no stacking
    %     for r=1:numel(xvalues)
    %         circledata(r)=aggregate(y(x==xvalues(r)),udata.view.aggregateby);
    %     end
    circledata= accumarray(x,y,[],aggfcn);
    
end

idx=(1:size(circledata,1))';

if udata.format.sortby==2 %largest in center
    [circledata,idx]=sort(circledata,'descend');
elseif udata.format.sortby==3 %smallest in center
    [circledata,idx]=sort(circledata);
end

udata.view.index=idx;


circledata(circledata<0) = 0;
circledata = sqrt(circledata);


%calculate circle packing:
for i = 1:size(circledata,2)
    keep = circledata(:,i)>0;
    [xcenter_temp,ycenter_temp]=ravesimplecirclepack(circledata(:,i));
    xcenter = zeros(size(circledata,1),1);
    ycenter = zeros(size(circledata,1),1);
    xcenter(keep) = xcenter_temp;
    ycenter(keep) = ycenter_temp;
    
    udata.view.centers=[xcenter,ycenter];
    udata.view.radii=circledata;
    
    %make patch data:
    nverts=40;
    thetas=linspace(0,2*pi,nverts);
    
    allverts=zeros(nverts*length(xcenter),2);
    faces=zeros(length(xcenter),nverts);
    
    for a=1:numel(xcenter)
        allverts((a-1)*nverts+1:a*nverts,1)=circledata(a)*cos(thetas)+xcenter(a);
        allverts((a-1)*nverts+1:a*nverts,2)=circledata(a)*sin(thetas)+ycenter(a);
        faces(a,:)=(a-1)*nverts+1:a*nverts;
    end
    
    patchdata.Vertices=allverts;
    patchdata.Faces=faces;
    
    %make cdata:
    if udata.format.colormode==1 %color by radius
        c=circledata(:,i);
        %normalize cdata to desired colormap:
        if ~udata.format.reversecolors
            c=(c-min(c))/(max(c)-min(c))*127+1+128*(udata.format.mcmap-1);
        else
            c=-(c-min(c))/(max(c)-min(c))*127+128*(udata.format.mcmap);
        end
        facecolors=c;
    else
        bc=1932:length(get(handles.fig,'colormap'));
        facecolors=repmat(bc(:),ceil(length(circledata)./size(bc,1)),1);
        facecolors=facecolors(1:length(circledata),:);
    end
    
    set(targetaxes,'climmode','manual','clim',[1 size(get(handles.fig,'colormap'),1)]);
    
    
    if udata.format.drawedge
        edgecolor=udata.format.edgecolor;
    else
        edgecolor='none';
    end
    
    
    udata.view.mainhandles(i)=patch(patchdata,'facecolor','flat','cdata',facecolors,'parent',targetaxes,'hittest','on',...
        'edgecolor',edgecolor);
end

set(targetaxes,'clim',[1 length(get(handles.fig,'colormap'))])
axis(targetaxes,'equal')

set(targetaxes,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')
set(handles.fakerax(handles.activeindex),'visible','off')
set(handles.fakeax(handles.activeindex),'userdata',udata);

%do mouseovers
if udata.explore.mouseover
    l1=patch(nan,nan,handles.colors.mouseoverhighlight,'facecolor','none','edgecolor',handles.colors.mouseoverhighlight,'linewidth',2,...
        'visible','off','parent',targetaxes,'hittest','off');
    t1=text(0,0,100,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t2=text(0,0,100,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t3=text(0,0,100,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t4=text(0,0,100,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'color',[1 1 1],'fontweight','normal','fontsize',14);
    t5=text(0,0,100,'name','visible','off','hittest','off','backgroundcolor','none','margin',1,'parent',targetaxes,'fontsize',14);
    
    udata.explore.mouseoverhandles=[l1 t1 t2 t3 t4 t5];
    udata.explore.cbarmouseovervalue=[]; %initialize these in case of no colorbar
    udata.explore.cbarmouseovertext=[];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    handles.mouseoverhandles=[handles.mouseoverhandles;handles.allblockers(handles.activeindex);udata.view.mainhandles(:)];
    handles.mouseoverfunctions=[handles.mouseoverfunctions;{{@ravemouseoverbubbleaxes,handles.activeindex}};repmat({{@ravemouseoverbubbledata,handles.activeindex}},numel(udata.view.mainhandles),1)];
    handles.mouseoverdata=[handles.mouseoverdata;{[]};{[]}];
    clear ravedefaultwindowbuttonmotionfcn ravemouseoverbubbleaxes ravemouseoverbubbledata
end

ravemakecolorbar(handles.fig,[]);

guidata(handles.fig,handles);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects




