function ravecreateimage(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.


handles=guidata(src);

udata=get(handles.fakeax(handles.activeindex),'userdata');

%backward compatibility:
if ~isfield(udata.view,'fastresize')
   udata.view.fastresize=0;
   set(handles.fakeax(handles.activeindex),'userdata',udata);
end


targetaxes=handles.activegraph;
targetindex=find(handles.allgraphs==targetaxes);
%raveresizeaxes(src,[],targetindex) %fix margins

filename=udata.view.filename;
path=udata.view.path;
% pscales=inputs.pscale;
% gscales=inputs.gscale;
analysis=udata.view.analysis;
d=udata.view.dataset; %data set index

%quit if no file selected:
if udata.view.invalid || isempty(filename)
    text(.5,.5,'No File Selected','parent',targetaxes,'fontname','arial','fontsize',18,...
        'horizontalalignment','center','verticalalignment','middle','color',[0 0 0]);
    guidata(src,handles)
   
    return
end

numpaths=numel(filename);

%clear existing graph
cla(targetaxes);
cla(handles.fakeax(handles.activeindex));
cla(handles.fakerax(handles.activeindex));
% legend(targetaxes,'off');

set(targetaxes,'xlimmode','auto','ylimmode','auto');


%% make the graph. %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%combine path and filenames to get full paths:
if path(end)~='\'
    path=[path,'\'];
end

fullpath=fullfile(path,udata.view.filename{1});

%load image
if strcmpi(fullpath(end-2:end),'png')
    imdata=imread(fullpath,'BackgroundColor',handles.colors.wsbg);
else
    imdata=imread(fullpath);
end

udata.view.imdata=imdata;

%determine how to resize axes:
maxsize=get(handles.allblockers(handles.activeindex),'position');
maxsize=maxsize(3:4)-[24 20];
maxratio=maxsize(1)/maxsize(2);
ratio=size(imdata,2)/size(imdata,1);
if ratio>maxratio %width constrained
    newheight=maxsize(1)/ratio;
    if newheight>size(imdata,1)
        newheight=size(imdata,1);
    end
    heightdiff=maxsize(2)-newheight;
%    oldpos=get(handles.activegraph,'position');
      oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
    if oldpos(3)>size(imdata,2)
        widthdiff=oldpos(3)-size(imdata,2);
        oldpos(3)=size(imdata,2);
    else
        widthdiff=0;
    end
    set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 oldpos(3) newheight])   
else %height constrained
    newwidth=maxsize(2)*ratio;
    if newwidth>size(imdata,2)
        newwidth=size(imdata,2);
    end
    
    widthdiff=maxsize(1)-newwidth;
    %oldpos=get(handles.activegraph,'position');
      oldpos=get(handles.allblockers(handles.activeindex),'position')+[10 10 -24 -20];
    if oldpos(4)>size(imdata,1)
        heightdiff=oldpos(4)-size(imdata,1);
        oldpos(4)=size(imdata,1);
    else
        heightdiff=0;
    end
    set(handles.activegraph,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 newwidth oldpos(4)])
end

%try to resize image to fit axes:
axpos=get(handles.activegraph,'position');
if ~udata.view.fastresize && ~isempty(which('imresize')) && size(imdata,1)>axpos(4)
   imdata=imresize(imdata,[axpos(4) axpos(3)]); 
end
    
image(imdata,'parent',handles.activegraph);
set(handles.activegraph,'visible','off','userdata','image');
set(handles.activegraph,'xlim',[0.5 size(imdata,2)-0.5],'ylim',[0.5 size(imdata,1)-0.5])

set(handles.fakeax(handles.activeindex),'userdata',udata);

%update da handles
guidata(src,handles)

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects
