function ravecreatetableallpoints(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%Allow update functions to supply a copy of handles:
if nargin<3
    handles=guidata(src);
end


udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;
xcol=udata.view.x;

if isempty(xcol) %empty table
    udata.view.vscroll=[];
    udata.view.hscroll=[];
    udata.format.titleaxes=[];
    set(handles.fakeax(handles.activeindex),'userdata',udata);
    guidata(handles.fig,handles);
    return
end

%make data:
data=udata.view.data;
numfun=@(x) sprintf('%-12.5g',x);
%convert data into a cell array of strings:
if ~isempty(data)
    data=num2cell(data);
    for c=1:numel(xcol)
        if ~strcmp(handles.datatype{d}{xcol(c)},'string')
            data(:,c)=cellfun(numfun,data(:,c),'uniformoutput',0);
        else
            data(:,c)=handles.discretevalues{d}{a}{xcol(c)}(cell2mat(data(:,c)));
        end
    end
end

rownames=udata.view.rownames;
colnames=udata.view.colnames;
colwidths=udata.view.colwidths(xcol);
rowheight=udata.view.rowheight;
headerheight=udata.view.headerheight;
headerwidth=udata.view.headerwidth;

%make row header colors:
if ~udata.format.userowcolors
    rhcolors=udata.format.rowheadercolor;
else
    rowcolors=handles.prefs.rowcolors;
    rhcolors=rowcolors(handles.rowcolors{d}{a}(udata.view.allrows),:);
end
  
%make background colors:
isselected=handles.isselected{d}{a}(udata.view.allrows);
isvisible=handles.isvisible{d}{a}(udata.view.allrows);
isfiltered=handles.isfiltered{d}{a}(udata.view.allrows);

rowcolors=repmat(udata.format.unselectedcolor,size(data,1),1);
rowcolors(isfiltered,:)=repmat(udata.format.filteredcolor,sum(isfiltered),1);
rowcolors(~isvisible,:)=repmat(udata.format.hiddencolor,sum(~isvisible),1);
rowcolors(isselected,:)=repmat(udata.format.selectedcolor,sum(isselected),1);

ravemaketablecore(handles.fig,[],handles.activeindex,data,rownames,rowheight,colnames,colwidths,headerwidth,headerheight,rowcolors,rhcolors);

backgroundimg=findobj(handles.fakeax(handles.activeindex),'type','image');
%set(handles.fakeax(handles.activeindex),'hittest','on','buttondownfcn',@selecttableallpoints)
set(backgroundimg,'hittest','on','buttondownfcn',@selecttableallpoints)

if handles.ishg2
    set(backgroundimg,'pickableparts','visible')
end


raveresizetable(handles.fig,[],handles.activeindex);
%fix scrolling
verttablescroll(handles.activegraph,[],0)
% 
% return
% 
% 
% 
% p=handles.fakerax(handles.activeindex); %the panel
% pos=get(p,'position');
% 
% numrows=size(data,1);
% numcols=numel(xcol);
% %note, numrows/numcols refer to the main data table only, headers may have different number of
% %elements.
% 
% nudge=3; %number of pixels occupied by border of uipanel.. avoid drawing into these.
% 
% %clear existing table
% delete(get(handles.activegraph,'children'));
% delete(get(handles.fakeax(handles.activeindex),'children'));
% 
% pkidz=setdiff(get(handles.fakerax(handles.activeindex),'children'),[handles.activegraph handles.fakeax(handles.activeindex)]);
% delete(pkidz);
% delete(udata.view.vscroll);
% delete(udata.view.hscroll);
% delete(udata.format.titleaxes);
% 
% 
% %make title
% udata.format.titleaxes=axes('units','pixels','parent',parent,'position',[pos(1) pos(2)+pos(4)+5 pos(3) 30],'xlim',[0 1],'ylim',[0 1],'visible','off');
% text(0,0,udata.format.title,'parent',udata.format.titleaxes,'fontname',handles.fonts.graphtitle,...
%     'fontsize',handles.fonts.graphtitlesize,'ver','baseline');
% 
% 
% 
% %make column header image:
% if ~isempty(colnames)
%     ch=ones(1,sum(colwidths),3);
%     ch(:,:,1)=format.colheadercolor(1);
%     ch(:,:,2)=format.colheadercolor(2);
%     ch(:,:,3)=format.colheadercolor(3);
%     
%     %color column gridlines:
%     collines=[1 cumsum(colwidths)];
%     ch(:,collines,1)=format.gridlinecolor(1);
%     ch(:,collines,2)=format.gridlinecolor(2);
%     ch(:,collines,3)=format.gridlinecolor(3);
%     
%     chax=axes('parent',p,'units','pixels','position',[headerwidth pos(4)-headerheight sum(colwidths) headerheight],'tag','colheader');
%     image(ch,'parent',chax)
%     for c=1:numel(colnames)
%         text(collines(c)+3,1,colnames{c},'parent',chax,'fontname',udata.format.fontname,...
%             'fontsize',udata.format.fontsize,'color',udata.format.textcolor,...
%             'color',udata.format.textcolor,'interpreter','none');
%     end
%     
%     set(chax,'visible','off')
%     
%     line([0.5 sum(colwidths)+.5],[1.5 1.5],'color',udata.format.gridlinecolor,'parent',chax);
%     
% else
%     chax=[];
% end
% 
% %make row header image:
% if ~isempty(rownames) && ~isempty(data)
%     
%     
%     rh=ones(rowheight*numrows,1,3);
%     rhcolors=repmat(udata.format.textcolor,numrows,1);
%     
%     if ~udata.format.userowcolors
%         rh(:,:,1)=format.rowheadercolor(1);
%         rh(:,:,2)=format.rowheadercolor(2);
%         rh(:,:,3)=format.rowheadercolor(3);
%         
%     else
%         rowcolors=handles.prefs.rowcolors;
%         for c=1:10
%             theserows=udata.view.allrows(handles.rowcolors{d}{a}(udata.view.allrows)==c);
%             if ~isempty(theserows)
%                 adders=repmat(0:rowheight-1,size(theserows,1),1);
%                 pixes=theserows(:).*rowheight-rowheight;
%                 pixes=bsxfun(@plus,pixes,adders);
%                 pixes=pixes(:)+1;
%                 rh(pixes,:,1)=rowcolors(c,1);
%                 rh(pixes,:,2)=rowcolors(c,2);
%                 rh(pixes,:,3)=rowcolors(c,3);
%             end
%         end
%         
%         shade = bsxfun(@times,[0.299 0.587 0.114],rowcolors(handles.rowcolors{d}{a}(udata.view.allrows),:));
%         shade=sum(shade,2);
%         rhcolors(shade>=0.5,:)=0;
%         rhcolors(shade<0.5,:)=1;
%     end
%     
%     %color row gridlines:
%     rowlines=[rowheight:rowheight:numrows*rowheight];
%     rh(rowlines,:,1)=format.gridlinecolor(1);
%     rh(rowlines,:,2)=format.gridlinecolor(2);
%     rh(rowlines,:,3)=format.gridlinecolor(3);
%     
%     rhax=axes('parent',p,'units','pixels','position',[0 pos(4)-size(rh,1)-headerheight headerwidth numrows*rowheight],...
%         'tag','rowheader','xlim',[.5 headerwidth+.5]);
%     image(rh,'parent',rhax,'tag','rowheaderimage')
%     %set(rhax,'xlim',[.5 headerwidth+.5]) %this must come after image call.
%     for r=1:numel(rownames)
%         text(.5+3/headerwidth,r*rowheight,rownames{r},'parent',rhax,'verticalalignment','bottom',...
%             'fontname',udata.format.fontname,'fontsize',udata.format.fontsize,...
%             'color',udata.format.textcolor,'interpreter','none')
%     end
%     set(rhax,'visible','off')
%     
%     line([1.5 1.5],[.5 rowheight*numrows+.5],'parent',rhax,'color',udata.format.gridlinecolor);
% else
%     rhax=[];
% end
% 
% 
% handles.subax{handles.activeindex}=[chax rhax];
% 
% if ~isempty(colnames) &&  ~isempty(rownames) && ~isempty(data)
%     %make a little block in upper corner to scroll both at once:
%     cornerax=axes('parent',p,'units','pixels','position',[0 pos(4)-headerheight headerwidth headerheight],'color',[1 1 1],'tag','cornerblocker');
%     image(ones(1,1,3)*.95,'parent',cornerax);
%     set(cornerax,'visible','off','tag','cornerblocker');
%     line([.5 1.5],[1.5 1.5],'parent',cornerax,'color',udata.format.gridlinecolor);
%     line([.5 .5],[.5 1.5],'parent',cornerax,'color',udata.format.gridlinecolor);
% end
% 
% 
% %% make main table image:
% 
% if ~isempty(data)
%     rownumz=1:numel(udata.view.allrows);
%     t=ones(numrows*rowheight,1,3);
%     t(:,:,1)=udata.format.unselectedcolor(1);
%     t(:,:,2)=udata.format.unselectedcolor(2);
%     t(:,:,3)=udata.format.unselectedcolor(3);
%     
%     %color according to row state:
%     selectedrows=rownumz(isselected(udata.view.allrows));
%     adders=repmat(0:rowheight-1,size(selectedrows,1),1);
%     pixes=selectedrows(:).*rowheight-rowheight;
%     pixes=bsxfun(@plus,pixes,adders);
%     pixes=pixes(:)+1;
%     t(pixes,:,1)=udata.format.selectedcolor(1);
%     t(pixes,:,2)=udata.format.selectedcolor(2);
%     t(pixes,:,3)=udata.format.selectedcolor(3);
%     
%     hiddenrows=rownumz(~isvisible(udata.view.allrows));
%     adders=repmat(0:rowheight-1,size(hiddenrows,1),1);
%     pixes=hiddenrows(:).*rowheight-rowheight;
%     pixes=bsxfun(@plus,pixes,adders);
%     pixes=pixes(:)+1;
%     t(pixes,:,1)=udata.format.hiddencolor(1);
%     t(pixes,:,2)=udata.format.hiddencolor(2);
%     t(pixes,:,3)=udata.format.hiddencolor(3);
%     
%     %color horizontal grid lines:
%     rowlines=[rowheight:rowheight:(rowheight*numrows)];
%     t(rowlines,:,1)=udata.format.gridlinecolor(1);
%     t(rowlines,:,2)=udata.format.gridlinecolor(2);
%     t(rowlines,:,3)=udata.format.gridlinecolor(3);
%     
%     %display:
%     set(handles.fakeax(handles.activeindex),'position',[headerwidth pos(4)-size(t,1)-headerheight sum(colwidths) size(t,1)],'color',[1 0 1]);
%     
%     tax=handles.fakeax(handles.activeindex);
%     image(t,'parent',handles.fakeax(handles.activeindex),'tag','mainimage','buttondownfcn',@selecttableallpoints,'hittest','on');
%     set(handles.fakeax(handles.activeindex),'visible','off');
%     tpos=get(handles.fakeax(handles.activeindex),'position');
%     set(tax,'buttondownfcn',@selecttableallpoints,'hittest','on')
%     
%     
%     %% make text for table:
%     texthandles=zeros(size(data));
%     
%     collines=[1 cumsum(colwidths)];
%     rowlines=rowheight:rowheight:(rowheight*numrows);
%     rowcoords=rowlines-rowheight/2+1;
%     for r=1:numrows
%         for c=1:numcols
%             texthandles(r,c)=text(collines(c)+3,-rowcoords(r),data{r,c},'parent',handles.activegraph,'erasemode','normal',...
%                 'fontname',udata.format.fontname,'fontsize',udata.format.fontsize,'color',udata.format.textcolor,'hittest','off','interpreter','none');
%         end
%     end
%     
%     set(handles.activegraph,'position',get(handles.fakeax(handles.activeindex),'position')+[0 0 0 1],'color','none','hittest','off');
%     ylim=get(handles.fakeax(handles.activeindex),'ylim');
%     set(handles.activegraph,'ylim',[-ylim(2) -ylim(1)+1],'visible','off','xlim',[.5 sum(colwidths)+.5]);
%     
%     %make vertical gridlines:
%     for cc=1:numel(collines)
%         line([collines(cc) collines(cc)],[-ylim(2) -ylim(1)+1],'color',udata.format.gridlinecolor,'tag','gridlines','parent',handles.activegraph,'hittest','off');
%     end
%     
%     dax=handles.activegraph;
%     
%     %hide invisible text:
%     lastvisiblerow=ceil((pos(4)-headerheight)/rowheight);
%     lastvisiblecol=find([0 cumsum(colwidths)]>pos(3)-headerwidth,1);
%     
%     set(texthandles(lastvisiblerow+3:end,:),'visible','off');
%     set(texthandles(:,lastvisiblecol+1:end),'visible','off');
%     
%     if udata.view.tight && tpos(1)+tpos(3)<maxpos(3) && tpos(4)+headerheight<maxpos(4)
%         %table is smaller than the uipanel, so shrink uipanel to just hold table.
%         set(p,'position',[pos(1) pos(2) tpos(1)+tpos(3) tpos(4)]);
%         pos=get(p,'position');
%         
%         set(chax,'position',[0 pos(4)-headerheight sum(colwidths) headerheight]);
%         set(rhax,'position',[0 pos(4)-size(rh,1)-nudge headerwidth numrows*rowheight]);
%         set(tax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);
%         set(dax,'position',[headerwidth pos(4)-size(t,1)-nudge tpos(3) tpos(4)]);
%         
%     end
% 
%     udata.view.texthandles=texthandles;
% else
%     udata.view.texthandles=[];
% end
% 
% %% make scrollbars:
% 
% %image command resets userdata:
% 
% set(handles.fakeax(handles.activeindex),'userdata',udata);
% set(handles.activegraph,'userdata',activetype);
% set(rhax,'tag','rowheader');
% set(chax,'tag','colheader');
% 
% hstep=(pos(3)-headerwidth);
% vstep=(pos(4)-headerheight);
% 
% 
% udata.view.rowheader=rhax;
% udata.view.colheader=chax;
% set(handles.fakeax(handles.activeindex),'userdata',udata);
% 
% if ~isempty(data)
%     hscroll=drawhscroll(parent,targetaxes,[pos(1) pos(2)-16 pos(3)-1 17],0,tpos(3),hstep,0,@horztablescroll,1);
%     vscroll=drawvscroll(parent,targetaxes,[pos(1)+pos(3)-1 pos(2)+1 17 pos(4)-1],0,tpos(4),vstep,tpos(4),@verttablescroll,1);
% else
%     hscroll=[];
%     vscroll=[];
% end
% 
% udata.view.vscroll=vscroll;
% udata.view.hscroll=hscroll;
% 
% set(handles.fakeax(handles.activeindex),'userdata',udata);
% guidata(handles.fig,handles);
% 
% raveresizetable(src,[],handles.activeindex);
% return
% 
% 
% %%  fix table sizing (copied from ravemovetable)
% 
% bpos=[get(handles.grabbers{handles.activeindex}(1),'xdata')-2 get(handles.grabbers{handles.activeindex}(1),'ydata')-3 ...
%     get(handles.grabbers{handles.activeindex}(3),'xdata')-get(handles.grabbers{handles.activeindex}(1),'xdata')+5 ...
%     get(handles.grabbers{handles.activeindex}(3),'ydata')-get(handles.grabbers{handles.activeindex}(1),'ydata')+5];
% 
% p=handles.fakerax(handles.activeindex); %uipanel of this table
% 
% if isempty(udata.format.title)
% maxpos=bpos+[24 26 -46 -41];
% else
% maxpos=bpos+[24 26 -46 -61];    
% end
% 
% tpos=get(handles.activegraph,'position');
% tablesize=[0 0 udata.view.headerwidth+tpos(3) udata.view.headerheight+tpos(4)];
% 
% handles.temp.oldpos=get(p,'position');
% 
% if udata.view.tight && tablesize(3)<maxpos(3)
%     %table is smaller than the uipanel, so shrink uipanel to just hold table.
% %     newpos(1)=bpos(1)+(maxpos(3)-tablesize(3))/2+24;
%     newpos(1)=bpos(1)+24;
%     newpos(3)=tablesize(3);
% else
%     newpos(1)=bpos(1)+24;
%     newpos(3)=maxpos(3);
% end
% 
% if udata.view.tight && tablesize(4)<maxpos(4)
%     %table is smaller than the uipanel, so shrink uipanel to just hold table.
%     %newpos(2)=bpos(2)+(maxpos(4)-tablesize(4))/2+26;
%     newpos(2)=bpos(2)+maxpos(4)-tablesize(4)+26;
%     newpos(4)=tablesize(4);
% else
%     newpos(2)=bpos(2)+26;
%     newpos(4)=maxpos(4);
% end
% 
% set(p,'position',[newpos(1) newpos(2) newpos(3) newpos(4)]);
% pos=get(p,'position');
% 
% %tpos=get(handles.fakeax(handles.activeindex),'position');
% 
% t=get(handles.fakeax(handles.activeindex),'position');
% hstep=(pos(3)-udata.view.headerwidth);
% vstep=(pos(4)-udata.view.headerheight);
% 
% if ~isempty(udata.view.hscroll)
%     userdata=get(udata.view.hscroll(4),'userdata');
%     hslidevalue=userdata(2);
%     [haxhandles]=drawhscroll(parent,handles.activegraph,[pos(1) pos(2)-17 pos(3)-1 17],0,t(3),hstep,hslidevalue,@horztablescroll,1);
%     delete(udata.view.hscroll);
%     udata.view.hscroll=haxhandles;
% end
% 
% if ~isempty(udata.view.vscroll)
%     userdata=get(udata.view.vscroll(4),'userdata');
%     vslidevalue=userdata(2);
%     [vaxhandles]=drawvscroll(parent,handles.activegraph,[pos(1)+pos(3) pos(2)+1 17 pos(4)-1],0,t(4),vstep,vslidevalue,@verttablescroll,1);
%     delete(udata.view.vscroll);
%     udata.view.vscroll=vaxhandles;
% end
% 
% set(handles.fakeax(handles.activeindex),'userdata',udata);
% 
% guidata(handles.fig,handles);
% 
% p=handles.fakerax(handles.activeindex);
% 
% chax=findobj(p,'tag','colheader');
% cb=findobj(p,'tag','cornerblocker');
% 
% if isempty(chax);chax=[];end;
% 
% newpos=get(p,'position');
% 
% try
%     oldpos=handles.temp.oldpos;
% catch %#ok<CTCH>
%     oldpos=newpos;
% end
% 
% set(udata.format.titleaxes,'position',[newpos(1) newpos(2)+newpos(4)+5 newpos(3) 30]);
% 
% deltah=newpos(4)-oldpos(4);
% 
% chaxpos=get(chax,'position');
% chaxpos=chaxpos+[0 deltah 0 0];
% set(chax,'pos',chaxpos);
% 
% if ~isempty(cb) %isempty if no data
%     cbpos=get(cb,'position');
%     cbpos(2)=chaxpos(2);
%     set(cb,'pos',cbpos);
% end
% 
% clear ravearrangeobjects
