function [model,rhmodel]=ravecreatemaintablemodel(src,ev,d,a)
%makes a RaveTableModel of dataset d for the main data table, using
%analysis a.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

if isfield(handles,'indata');
    handles=handles.indata;
end


data=datasets.alldata{d};
varnames=handles.allnames{d};
isfunction=handles.isfunction{d};
isnoise=handles.isnoise{d};
isconstant=handles.isconstant{d};
isvirtual = datasets.isvirtual{d};
isrowshaded=~handles.isvisible{d}{a};
rowcolors=handles.rowcolors{d}{a};
isvalid=datasets.isvalid{d};
isfiltered=handles.isfiltered{d}{a};





if size(data,1)>0
    datatypes=handles.datatype{d};
    
    isnumerical = ismember(datatypes,{'continuous','integer','logical','discrete'});
    
    
    textdataidx = data(:,~isnumerical);
    textdata=cell(size(textdataidx));
    
    data=data(:,isnumerical);
    
    formatstring=char(110+zeros(size(isnumerical)));

    idx = find(~isnumerical);
     for c=1:numel(idx);
        switch datatypes{idx(c)}
            case {'string','bitstring','list'}
                textdata(:,c)=handles.discretevalues{d}{a}{idx(c)}(textdataidx(:,c));
                formatstring(idx(c))='t';
            otherwise %do nothing, values are double by default.
                
        end
     end

    datprep = RaveTableModelDataPreparator();

    %Crappy workaround: datprep uses the rowcount of "data" so it breaks when data is empty, as in the case of an all-text data set. As a crappy
    %workaround, just use a dummy data vector to avoid having to fix this in java:
    if isempty(data)
        data = ones(size(data,1),1);
    end
    
    
    dblArray = datprep.getDataObject(data,textdata,formatstring);
    
    isFunction=logical(isfunction);
    isNoise=logical(isnoise);
    isConstant=logical(isconstant);
    isVirtual = logical(isvirtual);
    isDataValid=logical(isvalid);
    isRowShaded=logical(isrowshaded);
    isRowFiltered=logical(isfiltered);
    rowNumbers=num2cell(int32(1:size(data,1)))';
    jRowColors=num2cell(int32(rowcolors-1));
    
else
    dblArray=[];
end

if ~isempty(dblArray)
    dmodel=javax.swing.table.DefaultTableModel(dblArray,varnames);
    javaObjectEDT(dmodel);
    
    clear datprep dblArray
    model1=RaveTableSorter(dmodel);
    javaObjectEDT(model1);
    model1.setIsFunction(isFunction);
    model1.setIsNoise(isNoise);
    model1.setIsConstant(isConstant);
    model1.setIsVirtual(isVirtual);
    model1.setIsRowShaded(isRowShaded);
    model1.setIsRowFiltered(isRowFiltered);
    model1.setRowColors(jRowColors);
    model1.setIsDataValid(isDataValid);
    rhmodel=javax.swing.table.DefaultTableModel(rowNumbers,{' '});
    javaObjectEDT(rhmodel);
else %data set is empty
    dmodel=javax.swing.table.DefaultTableModel(varnames,0); %make an empty table
    javaObjectEDT(dmodel);
    model1=RaveTableSorter(dmodel);
    javaObjectEDT(model1);
    rhmodel=javax.swing.table.DefaultTableModel(0,{' '});
end

deezcolors=handles.prefs.rowcolors;

for c=1:10
    R=deezcolors(c,1);
    G=deezcolors(c,2);
    B=deezcolors(c,3);
    model1.setRowColorSpec(c-1, java.awt.Color(R,G,B));
end

model=model1;
set(handle(model1,'callbackproperties'),'TableChangedCallback',{@ravemaintablechangedcallback,handles.fig});

%% fix column visibility

try %gives error when creating model for a new data set.
    shows=handles.maintable.x{handles.maintable.d};
catch
    return
end

cols=handles.maintable.jtable.getColumnModel().getColumns;
colcount=handles.maintable.jtable.getColumnModel().getColumnCount;

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    if ismember(modelIdx+1,shows) %show this column
        thisCol.setMinWidth(15);
        thisCol.setPreferredWidth(75);
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);
    end
end
