function ravedisplaymaintable(src,ev)
%shows the current tablemodel in the main table.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
d=handles.maintable.d;
a=handles.maintable.a;

if d==0;return;end;

set(handles.infobar,'string','Preparing data table...');
drawnow;


%First replace the existing model with an empty model in order to free up java memory used by the existing model:
handles.maintable.models{d}=javax.swing.table.DefaultTableModel(handles.allnames{d},0); %make an empty table
javaObjectEDT(handles.maintable.models{d});
handles.maintable.rhmodels{d}=javax.swing.table.DefaultTableModel(0,{' '});
jtable=handles.maintable.jtable;
jtable.setModel(handles.maintable.models{d});
guidata(handles.fig,handles);

%Make new model:

%if ismember('model',handles.tablemodelupdatespending{d})
    %make new model:
    [handles.maintable.models{d} handles.maintable.rhmodels{d}]=ravecreatemaintablemodel(handles.fig,[],d,a);
%    handles.handles.tablemodelupdatespending{d}=setdiff(handles.handles.tablemodelupdatespending{d},'model');
    guidata(handles.fig,handles);
%end


%resize scroll pane:
if handles.tabsvisible
    left=30+315*handles.prefs.tabscale;
    bottom=1;
    top=40;
    height=handles.height-40;
    width=handles.width-(30+315*handles.prefs.tabscale);
else
    left=1;
    bottom=1;
    top=35;
    height=handles.height-2;
    width=handles.width-2;
end

jscroll=handles.maintable.jscroll;
panel=jscroll.getParent; %although jscroll was initially created with handles.fig as its parent, it looks like matlab makes a uipanel to hold it. need to move this panel to move jscroll
javaObjectEDT(panel);
%not sure why this works but it does: if you just setLocation, it will move
%back to its original position when it redraws. Seems like you can only set
%position once for every time you set location, so you need to alternate
%them.
% drawnow; %avoid java crash
panel.setLocation(left,top) %java coordinates from top left
try %has been removed in 2011b... seems like setLocation is all you need 
    panel.setPosition(left,top,width,height) %matlab coordinates from bottom left
end


jtable=handles.maintable.jtable;
sorter=handles.maintable.models{d};
jtable.setModel(handles.maintable.models{d});
jjj=jtable.getTableHeader();
sorter.setTableHeader(jjj);

%set column widths:
cols=jtable.getColumnModel().getColumns;
colcount=jtable.getColumnModel().getColumnCount;
shows=handles.maintable.x{handles.maintable.d};

for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    
    if ismember(modelIdx+1,shows) %show this column
       thisCol.setMinWidth(15);
       thisCol.setPreferredWidth(handles.maintable.colwidths{handles.maintable.d}(c+1));        
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);        
    end    
end

%set callback for table:
set(handle(jtable,'callbackproperties'),'MouseReleasedCallback',{@raveTableMouseReleaseCallback,handles.fig})

%set callback for column headers:
jhead=jtable.getTableHeader();
jhead.setReorderingAllowed(1); %allow column reordering %NOTE: this might break some things in RaveTableSorter that assume no reordering.
set(handle(jhead,'callbackproperties'),'MouseReleasedCallback',{@raveTableHeaderCallback,handles.fig})
set(handle(jhead,'callbackproperties'),'MouseDraggedCallback',{@raveTableHeaderDraggedCallback,handles.fig})


%set up row headers:
rowHeaderViewport=handles.maintable.jscroll.getComponent(3);
rowHeader=rowHeaderViewport.getComponent(0);
rowHeader.setModel(handles.maintable.rhmodels{d});

nrows=sorter.getRowCount;
rheight=rowHeader.getRowHeight;
ndigits=ceil(log10(nrows));

rowHeader.setPreferredSize(java.awt.Dimension(max(10*ndigits+6,20),nrows*rheight))
rowHeader.setSize(java.awt.Dimension(max(10*ndigits+6,20),nrows*rheight))

rowHeaderViewport.setPreferredSize(java.awt.Dimension(max(10*ndigits+6,20),rowHeaderViewport.getHeight));

handles.maintable.jscroll.doLayout;

   
%ROW HEADER RENDERER:
%(NOTE: Column header renderer is built into RaveTableSorter.java)
rrrr=RaveDefaultRowHeaderRenderer();
javaObjectEDT(rrrr);

tm=rowHeader.getColumnModel();
tc=tm.getColumn(0);

if handles.maintable.colorrows 
    tc.setCellRenderer(rrrr);
else
    tc.setCellRenderer(handles.maintable.defaultRowHeaderRenderer);
end

%callbacks:
set(handle(rowHeader,'callbackproperties'),'MousePressedCallback',{@raveRowHeaderCallback,handles.fig})

%CELL RENDERERS:
rdouble=RaveDefaultTableCellRenderer();
obj=java.lang.Double(1);
class=obj.getClass();
jtable.setDefaultRenderer(class,rdouble);
obj=java.lang.Object;
class=obj.getClass();
jtable.setDefaultRenderer(class,rdouble);

rstring=RaveStringTableCellRenderer();
obj=java.lang.String;
class=obj.getClass();
jtable.setDefaultRenderer(class,rstring);
obj=java.lang.Character('a'); %for length==1 strings
class=obj.getClass();
jtable.setDefaultRenderer(class,rstring);


rinteger=RaveIntegerTableCellRenderer();
obj=java.lang.Integer(1);
class=obj.getClass();
jtable.setDefaultRenderer(class,rinteger);

rnoise=RaveNoiseTableCellRenderer();
obj=java.awt.Rectangle(0,0,1,1);
class=obj.getClass();
jtable.setDefaultRenderer(class,rnoise);


%select rows that are selected in this analysis:
selectedRows=find(handles.isselected{d}{a}); 
if ~isempty(selectedRows)
    selectedRows=selectedRows-1;
    
    for g=1:numel(selectedRows);
        selectedRows(g)=handles.maintable.models{d}.viewIndex(selectedRows(g));
    end
    ncols=handles.maintable.models{d}.getColumnCount();
    
    jtable.setColumnSelectionInterval(0,ncols-1);
    jtable.setRowSelectionInterval(selectedRows(1),selectedRows(1));
    for i=2:numel(selectedRows)
        jtable.addRowSelectionInterval(selectedRows(i),selectedRows(i));
    end
end

%display visible columns for this model:
shows=handles.maintable.x{handles.maintable.d};
cols=handles.maintable.jtable.getColumnModel().getColumns;
colcount=handles.maintable.jtable.getColumnModel().getColumnCount;
for c=0:(colcount-1)
    thisCol=cols.nextElement;
    modelIdx=thisCol.getModelIndex;
    thiswidth=thisCol.getWidth;
    if thiswidth>0
        handles.maintable.colwidths{handles.maintable.d}(modelIdx+1)=thisCol.getWidth;
    end
    
    if ismember(modelIdx+1,shows) %show this column
       thisCol.setMinWidth(15);
       thisCol.setPreferredWidth(handles.maintable.colwidths{handles.maintable.d}(c+1));        
    else
        thisCol.setMinWidth(0);
        thisCol.setPreferredWidth(0);        
    end    
end

set(handles.maintable.handle,'visible','on');

set(handles.infobar,'string','')
%make sure renderers match current state:
%rowHeader.repaint;
%jtable.repaint;
jscroll.repaint; %this seems to take care of all the repainting that is needed.




