function results=raveoptgamultiobj(src,ev,settings)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

raveinitializeoptimizeranimation;

d=settings.d;
a=settings.a;


%% make objective/constraint functions:
[functionname,index,mins,maxs]=ravemakeminimizefunction(src,ev,settings);
nvars=numel(index);
FITNESSFCN=str2func(functionname);
settings.normalizationfactors=[mins;maxs-mins];

%% make options structure

options=gaoptimset(@gamultiobj);
options.PopulationSize=settings.pop;
options.CrossoverFraction=settings.pcross;
options.ParetoFraction=settings.pfrac;
options.MigrationDirection=settings.migrationdirection;
options.MigrationInterval=settings.migrationinterval;
options.MigrationFraction=settings.migrationfraction;
options.Generations=settings.gens;
options.TimeLimit=settings.timelimit;
options.StallGenLimit=settings.stallgenlimit;
options.TolFun=settings.tolfun;
options.TolCon=settings.tolcon;
options.SelectionFunction=settings.selectionfun;

if settings.plotprogress
    options.PlotFcns=@raveanimategads;
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';



%% do optimizer:
%tic
[X,FVAL,EXITFLAG,OUTPUT,population] = gamultiobj(FITNESSFCN,nvars,[],[],[],[],zeros(nvars,1),ones(nvars,1),options); %#ok<ASGLU>

handles=guidata(src);

if settings.plotprogress && ishandle(handles.temp.crapwindow)
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
indexx=settings.funvars;

if strcmp(settings.method,'simulatedannealing') || strcmp(settings.method,'patternsearch')
    population=X;
end


finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
finalpop(:,indexx)=population;

%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});
objective=raveevaluatefunction(src,settings.d,respvars,finalpop,handles);
finalpop(:,respvars)=objective;

guidata(handles.fig,handles);

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

ravefinishoptimizeranimation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
objs=handles.allnames{settings.d}(settings.y);

for o=1:numel(objs)
    objs{o}=[objs{o},', '];
end

objs=cell2mat(objs);

results.name=['GAMultiObj ',objs,num2str(settings.pop),'p ',num2str(settings.gens),'g   (',handles.datasetnames{settings.d},')'];
[sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=sortedpop;
results.finalobjective=objective(index,:);
results.settings=settings;


