function results=raveoptnbi(src,ev,settings)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);

%need two settings structures, one for nbi and one for the single objective
%optimizer:
nbisettings=settings;
d=nbisettings.d;
a=nbisettings.a;

%start animation:
raveinitializeoptimizeranimation;

%prepare single obj opt:
alg=nbisettings.subalgorithm;

%this is going to overwrite the settings structure:
if ~isempty(which(['optimizersettings',alg]))
    eval(['optimizersettings',alg]);
else
    errordlg(['No optimizer named "',alg,'" was found in your Rave directory'])
end

clear settings; %clear settings to avoid using it later since it's name is now ambiguous.
%load settings for single objective optimizer:
[tabsettingnames,tabsettingvalues]=raveparseoptimizerobjectivetab(src,ev);

if strcmp(nbisettings.settingsfile,'default')
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
else
    [displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,ev,alg); %#ok<NASGU,ASGLU>
    [fdisplaynames,fsettingvalues]=raveparseoptimizersettingsfile(dest,headerindex);
    fileidx=[];
    for f=1:numel(fdisplaynames)
        idx=find(strcmp(fdisplaynames{t},displaynames));
        if ~isempty(idx)
            fileidx(end+1)=idx; %#ok<AGROW>
        end
    end
    settingnames=displaynames(fileidx);
    settingvalues=fsettingvalues;
end

settingnames=[tabsettingnames(:);settingnames(:)];
settingvalues=[tabsettingvalues(:);settingvalues(:)];

%turn into a structure:
namevaluepairs=[settingnames settingvalues]';
namevaluepairs=namevaluepairs(:)';
subsettings=struct(namevaluepairs{:});

%fix things that don't get autofilled correctly:
subsettings.method=alg;
subsettings.stopbutton=nbisettings.stopbutton;
subsettings.showinfo=0;
subsettings.Display='final';

%solve for univariate optima:
optfun=str2func(['raveopt',alg]);

numobj=numel(nbisettings.y);

infotext=findobj(nbisettings.plotaxes,'tag','optinfotext');
totalfunz=0; %total function evals

for i=1:numobj
    subsettings.y=nbisettings.y(i);
    subsettings.targets=nbisettings.targets(i);
    
    %get variables for this functions:
    indexx=ravegetindependentvars(handles,d,a,subsettings.y,subsettings.activeconstraints);
    
    [junk,thesevars]=ismember(indexx,nbisettings.funvars);
    subsettings.funvars=nbisettings.funvars(thesevars);
    subsettings.x0=nbisettings.x0(thesevars);
    subsettings.normalizationfactors=nbisettings.normalizationfactors(:,thesevars);
    
    if nbisettings.showinfo
        set(infotext,'string',{['Finding univariate optima: ',num2str(i),'/',num2str(numobj)]});
    end
    
    %do optimizer:
    overload.alreadyanimating=true;
    subresults{i}=optfun(src,[],subsettings,overload);
    subresults{i}.independentvars=indexx;
    individualoptima(:,i)=ravenbievaluateobjectives(nbisettings,subresults{i})';
    
    %prevent animations from being deleted:
    if nbisettings.plotprogress
        anihandles=findobj(nbisettings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','opthistory');
        set(anihandles,'tag','nbiopthandles');
        set(findobj(nbisettings.plotaxes,'tag','optcurrent'),'tag','nbifinalpoint','hittest','off');
    end
end

%hide opt history:
delete(findobj(nbisettings.plotaxes,'tag','nbiopthandles'));

%form weight grid:
weights=ravetrigridn(nbisettings.ndivs,numobj)';

%make search base points:
basepoints=zeros(size(weights));
for j=size(weights,2):-1:1
    basepoints(:,j)=individualoptima*(weights(:,j));
end

weightgridhandle=ravenbiplotweightgrid(nbisettings,basepoints);

%make somewhere to store optimization results:
finalobjectives=zeros(size(weights));
nbiresults=cell(1,size(weights,2));

%assign nbiresults of individual minima:
[r,c]=find(weights'==1);
for C=1:numobj
    nbiresults{r(c(C))}=subresults{c(C)};
    finalobjectives(:,r(c(C)))=individualoptima(:,c(C));
end

%shift points to have the utopia point at the origin:
utopiapoint=diag(individualoptima);
phi=bsxfun(@minus,individualoptima,utopiapoint(:));
Quasinorm=-phi*ones(size(phi,1),1);

%form objective function. THis objective function takes in x and t, and
%returns only t. This ensures that the optimizer uses all variables in
%the x vector (they are needed to calculate constraints)
%
%The format for the vector of independent variables will be [x1 x2 ... xn t],
%so the objective function just returns the last element of this
%vector, multipled by -1 since we want to maximize t.
obj=@(X) -100*X(:,end);

%form vector design analysis function:
F=@(xdata) raveevaluatefunction(src,d,nbisettings.y,xdata,handles)';

%form equality constraint function: (b is a column of weights, the barycentric
%coordinates)
eqconstraint= @(xdata,b,t) bsxfun(@plus,b,bsxfun(@times,t(:)',Quasinorm)) - F(xdata); %awkwardly vectorized to handle optimizers that evaluate multiple points simultaneously, e.g. genetic algorithms

%add a column to necessary fields of subsettings to represent the t
%variable:
%subsettings.normalizationfactors
subsettings.funvars=[nbisettings.funvars NaN];
subsettings.normalizationfactors=[nbisettings.normalizationfactors [0;1]]; %used by the constraint

%other optimizar settings:
subsettings.MaxIter=1000;
subsettings.Display='off';
subsettings.TolCon=1e-2;
subsettings.TolFun=1e-6;

%start the search at the first univariate optima (this is the first entry
%in the weights array):

%make overload structure:
overload.objectivefcn=obj;
overload.mins=[zeros(1,numel(nbisettings.funvars)) -1000];
overload.maxs=[ones(1,numel(nbisettings.funvars)) 1000];
overload.normalizationfactors=[nbisettings.normalizationfactors [0;1]];
overload.alreadyanimating=1;

%make starting point of independent variables:
lastbest=zeros(1,size(handles.funnames{d},2));
lastbest(subresults{1}.independentvars)=[subresults{1}.finalinputs];
lastbest=lastbest(nbisettings.funvars); %keep only the columns needed for nbi optimization
lastbest=[lastbest 0]; %append extra column for t.

isfinished=false(size(weights,2),1);

%do optimizations:
disp('Beginning optimizations along quasinormal vectors...');
for globalidx=1:size(weights,2)
    
    %draw search direction:
    ravenbiplotsearchdirection(nbisettings,basepoints,Quasinorm,globalidx);
    
    %make constraint function specific to this iteration:
    anoncon = @(X) ravenbi_constraint_calc(X,handles,subsettings,eqconstraint,basepoints(:,globalidx));
    overload.constraintfcn=anoncon;
    
    subsettings.x0=lastbest; %start searching at final point of last iteration.
    
    if nbisettings.showinfo
        set(infotext,'string',{['Solving 1-D search: ',num2str(globalidx),'/',num2str(size(basepoints,2))]});
    end
    
    %DO OPTIMIZER %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    nbiresults{globalidx}=optfun(src,[],subsettings,overload);
    nbiresults{globalidx}.finalinputs=nbiresults{globalidx}.finalinputs(1:end-1);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %prevent animations from being deleted:
    if nbisettings.plotprogress
        delete(findobj(nbisettings.plotaxes,'tag','nbisearchdir'));
        uistack(findobj(nbisettings.plotaxes,'tag','nbifinalpoint'),'top');
        delete(findobj(nbisettings.plotaxes,'tag','optinfeasible','-or','tag','oftfeasible','-or','tag','opthistory'));
        finalpointhandle=findobj(nbisettings.plotaxes,'tag','optcurrent');
        if ~isempty(finalpointhandle)
            
            set(finalpointhandle,'tag','nbifinalpoint','hittest','off');
            
            finalx=get(finalpointhandle,'xdata');
            finaly=get(finalpointhandle,'ydata');
            finalz=get(finalpointhandle,'zdata');
            
            if size(basepoints,1)==3
                LLL=ravenbiplotsearchdirection(nbisettings,basepoints,[finalx(1);finaly(1);finalz(1)]-basepoints(:,globalidx),globalidx);
            elseif size(basepoints,1)==2
                LLL=ravenbiplotsearchdirection(nbisettings,basepoints,[finalx(1);finaly(1)]-basepoints(:,globalidx),globalidx);
            end
        end
    end
    isfinished(globalidx)=true;
    
    %start next iteration at final point that is nearest to the next iteration's weight vector:
    if globalidx<size(weights,2)
        nextweights=weights(:,globalidx+1)';
        finishedweights=weights(:,1:globalidx)';
        dists=ipdm(nextweights,finishedweights);
        idx=find(dists==min(dists),1);
        lastbest=[nbiresults{idx}.finalinputs 0];
        lastbest=bsxfun(@minus,lastbest,overload.normalizationfactors(1,:));
        lastbest=bsxfun(@rdivide,lastbest,overload.normalizationfactors(2,:));
        lastbest(isnan(lastbest))=0; %this happens for constant variables (divide by zero);
        lastbest(isinf(lastbest))=0;
        lastbest(end)=0;
        
    end
end

delete(findobj(nbisettings.plotaxes,'tag','optinfeasible','-or','tag','oftfeasible','-or','tag','opthistory','-or','tag','nbisearchdir','-or','tag','nbigrid'));
%delete(weightgridhandle);
drawnow;

%finish animation:
settings=nbisettings; %<-- ravefinishanimation needs this to be called settings.
ravefinishoptimizeranimation;

%export results:
results.finalinputs=zeros(size(weights,2),numel(nbisettings.funvars));
results.finalobjective=zeros(size(weights,2),numel(nbisettings.y));
results.settings=settings;
results.name=[handles.datasetnames{settings.d},' NBI, ',num2str(nbisettings.ndivs),' pt grid, '];
for r=1:numel(nbisettings.y)
    results.name=[results.name,handles.allnames{d}{nbisettings.y(r)}];
    if r<numel(nbisettings.y)
        results.name=[results.name,' '];
    end
end

for b=1:size(weights,2);
    results.finalinputs(b,:)=nbiresults{b}.finalinputs;
end

%evaluate objectives:
xdata=zeros(size(weights,2),size(handles.funnames{d},2));
xdata(:,nbisettings.funvars)=results.finalinputs;
results.finalobjectives=raveevaluatefunction(src,d,nbisettings.y,xdata,handles);

results.weights=weights';

%make animations deletable:
finalpointhandle=findobj(nbisettings.plotaxes,'tag','nbifinalpoint');
set(finalpointhandle,'tag','optplots');
% delete(findobj(nbisettings.plotaxes,'tag','nbisearchdir'));
% delete(findobj(nbisettings.plotaxes,'tag','nbisearchdir'));






