function results=raveoptnsgaii(src,ev,settings)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%settings is a structure with the following fields:
% .animation= logical, export animation or not
% .gif = logical, export as gif vs. avi
% .fps = double, frames per second. or empty for real time.
% .loop = logical, loop gif vs. run once.
% .pcross =decimal, probability of crossover
% .pmutation =decimal, probability of mutation.
% .nbits = int, number of bits
% .method='nsgaii';
% .pop= population size
% .gens = number of generations
% .plotprogress = logical, animate progress on selected graph or not
% .plotvars = list of varaibles whose functions need to be evalualted in order to draw this graph
% .targets = best value for each objective
% .d = dataset to use
% .a = analysis to use
% .y = list of variables to use as objectives
% .activeconstraints = index of constraints to use
% .startbutton = handle of startbutton
% .stopbuttonpos = location in pixels to put pause button

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

settings.progresswindow=0;
genssincebetter=0;
minever=1;

% settings.crowdfcn='minx';

if settings.progresswindow
   fff=figure;
   aaa=subplot(1,2,1);
   ylim([0 settings.pop+1])
   bbb=subplot(1,2,2);
   set(bbb,'yscale','log')
end

pcross=settings.pcross;
pmutation=settings.pmutation;
nbits=settings.nbits; %number of bits to discretize each input var

stopbutton=findobj(handles.tabobjects,'tag','optimizestopbutton');

%prepare animation:
raveinitializeoptimizeranimation;

%tic %used in calculating fps for animations.
%tic has been moved below...


paretotarget=settings.targets;%[str2double(get(targ1edit,'string')) str2double(get(targ2edit,'string'))];
paretotarget=-paretotarget;

if any(~isinf(paretotarget));
    errordlg('NSGA-II solver currently only supports -inf and inf as Objective Goals')
    error('NSGA-II solver currently only supports -inf and inf as Objective Goals')
end

%constraintlist=findobj(handles.tabobjects,'tag','constraintlist');
handles.temp.activeconstraints=settings.activeconstraints; %get(constraintlist,'value')-1;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.

d=settings.d;
a=settings.a;

%% convert numerical options to numbers:
nVars=numel(settings.x0); %#ok
numberofinequalities=nnz(strcmp('inequality',handles.constrainttype{d}{a})); %#ok
numberofequalities=nnz(strcmp('equality',handles.constrainttype{d}{a})); %#ok

numericaloptions={'pop','gens','nbits','pmutation','nmutation','pcross'};
for n=1:numel(numericaloptions) 
    if ischar(settings.(numericaloptions{n})) && ~isempty(settings.(numericaloptions{n}))
        settings.(numericaloptions{n})=eval(settings.(numericaloptions{n}));
    end
end

M=numel(settings.y); %number of objectives
N=floor(settings.pop);
T=floor(settings.gens);
if T<1;T=1;end

%get variables for these functions:
indexx=settings.funvars;
nvars=numel(indexx);

ranges=[handles.minlimit{d}{a}; handles.maxlimit{d}{a}];
    types=handles.datatype{settings.d}(settings.funvars);
    dofloor=false(1,numel(types));
    for u=1:numel(types)
        switch types{u}
            case {'integer','logical','discrete','string'}
                ranges(2,u)=ranges(2,u)+0.999;
                dofloor(u)=1;
        end
    end
rangestep=(ranges(2,:)-ranges(1,:))/(2^nbits-1);

%need to replace columns for discrete variables in varTable with only discrete values:
%but first need to decide how to determine which variables are discrete.
%let this slide for now.

%generate initial population:

domakepopulation=true;
if ~isempty(settings.popinitial)
    if ischar(settings.popinitial)
        dapath=which(settings.popinitial);
        if ~isempty(dapath)
            data=importdata(dapath);
            if isfield(data,'data');
                domakepopulation=false;
                Preal=data.data;
                settings.pop=size(Preal,1);
                N=settings.pop;
                
                
                P=zeros(N,nvars*nbits,'uint8'); %P is binary representation of population
              
                
                %normalize 0 to 1:
                Preal=bsxfun(@minus,Preal,ranges(1,:));
                Preal=bsxfun(@rdivide,Preal,(ranges(2,:)-ranges(1,:)));
                %convert to binary:
                Preal(isnan(Preal))=0;
                Preal(isinf(Preal))=0;
                Preal=round(Preal*(2^nbits-1));
                
                for g=1:numel(indexx)
                    p=Preal(:,indexx(g));
                    P(:,nbits*(g-1)+1:nbits*g)=rem(floor(p*pow2(1-nbits:0)),2);
                end

                %check, convert back to decimal:
                P3d=reshape(P,[size(P,1),nbits,nvars]);
                
                Prowindex=ones(N,size(ranges,2));
                
                colnums=(size(P3d,2):-1:1)-1;
                
                if nbits<=16.1
                    colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                elseif nbits<=32.1
                    colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                else
                    colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
                end
                
                prodz=P3d.*colz;
                
                expoz=2.^prodz.*P3d;
                
                sumz=sum(expoz,2);
                
                %convert back to double here:
                Prow2D=permute(sumz,[1 3 2]);
                Prowindex(:,indexx)=Prow2D;
                Preal=bsxfun(@times,Prowindex,rangestep);
                Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population

            end
        end
    end
end

startgen=0;
starttime=0;

if domakepopulation
    thistag=get(src,'tag');
    if strcmp(thistag,'startbutton')
        
            if nbits<=16.1
                P=uint16(round(rand(N,nvars*nbits))); %P is binary representation of population
            elseif nbits<=32.1
                P=uint32(round(rand(N,nvars*nbits)));
            else
                P=uint64(round(rand(N,nvars*nbits)));
            end
      
        startgen=0;
        starttime=0;
    elseif strcmp(thistag,'continuebutton')
        P=handles.optimizerresults(end).finalbin;
        
        if nbits<=16.1
            P=uint16(P); %P is binary representation of population
        elseif nbits<=32.1
            P=uint32(P);
        else
            P=uint64(P);
        end
        
        startgen=handles.optimizerresults(end).lastgen;
        starttime=handles.optimizerresults(end).lasttime;
        delete(handles.temp.lasthandles(ishandle(handles.temp.lasthandles)));
        delete(src);
        handles.tabobjects=setdiff(handles.tabobjects,src);
        N=size(P,1);
    end
end
guidata(handles.fig,handles);

%% evaluate this population:
[objective cviolation]=evaluate_population(P,M,nbits,nvars,paretotarget,ranges,rangestep,indexx,handles,udata,settings,dofloor);

%% calculate sorting/crowding of initial population:
[F,rank]=fast_non_dominated_sort(objective,cviolation);

%% calculate crowding distance: (for all frontiers)
distance=cell(0);
crowding=zeros(N,1);

for i=1:numel(F)
    I=objective(F{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
    distance{i}=crowdingdist(I,settings,i);
    crowding(F{i})=distance{i};
end

Plast=P;
P=make_new_population(P,crowding,rank,pcross,pmutation,cviolation,handles,settings);

tic %used to calculate ETA and total time.
set(handles.infobar,'string','\fontname{monospace} 0 \fontname{comic sans ms} Generations Completed.    Calculating ETA...')
elapsed=0; %initialize timer


%% DO GENERATIONS:
for t=1:T
    %t;
    handles.temp.lastgen=t;
    guidata(handles.fig,handles);
    
    %% combine new/old population:
    R=[P;Plast]; %combine new/old popualtions
    
    
    objectivelast=objective;
    cviolationlast=cviolation;
    
    %evaluate population at R:
    [objectivenew cviolationnew]=evaluate_population(P,M,nbits,nvars,paretotarget,ranges,rangestep,indexx,handles,udata,settings,dofloor);
    
    objective=[objectivenew;objectivelast];
    cviolation=[cviolationnew;cviolationlast];
    
    
    %% calculate sorting/crowding of initial population:
    [F,rank]=fast_non_dominated_sort(objective,cviolation);
    %trim down to N members using rank/crowding sort:
    
    %% calculate crowding distance: (for all frontiers)
    distance=cell(0);
    crowding=zeros(N,1);
    for i=1:numel(F)
        I=objective(F{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
        distance{i}=crowdingdist(I,settings,i);
        crowding(F{i})=distance{i};
    end

    survivors=[];
    i=1;
    while numel(survivors)<N && i<=N
        if numel(survivors)+numel(F{i})<=N
            survivors=union(survivors,F{i}); %add all members of F to survivors
        else %add only enough members to make numel(survivors)==N
            shortage=N-numel(survivors);
            
            %add this many elements to survivors:
            cdist=crowding(F{i});
            index=F{i}; %index of cdist points into R
            data=[cdist index'];
            data=sortrows(data);
            %take the last (shortage) rows of data (these are least crowded)
            survivors=union(survivors,data(end-shortage+1:end,2));
            
        end
        i=i+1;
    end
    
    %pull out these rows of R:
    Plast=R(survivors,:);
    objective=objective(survivors,:);
    cviolation=cviolation(survivors,:); 
    rank=rank(survivors);
    crowding=crowding(survivors);
    
    P=make_new_population(Plast,crowding,rank,pcross,pmutation,cviolation,handles,settings);
    
    %bookkeeping stuff:
    lasttime=elapsed;
    elapsed=toc;
    
    timethisgen=elapsed-lasttime;
    timetogo=(timethisgen*(T-t))/(24*3600); %in fractional days.
    timestr=datestr(timetogo,13); %13 is format for hhmmss.
    
    if max(rank)==1 && mean(crowding(~isinf(crowding)))>minever
        
        %minever=mean(crowding(~isinf(crowding)));
        genssincebetter=genssincebetter+1;
    else
        genssincebetter=0;
        
    end
    
    if minever>mean(crowding(~isinf(crowding)));
        minever=mean(crowding(~isinf(crowding)));
    end
    
    if mod(t+startgen,5)<1
        set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t+startgen),'\fontname{',handles.fonts.tab,'} Generations Completed.    ETA: ' timestr '   Gens since improvment: '  num2str(genssincebetter)])
        drawnow update
    end

    if settings.progresswindow
        rankcounts=histc(rank,[0.5 1.5 2.5 3.5 4.5 5.5 inf]);
            colurz=[1 0 0;1 .5 0; 1 1 0; 0 1 0; 0 1 1 ; 0 0 1];
            if t==1
                for rc=1:6
                    LL(rc)=line(t,rankcounts(rc),'color',colurz(rc,:),'parent',aaa);
                    cline=line(t,mean(crowding(~isinf(crowding))),'color',[0 0 1],'parent',bbb);
                    drawnow;
                end
            else
                xcdata=get(cline,'xdata');
                ycdata=get(cline,'ydata');
                xcdata(end+1)=t;
                ycdata(end+1)=mean(crowding(~isinf(crowding)));

                set(cline,'xdata',xcdata,'ydata',ycdata);
                for rc=1:6
                    xdata=get(LL(rc),'xdata');
                    ydata=get(LL(rc),'ydata');
                    xdata(end+1)=t;
                    ydata(end+1)=rankcounts(rc);
                    set(LL(rc),'xdata',xdata,'ydata',ydata);
                    drawnow;
                end
                
                if t>20000
                    set(bbb,'xlim',[t-20000 t]);
                end
            end
    end
    
    %premature stopping:
    if ~isempty(settings.stopbuttonpos) && get(stopbutton,'value')
        break
    end
    if ~settings.plotprogress
        if rem(t,5)==0
            drawnow %pause button requires a drawnow to register, so check every 5 gens if pause has been pushed.
        end
        continue
    end
    
    %convert final population back to real numbers:
    %convert gray coding to decimal:
    P3d=reshape(Plast,[size(P,1),nbits,nvars]);
    Prowindex=ones(N,size(ranges,2));
    colnums=(size(P3d,2):-1:1)-1;
    
    if nbits<=16.1
        colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    elseif nbits<=32.1
        colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    else
        colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
    end
    
    prodz=P3d.*colz;
    expoz=2.^prodz.*P3d;
    sumz=sum(expoz,2);
    
    %convert back to double here:
    Prow2D=permute(sumz,[1 3 2]);
    
    Prowindex(:,indexx)=Prow2D;
    
    Preal=bsxfun(@times,Prowindex,rangestep);
    Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population
    Preal(:,dofloor)=floor(Preal(:,dofloor));
    
    %draw progress:
    zerodata.settings=settings;
    zerodata.Preal=Preal;
    pfun=str2func(settings.plotfunction);
    pfun(handles.fig,t,settings,Preal,cviolation,rank);
    
end

%% finish up:
%remove dominated points from final population:
%objective=objective(survivors,:);
[F,rank]=fast_non_dominated_sort(objective,cviolation);

%convert final population back to real numbers:
P3d=reshape(Plast,[size(P,1),nbits,nvars]);
Prowindex=ones(N,size(ranges,2));
colnums=(size(P3d,2):-1:1)-1;

if nbits<=16.1
    colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
elseif nbits<=32.1
    colz=uint32(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
else
    colz=uint64(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));
end

prodz=P3d.*colz;
expoz=2.^prodz.*P3d;
sumz=sum(expoz,2);

%convert back to double here:
Prow2D=permute(sumz,[1 3 2]);

Prowindex(:,indexx)=Prow2D;

Preal=bsxfun(@times,Prowindex,rangestep);
Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population
Preal(:,dofloor)=floor(Preal(:,dofloor));

%evaluate this for ALL variables:
%respvars=find(handles.isfunction{settings.d});
respvars=settings.y;
objective=raveevaluatefunction(handles.fig,settings.d,respvars,Preal,handles);

%make continue button:
settings.gens=T-t;
if settings.gens==0
    settings.gens=T;
end

guidata(handles.fig,handles);

elapsed=toc/(24*3600);
timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.
set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t+startgen),'\fontname{',handles.fonts.tab,'} Generations Completed.    Elapsed Time: ' timestr])

%remove pareto tag so these objects dont get deleted on future runs:
paretos=findobj(handles.activegraph,'tag','pareto');
handles.temp.lasthandles=paretos;
set(paretos,'tag','optplots');

%save animation if needed:
ravefinishoptimizeranimation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Preal=double(Preal);

%generate name:
objs=handles.allnames{settings.d}(settings.y);

for o=1:numel(objs)
    objs{o}=[objs{o},', '];
end

objs=cell2mat(objs);

results.name=[handles.datasetnames{settings.d},' NSGA-II ',objs,num2str(settings.pop),'p ',num2str(settings.gens),'g'];
results.lasttime=elapsed;
results.lastgen=t+startgen;

Preal(:,respvars)=objective;
[sortedpop,index]=sortrows(Preal,settings.y);
%results.finalbin=P(index,:);
results.finalinputs=sortedpop(:,settings.funvars);
results.finalobjective=objective(index,:);
results.settings=settings;

continuebutton=findobj(handles.tabobjects,'tag','continuebutton');
set(continuebutton,'enable','on');


