function stop = raveanimateotb(x,state,event)
%animation functions for the optimizers in the Optimization Toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
settings=get(0,'userdata');
src=settings.src;
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%delete pop up window that appears:

if settings.targets(1)==-Inf %obj fun multiplier for displaying results:
   mult=-1;
else
    mult=1;
end

handles.temp.crapwindow=gcf;
set(gcf,'position',[-10000 -10000 400 400])
guidata(src,handles);

isfeasible=state.constrviolation<=settings.TolCon;

plotvars=settings.plotvars;

N=size(x,1);

%denormalize x:
x=bsxfun(@times,x,settings.normalizationfactors(2,:));
x=bsxfun(@plus,x,settings.normalizationfactors(1,:));

%distribute variables to correct columns:
index=~isnan(settings.funvars);
indexx=settings.funvars(index);
%indata=zeros(N,max(indexx));

indata=repmat(handles.xdials{udata.view.dataset}{udata.view.analysis},N,1); %need this in case objective fun and plot fun's dont have same input variable set.
indata(:,indexx)=x(index);

plotvalues=raveevaluatefunction(src,settings.d,plotvars,indata,handles);

switch settings.method
    case 'fmincon' %MULTIOBJECTIVE METHODS, USE RANK
        if strcmp(event,'init')
            trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
            delete(trash);
            
            %initialize plot:
            L(1)=line(nan,nan,'color',handles.colors.opthistory(1,:),'tag','opthistory','parent',settings.plotaxes,'marker','none','linewidth',1,...
                'hittest','off','visible','off');
            L(2)=line(nan,nan,'tag','optinfeasible','parent',settings.plotaxes,'marker','o','markerfacecolor',handles.colors.optinfeasible(1,:),...
                'markersize',handles.prefs.optmarkersize,'markeredgecolor',handles.colors.optinfeasible(2,:),'linestyle','none','hittest','off');
            L(3)=line(nan,nan,'tag','optfeasible','parent',settings.plotaxes,'marker','o','markerfacecolor',handles.colors.optfeasible(1,:),...
                'markersize',handles.prefs.optmarkersize,'markeredgecolor',handles.colors.optfeasible(2,:),'linestyle','none','hittest','off');
            L(4)=line(nan,nan,'tag','optcurrent','parent',settings.plotaxes,'marker','o','markerfacecolor',handles.colors.optoptimal(1,:),...
                'markersize',handles.prefs.optmarkersize,'markeredgecolor',handles.colors.optoptimal(2,:),'linestyle','none','hittest','off');
            
            if strcmp(settings.plottype,'3D')
                set(L,'zdata',nan);
            end
        end
        
        optinfeasible=findobj(settings.plotaxes,'tag','optinfeasible');
        optfeasible=findobj(settings.plotaxes,'tag','optfeasible');
        optcurrent=findobj(settings.plotaxes,'tag','optcurrent');
        opthistory=findobj(settings.plotaxes,'tag','opthistory');
        
        if isfeasible
            set(optcurrent,'markerfacecolor',handles.colors.optoptimal(1,:),'markeredgecolor',handles.colors.optoptimal(2,:))
        else
            set(optcurrent,'markerfacecolor',handles.colors.optinfeasible(1,:),'markeredgecolor',handles.colors.optinfeasible(2,:))
        end
        
        set(handles.infobar,'string',['Iteration: \fontname{monospace} ',sprintf('%4.0f',state.iteration),'\fontname{',handles.fonts.tab,'}   Function Evaluations: \fontname{monospace} ',sprintf('%4.0f',state.funccount),'\fontname{',handles.fonts.tab,'}   Objective Value: \fontname{monospace} ',sprintf('%8.4f',state.fval)])
        
        if settings.showinfo
            txt=findobj(settings.plotaxes,'tag','optinfotext');
            set(txt,'string',{['Iteration: ',sprintf('%6.0f',state.iteration)];['Function Evals: ',sprintf('%6.0f',state.funccount)];['Objective Value: ',sprintf('%6.6f',mult*state.fval)]},...
                'backgroundcolor',get(handles.fakerax(handles.activeindex),'color'))
        end
        
        
        switch settings.plottype
            case 'single'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                
                if isfeasible
                    oldxdata=get(optfeasible,'xdata');
                    oldydata=get(optfeasible,'ydata');
                    set(optfeasible,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)]);
                else
                    oldxdata=get(optinfeasible,'xdata');
                    oldydata=get(optinfeasible,'ydata');
                    set(optinfeasible,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)]);
                end
                
                oldxdata=get(opthistory,'xdata');
                oldydata=get(opthistory,'ydata');
                set(opthistory,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)]);
                
                set(optcurrent,'xdata',plotvalues(1),'ydata',plotvalues(2),'zdata',100*ones(size(plotvalues(2))));
                
            case '3D'
                if isfeasible
                    oldxdata=get(optfeasible,'xdata');
                    oldydata=get(optfeasible,'ydata');
                    oldzdata=get(optfeasible,'zdata');
                    set(optfeasible,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)],'zdata',[oldzdata(:);plotvalues(3)]);
                else
                    oldxdata=get(optinfeasible,'xdata');
                    oldydata=get(optinfeasible,'ydata');
                    oldzdata=get(optinfeasible,'zdata');
                    set(optinfeasible,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)],'zdata',[oldzdata(:);plotvalues(3)]);
                end
                
                oldxdata=get(opthistory,'xdata');
                oldydata=get(opthistory,'ydata');
                oldzdata=get(opthistory,'zdata');
                set(opthistory,'xdata',[oldxdata(:);plotvalues(1)],'ydata',[oldydata(:);plotvalues(2)],'zdata',[oldzdata(:);plotvalues(3)]);
                
                set(optcurrent,'xdata',plotvalues(1),'ydata',plotvalues(2),'zdata',plotvalues(3));
                
        end
        
        %upate graph to show variations in ALL dimensions:
        if settings.updateanalysis && ismember(settings.plotaxes,handles.allgraphs)
            if ~isempty(which(['raveupdate',get(settings.plotaxes,'userdata'),'xdials']))
                handles.xdials{settings.d}{settings.a}(indexx)=x(index);
                guidata(handles.fig,handles);
                fhandle=str2func(['raveupdate',get(settings.plotaxes,'userdata'),'xdials']);
                handles=fhandle(handles.fig,[],handles);
                guidata(handles.fig,handles);
            end
        end
        
end

%fix axes limits:
set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
    'xlim',get(settings.plotaxes,'xlim'),'ylim',get(settings.plotaxes,'ylim'),'zlim',get(settings.plotaxes,'zlim'))
drawnow;

if settings.animation
    Framez=get(handles.fig,'userdata');
    Framez(end+1) = getscreen(handles.fig,settings.framebounds);
    set(handles.fig,'userdata',Framez);
end

if ismember(settings.method,{'fmincon'})
    stop=false; %set this to true to stop at the current iteration.
    if get(settings.stopbutton,'value');stop=true;end
end

if settings.pause
    waitforbuttonpress;
    if uint8(get(gcf,'currentcharacter'))== 27
        settings.pause=0;
        set(0,'userdata',settings)
    end
    figure(handles.temp.crapwindow);
end



