function [functionname,index,mins,maxs]=ravemakeminimizefunction(src,ev,settings)
%makes a temporary function that can be used in a MATLAB optimizer.
%distributes inputs and makes all functions "min the best". This function
%takes all inputs normalized for the range 0-1, then restores them to their
%full ranges (handles.minlimit/handles.maxlimit values when the function
%was created) before evaluating regressions. This avoids scaling issues
%that some optimizers may have. The min/max of the inputs are ouputs to
%this function, and can be used to make the min/range inputs to ravenonlcon
%for correctly denormalize variables when calculating constraints. 
%
%outputs: functionname= name of the function created.
%         index = indices of the independent variables that are inputs to
%         this function. 
%         mins = vector of min bounds for each input
%         maxs = vector of max bounds for each input
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);

d=settings.d;
a=settings.a;
y=settings.y;
activecons=settings.activeconstraints(settings.activeconstraints>0);

%get variables for these functions:
index=ravegetindependentvars(handles,d,a,y,activecons);

%nvars=numel(index);%(number of input vars) not the same as size(d,2)
mins=handles.minlimit{d}{a}(index);
maxs=handles.maxlimit{d}{a}(index);

%make fitness functions:
PathName=handles.prefs.defaultdir;
FileName=['RAVEtempobjective','.m'];
clear(FileName); %ensure that this function gets freshly read (IMPORTANT)
fid=fopen(fullfile(PathName,FileName),'w+'); %note, this line gives an error on new matlab installs, maybe because old defaultdir isnt on new path.

functionname=FileName(1:end-2);

%print a help file for this function:
fprintf(fid,'function response=%s(inputs)\n',functionname); %error here on new install, see fid=fopen... line above. 
fprintf(fid,'%%This is a temporary objective function for using an optimizer in RAVE.\n');
fprintf(fid,'%%This file was auto-generated by running an optimizer in RAVE.\n');
fprintf(fid,'%%This function is not meant to be called from the command line.\n');
fprintf(fid,'%%This function will be deleted when the optimizer finishes.\n');
fprintf(fid,'%%If no optimizers are currently running, you can safely delete this file.\n');
fprintf(fid,'%%This function was created with RAVEMAKEMINIMIZEFUNCTION.m\n');
fprintf(fid,'%%Copyright 2010, Matt Daskilewicz\n');
fprintf(fid,'\n\n');
fprintf(fid,'settings=get(0,''userdata'');\n');
fprintf(fid,'src=settings.src;\n');
fprintf(fid,'d=%s;\n',num2str(d));
fprintf(fid,'narg=%s;\n',mat2str(handles.nargout{d}(y)));
fprintf(fid,'prefs=%s;\n',mat2str(handles.preferences{d}));
fprintf(fid,'targets=%s;\n',mat2str(handles.targetvalues{d}));
fprintf(fid,'mins=%s;\n',mat2str(mins));
fprintf(fid,'ranges=%s;\n',mat2str(maxs-mins));
fprintf(fid,'inputs=bsxfun(@times,inputs,ranges);\n');
fprintf(fid,'inputs=bsxfun(@plus,inputs,mins);\n');
fprintf(fid,'data=zeros(size(inputs,1),%s);\n',num2str(max(index)));
fprintf(fid,'data(:,%s)=inputs;\n',mat2str(index));
fprintf(fid,'num=%s;\n',mat2str(y));
fprintf(fid,'response=raveevaluatefunction(src,d,num,data);\n');

%account for different targets:
for g=1:numel(settings.targets)
   if settings.targets(g)==-inf
       %do nothing
   elseif settings.targets(g)==inf
       %multiply by -1
       fprintf(fid,'response(:,%1.0f)=-1*response(:,%1.0f);\n',g,g);
   else
       %nominal objective, minimize discrepency from target
       fprintf(fid,'response(:,%1.0f)=abs(response(:,%1.0f)-%s);\n',g,g,num2str(settings.targets(g)));
   end
end

fclose(fid);
rehash




