function results=raveoptfminsearch(src,ev,settings,overload)
%optimize in rave using the functions from the optimization toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

if nargin<4
    overload=struct();
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end

d=settings.d;
a=settings.a;

%initial point:
x0=settings.x0;


%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

if ~isfield(overload,'mins')
    mins=zeros(size(x0));
else
    mins=overload.mins;
end

if ~isfield(overload,'maxs')
    maxs=ones(size(x0));
else
    maxs=overload.maxs;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure
options=optimset(@fminsearch);

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

%plot results?
if settings.plotprogress
    options.PlotFcns=str2func(settings.PlotFcns);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.UseParallel='always';




%% make variables:
nVars=numel(x0); %#ok


%% convert numerical options to numbers:
numericaloptions={'MaxIter';'MaxFunEvals';'TolFun';'TolX'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end

%% do optimizer:
tic

%independent variables are normalized as per ravemakeminimizefunction:

handles.temp.crapwindow=nan;
guidata(handles.fig,handles);

%make settings retrievable for plotfcns:
set(0,'userdata',settings);

[X,FVAL,EXITFLAG,OUTPUT] = fminsearch(FITNESSFCN,x0,options); %#ok<ASGLU>

%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));

handles=guidata(src);
if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

% elapsed=toc/(24*3600);
% timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    ravefinishoptimizeranimation;
end

results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
results.name=['fminbnd, ',handles.allnames{d}{settings.y}];
