function results=raveoptpatternsearch(src,ev,settings,overload)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
%starttime=0;
if nargin<4
    overload=struct();
end

if isempty(which('patternsearch'));
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, which does not appear to be installed.')
    return
end

if ~license('test','gads_toolbox')
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, which appears to be installed but unlicensed.')
    return
end

if ~license('checkout','gads_toolbox')
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, but I could not check out a toolbox license.')
    return
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end
  
d=settings.d;
a=settings.a;

guidata(handles.fig,handles);

%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

x0=settings.x0;


if ~isfield(overload,'mins')
    mins=zeros(size(x0));
else
    mins=overload.mins;
end

if ~isfield(overload,'maxs')
    maxs=ones(size(x0));
else
    maxs=overload.maxs;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure

options=psoptimset;
options.Cache=settings.cache;
options.CacheSize=settings.cachesize;
options.CacheTol=settings.cachetol;
options.CompletePoll=settings.completepoll;
options.CompleteSearch=settings.completesearch;
options.Display='final';
options.InitialMeshSize=settings.initmeshsize;
options.InitialPenalty=10;
options.MaxFunEvals=settings.maxfun;
options.MaxIter=settings.maxiter;
options.MaxMeshSize=settings.maxmeshsize;
options.MeshAccelerator=settings.meshaccel;
options.MeshContraction=settings.meshcontract;
options.MeshExpansion=settings.meshexpand;
options.MeshRotate=settings.meshrotate;
options.OutputFcn=[];
options.PenaltyFactor=settings.penalty;
options.PollingOrder=settings.pollorder;
options.PollMethod=settings.pollmeth;
options.ScaleMesh=settings.scalemesh;
options.SearchMethod=settings.searchmeth;
options.TimeLimit=settings.timelimit;
options.TolBind=settings.bindtol;
options.TolCon=settings.consttol;
options.TolFun=settings.objtol;
options.TolMesh=settings.meshtol;
options.TolX=settings.inputvartol;

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
    trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
    delete(trash);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='always';

%% make variables:
nVars=numel(x0); %#ok
numberofinequalities=nnz(strcmp('inequality',handles.constrainttype{d}{a})); %#ok
numberofequalities=nnz(strcmp('equality',handles.constrainttype{d}{a})); %#ok
numberofbounds=2*numel(x0); %#ok


%% convert numerical options to numbers:
numericaloptions={'TolMesh','TolCon','TolX','TolFun','TolBind','MaxIter','MaxFunEvals','TimeLimit','MeshContraction','MeshExpansion',...
    'InitialMeshSize','MaxMeshSize','InitialPenalty','PenaltyFactor','CacheSize','CacheTol'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end


%% make nonlinear constraints:
if ~isfield(overload,'constraintfcn')
    %anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,mins,maxs-mins);
    anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,settings.normalizationfactors(1,:),settings.normalizationfactors(2,:));
else
    anonnonlcon=overload.constraintfcn;
end

%% do optimizer:
%tic;

% startpt=handles.xdials{d}{a}(indexx);
% startpt=bsxfun(@minus,startpt,mins);
% startpt=bsxfun(@rdivide,startpt,maxs-mins);
% startpt(isnan(startpt))=0;


if any(settings.activeconstraints) || isfield(overload,'constraintfcn');
%     [X,FVAL,EXITFLAG,OUTPUT] = fmincon(FITNESSFCN,x0,[],[],[],[],mins,maxs,anonnonlcon,options); %#ok<ASGLU>
    [X,FVAL,EXITFLAG,OUTPUT] = patternsearch(FITNESSFCN,x0,[],[],[],[],zeros(size(mins)),ones(size(mins)),anonnonlcon,options); %#ok<ASGLU>
else
%     [X,FVAL,EXITFLAG,OUTPUT] = fmincon(FITNESSFCN,x0,[],[],[],[],mins,maxs,[],options); %#ok<ASGLU>
    [X,FVAL,EXITFLAG,OUTPUT] = patternsearch(FITNESSFCN,x0,[],[],[],[],zeros(size(mins)),ones(size(mins)),[],options);  %#ok<ASGLU>
    
end


%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));


ph=findobj(settings.plotaxes,'tag','patternhist');
set(ph,'tag','optplots');

handles=guidata(src);

if settings.plotprogress && ishandle(handles.temp.crapwindow)
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
% indexx=settings.funvars;

% population=X;


% finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
% finalpop(:,indexx)=population;

%evaluate this for ALL variables:
% respvars=find(handles.isfunction{settings.d});
% objective=raveevaluatefunction(src,settings.d,respvars,finalpop);
% finalpop(:,respvars)=objective;

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    ravefinishoptimizeranimation;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
objs=handles.allnames{settings.d}{settings.y};

results.name=['Pattern Search ',objs,'   (',handles.datasetnames{settings.d},')'];

% [sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
