function results=raveoptsimulannealbnd(src,ev,settings,overload)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;
%starttime=0;
if nargin<4
    overload=struct();
end


if isfield(overload,'constraintfcn')
    error('Simulated Annealing does not support constraints.')
end

if isempty(which('patternsearch'));
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, which does not appear to be installed.')
    return
end

if ~license('test','gads_toolbox')
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, which appears to be installed but unlicensed.')
    return
end

if ~license('checkout','gads_toolbox')
    errordlg('The optimizer you selected requires the MATLAB GADS Toolbox, but I could not check out a toolbox license.')
    return
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end
  
d=settings.d;
a=settings.a;

%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

x0=settings.x0;


if ~isfield(overload,'mins')
    mins=zeros(size(x0));
else
    mins=overload.mins;
end

if ~isfield(overload,'maxs')
    maxs=ones(size(x0));
else
    maxs=overload.maxs;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure
options=saoptimset('simulannealbnd');
options.AnnealingFcn=str2func(settings.annealfun);
options.TemperatureFcn=str2func(settings.tempfun);
options.AcceptanceFcn=str2func(settings.acceptfun);
options.TolFun=settings.tolfun;
options.StallIterLimit=settings.stalliter;
options.MaxFunEvals=settings.maxfun;
options.TimeLimit=settings.timelimit;
options.MaxIter=settings.maxiter;
options.ObjectiveLimit=settings.objlimit;
options.InitialTemperature=settings.tinit;
options.ReannealInterval=settings.reannealinterval;

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';

%% make variables:
nVars=numel(x0); 
numberofvariables=nVars; %#ok
numberofinequalities=nnz(strcmp('inequality',handles.constrainttype{d}{a})); %#ok
numberofequalities=nnz(strcmp('equality',handles.constrainttype{d}{a})); %#ok
numberofbounds=2*numel(x0); %#ok


%% convert numerical options to numbers:
numericaloptions={'TolFun','MaxIter','MaxFunEvals','TimeLimit','StallIterLimit',...
    'DisplayInterval','PlotInterval','InitialTemperature','ReannealInterval'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end




%% do optimizer:
%tic
handles.temp.crapwindow=rand;
guidata(handles.fig,handles);
[X,FVAL,EXITFLAG,OUTPUT] = simulannealbnd(FITNESSFCN,handles.xdials{d}{a}(settings.funvars),zeros(size(mins)),ones(size(mins)),options);  %#ok<ASGLU>


%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));

sah=findobj(settings.plotaxes,'tag','annealhist','-or','tag','optbest');
set(sah,'tag','optplots');


handles=guidata(handles.fig);
if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
indexx=settings.funvars;

population=X;

finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
finalpop(:,indexx)=population;

%evaluate this for ALL variables:
respvars=find(handles.isfunction{settings.d});
objective=raveevaluatefunction(src,settings.d,respvars,finalpop,handles);
finalpop(:,respvars)=objective;

%guidata(handles.fig,handles);

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

ravefinishoptimizeranimation;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%generate name:
objs=handles.allnames{settings.d}{settings.y};

results.name=['Simulated Annealing ',objs,'   (',handles.datasetnames{settings.d},')'];
[sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
