%Fonts
function raveeditprefsfonts(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);




txt0=uicontrol('style','text','string','Loading font list... please wait...',...
    'position',[230 428 300 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
drawnow;

set(txt0,'visible','off')

txt1=uicontrol('style','text','string','Select a font to change:',...
    'position',[230 428 300 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');

m1=uicontrol('style','popupmenu','string',{'User Interface Primary';'User Interface Secondary';'Graph Titles';'Graph Axis Labels';'Graph Tick Labels';'Matrix Graph Tick Labels';'Mouseover Effects'},...
    'userdata',{'title';'tab';'graphtitle';'graph';'tick';'matrixtick';'mouseover'},...
    'parent',src,'position',[240 410 200 18],'backgroundcolor',[1 1 1],'callback',@menu_callback,'tag','m1');

txt2=uicontrol('style','text','string','Preview:',...
    'position',[530 428 200 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');

txt3=uicontrol('style','text','string','Font:',...
    'position',[230 333 100 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');


txt4=uicontrol('style','text','string','Size:',...
    'position',[230 280 100 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');

txt5=uicontrol('style','text','string','Adjust font size so that the preview texts fill their boxes. This will ensure that the text fits in the space Rave has available.',...
    'position',[530 120 270 60],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal','tag','txt5');

%make html font list (from Yair Altman's book, pg 109)
fonts=handles.fonts.listfonts;
fontz=handles.fonts.htmllistfonts;

%Get index of title font:
[junk,idx]=ismember(lower(handles.fonts.title),lower(fonts));

if idx==0
    [junk,idx]=ismember('arial',lower(fonts));
end

if idx==0
    idx=1;
end

color=handles.colors.frame;
R=color(1);
G=color(2);
B=color(3);

a1=axes('units','pixels','parent',src,'position',[540 180 200 300],'ylim',[0 300],'xlim',[0 200],'visible','off','color','none','tag','a1');
l1=uicontrol('style','popup','pos',[250 310 180 20],'string',fontz,'back',[1 1 1],'callback',@fontlist_Callback,'value',idx,'tag','l1');
[jslider,h1]=javacomponent('javax.swing.JSlider',[250 250 180 30]);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(0)
jslider.setMinimum(6)
jslider.setMaximum(40)
jslider.setMajorTickSpacing(2);
jslider.setValue(handles.fonts.titlesize);
set(jslider,'background',java.awt.Color(R,G,B))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@slider_Callback,src});
set(h1,'tag','h1');

rectangle('position',[0 18 117 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
rectangle('position',[0 68 160 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
rectangle('position',[0 118 81 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
rectangle('position',[0 168 178 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
rectangle('position',[0 218 128 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);

text(0,25,'Legends','parent',a1,'fontname',fonts{idx},'fontsize',handles.fonts.titlesize,'hor','left','tag','t1');
text(0,75,'Data Series','parent',a1,'fontname',fonts{idx},'fontsize',handles.fonts.titlesize,'hor','left','tag','t1');
text(0,125,'Graph','parent',a1,'fontname',fonts{idx},'fontsize',handles.fonts.titlesize,'hor','left','tag','t1');
text(0,175,'Control Panel','parent',a1,'fontname',fonts{idx},'fontsize',handles.fonts.titlesize,'hor','left','tag','t1');
text(0,225,'Navigator','parent',a1,'fontname',fonts{idx},'fontsize',handles.fonts.titlesize,'hor','left','tag','t1');

e1=uicontrol('style','edit','pos',[240+50+150 260 30 20],'string',num2str(handles.fonts.titlesize),'back',[1 1 1],...
    'callback',@sizeedit_Callback,'tag','e1');

handles.temp.tabobjects=[txt0 txt1 txt2 txt3 txt4 txt5 m1 a1 l1 h1 e1];
guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function menu_callback(src,ev)
handles=guidata(src);

val=get(src,'value');
[opt,name,size]=getparams(val);

txt5=findobj(gcbf,'tag','txt5');
a1=findobj(gcbf,'tag','a1');
delete(get(a1,'children'));
l1=findobj(gcbf,'tag','l1');
e1=findobj(gcbf,'tag','e1');
h1=findobj(gcbf,'tag','h1');

%make html font list (from Yair Altman's book, pg 109)
fonts=handles.fonts.listfonts;

%Get index of title font:
[junk,idx]=ismember(lower(handles.fonts.(name)),lower(fonts));

if idx==0
    [junk,idx]=ismember('arial',lower(fonts));
end

if idx==0
    idx=1;
end

set(e1,'string',num2str(handles.fonts.(size)))
jslider=get(h1,'javapeer');
jslider.setValue(handles.fonts.(size));

set(l1,'value',idx);


if opt==1 %User interface fonts
    set(txt5,'visible','on')
    
    rectangle('position',[0 18 117 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 68 160 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 118 81 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 168 178 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 218 128 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    
    text(0,25,'Legends','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t1');
    text(0,75,'Data Series','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t1');
    text(0,125,'Graph','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t1');
    text(0,175,'Control Panel','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t1');
    text(0,225,'Navigator','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t1');

elseif opt ==2 %Tab fonts
    set(txt5,'visible','on')
    
    rectangle('position',[0 18 85 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 68 50 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 118 110 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 168 80 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    rectangle('position',[0 218 98 18],'facecolor',[1 1 1],'edgecolor',[0 0 1],'parent',a1);
    
    text(0,27,'Manage','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t2');
    text(0,77,'View','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t2');
    text(0,127,'Constrain','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t2');
    text(0,177,'Explore','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t2');
    text(0,227,'Optimize','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t2');

else %Workspace fonts
    set(txt5,'visible','off')
    
    text(0,27,'NOPQRSTUVWXYZ','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t3');
    text(0,77,'ABCDEFGHIJKLM','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t3');
    text(0,127,'nopqrstuvwxyz','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t3');
    text(0,177,'abcdefghijklm','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t3');
    text(0,227,'0123456789','parent',a1,'fontname',handles.fonts.(name),'fontsize',handles.fonts.(size),'hor','left','tag','t3');

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fontlist_Callback(src,ev)
handles=guidata(src);

m1=findobj(gcbf,'tag','m1');
a1=findobj(gcbf,'tag','a1');
txts=findobj(a1,'type','text');
val=get(m1,'value');
[opt,name,size]=getparams(val);

fonts=handles.fonts.listfonts;
handles.fonts.(name)=fonts{get(src,'value')};
set(txts,'fontname',fonts{get(src,'value')});

guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeedit_Callback(src,ev)

handles=guidata(src);

m1=findobj(gcbf,'tag','m1');
a1=findobj(gcbf,'tag','a1');
h1=findobj(gcbf,'tag','h1');
txts=findobj(a1,'type','text');
val=get(m1,'value');
[opt,name,size]=getparams(val);

handles.fonts.(size)=str2double(get(src,'string'));
set(txts,'fontsize',str2double(get(src,'string')));

jslider=get(h1,'javapeer');

jslider.setValue(str2double(get(src,'string')));

guidata(src,handles);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function slider_Callback(jsrc,jev,src)
handles=guidata(src);

%Note: can't use gcbf in a java callback... trust that gcf is sufficent here:
m1=findobj(gcf,'tag','m1');
a1=findobj(gcf,'tag','a1');
e1=findobj(gcf,'tag','e1');
txts=findobj(a1,'type','text');
val=get(m1,'value');
[opt,name,size]=getparams(val);

handles.fonts.(size)=jsrc.getValue;
set(txts,'fontsize',jsrc.getValue);

set(e1,'string',num2str(jsrc.getValue));

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [opt,name,size]=getparams(val)
switch val
    case 1
        opt=1;
        name='title';
        size='titlesize';
    case 2
        opt=2;
        name='tab';
        size='tabsize';
    case 3
        opt=3;
        name='graphtitle';
        size='graphtitlesize';
    case 4
        opt=3;
        name='graph';
        size='graphsize';
    case 5
        opt=3;
        name='tick';
        size='ticksize';
    case 6
        opt=3;
        name='matrixtick';
        size='matrixsize';
    case 7
        opt=3;
        name='mouseover';
        size='mouseoversize';
end

