%Interface Colors
function raveeditprefsthemecolors(src,ev)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

txt1=uicontrol('style','text','string','Use the dropdown menu to select a component to change its color.',...
    'position',[230 428 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src);

a1=axes('units','pixels','parent',src,'position',[230 190 528 216],'ylim',[0 300],'xlim',[0 200],'visible','on','color','none');
idata=imread('setcolor.png');


colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];

%original image:
R=idata(:,:,1);
G=idata(:,:,2);
B=idata(:,:,3);

%updated image:
r=R;
g=G;
b=B;

for f=1:8
    thiscolor=handles.colors.(colornames{f});
    matchcolor=colorz(f,:);
    
    idx = R==matchcolor(1) & G==matchcolor(2) & B==matchcolor(3);
    r(idx)=round(thiscolor(1)*255);
    g(idx)=round(thiscolor(2)*255);
    b(idx)=round(thiscolor(3)*255);
    
    if f==4
        matchcolor=[255 0 255];
        idx = R==matchcolor(1) & G==matchcolor(2) & B==matchcolor(3);
        r(idx)=round((handles.colors.wsgrid(1)+handles.colors.wsbg(1))./2*255);
        g(idx)=round((handles.colors.wsgrid(2)+handles.colors.wsbg(2))./2*255);
        b(idx)=round((handles.colors.wsgrid(3)+handles.colors.wsbg(3))./2*255);
    end
    
end

%draw it:

idata(:,:,1)=r;
idata(:,:,2)=g;
idata(:,:,3)=b;

i=image(idata,'parent',a1);
set(i,'hittest','off');

set(a1,'xtick',[],'ytick',[],'hittest','off','buttondownfcn',@imageclick);

txt2=uicontrol('style','popup','string',{'Tab Background';'Tab Border';'Workspace Background';'Grid';'Frame';'Title';'Text';'Text2'},...
    'position',[230 150 200 18],'backgroundcolor',[1 1 1],'hor','left','callback',@colormenu_Callback);

handles.temp.tabobjects=[txt1 a1 txt2];

guidata(src,handles);


function imageclick(src,ev)

clicked=get(src,'currentpoint');
xclick=clicked(1,1,1);
yclick=clicked(1,2,1);

idata=imread('setcolor.bmp');

r=idata(xclick,yclick,1);
g=idata(xclick,yclick,2);
b=idata(xclick,yclick,3);

colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];


function colormenu_Callback(src,ev)
handles=guidata(src);

value=get(src,'value');

idata=imread('setcolor.png');
R=idata(:,:,1);
G=idata(:,:,2);
B=idata(:,:,3);

setcolorimage=findobj(gcf,'type','image');
newdata=get(setcolorimage,'cdata');

r=newdata(:,:,1);
g=newdata(:,:,2);
b=newdata(:,:,3);


displaynames={'Tab Background';'Tab Border';'Workspace Background';'Grid';'Frame';'Title';'Text';'Text2'};
colornames={'tabbg';'tabfg';'wsbg';'wsgrid';'frame';'title';'text';'texttint'};
colorz=[0 255 255;255 0 0;255 255 255;0 0 255;0 255 0;255 255 0;128 128 0;0 128 128];

idx= R==colorz(value,1) & G==colorz(value,2) & B==colorz(value,3);
newcolor=ravecolorchooser(gcf,handles.colors.(colornames{value}),['Choose ',displaynames{value},' Color']);

r(idx)=round(newcolor(1).*255);
g(idx)=round(newcolor(2).*255);
b(idx)=round(newcolor(3).*255);


newdata(:,:,1)=r;
newdata(:,:,2)=g;
newdata(:,:,3)=b;

set(setcolorimage,'cdata',newdata);

handles.colors.(colornames{value})=newcolor;
guidata(src,handles);

