classdef raveDataManager < handle

    properties
        datatype=cell(0);
        vectordata=cell(0);
        isvirtual=cell(0);
        virtuald=cell(0); 
        linkfrom=cell(0); 
        linkto=cell(0); 
        virtualc=cell(0); 
    end
    
    properties (Access = protected)
        alldiscretevalues=cell(0);
        alltextvalues=cell(0); 
        realdata=cell(0);
        realisvalid=cell(0);
        numrows=[]; %dx1 vector indicating number of rows in each data set
        numcols=[]; %dx1 vector indicating number of variables in each data set
    end
    
    methods
        %Constructor:
        function obj = raveDataManager()
            
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function obj = subsasgn(obj,S,B)
            if S(1).type(1)~='.';error('Invalid raveDataManager subsref error 1');end
            if ~ischar(S(1).subs);error('Invalid raveDataManager subsref error 2');end
            
            switch S(1).subs
                case 'alldata'
                    if ~isnumeric(B);error('raveDataManager.alldata{d}(x,y) = z requires z to be a numeric array');end
                    if numel(S)==1 %Not legal in Rave... you must always use {d} to tell what data set to replace
                        error('Subsasgn must be of the form raveDataManager.alldata{d} = ...');
                    elseif numel(S)==2 %Replacing an entire data set. This is the only form that allows numrows to be changed.
                        %Each data set must be initialized using this syntax.
                        d = S(2).subs{1};
                        
                        [numr,numc] = size(B);
                        obj.numrows(d) = numr;
                        if obj.numcols(d) ~= numc
                           error('Number of columns in new data does not match number of variables in dataset.') 
                        end
                        
                        obj.isvirtual{d} = false(1,numc);
                        obj.realdata{d} = cell(1,numc);
                        obj.alldiscretevalues{d}=cell(1,numc);
                        [obj.alldiscretevalues{d}{:}]=deal([]);
                        
                        for col = 1:numc
                            %                            obj.realdata{d}{col} = B(:,col);
                            obj.setData(d,1:numr,col,B(:,col));
                        end
                    elseif numel(S)==3 %replacing some rows/columns of the data set
                        d = S(2).subs{1};
                        
                        r = S(3).subs{1};
                        c = S(3).subs{2};
                        
                        if ischar(c) && c==':' %"All columns" could mean either every existing column, or that we are growing the dataset.
                            maxc = max(obj.numcols(d),size(B,2));
                            c = 1:maxc;
                        elseif islogical(c)
                            c = find(c);
                        end
                        
                        %update numcols if needed:
                        if max(c)>obj.numcols(d)
                            obj.isvirtual{d}(obj.numcols(d)+1:max(c)) = false;
                            obj.numcols(d) = max(c);
                        end
                        
                        %Check for colon: %"All rows" could mean either every existing row, or that we are growing the dataset. If we are growing the
                        %rows we must be replacing EVERY column
                        if ischar(r) && r==':'
                            maxr = max(obj.numrows(d),size(B,1));
                            r = 1:maxr;
                            
                            if maxr>obj.numrows(d)
                                obj.numrows(d) = maxr;
                                %make sure we're growing every column TO DO: this will need a workaround to handle deleted variables, so don't implement until deleted variables are implemented.
                                %                                 if ~isempty(setdiff(1:obj.numcols(d),c))
                                %                                     error('raveDataManager attempted to add rows to a subset of the columns of a dataset in a way that would result in different columns having different numbers of rows.')
                                %                                 end
                            end
                        end
                        
                        %Remove virtual columns from the list of columns we are trying to set:
                        c = c(~obj.isvirtual{d}(c));

                        for col = 1:numel(c)
                            obj=setData(obj,d,r,c(col),B(:,col));
                            %                             obj.realdata{d}{c(col)}(r(:)) = B(:,col);
                        end
                        
                    else %4 subs?
                        error('Illegal subsasgn subscript for raveDataManager object')
                    end
                case 'isvalid'
                    if ~(islogical(B) || isnumeric(B));error('raveDataManager.isvalid{d}(x,y) = z requires z to be a logical array');end
                    if isnumeric(B);B = logical(B);end
                    if numel(S)==1 %Not legal in Rave... you must always use {d} to tell what data set to replace
                        error('Subsasgn must be of the form raveDataManager.isvalid{d} = ...');
                    elseif numel(S)==2 %Replacing an entire data set. This is the only form that allows numrows to be changed.
                        %Each data set must be initialized using this syntax.
                        d = S(2).subs{1};
                        
                        [numr,numc] = size(B);

                        for col = 1:numc
                            obj.setIsValidData(d,1:numr,col,B(:,col));
                        end
                    elseif numel(S)==3 %replacing some rows/columns of the data set
                        d = S(2).subs{1};
                        
                        r = S(3).subs{1};
                        c = S(3).subs{2};
                        
                        if ischar(c) && c==':' %"All columns" could mean either every existing column, or that we are growing the dataset.
                            maxc = max(obj.numcols(d),size(B,2));
                            c = 1:maxc;
                        elseif islogical(c)
                            c = find(c); 
                        end
                       
                        %Check for colon: %"All rows" could mean either every existing row, or that we are growing the dataset. If we are growing the
                        %rows we must be replacing EVERY column
                        if ischar(r) && r==':'
                            maxr = max(obj.numrows(d),size(B,1));
                            r = 1:maxr;
                        end
                        
                        %Remove virtual columns... you can't set isvalid for them:
                        c = c(~obj.isvirtual{d}(c));
                        
                        if any(obj.isvirtual{d}(c))
                            error('Attempted to change value(s) for virtual column(s) in raveDataManager.')
                        end
                        
                        for col = 1:numel(c)
                            obj=setIsValidData(obj,d,r,c(col),B(:,col));
                        end
                        
                    else %4 subs?
                        error('Illegal subsasgn subscript for raveDataManager object')
                    end
                    
                case 'datatype'
                    if ischar(B);B={B};end
                    if ~iscellstr(B);error('raveDataManager.datatype must be set using a cell array of strings');end
                    if numel(S)==1 %Not legal in Rave... you must always use {d} to tell what data set to replace
                        error('Subsasgn must be of the form raveDataManager.alldata{d} = ...');
                    elseif numel(S)==2 %Replacing an entire data set.
                        d = S(2).subs{1};
                        obj.datatype{d}=B;
                        numc = numel(B);
                        
                        %Is this a new dataset?
                        if d>numel(obj.numcols)
                            obj.numcols(d) = numc;
                            obj.isvirtual{d}(1:numc) = false;
                        else
                            %Check if dataset has grown:
                            if numc>obj.numcols(d)
                                obj.numcols(d) = numc;
                                obj.isvirtual{d}(obj.numcols(d)+1:numc) = false;
                            end
                        end
                        
                    elseif numel(S)==3 %replacing some subset of the columns
                        d = S(2).subs{1};
                        c = S(3).subs{1};
                        obj.datatype{d}(c)=B;
                        
                        %Check if dataset has grown:
                        if max(c)>obj.numcols(d)
                            obj.isvirtual{d}(obj.numcols(d)+1:max(c)) = false;
                            obj.numcols(d) = max(c);
                        end
                        
                    else %4 subs?
                        error('Illegal subsasgn subscript for raveDataManager object')
                    end
                otherwise
                    obj = builtin('subsasgn',obj,S,B);
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function [varargout] = subsref(obj,S)
            %The raveDataManager object is just like a structure. So any
            %valid reference must be a "." reference. (Ok technically you
            %could say raveDataManager(1).blah, but we won't allow that in
            %the interest of keeping this code simple)
            
            if S(1).type(1)~='.';error('Invalid raveDataManager subsref error 1');end
            if ~ischar(S(1).subs);error('Invalid raveDataManager subsref error 2');end
            
            switch S(1).subs
                case 'alldata'
                    if numel(S)==1 %This is not really a valid thing to do in Rave, but it may
                        %happen if someone asks for alldata{end} and the "end" needs to evaluate
                        %numel(alldata). So just return a cell array of the
                        %appropriate size. TO DO: eventually i'll make this
                        %work for real, but it's low priority since it is
                        %never used in Rave.
                        varargout = {cell(1,numel(obj.realdata))};
                    elseif numel(S)==2 %Return entire dataset. This is used to make the maintable java model in Rave.
                        d = S(2).subs{1};
                        
                        r = 1:obj.numrows(d);
                        c = 1:obj.numcols(d);
                        
                        if numel(d)>1 %Only one data set at a time may be indexed.
                            error('Dataset index must be a scalar with subscripting raveDataManager.alldata.')
                        end
                        
                        %Get data for each column:
                        data = zeros(numel(r),numel(c));
                        
                        
                        for col = 1:numel(c)
                            if c(col)<=numel(obj.realdata{d}) || obj.isvirtual{d}(c(col)) %since obj.numcols is based on number of datatype cells, it may exceed the size of realdata
                                %This happens if the users subsrefs raveDataManage in between setting datatype and setting alldata. So if this happens,
                                %just return zeros for the empty columns.
                                data(:,col) = obj.getData(d,r,c(col));
                            end
                        end
                        varargout = {data};
                        
                    elseif numel(S)==3 %A standard subsref in Rave
                        d = S(2).subs{1};
                        
                        if numel(d)>1 %Only one data set at a time may be indexed.
                            error('Dataset index must be a scalar with subscripting raveDataManager.alldata.')
                        end
                        
                        r = S(3).subs{1};
                        c = S(3).subs{2};
                        
                        %Check for colon:
                        if ischar(r) && r==':'
                            r = 1:obj.numrows(d);
                        end
                        
                        if ischar(c) && c==':'
                            c = 1:obj.numcols(d);
                        elseif islogical(c)
                            c=find(c); %don't feel like dealing with this. 
                        end
                        
                        %Check for empty rows:
                        if isempty(r) || (islogical(r) && ~any(r))
                            varargout={zeros(0,numel(c))};
                            return
                        end
                        
                        
                        %Get data for each column:
                        data = zeros(nnz(r),nnz(c));
                        for col = 1:numel(c)
                            data(:,col) = obj.getData(d,r,c(col));
                        end
                        varargout = {data};
                        
                    else %4 subsrefs???
                        error('Illegal subscript reference on raveDataManager object.')
                    end
                case 'isvalid'
                    if numel(S)==1 %This is not really a valid thing to do in Rave, but it may
                        %happen if someone asks for alldata{end} and the "end" needs to evaluate
                        %numel(alldata). So just return a cell array of the
                        %appropriate size. TO DO: eventually i'll make this
                        %work for real, but it's low priority since it is
                        %never used in Rave.
                        varargout = {cell(1,numel(obj.realdata))};
                    elseif numel(S)==2 %Return entire dataset. This is used to make the maintable java model in Rave.
                        d = S(2).subs{1};
                        
                        r = 1:obj.numrows(d);
                        c = 1:obj.numcols(d);
                        
                        if numel(d)>1 %Only one data set at a time may be indexed.
                            error('Dataset index must be a scalar with subscripting raveDataManager.isValid.')
                        end
                        
                        %Get data for each column:
                        data = zeros(nnz(r),nnz(c));
                        
                        
                        for col = 1:numel(c)
                            if c(col)<=numel(obj.realdata{d}) || obj.isvirtual{d}(c(col)) %since obj.numcols is based on number of datatype cells, it may exceed the size of realdata
                                %This happens if the users subsrefs raveDataManage in between setting datatype and setting alldata. So if this happens,
                                %just return zeros for the empty columns.
                                data(:,col) = obj.getIsValidData(d,r,c(col));
                            end
                        end
                        varargout = {data};
                        
                    elseif numel(S)==3 %A standard subsref in Rave
                        d = S(2).subs{1};
                        
                        if numel(d)>1 %Only one data set at a time may be indexed.
                            error('Dataset index must be a scalar with subscripting raveDataManager.isValid.')
                        end
                        
                        r = S(3).subs{1};
                        c = S(3).subs{2};
                        
                        %Check for colon:
                        if ischar(r) && r==':'
                            r = 1:obj.numrows(d);
                        end
                        
                        if ischar(c) && c==':'
                            c = 1:obj.numcols(d);
                        elseif islogical(c)
                            c = find(c);
                        end
                        
                        %Check for empty rows:
                        if isempty(r) || (islogical(r) && ~any(r))
                            varargout={zeros(0,numel(c))};
                            return
                        end
                        
                        
                        %Get data for each column:
                        data = zeros(nnz(r),nnz(c));
                        for col = 1:numel(c)
                            data(:,col) = obj.getIsValidData(d,r,c(col));
                        end
                        varargout = {data};
                        
                    else %4 subsrefs???
                        error('Illegal subscript reference on raveDataManager object.')
                    end
                case 'debug'
                    keyboard
                otherwise
                    varargout = {builtin('subsref',obj,S)};
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function obj=setData(obj,d,r,c,data)
            %Set data in row r, column c, dataset d to equal data.
            
            %Check data type of this column:
            if numel(obj.datatype{d})<c
                error('You must specify the column datatype in raveDataManager before you can assign data to that column.')
            end
            
            switch obj.datatype{d}{c}
                case {'continuous','logical','integer'}
                    %nothing fancy needs to happen:
                    obj.realdata{d}{c}(r(:)) = data(:);
                case {'discrete','string'}
                    %Check if this data contains any values that do not
                    %appear in alldiscretevalues:
                    if numel(obj.alldiscretevalues{d})<c
                        obj.alldiscretevalues{d}{c}=[];
                    end
                    
                    %Do not use union here, because we need to maintain the
                    %order of existing discrete values:
                    newdiscretevalues = setdiff(unique(data),obj.alldiscretevalues{d}{c});
                    obj.alldiscretevalues{d}{c} = [obj.alldiscretevalues{d}{c} newdiscretevalues];
                    
                    %Now index the new data into obj.alldiscretevlues{d}{c}:
                    [junk,data] = ismember(data,obj.alldiscretevalues{d}{c});
                    
                    %Finally, store the data:
                    obj.realdata{d}{c}(r(:)) = uint32(data(:));
                case ''
                    error('Attempted to set data for deleted variable')
                otherwise
                    error('Cannot set data for unknown data type')
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function obj=setIsValidData(obj,d,r,c,data)
            %Set data in row r, column c, dataset d to equal data.
            
            %Check data type of this column:
            if numel(obj.datatype{d})<c
                error('You must specify the column datatype in raveDataManager before you can assign isValid data to that column.')
            end
            
            switch obj.datatype{d}{c}
                case {'continuous','logical','integer','discrete','string','list','vector','parameter'}
                    %nothing fancy needs to happen:
                    obj.realisvalid{d}{c}(r(:)) = data(:);
                case ''
                    error('Attempted to set data for deleted variable')
                otherwise
                    error('Cannot set isValid data for unknown data type')
            end
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function data = getData(obj,d,r,c)
            %Get data from data set d, column c, rows r. d and c must be
            %scalar, r may be a vector.
            
            if numel(obj.datatype{d})<c
                error('Column index out of bounds.')
            end
            
            %Is this a virtual or simple variable?
            if ~obj.isvirtual{d}(c) %simple variable
                data = obj.realdata{d}{c}(r);
            else %virtual variable
               %does not exist in this version of rave
            end
            
            %For discrete variables, we need to deindex the integer values:
            switch obj.datatype{d}{c}
                case 'discrete'
                    data = obj.alldiscretevalues{d}{c}(data);
                case 'string'
                    %strings don't get deindexed, just promote them to double:
                    data=double(data);
                otherwise
                    %don't need to do anything special
            end
        end
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function data = getIsValidData(obj,d,r,c)
            %Get data from data set d, column c, rows r. d and c must be
            %scalar, r may be a vector.
            
            if numel(obj.datatype{d})<c
                error('Column index out of bounds.')
            end
            
            %Is this a virtual or simple variable?
            if ~obj.isvirtual{d}(c) %simple variable
                data = obj.realisvalid{d}{c}(r);
            else %virtual variable
               %does not exist in this version of rave
            end

        end
        
       
    end
    
end
