function [C,x,y,z]=raveadaptivecontourc(src,xcol,ycol,zcol,vals,f,r,n)
%Adaptive contourc. Inputs are:
%
% src - used to get handles
% d - dataset index
% a - analysis index
% xcol - variable index for horizontal axis
% ycol - variable index for vertical axis
% zcol - variable index for function being contoured
% vals - vector of which contour levels to calculate
% r - scalar initial grid size (recommended to be less than 10)
% n - number of refinement levels ( recommended to be less than 10)
%


handles=guidata(src);








%preprocessing:
x=sort(x(:)); %ensure these are monotonically increasing... this is assumed in the use of "find" below.
y=sort(y(:));

[X,Y]=meshgrid(x,y);

if isempty(Z)
    Z=f(X,Y);
end

%Draw initial grid:
close all
figure
axes
line(X(:),Y(:),'marker','.','markeredgecolor',[.8 .8 .8],'linestyle','none','tag','grid','markersize',6);
h=[]; %contour line handles
   
for q=1:n+1

    %first calculate initial contour matrix:
    C=contourc(x,y,Z,vals);

    delete(h); %clear existing contours
    h=[];

    %Draw contours:
    I=size(C,2);
    i=1;
    while i<I
        ncols=C(2,i);
        thisx=C(1,i+1:i+ncols);
        thisy=C(2,i+1:i+ncols);
        thisz=10.*ones(size(thisy)); %use z value to avoid slow uistack
        h(end+1)=line('xdata',thisx,'ydata',thisy,'zdata',thisz,'parent',gca,'color',[0 0 1],'clipping','on','marker','.','markersize',4);
        i=i+ncols+1;
    end
    
    pause(2);
    set(findobj(gca,'tag','grid'),'marker','.','markeredgecolor',[.8 .8 .8],'markersize',6);
    if q==n+1;break;end %final iteration only draws contours, does not recalculate
    
    %Divide contour matrix into individual contours:
    conts=cell(0);
    I=size(C,2);
    i=1;
    while i<I
        level=C(1,i);
        ncols=C(2,i);
        thisx=C(1,i+1:i+ncols);
        thisy=C(2,i+1:i+ncols);
        conts{end+1}=[thisx;thisy]';
        i=i+ncols+1;
    end
    
    %% Determine which edges are crossed by contours:
    
    %make a grid to represent the edges:
    horizedges=false(numel(y),numel(x)-1); %note reversal of x/y to match plot axes
    vertedges=false(numel(y)-1,numel(x));
    
    % The points in conts{} are intersections where the contour crosses an
    % edge. Since each edge has a constant x or constant y value, we can check
    % which edge was crossed by looking for the x or y value of each vertex in
    % conts that is approximately == to one of the discrete levels of x/y:
    for c=1:numel(conts) %for each contour...
        for v=1:size(conts{c},1) %for each vertex...
            %which x/y value is most similar to this vertex?
            vtx=conts{c}(v,:);
            diffsx=x-vtx(1);
            diffsy=y-vtx(2);
            
            [minx,idxx]=min(abs(diffsx));
            [miny,idxy]=min(abs(diffsy));
            
            if minx<miny %x is constant
                %find largest y that is smaller than vtx(2)
                idxy=find(y<=vtx(2),1,'last');
                vertedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
            else %y is constant
                idxx=find(x<=vtx(1),1,'last');
                horizedges(idxy,idxx)=true; %note reversal of x/y to match plot axes.
            end
            
        end
    end
    
    %Note that horizedges and vertedges are flipud compared to the plotted
    %contours (i.e. y increases bottom to top)
    
    %Loop through all grid squares. If any edge of the grid square is
    %intersected by the contour, refine that square:
    
    dorefine=false(numel(y)-1,numel(x)-1);
    
    for r=1:numel(y)-1
        for c=1:numel(x)-1
            if vertedges(r,c) || vertedges(r,c+1) || horizedges(r,c) || horizedges(r+1,c)
                dorefine(r,c)=true;
            end
        end
    end
    
    %Squares to refine:
    [R,C]=find(dorefine);
    
    %Determine where to place new refinement points:
    R=2*R-1;
    C=2*C-1;
    
    newpts=[R+1 C;R C+1; R+1 C+2; R+2 C+1; R+1 C+1];
    idx=sub2ind([2*numel(y)-1,2*numel(x)-1],newpts(:,1),newpts(:,2));
    
    %Refine grid
    newx=[x x+[diff(x)/2;0]]';
    newy=[y y+[diff(y)/2;0]]';
    
    newx=newx(:);
    newy=newy(:);
    
    [newX,newY]=meshgrid(newx(1:end-1),newy(1:end-1));
    
    line(newX(idx),newY(idx),'marker','+','color',[1 0 1],'linestyle','none','tag','grid','markersize',4)

    %Use linear interpolation to expand original Z matrix (no new information added)
    Z=interp2(X,Y,Z,newX,newY);
    
    %Resample space at the new refinement points (new information added)
    Z(idx)=f(newX(idx),newY(idx));
    
    %Go to next iteration
    x=newx(1:end-1);
    y=newy(1:end-1);
    X=newX;
    Y=newY;
    
end









