function [lowfilterblob highfilterblob]=raveaddfiltertoaxis(src,d,a,x,axis,varargin)
%Adds a filter to the axis in src using variable var from d/a. Varargin is
%pv pairs for the line (marker only) objects that comprise the filter

handles=guidata(src);
facecolor=[255 204 51]./255;
edgecolor=facecolor/2;


%get current filter values:
minf=handles.minfilter{d}{a}(x);
maxf=handles.maxfilter{d}{a}(x);

xlimz=get(src,'xlim');
ylimz=get(src,'ylim');
zlimz=get(src,'zlim');

switch axis
    case 'x'
        lowfilterblob=line(minf,ylimz(1),zlimz(1)+1,...
            'linewidth',1,'marker','>','hittest','on',...
            'markerfacecolor',facecolor,'markersize',8,'markeredgecolor',edgecolor,...
            'buttondownfcn',{@xfilterclick,d,a},'userdata',x,'parent',src,'tag','loxfilter','clipping','off');
        
        highfilterblob=line(maxf,ylimz(1),zlimz(1)+1,...
            'linewidth',1,'marker','<','hittest','on',...
            'markerfacecolor',facecolor,'markersize',8,'markeredgecolor',edgecolor,...
            'buttondownfcn',{@xfilterclick,d,a},'userdata',x,'parent',src,'tag','hixfilter','clipping','off');
        
        %are filters visible?
        if single(minf)+eps(single(minf))<single(xlimz(1))
            set(lowfilterblob,'visible','off')
        end
        
        if single(maxf)>single(xlimz(2))+eps(single(xlimz(2)))
            set(highfilterblob,'visible','off')
        end
        
    case 'y'
        
    case 'z'
        
end


function xfilterclick(src,ev,d,a)
handles=guidata(src);

uistack(src,'top'); %important: this prevents the filter from getting stuck under the opposing filter.

set(handles.fig,'windowbuttonmotionfcn',{@xfilterslide,src,d,a})
set(handles.fig,'windowbuttonupfcn',{@filterdone,src})

function xfilterslide(fig,ev,src,d,a) %#ok<INUSL>
handles=guidata(fig);

x=get(src,'userdata');
thistype=handles.datatype{d}{x};
allowed=handles.discretevalues{d}{a}{x};

clicked=get(get(src,'parent'),'currentpoint');
val=clicked(1,1);

%move marker within bounds
axlims=get(ancestor(src,'axes'),'xlim');
lims=axlims;

tag=get(src,'tag');
tag=tag(1:2);

switch tag
    case 'lo'
        lims(2)=handles.maxfilter{d}{a}(x);
    case 'hi'
        lims(1)=handles.minfilter{d}{a}(x);
end

if val>lims(2)
    val=lims(2);
elseif val<lims(1)
    val=lims(1);
end

%round to nearest allowable point:
if ~any(strcmp(thistype,{'continuous','integer'}))
    if iscellstr(allowed);allowed=1:numel(allowed);end
    
    val=allowed(abs(allowed-val)==min(abs(allowed-val)));
    val=val(1); %in case above line has 2 mins.
    val=find(abs(allowed-val)==min(abs(allowed-val)));
else
    range=[handles.minlimit{d}{a}(x) handles.maxlimit{d}{a}(x)];
    if range(1)-range(2)==0;return;end %avoids errors with constant variables.
end

set(src,'xdata',val)

%update global filters:
switch tag
    case 'lo'
        handles.minfilter{d}{a}(x)=val;
    case 'hi'
        handles.maxfilter{d}{a}(x)=val;
end

%update discrete filters:
switch handles.datatype{d}{x}
    case 'string'
        newval=false(size(handles.discretefilter{d}{a}{x}));
        newval(handles.minfilter{d}{a}(x):handles.maxfilter{d}{a}(x))=true;
        handles.discretefilter{d}{a}{x}=newval(:);
    case {'discrete','integer','logical'};
        discretevalues=handles.discretevalues{d}{a}{x};
        newval=discretevalues>=handles.minfilter{d}{a}(x) & discretevalues<=handles.maxfilter{d}{a}(x);
        handles.discretefilter{d}{a}{x}=newval(:);
end

handles.isfiltered{d}{a}=ravecalculatefilteredstate(src,[],d,a);
raveupdateanalysis(handles.fig,handles,'isfiltered',[],d,a);

if ~handles.prefs.filteredselectable
    handles.isselected{d}{a}=handles.isselected{d}{a} & ~handles.isfiltered{d}{a};
    raveupdateanalysis(handles.fig,handles,'isselected',[],d,a);
end






function filterdone(fig,eventdata,src)  %#ok<INUSD>
handles=guidata(fig);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')





