function raveappendrows(src,d,data,newvectordata,isselected,isvisible,rowcolors,evaluatefunctions,expandranges)
%appends rows in "newdata" to dataset d
%
% newdata is an rxc matrix of data. c must equal the number of columns of
%  handles.alldata{d}. if evaluatefunctions==true, the columns of data
%  corresponding to functions will be overwritten, so they may be zeros.
%
% newvectordata is a cell array the same size as data. generally every cell
% will be empty.
%
% newtextdata is a cell array the same size as data. generally every cell
% will be empty.
%
% isselected is an rx1 to indicate whether to select the new data, or is
%     empty to unselect all.
% isvisible is an rx1 vector indicating that new data is visible or not, or
%     is empty to show all
% rowcolors is a rx1 vector of row color indices, or is empty to use the
%     default color for all rows.
% minfilter is a 1xc vector of minfilter values, or is empty to use the min
%     limits
% maxfilter is the same thing for max filter.
% discrete filter is a cell array of which discrete values should be
% filtered, or is empty to unfilter all discrete values
% keepifsamerownum is 0 or 1. If 1, and newdata has the same number of rows
%     as the data it is replacing, "isselected" "isvisible" and "rowcolors" will be ignored and none of
%     these settings will be changed from their previous values. The only
%     change that will be made is that newdata replaces the old data.
% evaluatefunctions is 0 or 1, indicating whether newdata already includes
%     the correct values of all function variables (=0) or whether they should
%     be evaluated by this function (=1);
% expandranges = 1 to set new minlimit/maxlimit at outer bounds of current
% minlimt/maxlimit and new minlimit/maxlimit
%     = 0 to replace with new minlimit/maxlimit
%     = -1 to keep old minlimit/maxlimit   


handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

oldnrows=size(datasets.alldata{d},1);
nrows=size(data,1);
if nrows==0;return;end

if nargin<10
    expandranges=1;
end

if size(data,2)~=size(datasets.alldata{d},2)
    error('Attempted to replace data set, but the number of columns in the new data does not match the number of columns in the old data')
end

ncols=size(datasets.alldata{d},2);

%Fill in empty inputs:
if isempty(newvectordata)
    newvectordata=cell(nrows,ncols);
end

if isempty(rowcolors)
    rowcolors=handles.prefs.defaultrowcolor*ones(nrows,1);
end

if isempty(isvisible)
    isvisible=true(nrows,1);
end

if isempty(isselected)
    isselected=false(nrows,1);
end


%check that this is a valid replacement:
if size(newvectordata,1)~=nrows
    error('Attempted to replace data set, but new data and vectordata did not have the same number of rows')
end

%evaluate functions?
if evaluatefunctions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,data,handles);
    data(:,index)=outs;
end

if isscalar(rowcolors);rowcolors=rowcolors*ones(nrows,1);end
if isscalar(isselected);isselected=logical(isselected*ones(nrows,1));end
if isscalar(isvisible);isvisible=logical(isvisible*ones(nrows,1));end

%append data:
isvalid = datasets.isvalid{d}; %need to store this before setting datasets.alldata to avoid referencing error.
datasets.alldata{d}=[datasets.alldata{d};data];
% textdata.alltext{d}=[textdata.alltext{d};newtextdata];
vectordata.vectordata{d}=[vectordata.vectordata{d};newvectordata];

datasets.isvalid{d}=[isvalid;true(size(data))];

handles.ispointofinterest{d}=[handles.ispointofinterest{d};false(nrows,1)];

for a=1:numel(handles.analysisnames)
    handles.rowcolors{d}{a}=[handles.rowcolors{d}{a};uint16(rowcolors)];
    handles.isvisible{d}{a}=[handles.isvisible{d}{a};isvisible];
    handles.isfiltered{d}{a}=[handles.isfiltered{d}{a};false(nrows,1)];
    handles.isselected{d}{a}=[handles.isselected{d}{a};isselected];
end

raveguidata(handles.fig,datasets);
% raveguidata(handles.fig,textdata);
raveguidata(handles.fig,vectordata);

raveupdatedataset(handles.fig,handles,'addrows',[],d,expandranges);

