function ravechangetab(src,eventdata) %#ok<INUSD>
%this function executes first whenever you click any tab, and mostly serves
%to undo things that should be undone when you click *away* from the
%previous tab.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

lasttab=handles.activetab;

%Check if in full-screen mode, if so, do nothing:
if strcmp(get(handles.tablemodebutton,'visible'),'off') %seems to be a reliable check of full-screen mode...
    return
end

%unhighlight currently selected subgraph for a scatterplotmatrix:
if ~isempty(handles.activesubgraph) && ishandle(handles.activesubgraph)
    try %in case udata.format.bg doesnt exist
    fgcolor=handles.colors.axes{udata.format.bg}(1,:);
    txt=handles.colors.axes{udata.format.bg}(5,:);
    set(handles.activesubgraph,'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor);
    set([get(handles.activesubgraph,'xlabel') get(handles.activesubgraph,'ylabel') get(handles.activesubgraph,'zlabel')],...
        'color',txt);    
    end
end

%clean up unused mouseoverhandles (has nothing to do with changing tabs,
%but this ensures it happens every so often...)

keep=ishandle(handles.mouseoverhandles);
handles.mouseoverhandles=handles.mouseoverhandles(keep);
handles.mouseoverfunctions=handles.mouseoverfunctions(keep);
handles.mouseoverdata=handles.mouseoverdata(keep);
clear ravedefaultwindowbuttonmotionfcn


%undo any graph placement callbacks:
set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

%turn off the print region of interest:
set(handles.roi,'visible','off');

%if this tab contains a flexgrid, delete it:
% if iscom(handles.tabflexgrid) && ~strcmp(lasttab,'optimize')
%     delete(handles.tabflexgrid)
%     handles.tabflexgrid=[];
% end

% %disable 3d rotation:
% if isa(handles.rotater,'graphics.rotate3d')
%     set(handles.rotater,'enable','off')
% end

%delete temporary handles:
handles.temp=[];

%delete old tabobjects:
if ~strcmp(lasttab,'optimize')
    delete(handles.tabobjects(ishandle(handles.tabobjects)));
    drawnow; %NOTE: this avoids java crashes.
    
    handles.tabobjects=[];
    guidata(handles.fig,handles);
    
    %find objects that got left behind:
    junks=findobj(handles.fig,'type','uicontrol','visible','on','-not','parent',handles.boneyard);
    posses=cell2mat(get(junks,'pos'));
    origin=get(handles.tabaxes,'position');
    trash=posses(:,1)>origin(1) & posses(:,1)<origin(1)+origin(3) & posses(:,2)>origin(2) & posses(:,2)<origin(2)+origin(4);
    delete(junks(trash));
    
    junktxt=findobj(handles.tabaxes,'type','text','visible','on');
    delete(junktxt);

else %never delete optimizetab stuff, just hide it.
    handles.optimizetabobjects=handles.tabobjects(ishandle(handles.tabobjects));
    set(handles.tabobjects(ishandle(handles.tabobjects)),'visible','off');
    drawnow; %NOTE: this avoids java crashes.
    handles.tabobjects=[];
    guidata(src,handles);
end