function hhh = ravecreateobjectivegui(src,ev)
% RAVECREATEOBJECTIVEGUI
%      This GUI is for use with RAVE only.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

%scan directory for objective templates:

raveroot=which('rave');
raveroot=raveroot(1:end-6);
filez=dir(fullfile(raveroot,'objectivetemplates\'));
h=1;

templatenames=cell(0);
templatefunctions=cell(0);

for g=3:numel(filez)
    thisfile=filez(g).name;
    if strcmp(thisfile(end-1:end),'.m')
        fid=fopen(fullfile(raveroot,'objectivetemplates\',thisfile),'r');
        firstline=fgetl(fid);
        %secondline=fgetl(fid);
        fclose(fid);
        if ~isempty(firstline) && strcmp(firstline(1),'%')
            %this is a template. Store it.
            templatenames{h}=firstline(2:end); 
            templatefunctions{h}=thisfile(1:end-2); 
            h=h+1;
        end
    end
end

handles.objectivetemplatenames=templatenames;
handles.objectivetemplatefunctions=templatefunctions;

guidata(handles.fig,handles);

%make the gui:
origin=get(handles.fig,'position');

hhh=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-355 origin(2)+origin(4)/2-350 710 700],'color',handles.colors.frame,...
    'name','Create New Model','numbertitle','off','resize','off','windowstyle','normal');

% titleaxes=axes('units','pixels','position',[11 720-50 217 38],'parent',hhh,'visible','off');
% text(0,0,'Create Objective Function','parent',titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
%     'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

u=uipanel('units','pixels','position',[50 40 626 650],'backgroundcolor',handles.colors.tabbg,...
    'bordertype','etchedin');

topframeaxes=axes('units','pixels','pos',[5 652-50 330 45],'parent',u,'visible','off');

midframeaxes=axes('units','pixels','pos',[5 522-50 330 45],'parent',u,'visible','off');

bottomframeaxes=axes('units','pixels','pos',[5 302-30-14 330 45],'parent',u,'visible','off');

text(0,.4,'Configure','parent',topframeaxes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,...
    'color',handles.colors.text,'horizontalalignment','left','verticalalignment','baseline');
text(0,.4,'Create from Template (Optional)','parent',midframeaxes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,...
    'color',handles.colors.text,'horizontalalignment','left','verticalalignment','baseline');
text(0,.4,'Edit Function Text','parent',bottomframeaxes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,...
    'color',handles.colors.text,'horizontalalignment','left','verticalalignment','baseline');



uicontrol('style','text','string','Choose Parent Analysis:','parent',u,...
    'position',[20 640-50 200 22],'backgroundcolor',handles.colors.tabbg,'hor','left');
uicontrol('style','text','string','Choose a Data Set:','parent',u,...
    'position',[320 640-50 200 22],'backgroundcolor',handles.colors.tabbg,'hor','left');
uicontrol('style','text','string','Name this function:','parent',u,...
    'position',[20 590-50 200 22],'backgroundcolor',handles.colors.tabbg,'hor','left');
uicontrol('style','text','string','Choose objective function template:','parent',u,...
    'position',[20 510-50 200 22],'backgroundcolor',handles.colors.tabbg,'hor','left');
uicontrol('style','text','string','Choose variables to include in objective function:','parent',u,...
    'position',[20 465-50 300 22],'backgroundcolor',handles.colors.tabbg,'hor','left');

analysismenu=uicontrol('style','popup','string',handles.analysisnames,'parent',u,...
    'position',[30 620-50 260 22],'backgroundcolor',handles.colors.tabbg,'hor','left','tag','analysismenu');

datasetmenu=uicontrol('style','popup','string',handles.datasetnames,'parent',u,...
    'position',[330 620-50 260 22],'backgroundcolor',handles.colors.tabbg,'hor','left','tag','datasetmenu');

varnamebox=uicontrol('style','edit','string','','parent',u,...
    'position',[30 570-50 360 22],'backgroundcolor',handles.colors.tabbg,'hor','left','tag','varnamebox');

macromenu=uicontrol('style','popup','string',handles.objectivetemplatenames,'parent',u,...
    'position',[30 490-50 360 22],'backgroundcolor',handles.colors.tabbg,'hor','left','tag','macromenu');

loadmacrobutton=uicontrol('style','pushbutton','string','Load','parent',u,...
    'position',[400 489-50 88 24],'hor','left','fontsize',12,'fontname',handles.fonts.tab,'callback',@loadmacro);

varlist=uicontrol('style','listbox','string',handles.basenames{1},'parent',u,...
    'position',[30 350-50 360 120],'backgroundcolor',handles.colors.tabbg,'hor',...
    'left','max',100,'value',1:numel(handles.basenames{1}),'listboxtop',1,'tag','varlist');

txtbox=uicontrol('style','edit','string','','parent',u,...
    'position',[30 40 574 230],'backgroundcolor',handles.colors.tabbg,...
    'hor','left','max',10000,'tag','txtbox');

importbutton=uicontrol('style','pushbutton','string','Create this Function','parent',u,...
    'position',[170 8 290 26],'hor','left','fontsize',16,'fontname',handles.fonts.tab,'callback',@importbutton_callback);

donebutton=uicontrol('style','pushbutton','string','Done','parent',hhh,...
    'position',[600 6 80 26],'hor','left','fontsize',16,'fontname',handles.fonts.tab,'callback',@donebutton_callback);

showhelp=uicontrol('style','pushbutton','string','Show Help','parent',u,...
    'position',[215 317-44 120 24],'hor','left','fontsize',12,'fontname',handles.fonts.tab,'callback',@showhelp_callback);

guidata(hhh,handles);



function loadmacro(src,ev)
handles=guidata(src);
macromenu=findobj(gcf,'tag','macromenu');
datasetmenu=findobj(gcf,'tag','datasetmenu');
varlist=findobj(gcf,'tag','varlist');
txtbox=findobj(gcf,'tag','txtbox');

%options=get(handles.popup.macromenu,'string');
thistemplatefun=handles.objectivetemplatefunctions{get(macromenu,'value')};

d=get(datasetmenu,'value'); %#ok<NASGU>
index=get(varlist,'value');
nvars=numel(index); %#ok<NASGU>

eval(thistemplatefun); %run script for thistemplatefun

set(txtbox,'string',newstring);




function importbutton_callback(src,ev)
handles=guidata(src);
% macromenu=findobj(gcf,'tag','macromenu');
datasetmenu=findobj(gcf,'tag','datasetmenu');
% varlist=findobj(gcf,'tag','varlist');
varnamebox=findobj(gcf,'tag','varnamebox');
txtbox=findobj(gcf,'tag','txtbox');


%check that all fields are filled:
if isempty(get(txtbox,'string'))
    errordlg('You need to paste or type a function in the box, or generate one using the template menu.')
    return
end

if isempty(get(varnamebox,'string'))
    errordlg('You need to enter a name for the variable you are creating.')
    return
end

ravetxt2m(handles.fig,[],get(varnamebox,'string'),get(txtbox,'string'),get(datasetmenu,'value'));

%clear input fieldS:
set(varnamebox,'string','');
set(txtbox,'string','');
ravefinish
set(handles.infobar,'string','Function created successfully.')

%do not save handles, or reload them after ravetxt2m

function donebutton_callback(src,ev)
close(gcf);

function showhelp_callback(src,ev)
handles=guidata(src);
origin=get(handles.fig,'position');
% load(handles.prefs)

infos={'You can use the following special variables in objective functions:';...
    '%VARIABLENAME - The current preference value for VARIABLENAME';
    '$+VARIABLENAME - The best value for VARIABLENAME among all data points being evaluated';
    '$-VARIABLENAME - The worst value for VARIABLENAME among all data points being evalued';
    '#VARIABLENAME - The value of VARIABLENAME after normalizing from 0-1';
    '   = (v - vmin)/(vmax - vmin)';
    'Note that each line in your function must end in a semicolon even if not normally required.'};

f=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-250 origin(2)+origin(4)/2-80 500 160],'color',handles.colors.frame,...
    'name','RAVE Text Function Help','numbertitle','off','resize','off','windowstyle','normal');

uicontrol('style','text','units','pixels','parent',f,'pos',[20 0 460 160],...
    'string',infos,'hor','left','backgroundcolor',get(f,'color'));










