function f=ravedefinetypesgui(src,d)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
handles=guidata(src); %gcbo = "define roles" button in managedatagui


%make figure:
f=figure('units','pixels','position',[0 0 651 534],'name',...
    'Define Data Types','numbertitle','off','menubar','none','resize','off','windowstyle','modal');
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

set(f,'color',handles.colors.frame)

%make initial uicontrols:
uipanel('units','pixels','position',[60 80 540 400],'parent',f,'backgroundcolor',[1 1 1])
axes('units','pixels','position',[88 55 506 427],'visible','off','xlimmode','manual','ylimmode','manual');
%handles.tabaxes=axes('units','pixels','position',[58 207 30 275],'parent',handles.gui,'visible','off','tag','tabaxes');
titleaxes=axes('units','pixels','position',[11 489 485 41],'parent',f,'visible','off','tag','titleaxes');

uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Ok','callback',@donebutton_Callback);

%make titles
set(titleaxes,'color','none','xcolor',handles.colors.frame,...
    'ycolor',handles.colors.frame,'zcolor',handles.colors.frame);

text(0,.2,'Define the role of each column','parent',titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

%get data from managedatagui

% Update handles structure
guidata(f, handles);
guidata(handles.fig,handles); %the button that calls this will look at handles.fig for handles, so make sure to update them 

data=handles.datatype{d};

%the data types are:
%convert data to indices:

varnamez=handles.allnames{d};

dat=javaArray('java.lang.String',numel(varnamez),2);
for g=1:numel(varnamez)
    dat(g,1)=java.lang.String(varnamez{g});
    dat(g,2)=java.lang.String(data{g});
end

jtable=javax.swing.JTable(dat,{'Variable Name','Data Type'}); %initialize a table
javaObjectEDT(jtable);
j=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
javaObjectEDT(j);
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
[jscrollpane mtable]=javacomponent(j,[150 100 300 300],f); %place scrollpane on gui

%% Setting column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[200 80];
for c=1:2 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

%jtable=table.getTable;
cb = javax.swing.JComboBox({'continuous','integer','discrete','string','logical'});
javaObjectEDT(cb);
cb.setEditable(false);  % prepare an editable drop-down CellEditor
editor = javax.swing.DefaultCellEditor(cb);
jtable.getColumnModel.getColumn(1).setCellEditor(editor);  % assign this editor to second column (see Note 2)

set(handle(jtable.getModel,'callbackproperties'),'TableChangedCallback',{@tablevaluechanged_callback,f,d})

strnng={'Use popup menus to select the data type for each variable.';...
    'Note: Allowable discrete values are defined by the Analysis, and must be set in the Manage Analyses window.'};
uicontrol('style','text','units','pix','parent',f,'position',[110 410 450 60],...
    'backgroundcolor',[1 1 1],'string',strnng,'hor','left');



function tablevaluechanged_callback(jsrc,jev,src,d)
handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

%determine which cell is selected:
eventDetails = jev;

c=eventDetails.getColumn+1;
r=eventDetails.getFirstRow+1;

newtype = jsrc.getValueAt(r-1,c-1);
oldtype=handles.datatype{d}{r};

handles.datatype{d}{r}=newtype;
datasets.datatype{d}{r}=newtype;

switch newtype
    case 'continuous'
        minval=min(datasets.alldata{d}(:,r));
        maxval=max(datasets.alldata{d}(:,r));
        if minval==maxval
            handles.isconstant{d}(r)=true; %otherwise, use previous value of isconstant;
        end
        [handles.minlimit{d}{:}(r)]=deal(minval);
        [handles.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.discretevalues{d}{:}(r)]=deal({[]});
        
    case 'integer'
        minval=floor(min(datasets.alldata{d}(:,r)));
        maxval=ceil(max(datasets.alldata{d}(:,r)));
        
        if minval==maxval
            handles.isconstant{d}(r)=true; %otherwise, use previous value of isconstant;
        end
        [handles.minlimit{d}{:}(r)]=deal(minval);
        [handles.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.discretevalues{d}{:}(r)]=deal({minval:maxval});
        [handles.xdials{d}{:}(r)]=deal(minval);
        [handles.discretefilter{d}{:}(r)]=deal({true(numel(minval:maxval),1)});
        
    case 'discrete'
        %minval=floor(min(handles.indata.indata.alldata{d}(:,r)));
        %maxval=ceil(max(handles.indata.indata.alldata{d}(:,r)));
        uniquez=unique(datasets.alldata{d}(:,r));
        minval=min(uniquez);
        maxval=max(uniquez);
        
        if minval==maxval
            handles.isconstant{d}(r)=true; %otherwise, use previous value of isconstant;
        end
        [handles.minlimit{d}{:}(r)]=deal(minval);
        [handles.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.discretevalues{d}{:}(r)]=deal({uniquez});
        [handles.xdials{d}{:}(r)]=deal(minval);
        [handles.discretefilter{d}{:}(r)]=deal({true(numel(uniquez),1)});
        
    case 'string'
        switch oldtype
            case 'string'
                
                %UPDATE TO WORK WITHOUT textdata.alltext WHICH HAS BEEN REMOVED
                
                [uniquez,ia,ic]=unique(textdata.alltext{d}(:,r));
                uniquez=strtrim(uniquez);

                %convert text values to numerical indexes:
                datasets.alldata{d}(:,r)=ic;
                
            otherwise %convert numbers to strings...
                
                
                
                
        end
        
        minval=floor(min(datasets.alldata{d}(:,r)));
        maxval=ceil(max(datasets.alldata{d}(:,r)));
        
        if minval==maxval
            handles.isconstant{d}(r)=true; %otherwise, use previous value of isconstant;
        end
        [handles.minlimit{d}{:}(r)]=deal(minval);
        [handles.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.discretevalues{d}{:}(r)]=deal({uniquez});
        [handles.xdials{d}{:}(r)]=deal(minval);
        [handles.discretefilter{d}{:}(r)]=deal({true(numel(uniquez),1)});
        
        
    case 'logical'
        minval=0;
        maxval=1;
        
        if minval==maxval
            handles.indata.indata.isconstant{d}(r)=true; %otherwise, use previous value of isconstant;
        end
        [handles.minlimit{d}{:}(r)]=deal(minval);
        [handles.maxlimit{d}{:}(r)]=deal(maxval);
        [handles.discretevalues{d}{:}(r)]=deal({[0 1]});
        [handles.xdials{d}{:}(r)]=deal(minval);
        [handles.discretefilter{d}{:}(r)]=deal({true(2,1)});

end
guidata(src,handles);
guidata(handles.fig,handles);
raveguidata(handles.fig,datasets);
raveguidata(handles.fig,vectordata);

function donebutton_Callback(src, eventdata)
close(ancestor(src,'figure'));



