function f=raveeditpreferencesgui(src,ev,startpage)
%main gui for editing everything in raveprefs.mat file.
%optional 3rd input is string keyword of the page to display initially. 

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
screensize=get(handles.fig,'position');


f=figure('units','pixels','position',[screensize(1)+screensize(3)/2-400 screensize(2)+screensize(4)/2-250 800 500],...
    'numbertitle','off','menubar','none','name','Change Preferences','color',handles.colors.frame,...
    'resize','off','windowstyle','normal');

uicontrol('style','text','string','Select a category:',...
    'position',[20 430 200 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

[categories,keywords]=getCategories(src,ev);

categorymenu=uicontrol('style','listbox','string',categories,...
    'position',[20 60 180 370],'backgroundcolor',handles.colors.frame,'fontsize',8,'hor','left',...
    'backgroundcolor',[1 1 1],'userdata',keywords,'callback',@categorymenu_Callback);

titleaxes=axes('units','pixels','position',[12 453 312 39],'parent',f,'visible','off');

text(0,.4,'Preferences','parent',titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

uicontrol('style','pushbutton','position',[520 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
uicontrol('style','pushbutton','position',[660 10 120 26],'string','Done','fontsize',12,'callback',@donebutton_Callback);

handles.temp.tabobjects=[];
guidata(f,handles);

%Load first preference page:
if nargin<3
    startpage=1;
else
    startpage=find(strcmp(startpage,keywords));
    if isempty(startpage);startpage=1;end
end
set(categorymenu,'value',startpage)
categorymenu_Callback(categorymenu,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function categorymenu_Callback(src,ev)
handles=guidata(src);
delete(handles.temp.tabobjects(ishandle(handles.temp.tabobjects)));
handles.temp.tabobjects=[];
guidata(src,handles);

%Form function:
keywords=get(src,'userdata');
thiskeyword=keywords{get(src,'value')};

%Display selected prefs:
fun=str2func(['raveeditprefs',thiskeyword]);
fun(ancestor(src,'figure'),[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [categories,keywords]=getCategories(src,ev)

categories={};
keywords={};
handles=guidata(src);

raveroot=which('rave');
raveroot=raveroot(1:end-6);
handles.exploreinfo=struct;

maindir=fullfile(raveroot,'preferences');
preffiles=dir(maindir);

for g=3:numel(preffiles)
    if ~preffiles(g).isdir
        thisname=preffiles(g).name;
        if ~strcmp(thisname(end-1:end),'.m') || ~isempty(strfind(thisname,'ravemakedefault'));continue;end
        keywords{end+1}=thisname(14:end-2); %#ok<AGROW>
        fid=fopen(thisname,'r');
        tline=strtrim(fgetl(fid));
        fclose all;
        categories{end+1}=strtrim(tline(2:end)); %#ok<AGROW>
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cancelbutton_Callback(src,ev)
answer=questdlg('Are you sure you want to cancel your changes? (Note, some changes such as colors/fonts that take effect immediately will not be cancelled','Cancel changes?');
if strcmp(answer,'Yes')
   close(gcf); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function donebutton_Callback(src,ev)
%Send all changes to the main rave window:
handles=guidata(src);
guidata(handles.fig,handles);
close(gcbf);

%update raveprefs.mat file:
saveraveprefs(handles.fig);




