function varargout=raveguidata(src,variable)
%Retrieve Rave app data of a specified type. The first input is always a
%handle to a Rave figure (or an object in a Rave figure). The second input
%is either a string from the list: 
% 'backupdata'
% 'datasets'
% 'datanotes'
% 'vectordata'
% 'discretevalues'
%
% Or it is a variable whose name is an item in the above list. (Must be one
% of these names, as Rave uses the variable's name to determine where to
% store it)
% When called with a string input, the output is the corresponding data
% structure, which you should give the same name as the input string, e.g.
% datasets=raveguidata(handles.fig,'datasets');
% When called with a variable input, there is no output. 


if nargin<1
    error('Too few input arguments')
end

src=ancestor(src,'figure');




if ischar(variable)
    if ~isappdata(src,variable)
       error('raveguidata was called with an input that is not a handle to an object in a Rave window, or with an invalid data string'); 
    end
    
    varargout{1}=getappdata(src,variable);
else
    varname=inputname(2);
    
    if ~ismember(varname,{'datasets','vectordata','discretevalues','backupdata','datanotes'})
       error('raveguidata was called with an invalid datatype string'); 
    end

    setappdata(src,varname,variable);
end
  