function [maincount,selectedcount,filtcount,nbars,edges,mainbins]=ravehistogramcount(handles,d,a,xcol,idx,mainrows,...
    selectedrows,filteredrows,nbars,discreteascategory,divideby)
%generates histogram data for column xcol of dataset d. Calculations are
%made so the resulting histogram has nbars bars. Only rows for which
%idx=true are considered. Discrete varaibles are treated as strings if
%discreteascategory=true. Output nbars may differ from input nbars if the
%variable is discrete/string. Output vectors have length =sum(idx)
%
% divideby=[] to not divide bars, or =0 to divide by row colors or = x to
% divide by discrete or string variable x. If ~isempty(divideby), first 3
% outputs have as many rows as x has discrete values. 

% handles=guidata(src);
% allnothidden=handles.isvisible{d}{a};
% allisselected=handles.isselected{d}{a};
% allisfiltered=handles.isfiltered{d}{a};

datasets=raveguidata(handles.fig,'datasets');

x=datasets.alldata{d}(:,xcol);

%determine bounds before applying idx:
minx=min(x);
maxx=max(x);

if ~isempty(idx)
    x=x(idx);
    mainrows=mainrows(idx);
    selectedrows=selectedrows(idx);
    filteredrows=filteredrows(idx);
end

switch handles.datatype{d}{xcol}
    case {'continuous','integer'}
        if minx==maxx %x is constant
            edges=[x(1)-1 x(1)+1];
        else
            edges=linspace(minx,maxx,nbars+1);
        end
        
    case 'discrete'
        if discreteascategory
            nbars=numel(handles.discretevalues{d}{a}{xcol});
            if minx==maxx %x is constant
                edges=[x(1)-1 x(1)+1];
            else
                edges=linspace(minx,maxx,nbars+1);
            end
        else
            nbars=numel(handles.discretevalues{d}{a}{xcol});
            if minx==maxx %x is constant
                edges=[x(1)-1 x(1)+1];
            else
                edges=linspace(minx,maxx,nbars+1);
            end
        end
        
    case {'string','bitstring'}
        nbars=numel(handles.discretevalues{d}{a}{xcol});
        if minx==maxx %x is constant
            edges=[x(1)-1 x(1)+1];
        else
            edges=(0:maxx)+0.5;
        end
end

%check if for all practical purposes, variable is a constant.
if numel(unique(edges))<numel(edges)
    edges=[x(1)-1 x(1)+1];
end

filtcount=histc(x(mainrows|filteredrows),edges); %this is all data, basically this forms the background. Only modify maincount when filtering.
[maincount,mainbins]=histc(x(mainrows),edges);

if ~isempty(x);
    if any(selectedrows)
        selectedcount=histc(x(selectedrows),edges); %stupidly, histc can't handle empty matrix inputs.
    else
        selectedcount=zeros(size(maincount));
    end
    
else
    selectedcount=[0 0];
end

%histc appends an extra bin at the end for items that match the upper limit
%edge. Move these into the next lower bin:
filtcount(end-1)=filtcount(end-1)+filtcount(end);
maincount(end-1)=maincount(end-1)+maincount(end);
selectedcount(end-1)=selectedcount(end-1)+selectedcount(end);

filtcount=filtcount(1:end-1);
maincount=maincount(1:end-1);
selectedcount=selectedcount(1:end-1);

