function raveinstall
%This function installs RAVE. It does not put the RAVE directory on your search path. You must do that yourself.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


prefspath=which('raveprefs.mat');
if ~isempty(prefspath)
    load raveprefs
else
    raveprefs=ravemakedefaultprefs;
end



%% save and move on...
save('raveprefstemp.mat','raveprefs')


%%
screen0;

function screen0

figg=findobj(0,'name','Rave setup');
if ~isempty(figg);return;end; %don't allow multiple instances to run

a=which('raveprefs.mat');
if ~isempty(a);msgbox('It looks like you''ve already set up Rave. If you complete this setup again your current settings will be overwritten.','Notice - raveprefs.mat already exists');end;

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);
screensize(3)=screensize(3)-screensize(1)+1;  

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','Rave setup','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal','color',[1 1 1]);

aaa=axes('units','pixels','position',[10 360 300 76],'visible','off');
image(imread('ravelogo_300.png','BackgroundColor',[1 1 1]),'parent',aaa);
set(aaa,'visible','off')
bbb=axes('units','pixels','position',[320 360 300 76],'visible','off');
text(0,0.5,['Version ',ravever,' Setup'],'fontname','arial','fontsize',24,'fontangle','italic','color',[.3 .3 .3])

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton0_Callback);

ccc=axes('units','pixels','position',[00 20 500 350],'xlim',[0 500],'ylim',[0 350],'visible','off');


text(30,300,{'Thank you for choosing Rave for your data visualization, exploration, optimization,';...
    'and modeling needs. Before you can begin using Rave, you need to specify a few';...
    'options using this installer.'},...
    'fontsize',12,'hor','left','backgroundcolor','none');

text(30,250,'Step 1: Choose Installation Type','fontsize',10,'hor','left','fontweight','bold','backgroundcolor',[1 1 1]);

text(30,235,'Select the desired installation option below.','fontsize',10,'hor','left','fontweight','normal','backgroundcolor',[1 1 1],'ver','top');

uicontrol('style','radiobutton','position',[30 210 600 24],'tag','localcheck',...
    'fontsize',10,'callback',@localcheck_callback,...
    'string','Local Installation: Select this to install Rave on this computer.','value',1,'backgroundcolor',[1 1 1])
uicontrol('style','radiobutton','position',[30 190 600 24],'callback',@portablecheck_callback,...
    'fontsize',10,'tag','portablecheck',...
    'string',{'Portable Installation: Select this to install Rave to a USB or other portable storage device.'},'backgroundcolor',[1 1 1])
text(48,163,{'Use this option to take Rave with you to use wherever you use MATLAB.'},'backgroundcolor',[1 1 1],'hor','left','parent',ccc)

text(30,100,{'Portable Installation Note: If you have not already done so, move the "rave" directory to the location';'(drive and path) where you want Rave installed. Then quit this installer, and re-run it from Rave''s new';'location. Alternatively, you can finish this installer and then move the rave directory to its final location.'},...
    'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal','visible','off','tag','portablewarning','backgroundcolor',[1 1 1]);

if ~isempty(a);uistack(askbox,'down');end;


function screen1
figg=findobj(0,'name','Rave setup');
if ~isempty(figg);return;end; %don't allow multiple instances to run

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);
screensize(3)=screensize(3)-screensize(1)+1;  

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','Rave setup','color',raveprefs.colors.frame,...
    'resize','off','windowstyle','normal','color',[1 1 1]);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton1_Callback);

aaa=axes('units','pixels','position',[10 360 300 76],'visible','off');
image(imread('ravelogo_300.png','BackgroundColor',[1 1 1]),'parent',aaa);
set(aaa,'visible','off')
bbb=axes('units','pixels','position',[320 360 300 76],'visible','off');
text(0,0.5,['Version ',ravever,' Setup'],'fontname','arial','fontsize',24,'fontangle','italic','color',[.3 .3 .3])

ccc=axes('units','pixels','position',[0 0 700 450],'visible','off','xlim',[0 700],'ylim',[0 450]);

text(30,340,'Step 2: Choose a default directory for Rave files.',...
    'fontsize',10,'hor','left','fontweight','bold');

text(30,325,{'When Rave needs to create a new file, it will put it in this folder. This most often happens when Rave needs';'to create a MATLAB function file to run. It is recommended that you make a new folder specifically for this';'purpose. Name it "ravefiles" or anything you like. This folder should be below your usual MATLAB working';'directory and not below the Rave installation directory.'},...
    'fontsize',10,'hor','left','fontweight','normal','ver','top');

bb=uicontrol('style','pushbutton','string','Browse...','position',[30 200 80 24],'callback',@browsebutton_Callback);
de=uicontrol('style','edit','string','','position',[120 200 540 24],'callback','','hor','left','back',[1 1 1],'tag','defaultdiredit');

if raveprefs.isportable
    text(30,170,{'Note: Since you selected Portable Installation, you cannot choose the location of this directory.';'It must be the "ravefiles" folder under your main "rave" directory. Click "Next" to continue.'},...
        'fontsize',10,'hor','left','fontweight','normal');
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    if ~isdir(fullfile(raveroot,'ravefiles'))
        mkdir(raveroot,'ravefiles');
    end
    set(de,'string',fullfile(raveroot,'ravefiles'),'enable','off');
    set(bb,'enable','off')
    
end

function screen2

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);
screensize(3)=screensize(3)-screensize(1)+1;  

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','Rave setup','resize','on','windowstyle','normal','resizefcn',@windowresize_Callback,'color',[1 1 1]);

aaa=axes('units','pixels','position',[10 360 300 76],'visible','off');
image(imread('ravelogo_300.png','BackgroundColor',[1 1 1]),'parent',aaa);
set(aaa,'visible','off')
bbb=axes('units','pixels','position',[320 360 300 76],'visible','off');
text(0,0.5,['Version ',ravever,' Setup'],'fontname','arial','fontsize',24,'fontangle','italic','color',[.3 .3 .3])

ccc=axes('units','pixels','position',[0 0 700 450],'visible','off','xlim',[0 700],'ylim',[0 450]);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Next','fontsize',12,'callback',@nextbutton2_Callback);

text(30,340,'Step 3: Specify size and position of RAVE window.',...
    'fontsize',10,'hor','left','fontweight','bold');

text(30,325,{'Resize and drag *this window* to the desired size and position that you wish the Rave window to appear';'on your screen. It is recommended that you simply maximize this window on the monitor on which you';'want Rave to appear. In order to function properly, the window must be at least 850 pixels high.'},...
    'fontsize',10,'hor','left','fontweight','normal','ver','top');

text(30,220,'Current size is too small. Make this window at least 400 pixels taller. Click "Next" when you are finished.',...
'fontsize',10,'hor','left','fontweight','bold','color',[1 0 0],'tag','windowsizetext');


function screen3
% hhh=raveeditthemefontsgui([],1);
% uiwait(hhh);
screen4

function screen4
% hhh=raveeditthemecolorsgui([],1);
% uiwait(hhh);
screen5

function screen5

load raveprefstemp

screensize=get(0,'monitorposition');
screensize=screensize(1,:);
screensize(3)=screensize(3)-screensize(1)+1;  

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
    'numbertitle','off','menubar','none','name','RAVE Setup','color',[1 1 1],...
    'resize','on','windowstyle','normal','resizefcn',@windowresize_Callback);

aaa=axes('units','pixels','position',[10 360 300 76],'visible','off');
image(imread('ravelogo_300.png','BackgroundColor',[1 1 1]),'parent',aaa);
set(aaa,'visible','off')
bbb=axes('units','pixels','position',[320 360 300 76],'visible','off');
text(0,0.5,['Version ',ravever,' Setup'],'fontname','arial','fontsize',24,'fontangle','italic','color',[.3 .3 .3])

ccc=axes('units','pixels','position',[0 0 700 450],'visible','off','xlim',[0 700],'ylim',[0 450]);

cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Finish','fontsize',12,'callback',@nextbutton5_Callback);

text(30,340,'Step 4: Set up file type associations',...
    'fontsize',10,'hor','left','fontweight','bold');

if ispc && ~raveprefs.isportable
    text(30,325,{'If you would like, the file extensions used by Rave (.rve and others) can be associated with MATLAB';'so that they have Rave icons and can be opened by double clicking. This requires adding';'some information to your registry. If you would like to do this, click the button below.';'(Note, you may need special user privileges to modify the registry.)';'When you are finished, click Next to continue.'},...
        'fontsize',10,'hor','left','fontweight','normal','ver','top');
    uicontrol('style','push','pos',[250 150 200 24],'string','Associate .rve file type with MATLAB','callback',@windowsfiletypes);
elseif raveprefs.isportable
    text(30,325,'File type associations cannot be set up for a Portable Installation. Click "Next" to continue.',...
        'fontsize',10,'hor','left','fontweight','normal','ver','top');
else
    text(30,325,{'Unfortunately I don''t know how to automatically set up file associations for your operating system.';'If you know how to do this from MATLAB, please contact mdaskilewicz@asdl.gatech.edu.';'Click Next to skip this step and continue.'},...
        'fontsize',10,'hor','left','fontweight','normal','ver','top');
end


% function screen6
% 
% load raveprefstemp
% 
% screensize=get(0,'monitorposition');
% screensize=screensize(1,:);
% screensize(3)=screensize(3)-screensize(1)+1;  
% 
% askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-350 screensize(2)+screensize(4)/2-225 700 450],...
%     'numbertitle','off','menubar','none','name','RAVE Setup','color',raveprefs.colors.frame,...
%     'resize','on','windowstyle','normal','resizefcn',@windowresize_Callback);
% 
% cancelbutton=uicontrol('style','pushbutton','position',[420 10 120 26],'string','Cancel','fontsize',12,'callback',@cancelbutton_Callback);
% donebutton=uicontrol('style','pushbutton','position',[560 10 120 26],'string','Finish','fontsize',12,'callback',@nextbutton6_Callback);
% 
% uicontrol('style','text','string','Step 6: Almost done...',...
%     'position',[30 380 640 40],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','bold');
% 
% uicontrol('style','text','string','There are a few more options that you must set up manually if you''d like to use them:',...
%     'position',[30 320 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
% 
% uicontrol('style','text','string','1. If you want optimizers to be able to send you text messages when they finish running, edit the function ravesetuptexting, update your information, and then run the function. (You may also need to edit ravesendtxtmessage.m if you don''t have a major US cell carrier)',...
%     'position',[30 280 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');
% 
% uicontrol('style','text','string','2. If you want to export graphs as pdfs, you must have ghostscript installed and on your operating system path. See the RAVE install manual for more info.',...
%     'position',[30 200 640 80],'backgroundcolor',raveprefs.colors.frame,'fontsize',10,'hor','left','fontweight','normal');


%enable texting?
%check that office web componenets is installed.
%set up PDF export

function browsebutton_Callback(src,ev)

mypath=uigetdir(pwd,'Choose a directory Rave can use to store any files it creates.');
editbox=findobj(gcf,'tag','defaultdiredit');
set(editbox,'string',mypath);



function nextbutton0_Callback(src,ev)

prefspath=which('raveprefstemp.mat');
load raveprefstemp

raveprefs.isportable=get(findobj(gcf,'tag','portablecheck'),'value');
save(prefspath,'raveprefs');


%% add rave directories to path:
if ~raveprefs.isportable
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    addpath(genpath(raveroot));
    writable=savepath;
    if ~writable
        savepath(which('pathdef.m'));        
    end
end

close(gcf);
screen1;



function nextbutton1_Callback(src,ev)
editbox=findobj(gcf,'tag','defaultdiredit');
mypath=get(editbox,'string');

if isdir(mypath)
    prefspath=which('raveprefstemp.mat');
    load raveprefstemp
    raveprefs.defaultdir=mypath;
    save(prefspath,'raveprefs');
    path(mypath,path); %add ravefiles to path
    savepath; %save path
    rehash;
    close(gcf);
    screen2;
else
    errordlg('You must enter a valid path.','Try again');
end

function nextbutton2_Callback(src,ev)
prefspath=which('raveprefstemp.mat');
load raveprefstemp
raveprefs.preferredposition=get(gcf,'position');

jf=findjobj(gcf,'class','FigureFrame');
if ~isempty(jf)
    raveprefs.ismaximized=jf.getExtendedState;
end

save(prefspath,'raveprefs');
close(gcf);
screen3;

function cancelbutton_Callback(src,ev)
close(gcf);

function windowresize_Callback(src,ev)
windowsizetext=findobj(src,'tag','windowsizetext');
if isempty(windowsizetext);return;end;
pos=get(src,'position');
height=pos(4);

if height>=850
    set(windowsizetext,'string','Window is large enough. Click "Next" when you are finished sizing.','color',[0 .8 .2]);
else
    set(windowsizetext,'string',['Current size is too small. Make this window at least ',num2str(850-height),' pixels taller. Click "Next" when you are finished.'],'color',[237 28 36]/255);
end
drawnow;



function windowsfiletypes(src,ev)

raveroot=which('rave');
raveroot=raveroot(1:end-7);
mlroot=matlabroot;

fid=fopen(fullfile(raveroot,'rve.reg.txt'));
raveroot=strrep(raveroot,'\','\\');
mlroot=strrep(mlroot,'\','\\');

fid2=fopen('rve.reg','wt');

tline = fgetl(fid); %skip 1st line
tline = fgetl(fid);
while ischar(tline)
    tline=strrep(tline,'XXXXXX',raveroot);
    tline=strrep(tline,'YYYYYY',mlroot);
    if strcmp(computer,'PCWIN64')
        tline=strrep(tline,'win32','win64');
    end
    fprintf(fid2,'%s\n',tline);
    tline = fgetl(fid);
end

fclose all

regpath=which('rve.reg');
system(['regedit "',regpath,'"']);
delete(regpath);

function nextbutton5_Callback(src,ev)
regpath=which('rve.reg');
if ~isempty(regpath);delete(regpath);end;
close(gcf);
%screen6;

%function nextbutton6_Callback(src,ev)
load raveprefstemp

if ~raveprefs.isportable
    %path=uigetdir(pwd,'Choose where to put the Rave preferences file that was just created. The best place is in your default MATLAB working directory.');
    path=raveprefs.defaultdir;
    if ~ischar(path);return;end;
    prefspath=fullfile(path,'raveprefs.mat');
else
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    prefspath=fullfile(raveroot,'raveprefs.mat');    
end

save(prefspath,'raveprefs');
delete('raveprefstemp.mat');
close(gcf);
rave;

function localcheck_callback(src,ev)
set(findobj(gcf,'tag','portablecheck'),'value',0);
set(src,'value',1);
set(findobj(gcf,'tag','portablewarning'),'visible','off')

function portablecheck_callback(src,ev)
set(findobj(gcf,'tag','localcheck'),'value',0);
set(src,'value',1);
set(findobj(gcf,'tag','portablewarning'),'visible','on')
