function th=ravemakecustomticks(src,axis,ticks,strings)
%src must be handle of axes for which to apply text labels.
%axis is 'x','y',or 'z' for which axis to make ticks along.
%vals is the numerical locations along axis where ticks should be placed
%strings is the cell array of strings to put at each location.
%orient is 'horizontal' or 'vertical' indicating whether to rotate text.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

th=[];

% fakeax=handles.fakeax(handles.activeindex);
% fakerax=handles.fakerax(handles.activeindex);

ticks=ticks(:)'; %ensure row vector

if isfield(udata.format,'bg')
    txtcolor=handles.colors.axes{udata.format.bg}(5,:);
else
    txtcolor=[0 0 0];
end


%For speed, this function only adds/deletes from existing tick text objects as needed, rather than recreating all ticks:

switch axis
    case 'x'
        xticks=findall(src,'tag','xticklabel');
        ndiff = numel(xticks)-numel(strings);
        if ndiff>0 %delete excess ticks
            delete(xticks(numel(strings)+1:end))
        elseif ndiff<0 %make more ticks
            for s=numel(strings):-1:numel(xticks)+1
                xticks(s)=text(nan,nan,nan,'',...
                    'HorizontalAlignment','center','rotation',0,'parent',src,'hittest','off',...
                    'verticalalignment','top','handlevisibility','off','tag','xticklabel',...
                    'fontname',handles.fonts.tick,'fontsize',handles.fonts.ticksize,...
                    'color',txtcolor,'interpreter',handles.prefs.interpreter);
            end
        end
        %Update strings on all objects:
        xticks=findall(src,'tag','xticklabel'); %need to re-get ticks to ensure they're in the same order that findobj will return when it is used in other functions. 
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'xtick',ticks,'xminortick','off','xticklabel','');
        set(xticks(:),{'string'},strings(:));
        
    case 'y'
        yticks=findall(src,'tag','yticklabel');
        ndiff = numel(yticks)-numel(strings);
        if ndiff>0 %delete excess ticks
            delete(yticks(numel(strings)+1:end))
        elseif ndiff<0 %make more ticks
            for s=numel(strings):-1:numel(yticks)+1
                yticks(s)=text(nan,nan,nan,'',...
                    'HorizontalAlignment','right','rotation',0,'parent',src,'hittest','off',...
                    'verticalalignment','middle','handlevisibility','off','tag','yticklabel',...
                    'fontname',handles.fonts.tick,'fontsize',handles.fonts.ticksize,...
                    'color',txtcolor,'interpreter',handles.prefs.interpreter);
            end
        end
        %Update strings on all objects:
        yticks=findall(src,'tag','yticklabel');
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ytick',ticks,'yticklabel','','yminortick','off');
        set(yticks(:),{'string'},strings(:));
        
    case 'z'
        zticks=findall(src,'tag','zticklabel');
        ndiff = numel(zticks)-numel(strings);
        if ndiff>0 %delete excess ticks
            delete(zticks(numel(strings)+1:end))
        elseif ndiff<0 %make more ticks
            for s=numel(strings):-1:numel(zticks)+1
                zticks(s)=text(nan,nan,nan,'',...
                    'HorizontalAlignment','right','rotation',0,'parent',src,'hittest','off',...
                    'verticalalignment','middle','handlevisibility','off','tag','zticklabel',...
                    'fontname',handles.fonts.tick,'fontsize',handles.fonts.ticksize,...
                    'color',txtcolor,'interpreter',handles.prefs.interpreter);
            end
        end
        %Update strings on all objects:
        zticks=findall(src,'tag','zticklabel');
        set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],'ztick',ticks,'zminortick','off');
        set(zticks(:),{'string'},strings(:));
end


