function ravemakedataset(src,D,infile,datasetname,datasettype,varnames,data,discretevalues,datatype)
%creates a basic data set with index D. If dataset D already exists, it
%will be overwritten.

handles=guidata(src);

numvars=size(data,2);
numrows=size(data,1);
numa=numel(handles.analysisnames);
mapname=handles.prefs.defaultvariablecolors;
newcolorz=getvariablecolors(numvars,mapname);


%get ranges:
if ~isempty(data)
    meandata=mean(data,1);
    mindata=min(data,[],1);
    maxdata=max(data,[],1);
else
    meandata=0.5*ones(1,numvars);
    mindata=zeros(1,numvars);
    maxdata=ones(1,numvars);
end

%store backupdata:
backupdata=getappdata(handles.fig,'backupdata');
backupdata{D}=data;
setappdata(handles.fig,'backupdata',backupdata);


%store datanotes:
datanotes=getappdata(handles.fig,'datanotes');
datanotes{D}=cell(numrows,1);
setappdata(handles.fig,'datanotes',datanotes);

%store data:
datasets=raveguidata(handles.fig,'datasets');
datasets.datatype{D} = datatype;
datasets.alldata{D} = data;
datasets.isvalid{D} = true(size(data));
datasets.isvirtual{D} = false(1,size(data,2));
datasets.virtuald{D} = zeros(1,size(data,2));
datasets.linkfrom{D} = zeros(1,size(data,2));
datasets.linkto{D} = zeros(1,size(data,2));
datasets.virtualc{D} = zeros(1,size(data,2));
datasets.alltextvalues{D} = discretevalues;

raveguidata(handles.fig,datasets);

% textdata=raveguidata(handles.fig,'textdata');
% textdata.alltext{D}=textdatas;
% raveguidata(handles.fig,textdata);

vectordata=raveguidata(handles.fig,'vectordata');
vectordata.vectordata{D}=cell(size(data));
raveguidata(handles.fig,vectordata);

%format/initialize data for output to main gui:
handles.datasettype{D}=datasettype;
handles.datasetfiles{D}=infile;

handles.allnames{D}=varnames;
handles.basenames{D}=varnames;
% handles.datasetlinks{D}=cell(1,numvars);
% handles.datasetlinkinfo{D}=cell(1,numvars);
handles.variablecolors{D}=newcolorz;
handles.allunits{D}=cell(1,numvars);

handles.isincluded{D}=true(1,numvars);
handles.isconstant{D}=mindata==maxdata;
handles.isnoise{D}=false(1,numvars);
handles.datasetnames{D}=datasetname;
handles.isfunction{D}=false(1,numvars);
handles.funclass{D}=cell(1,numvars);
[handles.funclass{D}{:}]=deal('independent');
handles.funattributes{D}=cell(1,numvars);
[handles.funattributes{D}{:}]=deal({''});
handles.funnames{D}=cell(1,numvars); %names of regressions for function columns.
handles.funvars{D}=num2cell(1:numvars); %variables in regressions for function columns.
handles.depvars{D}=cell(1,numvars);
handles.nargout{D}=zeros(1,numvars); %index of which output of its function this response is.
% handles.datatype{D}=cell(1,numvars);
handles.datatype{D}=datatype;
handles.targetvalues{D}=zeros(1,numvars);
handles.preferences{D}=zeros(1,numvars);
handles.normalizepreferences{D}=true;
handles.isobjective{D}=false(1,numvars);
handles.rankingcolumn{D}=0;
handles.rowlabelcolumn{D}=0;
handles.datasetupdatespending{D}=cell(0);
handles.analysisupdatespending{D}=cell(numa,1);
handles.tablemodelupdatespending{D}=cell(0);
handles.maintable.colwidths{D}=handles.prefs.colwidth*ones(1,numvars);
handles.ispointofinterest{D}=false(numrows,1);


%analysis variables:
handles.xdials{D}=cell(numa,1);
handles.minlimit{D}=cell(numa,1);
handles.maxlimit{D}=cell(numa,1);
handles.discretevalues{D}=cell(numa,1);
handles.rowcolors{D}=cell(numa,1);
handles.ineqconstraints{D}=cell(numa,1);
handles.isselected{D}=cell(numa,1);
handles.isvisible{D}=cell(numa,1);
handles.constrainthandles{D}=cell(numa,1);
handles.constrainttext{D}=cell(numa,1);
handles.constrainttype{D}=cell(numa,1);
handles.constraintvalues{D}=cell(numa,1);
handles.constraintadders{D}=cell(numa,1);
handles.constraintscalers{D}=cell(numa,1);
handles.constraintvars{D}=cell(numa,1);
handles.constraintcolors{D}=cell(numa,1);
handles.distributions{D}={};
handles.isfiltered{D}=cell(numa,1);
handles.minfilter{D}=cell(numa,1);
handles.maxfilter{D}=cell(numa,1);
handles.discretefilter{D}=cell(numa,1);
handles.rowcolorlabels{D}=cell(numa,1);

handles.aggregatingstatistic{D}=repmat({handles.prefs.defaultstatistic},1,numvars);

activedistributions=ones(1,numvars);
activedistributions(mindata==maxdata)=0;

handles.activedistribution{D}=activedistributions;

[handles.xdials{D}{:}]=deal(meandata);
[handles.minlimit{D}{:}]=deal(mindata);
[handles.maxlimit{D}{:}]=deal(maxdata);
[handles.discretevalues{D}{:}]=discretevalues; %this will contain all allowable values of non-continuous variables
[handles.rowcolors{D}{:}]=deal(ones(numrows,1,'uint16')*uint16(handles.prefs.defaultrowcolor));
[handles.isselected{D}{:}]=deal(false(numrows,1));
[handles.isvisible{D}{:}]=deal(true(numrows,1));
[handles.constrainthandles{D}{:}]=deal(cell(0)); %function handles must be stored in a cell
[handles.constrainttext{D}{:}]=deal(cell(0));
[handles.constrainttype{D}{:}]=deal(cell(0));
[handles.constraintvalues{D}{:}]=deal(cell(0));
[handles.constraintadders{D}{:}]=deal([]);
[handles.constraintscalers{D}{:}]=deal([]);
[handles.constraintvars{D}{:}]=deal(cell(0));
[handles.constraintcolors{D}{:}]=deal(cell(0));
[handles.isfiltered{D}{:}]=deal(false(numrows,1));
[handles.minfilter{D}{:}]=deal(mindata);
[handles.maxfilter{D}{:}]=deal(maxdata);
[handles.rowcolorlabels{D}{:}]=deal({'' '' '' '' '' '' '' '' '' ''});

discretefilter=discretevalues;
for q=1:numel(discretefilter)
    discretefilter{q}=true(numel(discretefilter{q}),1);
end
[handles.discretefilter{D}{:}]=discretefilter;

%make initial distributions:
for c=1:numvars
    dist=struct();
    dist.name='Default Distribution';
    switch datatype{c}
        case 'continuous'
            dist.type='uniform';
            dist.a=num2str(mindata(c),16);
            dist.b=num2str(maxdata(c),16);
            dist.serialnumber=rand*now;
        case 'integer'
            dist.type='discrete';
            dist.probs=mat2str(ones(1,(maxdata(c)-mindata(c)+1))/(maxdata(c)-mindata(c)+1));  
            dist.serialnumber=rand*now;
        case 'logical'
            dist.type='discrete';
            dist.probs='[.5 .5]';      
            dist.serialnumber=rand*now;
        otherwise
            dist.type='discrete';
            dist.probs=mat2str(ones(1,numel(discretevalues{c}))/numel(discretevalues{c}));            
            dist.serialnumber=rand*now;
    end
    
    for a=1:numa
        handles.distributions{D}{c}{1}=dist;
    end
end

%check for data types:
for c=1:numvars
    handles.funvars{D}{c}=c; %independent varibles are their own funvar.
    switch datatype{c}
        case {'string','bitstring','list'}
            [handles.xdials{D}{:}(c)]=deal(1);
        case 'logical'
            [handles.xdials{D}{:}(c)]=deal(0);
    end
end

guidata(src,handles);
