function f=ravemakenewfitcolumn(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

indata=guidata(src);

pf=get(src,'parent');
datasetmenu=findobj(pf,'tag','datasetmenu');
handles.d=get(datasetmenu,'value');

%make figure:
f=figure('units','pixels','position',[0 0 651 734],'name',...
    'Make Fit-Data Column','numbertitle','off','menubar','none','resize','off');
handles.gui=f;
origin=get(indata.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
load raveprefs
handles.prefs=raveprefs;
set(f,'color',raveprefs.colors.frame)

%make initial uicontrols:
handles.background=axes('units','pixels','position',[88 55 506 597],'visible','off','xlimmode','manual','ylimmode','manual');
   
handles.tabaxes=axes('units','pixels','position',[58 307 30 345],'parent',handles.gui,'visible','off','tag','tabaxes');  
handles.titleaxes=axes('units','pixels','position',[11 699 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

disclaimer=uicontrol('style','text','parent',f,'position',[26 689-36 560 46],'hor','left',...
    'string','Each tab below offers a different method to select a subset of rows in the data set. A new column will be added to the data set with the value "0" for fit rows, "1" for validation rows, and "2" for test rows. These values can then be used to indicate which rows to use as a basis for fitting a model.',...
    'backgroundcolor',get(f,'color'));



[x,y,v]=calculatetabprofile([1 1.2 .8],.1,2,1); 
handles.tabbg=patch(x,y,raveprefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],raveprefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',[.5 .5 .5]);

handles.random=text(.2,v(1),' Random','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',raveprefs.fonts.tabsize,...
    'color',raveprefs.colors.text,'vert','cap','buttondownfcn',@random_callback);
% handles.logic=text(.2,v(2),' Logical','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',raveprefs.fonts.tabsize,...
%     'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@logic_callback);
handles.selection=text(.2,v(2),' Row State','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',raveprefs.fonts.tabsize,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@selection_callback);
handles.range=text(.2,v(3),' Range','parent',handles.tabaxes,'rotation',90,'fontname',raveprefs.fonts.tab,'fontsize',raveprefs.fonts.tabsize,...
    'color',raveprefs.colors.tabfg,'vert','cap','buttondownfcn',@range_callback);


line([0 0 1 1 0],[.421 0 0 1 1],'color',raveprefs.colors.tabfg,'parent',handles.background);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Ok');
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
        'fontsize',14,'string','Cancel','callback',@cancelbutton_Callback);
    
%make titles
text(0,.2,'Make new fit-data column','parent',handles.titleaxes,'fontname',handles.prefs.fonts.title,'fontsize',handles.prefs.fonts.titlesize,...
   'color',raveprefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

%get data from managedatagui
handles.indata=indata; %gcbo = "define roles" button in managedatagui

handles.output = f;
handles.tabobjects=[];
% Update handles structure
guidata(f, handles);

%show data types tab:
random_callback(handles.random,[]);


function random_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1.2 .8],.1,1,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.selection handles.range],'color',handles.prefs.colors.tabfg);

%make uicontrols:
rulestext=uicontrol('position',[100 590 480 50],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string',{'Enter a number of rows in each box, or a decimal value to represent a fraction of the total number of rows. Any unassigned rows will be used for validation.'},'hor','left');

fittext=uicontrol('position',[120 560 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Fit Data:','fontsize',10,'fontweight','bold','hor','left');
valtext=uicontrol('position',[120 530 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Validation Data:','fontsize',10,'fontweight','bold','hor','left');
testtext=uicontrol('position',[120 500 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Test Data:','fontsize',10,'fontweight','bold','hor','left');

fitedit=uicontrol('position',[240 568 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.6','hor','left','tag','fitedit');
valedit=uicontrol('position',[240 538 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.2','hor','left','tag','valedit');
testedit=uicontrol('position',[240 508 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.2','hor','left','tag','testedit');


handles.tabobjects=[rulestext fittext valtext testtext fitedit valedit testedit];

guidata(src,handles);

set(handles.donebutton,'callback',@donebutton_random)



% function logic_callback(src,ev)
% %display uicontrols for defining column roles (constant, continuous, discrete)
% handles=guidata(src);
% delete(handles.tabobjects);
% 
% [x,y,v]=calculatetabprofile([1 1.2 .8],.1,2,1);
% set(handles.tablines,'xdata',x,'ydata',y);
% 
% set(src,'color',handles.prefs.colors.text);
% set([handles.random handles.selection handles.range],'color',handles.prefs.colors.tabfg);
% 
% %make uicontrols:
% handles.tabobjects=[];
% guidata(src,handles);
% 
% set(handles.donebutton,'callback',@donebutton_logical)

function selection_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1.2 .8],.1,2,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.random handles.range],'color',handles.prefs.colors.tabfg);


%make uicontrols:
rulestext=uicontrol('position',[100 590 480 50],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string',{'Select a unique value from each menu. Any unassigned rows will be used for validation. Any rows that belong to multiple categories that you select below will be assigned to Fit Data as the first priority and Validation Data as the second priority.'},'hor','left');

analysistext=uicontrol('position',[120 560 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Select an Analysis:','fontsize',10,'fontweight','bold','hor','left');
fittext=uicontrol('position',[120 560-30 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Fit Data:','fontsize',10,'fontweight','bold','hor','left');
valtext=uicontrol('position',[120 530-30 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Validation Data:','fontsize',10,'fontweight','bold','hor','left');
testtext=uicontrol('position',[120 500-30 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Test Data:','fontsize',10,'fontweight','bold','hor','left');

analysisedit=uicontrol('position',[260 568 250 22],'style','popupmenu','backgroundcolor',[1 1 1],...
    'string',handles.indata.analysisnames,...
    'hor','left','tag','analysismenu','value',1);
fitedit=uicontrol('position',[260 568-30 250 22],'style','popupmenu','backgroundcolor',[1 1 1],...
    'string',{'Plain Rows (Unselected, Visible, Unfiltered)','Selected Rows','Unselected Rows','Visible Rows','Hidden Rows','Filtered Rows','<None>'},...
    'hor','left','tag','fitedit','value',2);
valedit=uicontrol('position',[260 538-30 250 22],'style','popupmenu','backgroundcolor',[1 1 1],...
    'string',{'Plain Rows (Unselected, Visible, Unfiltered)','Selected Rows','Unselected Rows','Visible Rows','Hidden Rows','Filtered Rows','<None>'},...
    'hor','left','tag','valedit','value',7);
testedit=uicontrol('position',[260 508-30 250 22],'style','popupmenu','backgroundcolor',[1 1 1],...
    'string',{'Plain Rows (Unselected, Visible, Unfiltered)','Selected Rows','Unselected Rows','Visible Rows','Hidden Rows','Filtered Rows','<None>'},...
    'hor','left','tag','testedit','value',1);

handles.tabobjects=[rulestext fittext valtext testtext fitedit valedit testedit analysistext analysisedit];
guidata(src,handles);

set(handles.donebutton,'callback',@donebutton_selection)

function range_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
delete(handles.tabobjects);

[x,y,v]=calculatetabprofile([1 1.2 .8],.1,3,1);
set(handles.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.prefs.colors.text);
set([handles.selection handles.random],'color',handles.prefs.colors.tabfg);

%make uicontrols:
rulestext=uicontrol('position',[100 590 480 50],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string',{'Enter a number of rows in each box, or a decimal value to represent a fraction of the total number of rows. Left-over rows will be used for validation.'},'hor','left');

fittext=uicontrol('position',[120 560 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Fit Data:','fontsize',10,'fontweight','bold','hor','left');
valtext=uicontrol('position',[120 530 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Validation Data:','fontsize',10,'fontweight','bold','hor','left');
testtext=uicontrol('position',[120 500 120 28],'style','text','backgroundcolor',handles.prefs.colors.tabbg,...
    'string','Test Data:','fontsize',10,'fontweight','bold','hor','left');

fitedit=uicontrol('position',[240 568 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.6','hor','left','tag','fitedit');
valedit=uicontrol('position',[240 538 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.2','hor','left','tag','valedit');
testedit=uicontrol('position',[240 508 150 22],'style','edit','backgroundcolor',[1 1 1],...
    'string','0.2','hor','left','tag','testedit');


handles.tabobjects=[rulestext fittext valtext testtext fitedit valedit testedit];
guidata(src,handles);

set(handles.donebutton,'callback',@donebutton_range)

% 
% % --- Executes on button press in donebutton.
% function donebutton_Callback(src, eventdata)
% handles=guidata(src);
% 
% %isresponse=ismember(handles.names,get(handles.ybox,'string'));
% 
% %set(0,'userdata',handles.indata.indata);
% close(handles.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);




function donebutton_random(src,ev)
handles=guidata(src);

d=handles.d;

newname=nameprompt(src,[],d);
if isempty(newname);
    return
end
    
%distribute data:
trows=size(handles.indata.isselected{handles.d}{1},1);

fitedit=findobj(handles.tabobjects,'tag','fitedit');
valedit=findobj(handles.tabobjects,'tag','valedit');
testedit=findobj(handles.tabobjects,'tag','testedit');

nfitrows=str2double(get(fitedit,'string'));
ntestrows=str2double(get(testedit,'string'));

if nfitrows<=1
    nfitrows=round(nfitrows*trows);
end

if ntestrows<=1
    ntestrows=round(ntestrows*trows);
end

if nfitrows+ntestrows>trows %too many rows:
    nfitrows=round(trows*nfitrows./(nfitrows+ntestrows));
    ntestrows=trows-nfitrows;
end
    

rowdata=ones(trows,1);
rowdata(1:nfitrows)=0;
rowdata(nfitrows+1:nfitrows+ntestrows+1)=2;

rowdata=rowdata(randperm(trows));

appendcolumn(src,ev,rowdata,newname)

close(handles.gui);


% function donebutton_logical(src,ev)
% handles=guidata(src);
% close(handles.gui);


function donebutton_selection(src,ev)
handles=guidata(src);

d=handles.d;

newname=nameprompt(src,[],d);
if isempty(newname);
    return
end
    
%distribute data:
trows=size(handles.isselected{handles.d}{1},1);

analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
a=get(analysismenu,'value');

fitedit=findobj(handles.tabobjects,'tag','fitedit');
valedit=findobj(handles.tabobjects,'tag','valedit');
testedit=findobj(handles.tabobjects,'tag','testedit');

assigned=false(trows,1);
rowdata=ones(trows,1);

fitval=get(fitedit,'value');
valval=get(valedit,'value');
testval=get(testedit,'value');

if numel(unique([fitval valval testval]))~=3
    errordlg('Each menu entry you select must be unique.','Error')
    return
end
    
fig=handles.gui;
handles=handles.indata;

%Fit Data:
if fitval==1 %plain rows
    idx=~handles.isselected{d}{a} & handles.isvisible{d}{a} & ~handles.isfiltered{d}{a};
elseif fitval==2 %selected rows
    idx=handles.isselected{d}{a};
elseif fitval==3 %unselected
    idx=~handles.isselected{d}{a};
elseif fitval==4 %visible
    idx=handles.isvisible{d}{a};
elseif fitval==5 %hidden
    idx=~handles.isvisible{d}{a};
elseif fitval==6 %unfiltered
    idx=~handles.isfiltered{d}{a};
elseif fitval==7 %filtered
    idx=handles.isfiltered{d}{a};
elseif fitval==8 %none
    idx=false(trows,1);
end

rowdata(idx)=1;
assigned(idx)=true;

%Validation Data:
if valval==1 %plain rows
    idx=~handles.isselected{d}{a} & handles.isvisible{d}{a} & ~handles.isfiltered{d}{a};
elseif valval==2 %selected rows
    idx=handles.isselected{d}{a};
elseif valval==3 %unselected
    idx=~handles.isselected{d}{a};
elseif valval==4 %visible
    idx=handles.isvisible{d}{a};
elseif valval==5 %hidden
    idx=~handles.isvisible{d}{a};
elseif valval==6 %unfiltered
    idx=~handles.isfiltered{d}{a};
elseif valval==7 %filtered
    idx=handles.isfiltered{d}{a};
elseif valval==8 %none
    idx=false(trows,1);
end

idx=idx & ~assigned;
rowdata(idx)=0;
assigned(idx)=true;

%Test Data:
if testval==1 %plain rows
    idx=~handles.isselected{d}{a} & handles.isvisible{d}{a} & ~handles.isfiltered{d}{a};
elseif testval==2 %selected rows
    idx=handles.isselected{d}{a};
elseif testval==3 %unselected
    idx=~handles.isselected{d}{a};
elseif testval==4 %visible
    idx=handles.isvisible{d}{a};
elseif testval==5 %hidden
    idx=~handles.isvisible{d}{a};
elseif testval==6 %unfiltered
    idx=~handles.isfiltered{d}{a};
elseif testval==7 %filtered
    idx=handles.isfiltered{d}{a};
elseif testval==8 %none
    idx=false(trows,1);
end

idx=idx & ~assigned;
rowdata(idx)=2;

appendcolumn(src,ev,rowdata,newname)

close(fig);

function donebutton_range(src,ev)
handles=guidata(src);


close(handles.gui);



function newname=nameprompt(src,ev,d)
handles=guidata(src);

%make askbox:
screensize=get(handles.indata.fig,'position');

basename='Model Fit Flag';
name=basename;
q=2;
while ismember(name,handles.indata.basenames{d})
    name=[basename,' ',num2str(q)];  
    q=q+1;
end


askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-150 screensize(2)+screensize(4)/2-75 300 150],...
    'numbertitle','off','menubar','none','name','Name this variable','color',handles.prefs.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.prefs.colors.frame,'units','pixels',...
    'position',[30 100 250 40],...
    'string','Enter a name for this variable.',...
    'horizontalalignment','left')
u=uicontrol('style','edit','string',name,'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[60 70 180 22],'tag','variablename','hor','left');
uicontrol('style','pushbutton','string','Ok','parent',askbox,'units','pixels',...
    'position',[120 10 100 30],'callback',@makecolname_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[20 10 80 30],'callback',@makecolname_callback,'fontsize',10);

uicontrol(u);
uiwait(askbox)

newname=get(0,'userdata');


function makecolname_callback(src,ev)
varedit=findobj(get(src,'parent'),'tag','variablename');
set(0,'userdata',[]);
if strcmp(get(src,'string'),'Ok')
set(0,'userdata',get(varedit,'string'));
end
close(get(gcbo,'parent'))


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function appendcolumn(src,ev,newdata,newname)
handles=guidata(src);
d=handles.d;
handles=handles.indata;


datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

datasets.datatype{d}{end+1} = 'discrete';
idx=numel(datasets.datatype{d});
datasets.alldata{d}(:,idx)=newdata;
datasets.isvalid{d}(:,idx)=true;
% textdata.alltext{d}=[textdata.alltext{d},cell(length(newdata),1)];


% handles.datasetlinks{d}{end+1}=[];
% handles.datasetlinkinfo{d}{end+1}={};

vectordata.vectordata{d}=[vectordata.vectordata{d},cell(length(newdata),1)];
handles.allnames{d}{end+1}=newname;
handles.basenames{d}{end+1}=newname;
handles.variablecolors{d}(end+1,:)=rand(1,3);
handles.allunits{d}{end+1}=''; %havent implemented this yet
handles.isincluded{d}(end+1)=1;
handles.isconstant{d}(end+1)=min(newdata)==max(newdata);
handles.isfunction{d}(end+1)=0;
handles.funclass{d}{end+1}='independent';
handles.funattributes{d}{end+1}={''};
handles.funnames{d}{end+1}='';
handles.funvars{d}{end+1}=[];
handles.depvars{d}{end+1}=[];
handles.nargout{d}(end+1)=0;

handles.datatype{d}{end+1}='discrete';
handles.isnoise{d}(end+1)=false;

handles.preferences{d}(end+1)=0;
handles.targetvalues{d}(end+1)=-inf;
handles.isobjective{d}(end+1)=false;
handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;

handles.distributions{d}{end+1}{1}=struct();
handles.aggregatingstatistic{d}{end+1}=handles.prefs.defaultstatistic;
handles.activedistribution{d}(end+1)=0; 

for a=1:numel(handles.analysisnames)
    handles.xdials{d}{a}(end+1)=0;
    handles.minlimit{d}{a}(end+1)=0;
    handles.maxlimit{d}{a}(end+1)=2;
    handles.minfilter{d}{a}(end+1)=0;
    handles.maxfilter{d}{a}(end+1)=2;
    handles.discretevalues{d}{a}{end+1}=[0 1 2];
end

guidata(handles.fig,handles);
raveguidata(handles.fig,datasets);
% raveguidata(handles.fig,textdata);
raveguidata(handles.fig,vectordata);

%update maintable udata to display new columns:
%handles.maintable.x{dataset}=union(handles.maintable.x{dataset},(size(datasets.alldata{dataset},2)-numel(filenames)):size(datasets.alldata{dataset},2));
raveupdatedataset(handles.fig,handles,'addcolumns',[],d);
ravefinish; %make a done sound.





