function ravemovegraph(src,ev) %#ok<INUSD>
%RAVE: move the selected graph to a new location or resize graph by
%clicking movegrabbers.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);

%get starting point:
clicked=get(handles.mainaxes,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

set(handles.infobar,'string','To move without resizing, ctrl+click graph and drag to desired location.');
%NOTE: MOVING WITHOUT RESIZING IS DONE BY RAVESELECTGRAPH function.

set(handles.fig,'WindowButtonMotionFcn',@ravemovegrabber);
set(handles.fig,'WindowButtonUpFcn',@ravemovegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravemovegrabber(src,ev) %#ok<INUSD>

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

finish=get(handles.mainaxes,'currentpoint');

endx=finish(1,1,1);
endy=finish(1,2,1);

if handles.prefs.snaptogrid
    endx=handles.snapsize*round(endx/handles.snapsize);
    endy=handles.snapsize*round(endy/handles.snapsize);
end

if endx<1;endx=1;end
if endy<1;endy=1;end
if endx>handles.prefs.wswidth;endx=handles.prefs.wswidth;end
if endy>handles.prefs.wsheight;endy=handles.prefs.wsheight;end

minheight=handles.graphinfo.(handles.activetype).minheight;
minwidth=handles.graphinfo.(handles.activetype).minwidth;

%if this graph has legends, get their positions:
legdata = get(handles.fakerax(handles.activeindex),'userdata');
legdata = legdata(ishandle(legdata));
legdata = findobj(legdata,'flat','-not','tag','Colorbar');

if ~isempty(legdata)
    legposses = get(legdata,'position');
    if iscell(legposses);legposses=cell2mat(legposses);end
    legvertex=legposses(:,1:2);
    
    %determine which corner of the graph each legend is closest to (based on lower left corner of legend:
    
    %Get corner positions of the graph. Use handles.allgraphs since this is usually the one that determines the size:
    %corner order: [SW NW NE SE];
    
    axpos = get(handles.allgraphs(handles.activeindex),'position');
    %do math to get all 4 corners:
    axcorners = [axpos(1) axpos(2); axpos(1) axpos(2)+axpos(4);axpos(1)+axpos(3) axpos(2)+axpos(4);axpos(1)+axpos(3) axpos(2)];
    
    [junk,closest] = min(ipdm(legvertex,axcorners),[],2);
    
    %maintain the distance of each grabber to its closest corner:
    for i = size(legvertex,1):-1:1
        legdistance(i,:) = legvertex(i,:)-axcorners(closest(i),:);
    end
end

if ~strcmpi(handles.graphinfo.(handles.activetype).axesformat,'fixedsize')
    switch get(gco,'tag') %grabber order: [SW NW NE SE];
        case 'NE'
            %enforce minimum graph size:
            if endx<get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth
                endx=get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth;
            end
            
            if endy<get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1
                endy=get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1;
            end
            
            set(gco,'xdata',endx-3,'ydata',endy-2)
            set(handles.grabbers{handles.activeindex}(2),'ydata',endy-2)
            set(handles.grabbers{handles.activeindex}(4),'xdata',endx-3)
            
        case 'SE'
            %enforce minimum graph size:
            if endx<get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth
                endx=get(handles.grabbers{handles.activeindex}(1),'xdata')+minwidth;
            end
            
            if endy>get(handles.grabbers{handles.activeindex}(2),'ydata')-minheight
                endy=get(handles.grabbers{handles.activeindex}(2),'ydata')-minheight;
            end
            
            set(gco,'xdata',endx-3,'ydata',endy+3)
            set(handles.grabbers{handles.activeindex}(1),'ydata',endy+3)
            set(handles.grabbers{handles.activeindex}(3),'xdata',endx-3)
            
        case 'NW'
            %enforce minimum graph size:
            if endx>get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1
                endx=get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1;
            end
            
            if endy<get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1
                endy=get(handles.grabbers{handles.activeindex}(1),'ydata')+minheight-1;
            end
            
            set(gco,'xdata',endx+2,'ydata',endy-2)
            set(handles.grabbers{handles.activeindex}(3),'ydata',endy-2)
            set(handles.grabbers{handles.activeindex}(1),'xdata',endx+2)
        case 'SW'
            %enforce minimum graph size:
            if endx>get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1
                endx=get(handles.grabbers{handles.activeindex}(3),'xdata')-minwidth-1;
            end
            
            if endy>get(handles.grabbers{handles.activeindex}(3),'ydata')-minheight
                endy=get(handles.grabbers{handles.activeindex}(3),'ydata')-minheight;
            end
            set(gco,'xdata',endx+2,'ydata',endy+3)
            set(handles.grabbers{handles.activeindex}(4),'ydata',endy+3)
            set(handles.grabbers{handles.activeindex}(2),'xdata',endx+2)
    end
    
    
    pos=[get(handles.grabbers{handles.activeindex}(1),'xdata')-2 get(handles.grabbers{handles.activeindex}(1),'ydata')-3 ...
        get(handles.grabbers{handles.activeindex}(3),'xdata')-get(handles.grabbers{handles.activeindex}(1),'xdata')+5 ...
        get(handles.grabbers{handles.activeindex}(3),'ydata')-get(handles.grabbers{handles.activeindex}(1),'ydata')+5];
    
else
    set(gco,'xdata',endx+3,'ydata',endy+3)
    set(handles.grabbers{handles.activeindex}(4),'ydata',endy+3)
    set(handles.grabbers{handles.activeindex}(2),'xdata',endx+2)
    
    pos=[get(handles.grabbers{handles.activeindex}(1),'xdata')-2 get(handles.grabbers{handles.activeindex}(1),'ydata')-3 minwidth minheight];
    
end

%prevent unnecessary calls to raveresizeaxes when graphs are snapped to grid:
oldpos=get(handles.allblockers(handles.activeindex),'position');
if all(pos==oldpos);return;end

set([handles.allblockers(handles.activeindex) handles.expblockers(handles.activeindex)],'position',pos)

widthinches=sprintf('%5.2f',round(pos(3)/96*4)/4);
heightinches=sprintf('%5.2f',round(pos(4)/96*4)/4);
set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'" at 96dpi resolution'])

if strcmp(handles.graphinfo.(handles.activetype).axesformat,'table')
    %Get code from ravemovetable
else
    raveresizeaxes(handles.fig,[],handles.activeindex);
end

%Update legend positions:

if ~isempty(legdata)
    axpos = get(handles.allgraphs(handles.activeindex),'position');
    %do math to get all 4 corners:
    axcorners = [axpos(1) axpos(2); axpos(1) axpos(2)+axpos(4);axpos(1)+axpos(3) axpos(2)+axpos(4);axpos(1)+axpos(3) axpos(2)];

    %maintain the distance of each grabber to its closest corner:
    for i = size(legposses,1):-1:1
        legvertex(i,:) = legdistance(i,:) + axcorners(closest(i),:);
        set(legdata(i),'position',[legvertex legposses(i,3:4)]);
    end
end




function ravemovegraphdone(src,ev)

clear ravearrangeobjects %has persistent variables

handles=guidata(src);
guidata(handles.fig,handles);

set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'windowbuttonupfcn','');

set(handles.infobar,'string','');

