function output=raveparetofront(objective,dorank)
%determines pareto front of objective where each objective is to maximize.
% objective is a n x m matrix where each column contains the
% values of one objective.
% if dorank=false (or omited), output is n x 1 logical indicating
% whether that point is on the pareto front.
% if dorank=true, output is a n x 1 vector of integers indicating each
% point's non-domination rank. 1 = on the pareto front.

persistent domex

if nargin==1
    dorank=false;
end

isaccounted=false(size(objective,1),1);
ranks=nan(size(objective,1),1);
r=1;

if isempty(domex) || domex==true %attempt to use mex version
    try
        while any(~isaccounted);
            thesenondom=paretofront(-objective(~isaccounted,:));
            if ~dorank
                output=thesenondom;
                return
            end
            
            index=find(~isaccounted);
            ranks(index(thesenondom))=r;
            isaccounted(index(thesenondom))=true;
            r=r+1;
        end
        
        output=ranks;
        domex=true;
        return
    catch 
        domex=false;
        warning('Rave tried to use the mex version of paretofront.c, but it didn''t work. If you can compile the paretofront.c file into a .mex32 or .mex64 file that works on your computer then future multiobjective optimization runs will be much faster.');
    end
end

%If the c version fails, then do Deb's O(MN^2) scheme on pg 43 of Deb book.
S = false(size(objective,1));
count = zeros(size(objective,1),1);

for i=1:size(objective,1);
    testpt=objective(i,:);
    count(i)=dominancecount(testpt,objective);
    S(:,i)=finddominatees(testpt,objective);
end


while any(~isaccounted);
    idx=find(count==0 & ~isaccounted);
    isaccounted(idx)=true;
    ranks(idx)=r;
    nsubs=sum(S(:,idx),2);
    count=count-nsubs;
    r=r+1;
end

if dorank
    output=ranks;
else
    output=ranks==1;
end

function [count,idx]=dominancecount(testpt,allpts)
%counts how many points in allpts dominate testpt, assuming all objectives
%are maximize. Optionally also returns the idx of these points (rows in
%allpts).

idx=finddominators(testpt,allpts);
count=sum(idx);

function idx=finddominatees(testpt,allpts)
%returns a logical index into rows of allpts indicating which of those pts
%are dominated by the testpt.
%
%presumes that each objective is to maximize.
normmatrix=bsxfun(@minus,allpts,testpt);
idx=all(normmatrix<=0,2) & ~all(normmatrix==0,2);

function idx=finddominators(testpt,allpts)
%returns a logical index into rows of allpts indicating which of those pts
%dominate the testpt.
%
%presumes that each objective is to maximize.

normmatrix=bsxfun(@minus,allpts,testpt);
idx=all(normmatrix>=0,2) & ~all(normmatrix==0,2);
