function [varnames,data,textdata,discretevalues,datatype]=raveparsedatastructure(handles,in)
%Parses the data structure "in" and returns variables for data set
%creation.
% This is a helper function for raveparseXXX2dataset files. Use this
% function when the data file can easily be converted to a structure
% with the following fields:
%  .colheaders = cell array of strings
%  .textdata = cell array of strings
% in may be a vector structure that contains multiple data sets.


infobar=handles.infobar;

varnames=cell(1,numel(in));
data=cell(1,numel(in));
textdata=cell(1,numel(in));
discretevalues=cell(1,numel(in));
datatype=cell(1,numel(in));

for d=1:numel(in)
    varnames{d}=in(d).colheaders;
    ncols=numel(in(d).colheaders);
    
    isinteger=false(1,ncols);
    isbinary=false(1,ncols);
    isbitstring=false(1,ncols);
    discretevalues{d}=cell(1,ncols);
    
    %Determine which columns are numerical/string/logical/bitstring:
    
    set(infobar,'string',['Looking for numerical data in dataset ',num2str(d)]);drawnow;
    
    %First look for actual numerical values, which can come from xls files:
    isnumbers=cellfun(@isnumeric,in(d).textdata);
    isnans=cellfun(@cellisnan,in(d).textdata);
%     isnumerical=all(isnumbers | isnans,1);
    isnumerical = all(isnumbers,1);
    
    %Replace NaNs in text columns with empty string:
     isnans(:,isnumerical)=false;
     [in(d).textdata{isnans}]=deal('');
     in(d).textdata=reshape(in(d).textdata,size(isnans));
    
    %Look for numbers hiding in text columns:
    isnumbers=isnumbers & ~isnans;
    ismixed=any(isnumbers,1) & ~all(isnumbers,1); %columns that have some numbers and some text
    
    for c=find(ismixed)
        in(d).textdata(isnumbers(:,c),c)=cellfun(@formatnumber,in(d).textdata(isnumbers(:,c),c),'uniformoutput',false);
    end
    
    if all(isnumerical)
        istext=~isnumerical;
        isinteger=false(size(isnumerical));
        isbinary=false(size(isnumerical));
        isbitstring=false(size(isnumerical));
    else
        
        %look for numerical columns:
        istext=~isnumerical;
        
        %now check all rows for data that might be numeric:
        
        %To make this fast, it only checks the first 100 rows of data. If all of those are numerical it is assumed that the whole column is numerical. 
        
        maxidx = min(100,size(in(d).textdata,1));
        
        subtext=in(d).textdata(1:maxidx,~isnumerical);
        c=find(~isnumerical);
        
        isnumbers=cellfun(@checkifnumber,subtext);
        isnumbercolumn=all(isnumbers,1);
        
        istext(c(isnumbercolumn))=false;
        istext(c(~isnumbercolumn))=true;
        isnumerical=~istext;
        
        
        %look for integer data:
        
        %{
        set(infobar,'string',['Looking for integer data in dataset ',num2str(d)]);drawnow;
        for c=ncols:-1:1
            if ~isnumerical(c);isinteger(c)=false;continue;end
            if isnumeric(in(d).textdata{1,c});continue;end %can only detect integers for textual numbers
            b1=cellfun(@isempty,strfind(in(d).textdata(:,c),'.')); %looks for numbers with no decimal points
            b2=cellfun(@isempty,strfind(in(d).textdata(:,c),'e-')); %looks for numbers with no decimal points
            b3=cellfun(@isempty,strfind(in(d).textdata(:,c),'E-')); %looks for numbers with no decimal points
            isinteger(c)=all([b1;b2;b3]);
        end
        %}
        
        %look for binary data:
        set(infobar,'string',['Looking for binary data in dataset ',num2str(d)]);drawnow;
        for c=ncols:-1:1
            if ~isinteger(c);isbinary(c)=false;continue;end
            if isnumeric(in(d).textdata{1,c});continue;end %can only detect binary for textual numbers
            iszero=cellfun(@eq,in(d).textdata(:,c),repmat({'0'},size(in(d).textdata,1),1),'uniformoutput',false);
            isone=cellfun(@eq,in(d).textdata(:,c),repmat({'1'},size(in(d).textdata,1),1),'uniformoutput',false);
            isbin=cellfun(@or,iszero,isone,'uniformoutput',false);
            isbinary(c)=all(cellfun(@all,isbin));
        end
        
        %check logical vs bitstring:
        set(infobar,'string',['Looking for bitstring data in dataset ',num2str(d)]);drawnow;
        
        for c=ncols:-1:1
            if ~isbinary(c);isbitstring(c)=false;continue;end
            len=cellfun(@length,in(d).textdata(:,c));
            if all(len==1)
                isbitstring(c)=false;
            elseif all(len==len(1)); %all the same length: this is a bitstring
                isnumerical(c)=false;
                isbinary(c)=false;
                isbitstring(c)=true;
            else %treat as a regular string
                istext(c)=true;
                isnumerical(c)=false;
                isinteger(c)=false;
                isbinary(c)=false;
                isbitstring(c)=false;
            end
        end
    end
    
    %Process each column according to its data type:
    data{d}=nan(size(in(d).textdata));
    textdata{d}=cell(size(in(d).textdata));
    
    
    %Numerical data:
    set(infobar,'string',['Processing numerical data in dataset ',num2str(d)]);drawnow;
    
    if any(isnumerical | isbinary);
        
        %Check if columns are already numbers:
        isnumbers=all(cellfun(@isnumeric,in(d).textdata),1);
        
        for c=find(isnumbers & (isnumerical | isbinary)) %Directly store numerical data
            data{d}(:,c)=cell2mat(in(d).textdata(:,c));
        end
        
        for c=find(~isnumbers & (isnumerical | isbinary)) %Convert strings of numbers to doubles:
            data{d}(:,c)=cellfun(@eval,in(d).textdata(:,c));
        end
    end
    
    %String values:
    set(infobar,'string',['Processing text data in dataset ',num2str(d)]);drawnow;
    if any(istext | isbitstring)
        %convert text values to numerical indexes:
        for c=find(istext|isbitstring)
            textdata{d}(:,c)=in(d).textdata(:,c);
            [discretevalues{d}{c},junk,isthisvalue]=unique(in(d).textdata(:,c));
            data{d}(:,c)=isthisvalue;
            discretevalues{d}{c}=strtrim(discretevalues{d}{c});
        end
    end
    
    for c=1:ncols
        if istext(c)
            datatype{d}{c}='string';
        elseif isbitstring(c)
            datatype{d}{c}='bitstring';
        elseif isbinary(c)
            datatype{d}{c}='logical';
        elseif isinteger(c)
            datatype{d}{c}='integer';
        else
            datatype{d}{c}='continuous';
        end
    end
end


%look for empty names:
noname=find(cellfun(@isempty,varnames));
varnames(noname)=makecolumnnames(noname);

set(infobar,'string','All data imported successfully');drawnow;


function isnumber=checkifnumber(string)
if ~ischar(string)
    isnumber=false;
else
    isnumber=~isempty(regexp(string,'^[+-]?(\d+\.\d+|\d+\.|\.\d+|\d+)([eE][+-]?\d+)?$','match'));
end

function is=cellisnan(c) %determine if anything in a cell isnan
is= isnumeric(c) && isnan(c);


function s=formatnumber(n)
s=strtrim(sprintf('%16.16g',n));

function names=makecolumnnames(n)
names=cell(numel(n));
for i=n
    names{i}=['Column ',num2str(i)];
end