function info=raveparsemfilefunction(src,ev,d,filename)
%parse an m file for use as a user function:
handles=guidata(src);

info=[];
fid=fopen(filename);

%get file name from path:
[pathstr, functionname, ext] = fileparts(filename); %#ok

if isempty(which(functionname))
    addpath(pathstr);
    savepath;
    if isempty(which(functionname))
        warndlg('The function you selected is in a folder that is not on your MATLAB search path, and the folder could not be added to the path. You must move it onto your path before you can use it in RAVE.','I can''t let you do that');
        return
    end
end

if ~strcmp(filename,which(functionname))
    warndlg({['The function you selected could not be loaded because there is another function ',...
        'with the same name higher on your MATLAB search path. Rename the function or remove other ',...
        'functions with the same name and try again.']; '(To see all functions on your path with this name, type:';['"which ',...
        functionname,' -all" at the MATLAB command prompt.)']},'I can''t let you do that');
    return
end


firstline=fgetl(fid);
firstline=strtrim(firstline);

secondline = fgetl(fid);
secondline = strtrim(secondline);

if isempty(firstline)
    errordlg('Rave requires that the "function" declaration appear on the first line of the m-file. The file you are trying to load has one or more blank lines at the top. Remove them and try again.','Error');
end
%ignore comments at top of file
while firstline(1)==char(37) %this line is a comment
    firstline=fgetl(fid);
    firstline=firstline(~isspace(firstline));
end

if ~strcmpi(firstline(1:8),'function')
    warndlg('The file you selected appears to be a script, not a function. If it really is a function, the first line of the file must be the function declaration.','Not a Function')
    return
end

try
    if strcmp(firstline(10:17),'varargin') || strcmp(firstline(10:17),'varargin')
        warndlg('We like varargout as much as the next guy, but unfortunately RAVE requires each output to be listed explicitly on the first line of the m file.','You can''t do that')
        return
    end
catch
    warndlg('This function doesnt take any inputs. See the rave documentation for required input format for user-supplied functions in rave.','You can''t do that');
    return
end

%done with the file for now.
fclose(fid);

%GET OUTPUT VARIABLE NAMES
%ok, enough with the errors... check how many outputs this function has.
if firstline(10)=='[' %[ %function probably has multiple outputs
    %find matching ] and extract variable names inbetween.
    lastindex=find(firstline==']');
    outputsection=firstline(9:lastindex);
    outputsection(outputsection==char(32) | outputsection==char(44) |outputsection==';'|outputsection=='[')=';';
    outputsection=[';',outputsection(2:end-1),';]'];
    outputnamez=symvar(outputsection);
    %put back in original order:
    for q=1:numel(outputnamez)
        idx(q)=strfind(outputsection,[';',outputnamez{q},';']); %#ok<AGROW>
    end
    [junk,b]=sort(idx);
    outputnamez=outputnamez(b);
else
    lastindex=find(firstline==char(61));
    outputnamez={firstline(10:lastindex-1)};
end

%now check for inputs:
firstidx=find(firstline=='(',1,'last');
lastidx=find(firstline==')',1,'last');

inputsection=firstline(firstidx:lastidx);
inputsection(inputsection==char(32) | inputsection==char(44) |inputsection==';'|inputsection=='[')=';';
inputsection=[';',inputsection(2:end-1),';]'];
inputnames=symvar(inputsection);
%put back in original order:
idx=[];
for q=1:numel(inputnames)
    idx(q)=strfind(inputsection,[';',inputnames{q},';']); %#ok<AGROW>
end
[junk,b]=sort(idx);
inputnames=inputnames(b);


info.outputnames=outputnamez;
info.inputnames=inputnames;


%determine function input structure:
allnames=handles.allnames{d};
%remove spaces from varnames
 for j=length(allnames):-1:1
     nospacenames{j}=allnames{j}(allnames{j}~=char(32));
 end
%check if inputnames match names from variables in the data set:
matches=zeros(numel(inputnames),1);

for i=1:numel(inputnames)
    thisidx=find(strcmp(inputnames{i},allnames));
    if isempty(thisidx);
        thisidx=find(strcmpi(inputnames{i},allnames));
    end
    
    if isempty(thisidx);
        thisidx=find(strcmpi(inputnames{i},nospacenames));
    end

    if numel(thisidx)>1
        warndlg({['The function you selected could not be loaded because it specifies an input ',...
        'variable named "',inputnames{i},'" and there are multiple variables in your data set with this name. ',...
        'Input variable names must be unique so I can determine which values to send to the function.']},'I can''t let you do that');
    end
    
    if ~isempty(thisidx)
        matches(i)=thisidx;
    end
end

inputvars=[];
if all(matches~=0)
    %check if matches are identical to the selection the user specified in
    %the gui:
    if numel(matches)~=numel(inputvars) || ~all(sort(matches)==inputvars)
%          warndlg({['Notice: The input variable names specified in the function file do not match the ',...
%         'names you selected in the list on the right. The variables specified in the file will be used ',...
%         'in place of your selection.']},'Variable mismatch');
    end
    inputvars=matches;
    funtype='matlab_varnames';
elseif numel(inputnames)==1 && ~any(matches)
    funtype='matlab_singleinput';
    inputvars=1:nnz(~handles.isfunction{d});
elseif numel(inputnames)==3 && ~any(matches)
    funtype='matlab_threeinput';
    inputvars=1:nnz(~handles.isfunction{d});
else
    errordlg({['The function you selected could not be loaded because it does not properly declare ',...
        'the function''s inputs. You must declare the inputs in one of the following formats: 1) A list of variable names that '...
        'match variable names in your data set (variations in capitalization and spacing are allowed). 2) A single input, which will be a matrix of the columns '...
        'you selected in the listbox on the right. 3) Three inputs: A matrix as in (2), followed by a vector of targets for each variable in the data set, '...
        'followed by a vector of preferences for each varaible in the data set.']},'I can''t let you do that');
    return
end

info.dataset=d;
info.inputvars=inputvars;
info.funtype=funtype;

%Parse second line for function attributes
info.attributes={''};

if ischar(secondline)
    secondline=[secondline,' ']; %ensure there is a trailing space for regexp
    attributes = regexp(secondline,'#([\w]*)','tokens');
    if isempty(attributes)
        info.attributes={''};
        return
    end
    %Peel off outer layer of cells:
    attributes = [attributes{:}];
    info.attributes=lower(attributes);
end




