function ravereplacedataset(src,d,data,newvectordata,discretevalues,isselected,isvisible,rowcolors,keepifsamerownum,evaluatefunctions)
%Replaces the data set in index d with newdata.
% data is an rxc matrix of data. c must equal the number of columns of
%  handles.alldata{d}. if evaluatefunctions==true, the columns of data
%  corresponding to functions will be overwritten, so they may be zeros.
%
% vectordata is a cell array the same size as data. generally every cell
% will be empty.
%
% textdata is a cell array the same size as data. generally every cell
% will be empty.
%
% isselected is an rx1 to indicate whether to select the new data, or is
%     empty to unselect all.
% isvisible is an rx1 vector indicating that new data is visible or not, or
%     is empty to show all
% rowcolors is a rx1 vector of row color indices, or is empty to use the
%     default color for all rows.
% minfilter is a 1xc vector of minfilter values, or is empty to use the min
%     limits
% maxfilter is the same thing for max filter.
% discrete filter is a cell array of which discrete values should be
% filtered, or is empty to unfilter all discrete values
% keepifsamerownum is 0 or 1. If 1, and newdata has the same number of rows
%     as the data it is replacing, "isselected" "isvisible" and "rowcolors" will be ignored and none of
%     these settings will be changed from their previous values. The only
%     change that will be made is that newdata replaces the old data.
% evaluatefunctions is 0 or 1, indicating whether newdata already includes
%     the correct values of all function variables (=0) or whether they should
%     be evaluated by this function (=1);
% 
% IMPORTANT: This function does not call raveupdatedataset. You should do
% that immediately after you call this function.
%evaluate functions

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

oldnrows=size(datasets.alldata{d},1);
nrows=size(data,1);

if size(data,2)~=size(datasets.alldata{d},2)
    error('Attempted to replace data set, but the number of columns in the new data does not match the number of columns in the old data')
end

ncols=size(datasets.alldata{d},2);

%Fill in empty inputs:


if isempty(newvectordata)
    newvectordata=cell(nrows,ncols);
end

if isempty(rowcolors)
    rowcolors=handles.prefs.defaultrowcolor*ones(nrows,1);
end

if isempty(isvisible)
    isvisible=true(nrows,1);
end

if isempty(isselected)
    isselected=false(nrows,1);
end


%check that this is a valid replacement:
if size(newvectordata,1)~=nrows
    error('Attempted to replace data set, but new data and vectordata did not have the same number of rows')
end

%evaluate functions?
if evaluatefunctions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,data,handles);
    data(:,index)=outs;
end

%replace data:
datasets.alldata{d}=data;
% textdata.alltext{d}=newtextdata;
vectordata.vectordata{d}=newvectordata;
datasets.isvalid{d}=true(size(data));

raveguidata(handles.fig,datasets);
% raveguidata(handles.fig,textdata);
raveguidata(handles.fig,vectordata);

%ignore all?
if keepifsamerownum && oldnrows==nrows;
    return
end

%Update Analyses:
for a=1:numel(handles.analysisnames)

    handles.isselected{d}{a}=isselected;
    handles.isvisible{d}{a}=isvisible;
    handles.rowcolors{d}{a}=rowcolors;
    
    %check filters:
    guidata(handles.fig,handles);
    handles.isfiltered{d}{a}=ravecalculatefilteredstate(src,[],d,a);
    
    %Reset points of interest:
    handles.ispointofinterest{d}=false(nrows,1);
end

guidata(handles.fig,handles);
