function ravesavemetadata(src,ev)
%these functions are described in the rave readme file.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);


set(0,'userdata',handles.fig)

hhh=savemetadatagui(src,ev);
uiwait(hhh);

results=get(0,'userdata');


if ~isstruct(results) %user hit cancel
    return
end

d=results.d;
a=results.a;
dest=results.path;
makethese=results.makethese;



name=handles.datasetnames{d};




dest=fullfile(dest,[name '.rvm']);
fid=fopen(dest,'wt'); %clear file contents.
fclose(fid);

if makethese(1);save_rvefun(src,d,a,dest);end
if makethese(2);save_rveval(src,d,a,dest);end
if makethese(3);save_rvecon(src,d,a,dest);end
if makethese(4);save_rvecur(src,d,a,dest);end
if makethese(5);save_rveopt(src,d,a,dest);end
if makethese(6);save_rvecol(src,d,a,dest);end
if makethese(8);save_rvedis(src,d,a,dest);end
if makethese(9);save_rvepoi(src,d,a,dest);end
if makethese(10);save_rvente(src,d,a,dest);end
if makethese(7);save_rverow(src,d,a,dest);end %do row state last since it can be obnoxiously long


ravefinish;
set(handles.infobar,'string','Metadata exported successfully.')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvefun(src,d,a,dest)
handles=guidata(src);

thesevars=find(handles.isfunction{d});

if isempty(thesevars);return;end %no functions in this data set

fid=fopen(dest,'at');
fprintf(fid,'\n');
fprintf(fid,'%s\n','%#FUNCTIONS');

for g=1:numel(thesevars)
    thisname=handles.basenames{d}{thesevars(g)};
    
    if handles.isobjective{d}(thesevars(g))
        thisname=['*' thisname]; %#ok<AGROW>
    end

    thisfile=which(handles.funnames{d}{thesevars(g)});
    thisout=handles.nargout{d}(thesevars(g));
    thisins=handles.funvars{d}{thesevars(g)};
    thisformat=handles.funclass{d}{thesevars(g)};
    
    fprintf(fid,'%s\t%s\t%s\t%s\t',thisname,thisfile,thisformat,num2str(thisout));
    for f=1:numel(thisins)
        fprintf(fid,[handles.basenames{d}{thisins(f)} '\t']);
    end
    fprintf(fid,'\n');
end
fclose(fid);


%function save_rvetyp(src,d,a,dest)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rveval(src,d,a,dest)
handles=guidata(src);

thesevars=1:numel(handles.basenames{d});

fid=fopen(dest,'at');

for A=1:numel(a)
    
    fprintf(fid,'\n');
    fprintf(fid,'%s%s\n','%#RANGES:',num2str(A));
    
    for g=1:numel(thesevars)
        thisname=handles.basenames{d}{thesevars(g)};
        fprintf(fid,'%s\t',thisname);
        thistype=handles.datatype{d}{thesevars(g)};
        fprintf(fid,'%s\t',thistype);
        switch thistype
            case {'continuous','integer','logical'}
                fprintf(fid,'%16.15g\t',handles.minlimit{d}{A}(g));
                fprintf(fid,'%16.15g',handles.maxlimit{d}{A}(g));
            case 'discrete'
                N=numel(handles.discretevalues{d}{A}{g});
                for n=1:N
                    fprintf(fid,'%16.15g\t',handles.discretevalues{d}{A}{g}(n));
                end
            case {'string','list'}
                N=numel(handles.discretevalues{d}{A}{g});
                for n=1:N
                    fprintf(fid,'%s\t',handles.discretevalues{d}{A}{g}{n});
                end
                
        end
        fprintf(fid,'\n');
    end
    
end

fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvecon(src,d,a,dest)
handles=guidata(src);
N=numel(handles.constrainttext{d}{a(1)});
if N==0;return;end

fid=fopen(dest,'at');
fprintf(fid,'\n');

for A=1:numel(a)
    fprintf(fid,'%s%s\n','%#CONSTRAINTS:',num2str(A));
    for n=1:N
        fprintf(fid,'%s\t',handles.constrainttext{d}{a(1)}{n});
        fprintf(fid,'%s\n',mat2str(handles.constraintcolors{d}{a(1)}{n}));
    end
end

fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvecur(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'at');
fprintf(fid,'\n');

for A=1:numel(a)
    fprintf(fid,'%s%s\n','%#CURRENTPOINT:',num2str(A));
    for n=1:numel(handles.basenames{d})
        fprintf(fid,'%s\t',handles.basenames{d}{n});
    end
    fprintf(fid,'\n');
    for n=1:numel(handles.basenames{d})
        fprintf(fid,'%s\t',num2str(handles.xdials{d}{a(1)}(n)));
    end
   fprintf(fid,'\n'); 
end

fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rveopt(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'at');
fprintf(fid,'\n');
fprintf(fid,'%s\n','%#TARGETSANDPREFERENCES');

for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',handles.basenames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',num2str(handles.targetvalues{d}(n)));
end
fprintf(fid,'\n');
for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',num2str(handles.preferences{d}(n)));
end
fprintf(fid,'\n');
fclose(fid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvecol(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'at');
fprintf(fid,'\n');
fprintf(fid,'%s\n','%#COLORS');

for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',handles.basenames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',mat2str(handles.variablecolors{d}(n,:),4));
end
fprintf(fid,'\n');
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rverow(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'at');
fprintf(fid,'\n');

for A=1:numel(a)
    fprintf(fid,'%s%s\n','%#ROWSTATE:',num2str(a));
    for n=1:size(handles.isselected{d}{1},1)
        fprintf(fid,'%s\t',num2str(handles.rowcolors{d}{a(1)}(n)));
        fprintf(fid,'%s\t',num2str(handles.isvisible{d}{a(1)}(n)));
        fprintf(fid,'%s\n',num2str(handles.isselected{d}{a(1)}(n)));
    end
    fprintf(fid,'\n');
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvedis(src,d,a,dest)
handles=guidata(src);

fid=fopen(dest,'at');

fprintf(fid,'\n');
fprintf(fid,'%s\n','%#RANDOMVARS');

for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',handles.basenames{d}{n});
end
fprintf(fid,'\n');
for n=1:numel(handles.basenames{d})
    fprintf(fid,'%s\t',num2str(handles.isnoise{d}(n)));
end
fprintf(fid,'\n');


fprintf(fid,'\n');
fprintf(fid,'%s\n','%#DISTRIBUTIONS');

for n=1:numel(handles.basenames{d})
    for g=1:numel(handles.distributions{d}{n})
        if ~isempty(fieldnames(handles.distributions{d}{n}{g}))
            fprintf(fid,'%s\n',handles.basenames{d}{n});
            fff=fieldnames(handles.distributions{d}{n}{g});
            for f=1:numel(fff)
                if ~strcmp(fff{f},'serialnumber')
                    fprintf(fid,'%s = %s\n',fff{f},handles.distributions{d}{n}{g}.(fff{f}));
                else
                    fprintf(fid,'%s = %s\n',fff{f},mat2str(handles.distributions{d}{n}{g}.(fff{f})));
                end
            end
            
            fprintf(fid,'isactive = %s\n',num2str(g==handles.activedistribution{d}(n)));
            fprintf(fid,'\n');
        end
    end
end

fprintf(fid,'\n');
fprintf(fid,'%s\n','%#STATISTICS');

for n=1:numel(handles.basenames{d})
        fprintf(fid,'%s\t',handles.basenames{d}{n});
        for f=1:numel(handles.aggregatingstatistic{d}{n})
            fprintf(fid,'%s\t',mat2str(handles.aggregatingstatistic{d}{n}{f}));
        end
        fprintf(fid,'\n');
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvepoi(src,d,a,dest)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function save_rvente(src,d,a,dest);
handles=guidata(src);

fid=fopen(dest,'at');

fprintf(fid,'\n');
fprintf(fid,'%s\n','%#ROWNOTES');

datanotes=raveguidata(handles.fig,'datanotes');
notes=datanotes{d};

notestr=printVariable(notes);

fprintf(fid,'%s\n',notestr{1});
fclose(fid);
