function ravesavestate(src,eventdata)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
fig=handles.fig;
datasets=raveguidata(handles.fig,'datasets');

set(handles.infobar,'string','')
%ask user for a name for this project.
if handles.tablemode
    msgbox('You cannot save a project while viewing the data table. Switch back to the workspace view, and then save the project.',...
        'Sorry about that...','error');
    return
end

h=ravesavegui([],handles);
uiwait(h);
name=get(0,'userdata');

%did they cancel?
if isempty(name);return;end

set(handles.infobar,'string','\color[rgb]{1 0 0}Saving state... please wait...');
drawnow;

mkdir(handles.prefs.defaultdir,name);

fid=fopen(fullfile(handles.prefs.defaultdir,name,[name,'.rve']),'w');
fprintf(fid,'%%Rave Project File Generated By Rave v.');
fprintf(fid,ravever);
fprintf(fid,' Warning: Editing this file may cause it to become unloadable in Rave.\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Process graph info:
graphinfo=struct();

for i = 1:numel(handles.allgraphs)
    if ~ishandle(handles.allgraphs(i));continue;end
    graphtype = get(handles.allgraphs(i),'userdata');
    udata = get(handles.fakeax(i),'userdata');
    blockerpos=get(handles.allblockers(i),'position');
    
    %remove deletehandles from udata, because these will break if
    %you try to delete them in the opened state.
    if isfield(handles.graphinfo.(graphtype),'deletehandles')
        deletehandles=handles.graphinfo.(graphtype).deletehandles;
        for g=1:numel(deletehandles)
            eval([deletehandles{g},'=[];']);
        end
    end

    graphinfo(i).ai=i;
    graphinfo(i).graphtype=graphtype;
    graphinfo(i).udata=udata;
    graphinfo(i).position=blockerpos;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Process handles:

%remove unusable subfields of handles.maintable:
unsaveablefields={'handle','jtable','jscroll','models','rhmodels'};
unsaveablefields=intersect(unsaveablefields,fieldnames(handles.maintable));
handles.maintable=rmfield(handles.maintable,unsaveablefields);


%remove unsaveable things from handles:
unsaveablefields={'trayIcon','axiscanvas','mouseoverfunctions','sounds','mousetimer','fontmetrics',...
    'jvscroll','jhscroll','constrainthandles','mouseoverdata','activeindex','graphanalysis','graphdataset'};
unsaveablefields=intersect(unsaveablefields,fieldnames(handles));
handles=rmfield(handles,unsaveablefields);

%Remove useless fields that should always be recreated in the new rave
%session:
uselessfields={'contextmenus','output','helpbutton','tabobjects','hideshowbutton','openglbutton','showgridbutton','zoombutton','newbutton',...
    'tablemodebutton','dummyobj','stopbutton','alltabs','fronttabsfront','temp','backtablines','backtabbg','fronttablines','fronttabbg',...
    'backtabprofile','fronttabprofile','controltitle','backtabaxes','fronttabaxes','tabaxes','bgtabaxes','fronttabposition','backtabposition',...
    'titleaxes','hscroll','vscroll','explorertitle','currentview','explorergrid','explorer','pagecontrols','pageedit','lgrid','mgrid','sgrid',...
    'foregroundaxes','mainaxes','infobar','bottommargin','topmargin','rightmargin','leftmargin','wspanel','boneyard',...
    'distributioninfo','exploreinfo','modelinfo','graphinfo','optimizerinfo','helproot','tablemode','optimizetabobjects',...
    'rasters','rotater','activetab','activetype','newgraphtype','raveroot','axeslisteners','fig','defaultdir','ishg2',...
    'fakeax','fakerax','activedir','activesubgraph','subax','subfakeax','subfakerax'};
uselessfields=intersect(uselessfields,fieldnames(handles));
handles=rmfield(handles,uselessfields);


%Remove preferences that should use the value stored in the computer
%opening this file:
uselessprefs = {'defaultdir','listfonts','htmllistfonts','monitorpositions','isportable','alwaysupdate','startintablemode',...
    'gpgpu','cellcarrier','cellnumber','checkforupdates','currentversion','wvisual','xvisual',...
    'altgraphicsdir','altoptimizerdir','altmodelsdir','altobjectivesdir','altexploremethodsdir'};
uselessprefs=intersect(uselessprefs,fieldnames(handles.prefs));
handles.prefs=rmfield(handles.prefs,uselessprefs);
uselessprefs = {'listfonts','htmllistfonts'};
uselessprefs=intersect(uselessprefs,fieldnames(handles.prefs.fonts));
handles.prefs.fonts=rmfield(handles.prefs.fonts,uselessprefs);



%Convert to text:
handlesinfo=printVariable(handles);
%print to file:
for i = 1:numel(handlesinfo)
    fprintf(fid,'%s\n',handlesinfo{i});
end

%Convert to text:
graphinfotext=printVariable(graphinfo);
%print to file:
for i = 1:numel(graphinfotext)
    fprintf(fid,'%s\n',graphinfotext{i});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Process data sets:

for d = 1:numel(handles.datasetnames)
    datainfo(d).alldata=datasets.alldata{d};%#ok<*AGROW>
    datainfo(d).datatype=datasets.datatype{d}; 
    
    if numel(datasets.vectordata)>=d
        datainfo(d).vectordata=datasets.vectordata{d};
    end
    
    datainfo(d).isvirtual=datasets.isvirtual{d};
    
    if numel(datasets.linkfrom)>=d
        datainfo(d).linkfrom=datasets.linkfrom{d};
        datainfo(d).linkto=datasets.linkto{d};
        datainfo(d).virtualc=datasets.virtualc{d};
    end
    
    datainfo(d).alltextvalues=datasets.alltextvalues{d};
    datainfo(d).alldiscretevalues=datasets.alldiscretevalues{d};
end

%Convert to text:
datainfotext=printVariable(datainfo);
%print to file:
for i = 1:numel(datainfotext)
    fprintf(fid,'%s\n',datainfotext{i});
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
handles = guidata(fig);
ravefinish;
set(handles.infobar,'string','Project saved successfully.')
fclose(fid);



