function handles=raveupdateanalysis(varargin)
%raveupdateanalysis(src,handles,updatetype,skipthese,d,a,thesevars)
%applies the update in updatetype to all graphs that share the same
%analysis as the active graph. (or updates the analysis "a" if 6 inputs are
%used)
%
%options for updatetype:
%'xdials','visiblerows','constraints','isselected,'rowcolors',
%'variablecolors','isfiltered'
%
%
%skipthese is a list of indices of graphs to not update (i.e. graphs that
%have already been updated before calling raveupdateanalysis.)
%
%the fifth input, d, is only used if you also supply a sixth input, a.
%
%the seventh input, thesevars is a list of which variables' xdials
% were changed. This value is not directly used by this function, but it is passed
% to any raveupdatexdials<keyword> function as a 4th input. Those functions
% can then decide whether or not to use it.
%( It only affects xdials updates since those are the only analysis update
% that affects particular variable(s).)
%
%
% IMPORTANT: This function only calls guidata if it was called with 0
% outputs. If this function is outputting handles, it doesnt bother to call
% guidata. 
%
%
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

allinputs=varargin;

src=varargin{1};
handles=varargin{2};
updatetype=varargin{3};
skipthese=varargin{4};

if isempty(handles) %not all occurances of raveupdateanalysis include handles as 2nd input yet
    handles=guidata(src);
    allinputs{2}=handles; %for calling raveupdateanalysisdatasetlinks below...
end

if nargin<6
    udata=get(handles.fakeax(handles.activeindex),'userdata');
    thisanalysis=udata.view.analysis;
    thisdataset=udata.view.dataset;
else
    thisdataset=varargin{5};
    thisanalysis=varargin{6};
end

if nargin<7
    thesevars=[];
else
    thesevars=varargin{7};
end

%update maintable if in table mode (otherwise updates will be applied when you change to tablemode):
if handles.tablemode && ~isempty(handles.maintable) && thisdataset==handles.maintable.d && thisanalysis==handles.maintable.a
    switch updatetype
        case 'xdials'
            
        case 'visiblerows'
            isRowShaded=~handles.isvisible{thisdataset}{thisanalysis};
%             isRowShaded=javaArray('java.lang.Boolean',numel(isrowshaded));
%             for r=1:numel(isrowshaded)
%                 isRowShaded(r)=java.lang.Boolean(isrowshaded(r));
%             end
            handles.maintable.models{thisdataset}.setIsRowShaded(isRowShaded);
            
        case 'constraints'
            
        case 'isselected'
            
        case 'isfiltered'
            
        case 'rowcolors'
            rowcolors=handles.rowcolors{thisdataset}{thisanalysis};
             jRowColors=javaArray('java.lang.Integer',numel(rowcolors));
             for r=1:numel(rowcolors)
                 jRowColors(r)=java.lang.Integer(rowcolors(r)-1);
             end
            handles.maintable.models{thisdataset}.setRowColors(jRowColors);
        case 'variablecolors'
    end
end

%if any variables are decided, update data set:
if thisanalysis==1 %only applies to analysis 1.
    switch updatetype
        case 'xdials'
            if any(handles.isconstant{thisdataset} & ~handles.isnoise{thisdataset} & ~(handles.maxlimit{thisdataset}{1}==handles.minlimit{thisdataset}{1}))
                datasets=raveguidata(handles.fig,'datasets');
                decidedvars=find(handles.isconstant{thisdataset} & ~(handles.maxlimit{thisdataset}{1}==handles.minlimit{thisdataset}{1}));
                datasets.alldata{thisdataset}(:,decidedvars)=repmat(handles.xdials{thisdataset}{1}(decidedvars),size(datasets.alldata{thisdataset},1),1);
                
                %determine which functions need to update:
                updatefuns=[];
                for c=find(handles.isfunction{thisdataset})
                    if any(ismember(decidedvars,handles.funvars{thisdataset}{c}))
                        updatefuns(end+1)=c; %#ok<AGROW>
                    end
                end
                
                if ~isempty(updatefuns)
                    y=raveevaluatefunction(handles.fig,thisdataset,updatefuns,datasets.alldata{thisdataset},handles);
                    datasets.alldata{thisdataset}(:,updatefuns)=y;
                end
                
                raveguidata(handles.fig,datasets);
                handles=raveupdatedataset(handles.fig,handles,'datachanged',[],thisdataset,thesevars);
                
            end
    end
end


%find indices of other graphs with this dataset and analysis:
othergraphs=find(handles.graphanalysis==thisanalysis);

othergraphs=setdiff(othergraphs,skipthese);

if isempty(othergraphs);return;end

originalindex=handles.activeindex;

for ai=othergraphs
    othergraph=handles.allgraphs(ai);
    if ~ishandle(othergraph);continue;end %skip if this graph has been deleted.
    
    thistype=get(othergraph,'userdata'); %type of other graph
    udata=get(handles.fakeax(ai),'userdata'); %user data of othergraph
    if udata.view.dataset~=thisdataset;continue;end %skip if this graph is from a different data set.
    
    %check if thistype of graph gets updated:
    if ismember(updatetype,lower(handles.graphinfo.(thistype).updates));
        %check for a function to perform the update:
        handles.activeindex=ai;
        handles.activegraph=handles.allgraphs(ai);
        handles.activetype=thistype;
        
        if ~isempty(which(['raveupdate',thistype,updatetype]))
            if strcmp(updatetype,'xdials') %also send thesevars as input:
                handles=eval(['raveupdate',thistype,updatetype,'(handles.fig,[],handles,thesevars);']);
            else
                handles=eval(['raveupdate',thistype,updatetype,'(handles.fig,[],handles);']);
            end
            
        else %brute force method:
            guidata(handles.fig,handles);
            eval(['ravecreate',thistype,'(handles.fig,[]);'])
            handles=guidata(handles.fig);
        end
        
        if ai~=originalindex
            handles=raverasterize(handles,[],ai);
        end
    end
end

lastgraph=handles.allgraphs(originalindex);
thistype=get(lastgraph,'userdata'); %type of other graph in this analysis
handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=lastgraph;

if nargout==0
    guidata(handles.fig,handles);
end


%process dataset links:
allinputs{1}=handles.fig;
handles=raveprocessdatasetanalysislinks(allinputs{:});




