function makeuimontecarlogeneric(src,ev)
%Adds generic monte carlo type controls to the ravesamplinggui
handles=guidata(src);
u=handles.temp.u;
f=handles.temp.f;
uu=findobj(handles.temp.f,'tag','methodspanel');

infotext=uicontrol('parent',uu,'position',[30 240 500 60],'style','text',...
    'string',{'Select a variable in the table, then change its distribution using the menu to the right.'},'backgroundcolor',handles.colors.tabbg,'hor','left');

datasetmenu=findobj(u,'tag','datasetmenu');
analysismenu=findobj(u,'tag','analysismenu');
d=get(datasetmenu,'value');
a=get(analysismenu,'value');

nvars=sum(~handles.isfunction{d});
isvars=find(~handles.isfunction{d});

names=handles.allnames{d}(isvars);

distrs={};
for i=nvars:-1:1
    ad=handles.activedistribution{d}(isvars(i));
    
    if ad>0
        distrs{i}=handles.distributions{d}{isvars(i)}{ad}.name;
    else
        distrs{i}='<Hold constant at current value>';
    end
end

dat=javaArray('java.lang.Object',numel(names),2);
for g=1:numel(names)
    dat(g,1)=java.lang.String(names{g});
    dat(g,2)=java.lang.String(distrs{g});
end

tm=RaveUneditableTableModel(dat,{'Variable Name','Distribution'});
tm.setIsEditable({false false});
jtable=javax.swing.JTable(tm); %initialize a table
javaObjectEDT(jtable);
jscroll=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
javaObjectEDT(jscroll);
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
jtable.putClientProperty('terminateEditOnFocusLost',true); %doesnt seem to work...
pause(0.1)
[jhandle,mhandle]=javacomponent(jscroll,[30 50 300 220],uu);
pause(0.1)
handles.temp.jtable=jtable;
handles.temp.jscroll=jhandle;
handles.temp.x=[];

set(mhandle,'tag','jscroll');

heading=uicontrol('parent',uu,'position',[340 240+10 180 22],'style','text',...
    'string','Select a distribution:',...
    'backgroundcolor',handles.colors.tabbg,'hor','left','fontweight','bold',...
    'fontsize',10);
dmenu=uicontrol('parent',uu,'position',[350 220+10 180 22],'style','popup',...
    'string','blah','callback',@dmenu_Callback,...
    'backgroundcolor',handles.colors.tabbg,'hor','left','tag','dmenu','visible','off');

heading2=uicontrol('parent',uu,'position',[30 10 180 22],'style','text',...
    'string','Number of points to sample:',...
    'backgroundcolor',handles.colors.tabbg,'hor','left','fontweight','bold',...
    'fontsize',10);

nsamples=uicontrol('parent',uu,'position',[220 12 110 22],'style','edit',...
    'string','1000','backgroundcolor',[1 1 1],'hor','left','fontweight','normal',...
    'fontsize',10,'max',1,'tag','nsamples');

guidata(handles.fig,handles);
guidata(f,handles);

handles.temp.tabobjects=[handles.temp.tabobjects mhandle infotext heading dmenu heading2 nsamples];

guidata(handles.temp.f,handles);

mod=jtable.getSelectionModel;
mh=handle(mod,'callbackproperties');
set(mh,'ValueChangedCallback',{@table_selection_changed,handles.temp.f})

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function table_selection_changed(jsrc,jev,f)
if jev.getValueIsAdjusting;return;end %Prevent running callback on deselection events, only on new selection events

handles=guidata(f);
u=handles.temp.u;
uu=findobj(handles.temp.f,'tag','methodspanel');

datasetmenu=findobj(u,'tag','datasetmenu');
analysismenu=findobj(u,'tag','analysismenu');
dmenu=findobj(uu,'tag','dmenu');
d=get(datasetmenu,'value');
a=get(analysismenu,'value');

nvars=sum(~handles.isfunction{d});
isvars=find(~handles.isfunction{d});

%Which rows are selected?
isselected=false(nvars,1);
for i=1:nvars
    isselected(i)=jsrc.isSelectedIndex(i-1) ;
end

nselected=nnz(isselected);
selectedvars=isvars(isselected);

if nselected==0
    set(dmenu,'visible','off')
    return
else
    set(dmenu,'visible','on')
end


%What distribution names do selected variables have?
for i=nselected:-1:1
    for j=numel(handles.distributions{d}{selectedvars(i)})
        distrs{i}{j}=handles.distributions{d}{selectedvars(i)}{j}.name;
    end
end

%Update distribution menu
if nselected==1 %if only one variable is selected, just display its names in the menu:
    set(dmenu,'string',[{'<Hold constant at current value>'},distrs{1}],'value',handles.activedistribution{d}(selectedvars)+1);
else %Loop through distribution names of first selected variable.
    iscommon=false(numel(distrs{1}));
    for i=1:numel(iscommon)
        iscommon(i)=all(cellfun(@ismember,repmat(distrs{1}(i),1,numel(distrs)),distrs)); %Does every variable have this distribution?
    end
    if any(iscommon)
        set(dmenu,'string',[{'<Hold constant at current value>'},distrs{1}(iscommon)],'value',2)
    else
        set(dmenu,'string','<No Shared Distributions>','value',1)
    end
end

handles.temp.x=selectedvars;
guidata(handles.temp.f,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dmenu_Callback(src,ev)
handles=guidata(src);
u=handles.temp.u;
uu=findobj(handles.temp.f,'tag','methodspanel');

datasetmenu=findobj(u,'tag','datasetmenu');
analysismenu=findobj(u,'tag','analysismenu');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');
x=handles.temp.x;

if isempty(x);return;end

str=get(src,'string');

name=str{get(src,'value')};

if strcmp(name,'<Hold constant at current value>')
    handles.activedistribution{d}(x)=0;
else
    for i=1:numel(x)
        for j=1:numel(handles.distributions{d}{x(i)})
            if strcmp(name,handles.distributions{d}{x(i)}{j}.name)
                handles.activedistribution{d}(x)=j;
                break
            end
        end
    end
end


%Update table to show this value:
js=findobj(handles.temp.f,'tag','jscroll');
js=get(js,'JavaPeer');
jtable=js.getViewport.getComponent(0);

rows=jtable.getSelectedRows;

for r=1:numel(rows)
    jtable.setValueAt(name,rows(r),1)
end
jtable.repaint;

guidata(handles.temp.f,handles);
guidata(handles.fig,handles);




