function exporttab(src,eventdata)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);
handles=guidata(src);
set(handles.infobar,'string','');
handles.activetab='export';
guidata(src,handles);

%move front tabs to front if necessary

set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y);


%raise this tab to top
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,5,1); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

handles.exportdestination='file';
handles.exportlayout=1;
handles.exportformat='xls';

set(handles.infobar,'string','');

%% clear existing tab
delete(handles.tabobjects)

%% GRAPH STUFF

graphtext=text(.15-.10317,0.97,'Export Graphs','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

destinationtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Destination:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

filebutton=uicontrol('position',[origin(1)+50 origin(2)+540 35 35],...
    'style','togglebutton','string','','cdata',imread('image.png','png'),'value',1,'userdata','file',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg);
printbutton=uicontrol('position',[origin(1)+95 origin(2)+540 35 35],...
    'style','togglebutton','string','','cdata',imread('printer.png','png'),'userdata','print',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg);
popoutbutton=uicontrol('position',[origin(1)+95+45 origin(2)+540 35 35],...
    'style','togglebutton','string','','cdata',imread('matlab.png','png'),'userdata','newfig',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg);
pptbutton=uicontrol('position',[origin(1)+140 origin(2)+540 35 35],...
    'style','togglebutton','string','','cdata',imread('ppt.png','png'),'userdata','ppt',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');
wordbutton=uicontrol('position',[origin(1)+185 origin(2)+540 35 35],...
    'style','togglebutton','string','','cdata',imread('word.png','png'),'userdata','word',...
    'callback',@graphblob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');

handles.graphexportblobs=[filebutton printbutton popoutbutton pptbutton wordbutton];

destinationinfo=uicontrol('position',[origin(1)+25 origin(2)+518 220 16],...
    'style','text','string','Save as Image','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','center','tag','destinationinfo');

layouttext=uicontrol('position',[origin(1)+25 origin(2)+500 50 20],...
    'style','text','string','Layout:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

oneinone=uicontrol('position',[origin(1)+35 origin(2)+485 220 20],...
    'style','radiobutton','string','Selected graph and its legends','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',1,'callback',@layout_callback,...
    'tag','oneinone');

allinone=uicontrol('position',[origin(1)+35 origin(2)+465 220 20],...
    'style','radiobutton','string','All graphs within a region of interest','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','callback',@layout_callback,'tag','allinone');
allinonetext=uicontrol('position',[origin(1)+62 origin(2)+443 180 20],...
    'style','text','string','(maintains relative positions)','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left');

aatext=uicontrol('position',[origin(1)+25 origin(2)+400+20 150 20],...
    'style','text','string','Anti-Aliasing:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

nosmooth=uicontrol('position',[origin(1)+35 origin(2)+380+25 220 20],...
    'style','radiobutton','string','None','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','nosmooth','value',1,'callback',@aa_callback);
linesmooth=uicontrol('position',[origin(1)+35 origin(2)+360+25 220 20],...
    'style','radiobutton','string','LineSmoothing','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','linesmooth','value',0,'callback',@aa_callback);
mayasmooth=uicontrol('position',[origin(1)+35 origin(2)+340+25 220 20],...
    'style','radiobutton','string','MYAA','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','mayasmooth','value',0,'callback',@aa_callback);

rendtext=uicontrol('position',[origin(1)+155 origin(2)+400+20 80 20],...
    'style','text','string','Renderer:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
zbuff=uicontrol('position',[origin(1)+165 origin(2)+380+25 80 20],...
    'style','radiobutton','string','ZBuffer','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','zbuffer','value',0,'callback',@rend_callback);
opengl=uicontrol('position',[origin(1)+165 origin(2)+360+25 80 20],...
    'style','radiobutton','string','OpenGL','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','opengl','value',1,'callback',@rend_callback);

if handles.ishg2
    set([rendtext zbuff opengl],'visible','off');
end

whitecheck=uicontrol('position',[origin(1)+25 origin(2)+335 230 20],...
    'style','checkbox','string','Use workspace background color','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','tag','whitecheck','value',0);


%note, use handles.lastexport.antialias to maintain this setting between
%exports.
%use handles.lastexport.filetype to maintain filetype between exports.


exportgraphbutton=uicontrol('position',[origin(1)+30 origin(2)+210 220 30],...
    'style','pushbutton','string','Export Selected Graphs','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold','callback',@raveexportgraph);


if isempty(handles.activetype);set(exportgraphbutton,'enable','off');end


%% DATA STUFF

datatext=text(.15-.10317,.285*1.0317-.0317,'Export Data','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

formattext=uicontrol('position',[origin(1)+25 origin(2)+205-75 180 20],...
    'style','text','string','File Format:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

xlsbutton=uicontrol('position',[origin(1)+140 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('excel.png','png'),'value',0,'userdata','xls',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');
matbutton=uicontrol('position',[origin(1)+185 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('matlab.png','png'),'userdata','mat',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');
tabbutton=uicontrol('position',[origin(1)+50 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('tab.png','png'),'userdata','tab',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'value',1,'tag','datablob');
csvbutton=uicontrol('position',[origin(1)+95 origin(2)+168-75 35 35],...
    'style','togglebutton','string','','cdata',imread('csv.png','png'),'userdata','csv',...
    'callback',@datablob_callback,'backgroundcolor',handles.colors.togglebuttonbg,'tag','datablob');


selectedonlycheck=uicontrol('position',[origin(1)+35 origin(2)+50 220 20],...
    'style','checkbox','string','Only export selected rows','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',9,'horizontalalignment','left','value',0,'callback',[],...
    'tag','selectedonly');


exportdatabutton=uicontrol('position',[origin(1)+30 origin(2)+10 220 30],...
    'style','pushbutton','string','Export Data Set','backgroundcolor',handles.colors.buttonbg,...
    'fontname','arial','fontsize',12,'horizontalalignment','left','fontweight','bold','callback',@raveexportdata);

handles.dataexportblobs=[xlsbutton matbutton tabbutton csvbutton];

formatinfo=uicontrol('position',[origin(1)+25 origin(2)+144-75 220 16],...
    'style','text','string','Tab Delimited','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',8,'horizontalalignment','center','tag','formatinfo');

scaletext=uicontrol('position',[75+3.6*handles.lastexport.scale  origin(2)+250 50 30],...
    'style','text','string',{[num2str(10*handles.lastexport.scale),' %'];[num2str(round(10*handles.lastexport.scale*.96)) 'dpi']},'backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

scaler=uicontrol('position',[origin(1)+25 origin(2)+305 150 20],...
    'style','text','string','Exported Size:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');


%scale slider:
[jslider,hslider]=javacomponent('javax.swing.JSlider',[origin(1)+40 origin(2)+280 190 30]);
jslider.setPaintTicks(1)

jslider.setSnapToTicks(0)
jslider.setMinimum(10)
jslider.setMaximum(60)
jslider.setMajorTickSpacing(2);
%jslider.setMinorTickSpacing(1);
jslider.setValue(handles.lastexport.scale);
set(jslider,'background',java.awt.Color.white)
set(jslider,'StateChangedCallback',{@jslider_Callback,scaletext});

%handles.javacomps=jslider;





%% group items and finish up
handles.tabobjects=[graphtext datatext destinationtext layouttext filebutton printbutton pptbutton wordbutton...
    destinationinfo oneinone allinone allinonetext exportgraphbutton formattext xlsbutton matbutton tabbutton csvbutton...
    formatinfo exportdatabutton nosmooth linesmooth mayasmooth aatext scaletext scaler selectedonlycheck popoutbutton...
    rendtext zbuff opengl hslider whitecheck];

handles.activetab='export';

guidata(src,handles);
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function graphblob_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

destinationinfo=findobj(handles.tabobjects,'tag','destinationinfo');

set(handles.graphexportblobs,'value',0)
set(src,'value',1)

uicontrol(handles.dummyobj);

%set export desination string
handles.exportdestination=get(src,'userdata');

switch get(src,'userdata')
    case 'file'
        set(destinationinfo,'string','Save as Image')
    case 'print'
        set(destinationinfo,'string','Print')
    case 'newfig'
        set(destinationinfo,'string','Copy to New Figure')
    case 'ppt'
        set(destinationinfo,'string','Export to Powerpoint')
    case 'word'
        set(destinationinfo,'string','Export to Word')
end

guidata(gcf,handles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function layout_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

allinone=findobj(handles.tabobjects,'tag','allinone');
oneinone=findobj(handles.tabobjects,'tag','oneinone');

set([allinone oneinone],'value',0)
set(src,'value',1)
uicontrol(handles.dummyobj)

if src==allinone
    set(handles.roi,'visible','on')
    set(handles.infobar,'string','Click and drag in workspace to draw a region of interest.')
    set(handles.mainaxes,'buttondownfcn',{@raveplaceroi,handles})
    set(handles.explorer,'buttondownfcn',{@raveplaceroi,handles})
    %     buttons=findobj(handles.tabobjects,'type','uicontrol');
    %     set(buttons,'enable','inactive');
else
    set(handles.roi,'visible','off')
    set(handles.infobar,'string','');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function raveplaceroi(src,eventdata,handles) %#ok<INUSL>
%check that they clicked on exploreraxes or mainaxes:
if ~any([gco==handles.explorer,gco==handles.mainaxes])
    set(handles.mainaxes,'buttondownfcn',@ravedragview)
    set(handles.explorer,'buttondownfcn',@ravemoveview)
    return
end

set(handles.fig,'windowbuttondownfcn',[]);
buttons=findobj(handles.tabobjects,'type','uicontrol');
set(buttons,'enable','on');



%clicked=get(handles.mainaxes,'currentpoint');
clicked=get(gco,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

snapsize=1;

xclick=snapsize*floor(xclick/snapsize);
yclick=snapsize*floor(yclick/snapsize);


delete(handles.roi);
handles.roi=[];
guidata(src,handles);


handles.roi(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[0 1 .5],...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor','none');
handles.roi(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',[0 1 .5],...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizeroi);
set(handles.fig,'WindowButtonUpFcn',@ravesizeroidone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravesizeroi(src,eventdata) %#ok<INUSD>
handles=guidata(src);

finish=get(gco,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

snapsize=1;

pos1=snapsize*floor(min(startx,endx)/snapsize);
pos3=snapsize*ceil(max(startx,endx)/snapsize)-pos1;
pos2=snapsize*floor(min(starty,endy)/snapsize);
pos4=snapsize*ceil(max(starty,endy)/snapsize)-pos2;

set(handles.roi,'position',[pos1 pos2 pos3 pos4]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravesizeroidone(src,eventdata) %#ok<INUSD>
handles=guidata(src);
%stop dragging
set(handles.fig,'WindowButtonMotionFcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'WindowButtonUpFcn',[]);
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function datablob_callback(src,eventdata)
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

formatinfo=findobj(handles.tabobjects,'tag','formatinfo');

set(handles.dataexportblobs,'value',0)
set(src,'value',1)

uicontrol(handles.dummyobj);

%set export desination string
handles.exportformat=get(src,'userdata');

switch get(src,'userdata')
    case 'xls'
        set(formatinfo,'string','Excel Workbook')
    case 'mat'
        set(formatinfo,'string','Matlab (*.mat)')
    case 'tab'
        set(formatinfo,'string','Tab Delimited')
    case 'csv'
        set(formatinfo,'string','Comma Seperated Values')
end


guidata(gcf,handles)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function aa_callback(src,ev) %manage radio buttons.
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

nosmooth=findobj(handles.tabobjects,'tag','nosmooth');
linesmooth=findobj(handles.tabobjects,'tag','linesmooth');
mayasmooth=findobj(handles.tabobjects,'tag','mayasmooth');

set([nosmooth linesmooth mayasmooth],'value',0)
set(src,'value',1)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rend_callback(src,ev) %manage radio buttons.
handles=guidata(src);

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

opengl=findobj(handles.tabobjects,'tag','opengl');
zbuffer=findobj(handles.tabobjects,'tag','zbuffer');

set([opengl zbuffer],'value',0)
set(src,'value',1)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function jslider_Callback(src,ev,scaletext) %#ok<INUSL>
pos=get(scaletext,'position');
set(scaletext,'string',{[num2str(src.getValue*10),' %'];[num2str(round(src.getValue*10*.96)),'dpi']},'position',[75+3.60*src.getValue pos(2:4)]);
handles=guidata(scaletext);
handles.lastexport.scale=src.getValue;

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

guidata(scaletext,handles);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function raveexportdata(src,ev)
%export selected dataset to a file.
handles=guidata(src);
datasets=raveguidata(src,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)

formattype=findobj(handles.tabobjects,'tag','datablob','value',1);
selectedonly=findobj(handles.tabobjects,'tag','selectedonly');

try
    d=udata.view.dataset;
catch
    d=1;
end
allnames=handles.allnames{d};

if get(selectedonly,'value')
    alldata=datasets.alldata{d}(handles.isselected{d}{udata.view.analysis},:);
else
    alldata=datasets.alldata{d};
end

thisformat=get(formattype,'userdata');

switch thisformat
    case 'xls'
        [name,path]=uiputfile('*.xls','Choose a destination','rave_data.xls');
        if name==0;return;end;
        name=ravemakevalidfilename(name);
        dest=fullfile(path,name);
        xlswrite(dest, allnames,1,'A1');
        xlswrite(dest, alldata,1,'A2');
        
    case 'mat'
        [name,path]=uiputfile('*.mat','Choose a destination','rave_data.mat');
        if name==0;return;end;
        name=ravemakevalidfilename(name);
        dest=fullfile(path,name);
        data=alldata;
        column_names=allnames;
        save(dest,'data','column_names');
        
    case 'tab'
        [name,path]=uiputfile('*.txt','Choose a destination','rave_data.txt');
        if name==0;return;end;
        name=ravemakevalidfilename(name);
        dest=fullfile(path,name);
        
        if name==0;return;end;
        
        fid=fopen(dest,'wt');
        for t=1:numel(allnames)-1
            fprintf(fid,'%s\t',allnames{t});
        end
        fprintf(fid,'%s\n',allnames{end});
        fclose(fid);
        dlmwrite(dest,alldata,'-append','delimiter','\t','newline','pc','precision',8);
        
    case 'csv'
        [name,path]=uiputfile('*.csv','Choose a destination','rave_data.csv');
        if name==0;return;end;
        name=ravemakevalidfilename(name);
        dest=fullfile(path,name);
        
        fid=fopen(dest,'wt');
        for t=1:numel(allnames)-1
            fprintf(fid,'%s,',allnames{t});
        end
        fprintf(fid,'%s\n',allnames{end});
        fclose(fid);
        dlmwrite(dest,alldata,'-append','delimiter',',','newline','pc');
        
end

ravefinish

