function optimizetab(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ravechangetab(src,[]);
handles=guidata(src);
set(handles.infobar,'string','');
handles.activetab='optimize';
guidata(src,handles);

%move front tabs to front if necessary
set(handles.fronttabaxes,'position',handles.backtabposition)
set(handles.backtabaxes,'position',handles.fronttabposition)
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,0); %#ok<NASGU>
set(handles.fronttablines,'xdata',x,'ydata',y)


set(handles.infobar,'string','');

handles.opttemp.settings=cell(0);
handles.opttemp.filenames=cell(0);
handles.opttemp.batchname='';

delete(handles.tabobjects)

%raise this tab to top
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,3,1); %#ok<NASGU>
set(handles.backtablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');

udata=get(handles.fakeax(handles.activeindex),'userdata');
if ~isempty(udata)
    d=udata.view.dataset;
    a=udata.view.analysis;
else
    d=1;
    a=1;
end

%% make methods list:

if isempty(handles.optimizetabobjects) %reload optimizer folder structure on tab reset
    guidata(handles.fig,handles);
    raveupdateoptimizertypes(src,ev)
    handles=guidata(src);
end

optkeywords=fieldnames(handles.optimizerinfo);

for g=1:numel(optkeywords);
    optclass{g}=handles.optimizerinfo.(optkeywords{g}).class;
    longname{g}=handles.optimizerinfo.(optkeywords{g}).longname;
end

methodstring={'<Select algorithm from list>','-------------=SINGLE-OBJECTIVE=--------------',longname{strcmp(optclass,'single')},'--------------=MULTI-OBJECTIVE=---------------',longname{strcmp(optclass,'multi')},'--------------------=RANKING=-------------------',longname{strcmp(optclass,'ranking')}};
keywordstring={'','',optkeywords{strcmp(optclass,'single')},'',optkeywords{strcmp(optclass,'multi')},'',optkeywords{strcmp(optclass,'ranking')}};

if ~isempty(handles.optimizetabobjects)
    set(handles.optimizetabobjects,'visible','on')
    handles.tabobjects=handles.optimizetabobjects;
    guidata(handles.fig,handles);
    
    datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
    analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
    
    d=get(datasetmenu,'value');
    a=get(analysismenu,'value');
    
    set(datasetmenu,'string',handles.datasetnames)
    set(analysismenu,'string',handles.analysisnames)
    
    constraintmenu=findobj(handles.tabobjects,'tag','constraintmenu');
    activeconstraints=get(constraintmenu,'value');
    
    if isempty(handles.constrainttext{d}{a})
        conststring='No constraints have been created yet.';
    else
        conststring=[cellstr('') handles.constrainttext{d}{a}];
    end
    
    set(constraintmenu,'string',conststring,'value',activeconstraints);
    
    objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');
    objs=get(objectivemenu,'value');
    
    set(objectivemenu,'string',handles.allnames{d},'value',objs);
    
    if isempty(handles.activeindex)
        drawcheck=findobj(handles.tabobjects,'tag','drawcheck');
        set(drawcheck,'enable','off')
    end
    
    exportbutton=findobj(handles.tabobjects,'tag','exportbutton');
    if isempty(handles.optimizerresults)
        set(exportbutton,'enable','off')
    else
        set(exportbutton,'enable','on')
    end
    
    %allow animation if possible:
    anicheck=findobj(handles.tabobjects,'tag','anicheck');
    if isempty(udata) || udata.view.dataset~=d
        set(anicheck,'value',0,'enable','off')
    else
        set(anicheck,'enable','on') 
    end
    return
end


%% text headers
projecttext=text(.15-.10317,.97*1.0317-.0317,'Project','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
algorithmtext=text(.15-.10317,.85*1.0317-.0317,'Algorithm','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);
parameterstext=text(.15-.10317,.36,'Settings','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);


%% uicontrols

analysistext=text(.2-.10317,.895*1.0317-.0317,'Analysis:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');
datasettext=text(.2-.10317,.935*1.0317-.0317,'Data Set:','horizontalalignment','left',...
    'backgroundcolor','none',...
    'parent',handles.tabaxes,'fontsize',10,'fontname','arial','color',[0 0 0],'fontweight','bold');
analysismenu=uicontrol('position',[origin(1)+95 origin(2)+530+13 160 30],...
    'style','popupmenu','string',handles.analysisnames,'callback',@analysismenu_callback,...
    'backgroundcolor',[1 1 1],'tag','analysismenu','value',a);
datasetmenu=uicontrol('position',[origin(1)+95 origin(2)+556+13 160 30],...
    'style','popupmenu','string',handles.datasetnames,'callback',@datasetmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','value',d);

methodmenu=uicontrol('position',[origin(1)+40 origin(2)+464+22 215 30],...
    'style','popupmenu','string',methodstring,...
    'callback',@methodmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','methodmenu','userdata',keywordstring);

targetstext=text(.20-.10317,.77*1.0317-.0317,'Select Objective(s):','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
    'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

constraintstext=text(.20-.10317,.57*1.0317-.0317,'Select Constraints:','horizontalalignment','left',...
    'backgroundcolor',handles.colors.tabbg,'parent',handles.tabaxes,'fontsize',10,...
    'fontname','arial','color',[0 0 0],'fontweight','bold','backgroundcolor','none');

responsevars=handles.allnames{d};

objectivemenu=uicontrol('position',[origin(1)+40 origin(2)+370+20 215 80],...
    'style','listbox','string',responsevars,...
    'callback',@objectivemenu_callback,...
    'backgroundcolor',[1 1 1],'tag','objectivemenu','max',100);

editinputsbutton=uicontrol('position',[origin(1)+30 origin(2)+345+20 100 22],...
    'style','pushbutton','string','Edit Input Ranges',...
    'callback',@editinputs_callback,...
    'backgroundcolor',handles.colors.buttonbg,'tag','edittargets');

edittargetsbutton=uicontrol('position',[origin(1)+140 origin(2)+345+20 110 22],...
    'style','pushbutton','string','Edit Objective Goals',...
    'callback',@edittargets_callback,...
    'backgroundcolor',handles.colors.buttonbg,'tag','edittargets');

if isempty(handles.constrainttext{d}{a})
    conststring='No constraints have been created yet.';
else
    conststring=[cellstr('') handles.constrainttext{d}{a}];
end

constraintcheck=uicontrol('position',[origin(1)+30 origin(2)+322 226 22],...
    'style','checkbox','string','Enforce side constraints','callback',@drawcheck_callback,...
    'backgroundcolor',[1 1 1],'tag','constraintcheck','value',1,'fontsize',9,'enable','off');

constraintmenu=uicontrol('position',[origin(1)+40 origin(2)+240 215 80],...
    'style','listbox','string',conststring,...
    'callback',@constraintmenu_callback,...
    'backgroundcolor',[1 1 1],'tag','constraintmenu','max',100);

dat = {'',''};
cnames = {'Parameter','Value'};

jtable=javax.swing.JTable(dat,cnames); %initialize a table
javaObjectEDT(jtable);
j=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
[jscrollpane mtable]=javacomponent(j,[origin(1)+15 origin(2)+70 235 120],handles.fig);

%% Setting column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[120 100]; %some column widths

for c=1:2 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

%handles.tabflexgrid=jtable;
set(mtable,'tag','mtable');


%startbutton:
startbutton=uicontrol('position',[origin(1)+15 origin(2)+38 70 26],...
    'style','pushbutton','string','Start','backgroundcolor',handles.colors.buttonbg,...
    'callback',@startbutton_callback,'enable','off','tag','startbutton','fontsize',12,...
    'fontweight','bold','foregroundcolor',[0 .8 0],'enable','off');

continuebutton=uicontrol('position',[origin(1)+90 origin(2)+38 85 26],...
    'style','pushbutton','string','Continue','backgroundcolor',handles.colors.buttonbg,...
    'callback',@continuebutton_callback,'enable','off','tag','continuebutton','fontsize',12,...
    'fontweight','bold','foregroundcolor',[0 .8 0],'enable','off');

batchbutton=uicontrol('position',[origin(1)+180 origin(2)+38 70 26],...
    'style','pushbutton','string','Batch...','backgroundcolor',handles.colors.buttonbg,...
    'callback',@batchbutton_callback,'enable','off','tag','batchbutton','fontsize',12,...
    'fontweight','bold','foregroundcolor',[0 0 0],'enable','on','visible','on');


%% results:

anicheck=uicontrol('position',[origin(1)+30 origin(2)+70-8+150-15 226 22],...
    'style','checkbox','string','Animate Progress','callback',@anicheck_callback,...
    'backgroundcolor',[1 1 1],'tag','anicheck','value',0,'fontsize',9);

anisettings=uicontrol('position',[origin(1)+160 origin(2)+70-8+150-15 80 22],...
    'style','pushbutton','string','Settings','callback',@animationsettings_callback,...
    'backgroundcolor',[1 1 1],'tag','anisettings','value',0,'fontsize',9,'backgroundcolor',handles.colors.buttonbg,'enable','off');

exportbutton=uicontrol('position',[origin(1)+15 origin(2)+05 235 24],...
    'style','pushbutton','string','Save/Export Results','callback',@exportappenddata_callback,...
    'backgroundcolor',[1 1 1],'tag','exportbutton','value',0,'fontsize',12,...
    'backgroundcolor',handles.colors.buttonbg,'visible','on','enable','off',...
    'fontweight','bold');


%% save / load settings

savebutton=uicontrol('position',[origin(1)+190 origin(2)+168 16 16],'style','pushbutton','tag','savesettings',...
    'cdata',double(imread('smallsave.png','BackgroundColor',[0.9 0.9 0.9]))./255,'tooltip','Save Settings','callback',@savesettings);

loadbutton=uicontrol('position',[origin(1)+215 origin(2)+168 16 16],'style','pushbutton','tag','savesettings',...
    'cdata',double(imread('smallopen.png','BackgroundColor',[0.95 0.95 0.95]))./255,'tooltip','Load Settings','callback',@loadsettings);

handles.tabobjects=[projecttext algorithmtext parameterstext constraintstext methodmenu targetstext objectivemenu...
    analysistext analysismenu datasettext datasetmenu constraintmenu edittargetsbutton...
    anicheck anisettings startbutton continuebutton editinputsbutton mtable batchbutton exportbutton constraintcheck...
    savebutton loadbutton];

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function methodmenu_callback(src,ev)
handles=guidata(src);

keywordstring=get(src,'userdata');
thismethod=keywordstring{get(src,'value')};

startbutton=findobj(handles.tabobjects,'tag','startbutton');
batchbutton=findobj(handles.tabobjects,'tag','batchbutton');
objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');

if isempty(thismethod)
    set(handles.infobar,'string','This is a category. Choose again.');
    set([startbutton batchbutton],'enable','off');
    return
else
    set(handles.infobar,'string','');
end

set([startbutton batchbutton],'enable','on');

[displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,[],thismethod);

%limit selection of objectives based on optimizer class:
thisclass=handles.optimizerinfo.(thismethod).class;
if strcmp(thisclass,'single')
    if numel(get(objectivemenu,'value'))>1
        set(objectivemenu,'value',1)
    end
    set(objectivemenu,'max',1)
else
    set(objectivemenu,'max',100)
end

%remove parameters that the user can't edit:
removethese=ismember(settingnames,{'longname','class','enforcesideconstraints','allowconstraints','isstoppable','maxobjectives'});
settingnames=settingnames(~removethese);
displaynames=displaynames(~removethese);
settingvalues=settingvalues(~removethese);
isastring=isastring(~removethese);

%update table:
ravefilloptimizersettingstable(src,ev,displaynames,settingvalues)

%these are used by create_settings to parse the settings table;
handles.opttemp.settingnames=settingnames;
handles.opttemp.displaynames=displaynames;
handles.opttemp.isastring=isastring;

guidata(handles.fig,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function objectivemenu_callback(src,ev)
handles=guidata(src);
set(handles.infobar,'string','')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function constraintmenu_callback(src,ev)
handles=guidata(src);
set(handles.infobar,'string','')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function datasetmenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

anicheck=findobj(handles.tabobjects,'tag','anicheck');
if isempty(udata) || udata.view.dataset~=get(src,'value')
    set(anicheck,'value',0,'enable','off');
else
    set(anicheck,'enable','on');
end

analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
d=get(src,'value');
a=get(analysismenu,'value');

%update objective menu:
objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');
set(objectivemenu,'string',handles.allnames{d},'value',1);

%update constraint menu:
if isempty(handles.constrainttext{d}{a})
    conststring='No constraints have been created yet.';
else
    conststring=[cellstr('') handles.constrainttext{d}{a}];
end
constraintmenu=findobj(handles.tabobjects,'tag','constraintmenu');
set(constraintmenu,'string',conststring,'value',1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

drawcheck=findobj(handles.tabobjects,'tag','drawcheck');
anicheck=findobj(handles.tabobjects,'tag','drawcheck');
if ~isempty(udata) || udata.view.analysis~=get(src,'value')
    set(drawcheck,'value',0,'enable','off')
    set(anicheck,'value',0,'enable','off');
else
    set(drawcheck,'enable','on')
    set(anicheck,'enable','on');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function editinputs_callback(src,ev)
handles=guidata(src);
set(0,'userdata',get(handles.fig,'position'));

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

hhh=ravechangerangesgui(d,a,handles);
uiwait(hhh);
newhandles=get(0,'userdata');
if ~isstruct(newhandles);return;end
handles=newhandles;
guidata(handles.fig,handles);

set(0,'userdata',[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function edittargets_callback(src,ev)
handles=guidata(src);
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');

ravedefinetargetsgui(src,d);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function anicheck_callback(src,ev)
handles=guidata(src);
anisettings=findobj(handles.tabobjects,'tag','anisettings');

if get(src,'value')
    set(anisettings,'enable','on')
else
    set(anisettings,'enable','off')
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function exportappenddata_callback(src,ev)
handles=guidata(src);
%prompt for options:
set(0,'userdata',get(handles.fig,'position'));

exportappenddatagui(handles.fig,[]); %note, MUST call with handles.fig

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function animationsettings_callback(src,ev)
handles=guidata(src);
%prompt for options:
set(0,'userdata',get(handles.fig,'position'));

hhh=raveeditpreferencesgui(handles.fig,[],'optanimation');

uiwait(hhh);

saveraveprefs(handles.fig);



%{
function exportdata_callback(src,ev,settings)
%this function is used to save results when optimizing in Batch mode.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

isbatch=~isempty(handles.opttemp.batchname);

datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
d=get(datasetmenu,'value');

if isfield(handles.temp,'finalpop')
    if isempty(handles.temp.finalpop)
        return
    end
else
    return
end

%prompt for file name:
if ~isbatch
    filename=makefilename(src,ev,[]);
    [filename path]=uiputfile('*.txt','Choose a destination',filename);
    if filename==0;return;end
    batchpath='';
else
    filename=handles.temp.filename;
    path=cd;
    batchpath=handles.temp.batchname{1};
end

dest=fullfile(path,batchpath,filename);
if ~isdir(fullfile(path,batchpath))
    mkdir(fullfile(path,batchpath));
end
%make string of names:

allnames=handles.allnames{udata.view.dataset};
alldata=[handles.temp.finalpop];

%evaluate population for all responses:
index=find(handles.isfunction{d});

outs=raveevaluatefunction(src,d,index,alldata,handles);
alldata(:,index)=outs;


fid=fopen(dest,'wt');
for t=1:numel(allnames)-1
    fprintf(fid,'%s\t',allnames{t});
end
fprintf(fid,'%s\n',allnames{end});


fclose(fid);
dlmwrite(dest,alldata,'-append','delimiter','\t','newline','pc','precision',8);

%export settings:
%
% [pathstr, name, ext] = fileparts(dest);
% settingsdest=fullfile(pathstr,[name '.settings' ext]);
%
% fid=fopen(settingsdest,'wt');
% fprintf(fid,'%s\t',allnames{t});
%
% fclose(fid);

ravefinish;
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function startbutton_callback(src,ev)
handles=guidata(src);

mtable=findobj(handles.tabobjects,'tag','mtable');
jscroll=get(mtable,'javapeer');
jv=jscroll.getViewport;
jtable=jv.getView;
ce=jtable.getCellEditor;
if ~isempty(ce)
    ce.stopCellEditing;    
end

optimize_startbutton_callback(src,ev)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function continuebutton_callback(src,ev)
handles=guidata(src);

if isempty(handles.opttemp.batchname) %single run
    
    settings=create_settings(src,ev);
    handles.opttemp.settings=settings;
    handles.opttemp.batchname='';
    
    if ~handles.prefs.keepalloptimizerresults
        handles.optimizerresults=cell(0);
    end
    
    guidata(handles.fig,handles);
    do_optimizer(src,ev,settings);
    
else %batch mode
    handles.opttemp.batchname=inputdlg('Name this batch. Results will be saved in a folder with this name.');
    if isempty(handles.opttemp.batchname)
        errordlg('You must enter a name for the batch.')
    end
    
    guidata(src,handles);
    for j=1:numel(handles.opttemp.settings)
        handles.opttemp.filename=handles.opttemp.filenames{j};
        guidata(src,handles);
        settings=handles.opttemp.settings{j};
        do_optimizer(src,ev,settings);
        exportdata_callback(src,ev);
    end
end

txtcheck=findobj(handles.tabobjects,'tag','txtcheck');
if get(txtcheck,'value')
    ravesendtxtmessage(handles.fig,[],'Your optimizer has finished.');
end

if isjava(handles.trayIcon)
    trayIconClasses = handles.trayIcon.getClass.getClasses;
    MessageTypes = trayIconClasses(1).getEnumConstants;
    handles.trayIcon.displayMessage('Rave','Your optimizer has finished.',MessageTypes(3));
end

handles=guidata(handles.fig);
handles.opttemp.batchname='';
guidata(handles.fig,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function batchbutton_callback(src,ev)
handles=guidata(src);
set(handles.infobar,'string','Batch mode has been disabled temporarily while we remodel...');



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function filename=makefilename(src,ev,settings)
%make default file name for saving the results of this optimizer
handles=guidata(src);

if isempty(settings) %if not called from batch mode...
    
    methods=get(findobj(handles.tabobjects,'tag','methodmenu'),'string');
    settings.method=methods{get(findobj(handles.tabobjects,'tag','methodmenu'),'value')};
end

switch settings.method
    case {'NSGA-II','NSGA-II + Local Search'}
        allobjs=get(findobj(handles.tabobjects,'tag','objectivemenu'),'string');
        objs=allobjs(get(findobj(handles.tabobjects,'tag','objectivemenu'),'value'));
        
        for o=1:numel(objs)
            objs{o}=[objs{o},', '];
        end
        
        objs=objs';
        objs=cell2mat(objs);
        
        filename=objs;
        filename=filename(:)';
        
        mtable=findobj(handles.tabobjects,'type','hgjavacomponent');
jscroll=get(mtable,'javapeer');
jtable=jscroll.getViewport.getComponent(0);
        
        
        
        pop=jtable.getValueAt(0,1);
        gens=jtable.getValueAt(1,1);
        filename=['NSGA-II ',filename,pop,'p ',gens,'g.txt'];
        
    otherwise
        filename='optimizer_results.txt';
        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function savesettings(src,ev)
%create settings structure for optimizers.
handles=guidata(src);

mtable=findobj(handles.tabobjects,'type','hgjavacomponent');
jscroll=get(mtable,'javapeer');
jtable=jscroll.getViewport.getComponent(0);

methodmenu=findobj(handles.tabobjects,'tag','methodmenu');

allmethods=get(methodmenu,'userdata');
thismethod=allmethods{get(methodmenu,'value')};

displaynames=handles.opttemp.displaynames;

%prompt for where to put it:
[name,path]=uiputfile('*.txt','Choose a destination','opt_settings.txt');
if name==0;return;end;
name=ravemakevalidfilename(name);
dest=fullfile(path,name);
fid=fopen(dest,'wt');

fprintf(fid,'%s\n',['%#',thismethod,' <name these settings here>']);
for g=1:numel(displaynames);
    thisvalue=jtable.getValueAt(g-1,1);
    fprintf(fid,'%s = %s\n',displaynames{g},thisvalue);
end

fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function loadsettings(src,ev)
handles=guidata(src);

mtable=findobj(handles.tabobjects,'type','hgjavacomponent');
jscroll=get(mtable,'javapeer');
jtable=jscroll.getViewport.getComponent(0);

methodmenu=findobj(handles.tabobjects,'tag','methodmenu');

allmethods=get(methodmenu,'userdata');
thismethod=allmethods{get(methodmenu,'value')};

%prompt for file:
[name,path]=uigetfile('.txt','Select a settings file');

if name==0;return;end;
dest=fullfile(path,name);
fid=fopen(dest,'r');

tline=fgetl(fid);
linez={};

while ischar(tline)
    linez{end+1}=tline;
    tline=fgetl(fid);
end

fclose(fid);

%look for header lines:
headers=[];

for t=1:numel(linez)
   if length(linez{t})>=2 && strcmp(linez{t}(1:2),'%#')
       headers(end+1)=t;
   end
end

if isempty(headers)
    errordlg('No settings headers were found in this file. Headers must come at the beginning of a block of settings and must begin with ''%#'' followed by the optimizer''s keyword.','Invalid format');
    return
end

if numel(headers)>1
    warndlg('Optimization settings files with multiple headers are not yet supported. I will load the first header''s settings.')
end

[displaynames,settingvalues]=raveparseoptimizersettingsfile(handles,dest,1);

%compare with algorithm's settings:
algsettingnames=handles.opttemp.displaynames;

for l=1:numel(displaynames)
    [ishere,rownum]=ismember(displaynames{l},algsettingnames);
    if ~ishere;continue;end
    jtable.setValueAt(settingvalues{l},rownum-1,1)
end
