classdef varlistcontrol < hgsetget
    %An extension of uicontrol that offers more options for making controls
    %that list variable names. Intended to be used with the listbox and
    %popupmenu styles of uicontrol. Offers options to easily change the
    %order in which the list items are listed without affecting the values
    %returned by the control. 
    
    properties
        uipeer=[];
        displayOrder=[]; %a vector of length == length(get(this,'string')) indicating which order the elements of "String" should be displayed in on the underlying uicontrol.
        returnValue = []; %a vector of length == length(get(this,'string')) indicating which "Value" should be returned by get(this,'value')
        value=[];
    end

    properties (Dependent = true)
        globalValue
    end

    methods (Static = true)
        function [uipairs,pairs] = parsePvPairs(varargin)
            %determine which properties in varargin are builtin uicontrol
            %properties, and which are varlistcontrol properties
            
            %split into names/values:
            propNames = varargin(1:2:end)';
            propValues = varargin(2:2:end)';
            
            if numel(propNames)~=numel(propValues)
                error('Invalid property/value pairs')
            end
            
            %get list of properties for uicontrol:            
            uicontrolProps = {'BackgroundColor','BeingDeleted','BusyAction','ButtonDownFcn','Callback',...
                'CData','Children','Clipping','CreateFcn','DeleteFcn','Enable','Extent',...
                'FontAngle','FontName','FontSize','FontUnits','FontWeight','ForegroundColor',...
                'HandleVisibility','HitTest','HorizontalAlignment','Interruptible','KeyPressFcn',...
                'ListboxTop','Max','Min','Parent','Position','Selected','SelectionHighlight','SliderStep',...
                'String','Style','Tag','TooltipString','Type','UIContextMenu','Units','UserData','Value','Visible'};

            isUicontrolProp = ismember(lower(propNames),lower(uicontrolProps));
            
            %send the uicontrol props to the builtin set function for
            %uicontrols:
            uipairs = [propNames(isUicontrolProp) propValues(isUicontrolProp)]';
            uipairs = uipairs(:)';
%           builtin(@set,obj.uipeer,uipairs{:});
            
            %Deal with the non-uicontrol pairs:
            pairs = [propNames(~isUicontrolProp) propValues(~isUicontrolProp)]';
            pairs = pairs(:)';
        end
    end
    
    
    methods 
        function obj = varlistcontrol(varargin)
            [uipairs,pairs] = obj.parsePvPairs(varargin{:});
            obj.uipeer = uicontrol(uipairs{:});
            set(obj,pairs{:});
        end
        
        function set(obj,varargin)
            %Split up uicontrol pv pairs from varlist pvpairs:
            [uipairs,pairs] = obj.parsePvPairs(varargin{:});
            
            %Set the uicontrol pairs:
            builtin('set',obj.uipeer,uipairs{:});
            
            %Set everything else:
            
            
            
            
            
        end
        
        function val = get.value(obj)
            val = get(obj.uipeer,'value');
        end
        
        
    end
    
    methods (Access = protected)

    end
end